package jcreek;

/**
 * <p>Title: Launcher</p>
 * <p>Description: Default application to run in the JavaCreek.jar file. Launches the other applications.</p>
 * <p>Company: NTNU, IDI, DIS</p>
 * @author Frode Srmo
 * @version 1.0
 */

import javax.swing.*;
import java.awt.*;
import java.awt.image.ImageObserver;
import java.net.URL;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class Launcher implements ActionListener, WindowListener
{
  JFrame mainWin;
  JButton cliButton;
  JButton editorButton;
  JButton exitButton;
  JButton wekaButton;
  Vector processes = new Vector();

  public Launcher()
  {
    initLauncher();
  }

  public void initLauncher()
  {
    mainWin = new JFrame("Creek Launcher");
    JPanel panel = new JPanel();
    panel.setLayout(new BorderLayout());

    URL imageURL = getClass().getClassLoader().getResource("jcreek/cke/resources/LauncherImage.jpg");
    Image img = Toolkit.getDefaultToolkit().createImage(imageURL);

    MediaTracker tracker = new MediaTracker(mainWin);
    tracker.addImage(img, 1);
    try
    {
      tracker.waitForAll();
    }
    catch(InterruptedException e)
    {
    }

    panel.add(new ImageCanvas(img, mainWin), BorderLayout.CENTER);

    cliButton = new JButton("Command Line Interface");
    cliButton.addActionListener(this);
    editorButton = new JButton("Knowledge Editor");
    editorButton.addActionListener(this);
    wekaButton = new JButton("WEKA");
    wekaButton.addActionListener(this);
    exitButton = new JButton("Exit");
    exitButton.addActionListener(this);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new GridLayout(6,1));


    String version = "unknown version";
    String time = "";
    Properties versionProperties = new Properties();
    try
    {
      versionProperties.load(getClass().getResourceAsStream(
          "/jcreek/resources/version.properties"));
      version = versionProperties.getProperty("name")+" "+versionProperties.getProperty("branch")+" version "+
          versionProperties.getProperty("version")+" build "+versionProperties.getProperty("build.number");
      time = "Built "+versionProperties.getProperty("build.time");
    }
    catch(IOException e)
    {
      e.printStackTrace();
    }

    Font labelFont = new Font("Verdana;Arial;Helvetica", Font.PLAIN, 10);
    buttonPanel.setBackground(Color.black);
    JLabel versionLabel = new JLabel(version);
    versionLabel.setFont(labelFont);
    versionLabel.setForeground(Color.gray);
    versionLabel.setBorder(BorderFactory.createEmptyBorder(2,4,0,4));
    buttonPanel.add(versionLabel);
    JLabel timeLabel = new JLabel(time);
    timeLabel.setFont(labelFont);
    timeLabel.setForeground(Color.gray);
    timeLabel.setBorder(BorderFactory.createEmptyBorder(0,4,2,4));
    buttonPanel.add(timeLabel);
    buttonPanel.add(editorButton);
    buttonPanel.add(cliButton);
    buttonPanel.add(wekaButton);
    buttonPanel.add(exitButton);


//    buttonPanel.add(new JLabel

    panel.add(buttonPanel, BorderLayout.SOUTH);

    mainWin.setContentPane(panel);
    mainWin.setSize(205,320);
    mainWin.setLocation((mainWin.getGraphicsConfiguration().getBounds().width-205)/2, (mainWin.getGraphicsConfiguration().getBounds().height-260)/2);
    mainWin.setResizable(false);
    mainWin.show();
    mainWin.addWindowListener(this);


//    System.out.println(System.getProperty("java.class.path"));

    Runtime.getRuntime().addShutdownHook(new ProcessShutdown(this));

  }

  protected boolean checkForSDK()
  {
    boolean SDKfound = false;
    try
    {
      Class compiler = Class.forName("sun.tools.javac.Main");
      SDKfound = true;
    }
    catch(ClassNotFoundException e)
    {
    }

    if(SDKfound)
    {
      return true;
    }

    String javaHome = System.getProperty("java.home");
    File tools = new File(javaHome, "../lib/tools.jar");
    if(tools.exists())
    {
      return tryAddClasspath(tools);
    }
    else
    {
      tools = new File(javaHome, "lib/tools.jar");
      if(tools.exists())
      {
        return tryAddClasspath(tools);
      }
    }

    return false;
  }

  private boolean tryAddClasspath(File f)
  {
    try
    {
      String fileString = f.getCanonicalPath();
      String classPath = System.getProperty("java.class.path");
      if(classPath != null && classPath.toLowerCase().indexOf(fileString) == -1)
      {
        System.setProperty("java.class.path", fileString+File.pathSeparator+classPath);
        return true;
      }
    }
    catch(IOException e)
    {
      e.printStackTrace();
    }
    return false;
  }

  public static void main(String argv[])
  {
    new Launcher();
  }

  public void actionPerformed(ActionEvent e)
  {
    if(e.getSource() == exitButton)
      closeApplication();
    else if(e.getSource() == cliButton)
      openCLIApplication();
    else if(e.getSource() == editorButton)
      openEditorApplication();
    else if(e.getSource() == wekaButton)
      openWekaApplication();
  }

  public void cleanProcesses()
  {
    for(int i=processes.size()-1;i>=0;i--)
    {
      Process proc = (Process) processes.elementAt(i);
      proc.destroy();
    }
    processes = new Vector();
  }

  public void closeApplication()
  {
    System.exit(0);
  }


  public void openCLIApplication()
  {
    String java = findJavaExecutable();
    if(java != null)
    {
      try
      {
        Process proc = Runtime.getRuntime().exec( new String[] {
              java,
              "-classpath",
              System.getProperty("java.class.path"),
              "jcreek.cli.CreekJatha" });
        new TransferInputStream(proc.getErrorStream(),System.err);
        new TransferInputStream(proc.getInputStream(),System.out);

        processes.addElement(proc);
      }
      catch(java.io.IOException e)
      {
        e.printStackTrace();
      }
    }
    else
    {
      System.out.println("Failed to find Java executable.");
    }
//    new ApplicationThread(javacreek.gui.DemoApplet.class);
//    mainWin.hide();
  }

  public void openWekaApplication()
  {
    String java = findJavaExecutable();
    if(java != null)
    {
      try
      {
        Process proc = Runtime.getRuntime().exec( new String[] {
              java,
              "-classpath",
              System.getProperty("java.class.path"),
              "weka.gui.GUIChooser" });
        new TransferInputStream(proc.getErrorStream(),System.err);
        new TransferInputStream(proc.getInputStream(),System.out);

        processes.addElement(proc);
      }
      catch(java.io.IOException e)
      {
        e.printStackTrace();
      }
    }
    else
    {
      System.out.println("Failed to find Java executable.");
    }
//    new ApplicationThread(javacreek.gui.DemoApplet.class);
//    mainWin.hide();
  }


  private String findJavaExecutable()
  {
    String path = System.getProperty("java.home")+File.separator+"bin"+File.separator;
    File java = new File(path+"javaw.exe");
    if(java.exists())
      return java.getAbsolutePath();

    java = new File(path+"javaw");
    if(java.exists())
      return java.getAbsolutePath();

    java = new File(path+"java.exe");
    if(java.exists())
      return java.getAbsolutePath();

    java = new File(path+"java");
    if(java.exists())
      return java.getAbsolutePath();

    return null;
  }

  public void openEditorApplication()
  {
    String java = findJavaExecutable();
    if(java != null)
    {
      try
      {
        Process proc = Runtime.getRuntime().exec( new String[] {
              java,
              "-classpath",
              System.getProperty("java.class.path"),
              "jcreek.cke.Main" });
        new TransferInputStream(proc.getErrorStream(),System.err);
        new TransferInputStream(proc.getInputStream(),System.out);

        processes.addElement(proc);
      }
      catch(java.io.IOException e)
      {
        e.printStackTrace();
      }
    }
    else
    {
      System.out.println("Failed to find Java executable.");
    }
//    new ApplicationThread(javacreek.browser.Visualizer.class);
//    mainWin.hide();
  }

  public void windowDeactivated(WindowEvent e)
  {
  }

  public void windowActivated(WindowEvent e)
  {
  }

  public void windowClosed(WindowEvent e)
  {
  }

  public void windowClosing(WindowEvent e)
  {
    closeApplication();
  }

  public void windowOpened(WindowEvent e)
  {
  }

  public void windowDeiconified(WindowEvent e)
  {
  }

  public void windowIconified(WindowEvent e)
  {
  }
  class ImageCanvas extends JComponent
  {
    Image img;
    ImageObserver obs;
    ImageCanvas(Image img, ImageObserver obs)
    {
      this.img = img;
      this.obs = obs;
      setPreferredSize(new Dimension(img.getWidth(obs), img.getHeight(obs)));
    }

    public void paint(Graphics g)
    {
      g.drawImage(img,0,0,obs);
    }

  }

  class ProcessShutdown extends Thread
  {
    Launcher processKeeper;
    ProcessShutdown(Launcher l)
    {
      processKeeper = l;
    }

    public void run()
    {
      processKeeper.cleanProcesses();
    }
  }

  class TransferInputStream implements Runnable
  {
    InputStream in;
    OutputStream out;
    Thread transfer;
    TransferInputStream(InputStream in, OutputStream out)
    {
      this.in = in;
      this.out = out;
      transfer = new Thread(this);
      transfer.start();
    }

    public void run()
    {
      try
      {
        while(true)
        {
          while(in.available() > 0)
            out.write(in.read());
          try
          {
            Thread.sleep(500);
          }
          catch(InterruptedException e) { }
        }
      }
      catch(IOException e)
          { }
    }
  }
}

