package jcreek.cke;

import java.io.*;
import java.net.*;
import java.awt.Toolkit;
import java.awt.Dimension;
import javax.swing.JWindow;
import javax.swing.UIManager;
import jcreek.cke.visualizer.AppSettings;
import jcreek.cke.controls.SplashScreen;
//import com.oyoaha.swing.plaf.oyoaha.*;
//import com.incors.plaf.kunststoff.KunststoffLookAndFeel;

public class Main
{
    /**
     * Suffix added to file type names.
     */
    protected static String typeSuffix = "Type";

  /**
     * Main method for Creek program. Gets things started. Note that
     * this returns to the JVM but the app still runs. It is an event-
     * driven application and thus won't exit until we call
     * System.exit().
     *
     * @param  args  array of command-line arguments
     */
    public static void main( String[] args )
    {
        // Display Splash Screen
         SplashScreen splash = new SplashScreen();
         splash.advance();
        // Create a new Vizualizer object. This may fail
        // and cause a system exit.
        MainWindow mainWin = MainWindow.instanceOf();

        // Load the user preferences.
        AppSettings props = AppSettings.instanceOf();
        // xxx - reads from the working directory
        props.load( "creek.ini" );

        // Set the look & feel of the program to what the user
        // had last selected.
        try {

            String lafName = props.getString( AppSettings.LOOK_FEEL );
            if ( lafName != "" ) {
                UIManager.setLookAndFeel( lafName );
            } else {
                // If nothing saved, use cross-platform look & feel.

                UIManager.setLookAndFeel
                    ( UIManager.getCrossPlatformLookAndFeelClassName() );

/*
                UIManager.setLookAndFeel
                    ( new KunststoffLookAndFeel() );
*/
                splash.advance();

/*
                OyoahaLookAndFeel lnf = new OyoahaLookAndFeel();
                URL url = mainWin.getResource("lookAndFeelTheme");
                lnf.setOyoahaTheme(url);
                UIManager.setLookAndFeel( lnf );
*/

            }
        } catch ( Exception e ) {
            System.err.println( "ERROR: Couldn't load look & feel: " + e );
            if(splash != null)
                splash.dispose();
        }

        // Instantiate View document group so it has a chance
        // to add the undo/redo actions to the command table.
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        splash.advance();
        // Register all the supported document types.
        String[] fileTypes = MainWindow.tokenize
            ( mainWin.getResourceString( "fileTypes" ) );
        for ( int i = 0; i < fileTypes.length; i++ ) {
            try {
                String type = mainWin.getResourceString
                    ( fileTypes[i] + typeSuffix );
                Class fileType = Class.forName
                    ( "jcreek.cke.document." + type );
                dg.registerkm( fileTypes[i], fileType );
                splash.advance();
            } catch ( Exception e ) {
                System.err.println( "ERROR: Couldn't create file type: "
                                    + fileTypes[i] + ": " + e );
                if(splash != null)
                    splash.dispose();

            }
        }
        String[] docTypes = MainWindow.tokenize
            ( mainWin.getResourceString( "docTypes" ) );
        for ( int i = 0; i < docTypes.length; i++ ) {
            try {
                String type = mainWin.getResourceString
                    ( docTypes[i] + typeSuffix );
                Class docType = Class.forName
                    ( "jcreek.cke.document." + type );
                dg.registerDocument( docTypes[i], docType );
                splash.advance();
            } catch ( Exception e ) {
                System.err.println( "ERROR: Couldn't create file type: "
                                    + fileTypes[i] + ": " + e );
                if(splash != null)
                    splash.dispose();
            }
        }
    //check window then chech document group
    // Build and show a window to begin user interaction.
        mainWin.newWindow();
        if(splash != null)
            splash.dispose();

    } // main
}
