package jcreek.cke.browser;

import jcreek.cke.document.CreekDocument;
import jcreek.cke.command.CreekCommand;

import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.*;


/**
 * Our specialized undo manager that adds functionality to
 * remove undoable edits that are associated with a specific
 * Creek document.
 *
 * @author
 * @version
 */
public class CreekUndoManager extends UndoManager {

    /**
     * Creates a new CreekUndoManager object.
     */
    public CreekUndoManager() {
        super();
    } // CreekUndoManager

    /**
     * Tell the edits associated with the old document that they
     * should reattach themselves to the new document.
     *
     * @param  oldDoc  previous document
     * @param  newDoc  new document
     */
    public void reattachEdits( CreekDocument oldDoc,
                               CreekDocument newDoc ) {
        int size = edits.size();
        for ( int i = 0; i < size; i++ ) {
            Object e = edits.elementAt( i );
            if ( e instanceof CreekCommand ) {
                CreekCommand c = (CreekCommand)e;
                if ( c.getDocument() == oldDoc ) {
                    // set command's document to new one
                    c.setDocument( newDoc );
                }
            }
        }
    } // reattachEdits

    /**
     * Removes all edits associated with the given document.
     *
     * @param  doc  document for which all edits should be removed
     */
    public void trimEdits( CreekDocument doc ) {
        int size = edits.size();
        // Remove the edits in reverse order, of course.
        for ( int i = size - 1; i >= 0; i-- ) {
            Object e = edits.elementAt( i );
            if ( e instanceof CreekCommand &&
                 ((CreekCommand)e).getDocument() == doc ) {
                // remove this command from the queue
                trimEdits( i, i );
            }
        }
    } // trimEdits
} // CreekUndoManager