/*********************************************************************
 *
 * DESCRIPTION:
 *      This file defines the main window of the application, along
 *      with classes the support the window.
 *
 *
 ********************************************************************/

package jcreek.cke.browser;

import jcreek.cke.MainWindow;
import jcreek.cke.tool.*;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.visualizer.AppSettings;
import jcreek.cke.visualizer.EntityList;
import jcreek.cke.visualizer.ScrollView;
import jcreek.cke.visualizer.TreeView;
import jcreek.cke.visualizer.ViewportScroller;
import jcreek.cke.visualizer.FrameView;
import jcreek.cke.visualizer.StatusBar;
import jcreek.cke.visualizer.PropertyControl;
import jcreek.cke.visualizer.MultiDeskTopPane;
import jcreek.cke.visualizer.MultiInternalFrame;
import jcreek.cke.visualizer.FrameProperty;
import jcreek.representation.Entity;
import jcreek.cke.controls.SideBar;
import jcreek.cke.conceptview.*;

import java.awt.*;
import java.awt.event.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.net.URL;
import java.util.Hashtable;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.InternalFrameListener;
import javax.swing.event.InternalFrameEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.datatransfer.*;
import java.awt.dnd.*;
import java.util.ArrayList;

/**
 * This class implements that main window of the application. There is
 * typically only one of these in each session. It acts as a
 * window listener for the internal windows it creates, notifying
 * the document group when windows are closed and when they are
 * activated.
 *
 * @author
 * @version
 */

public class Visualizer extends JFrame implements InternalFrameListener, WindowListener
{

    /**
     * Common Area to hold Standard components
     * or display Status Messages
     */
     protected StatusBar sb;


     protected PropertyControl pc;

     protected JInternalFrame propWin;
    /**
     * Indicates the position of the mouse over a canvas.
     * @serial
     */
    protected MousePositionIndicator mpi;

    /**
     * Holds all the internal windows within the main window.
     * @serial
     */
    protected JDesktopPane dtp;

//    protected JSplitPane entityView;

    protected EntityList findEnt; //= new EntityList();



    /**
     * Holds a tree view of all the available views
     * It allows to switch between the different views
     * @serial
     */
    protected TreeView treeView;

    protected FrameView fView;

    public JInternalFrame toolWin;
    /**
     * Holds the names of the internal windows. Allows switching
     * between available document windows.
     * @serial
     */
    protected WindowMenu winMenu;


    protected ViewMenu viewMenu;
    /**
     * Our window's tool manager. Builds and manages the tools.
     * @serial
     */
    protected ToolManager toolMan;
    /**
     * Suffix added to command strings to find images.
     * @serial
     */
    public static String imageSuffix = "Image";
    /**
     * Suffix added to command strings to find labels.
     * @serial
     */
    public static String labelSuffix = "Label";
    /**
     * Suffix added to command strings to find tooltips.
     * @serial
     */
    public static String tipSuffix = "Tooltip";
    /**
     * Suffix added to command strings to find mnemonics.
     * @serial
     */
    public static String memSuffix = "Mem";
    /**
     * Suffix added to command strings to find accelerator.
     * @serial
     */
    public static String accelSuffix = "Accel";
    /**
     * Suffix added to command strings to find title.
     * @serial
     */
    public static String titleSuffix = "Title";

    private   ArrayList       dropTargets = null;

    //for tool menus
    private Hashtable toolMenu;

    //for tool menus
    private ButtonGroup toolGroup;


    //UI for SplitPane
    protected JScrollPane sPane;
    protected JSplitPane  splitter;
    protected JPanel propPane;
    protected JPanel content;
    protected SideBar sideBar;
    protected int divLocation=0;

    protected boolean isPropVisible = false;
    protected String  propKey;

    //for dockable Properties
    private Hashtable props;

    protected FrameProperty frameProperty =  new FrameProperty();

    /**
     * Creates a new Visualizer object and puts up the main window.
     * Also creates a window listener to close the program when the
     * close button is activated.
     *
     * @param  title  title for the main window
     */
    public Visualizer( String title ) {
        // Call superclass for default behavior.
        super( title );

        //initialize table to hold tool menus
        toolMenu = new Hashtable();

        toolGroup = new ButtonGroup();

        addWindowListener( this );

        // Let the ExitAction decide when to close us.
        setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );

        // Set the size and position of the main window.
//        setSize( 600, 500 );
        setSize(Toolkit.getDefaultToolkit().getScreenSize());
	      setLocation( 0, 0 );
            dtp = new MultiDeskTopPane();
//            dtp.putClientProperty("JDesktopPane.dragMode","outline");
            newToolWindow("Tool Window");

        try {

            // Create a menubar and add all the menus to it.
            JMenuBar mb = createMenubar();
            MainWindow mw = MainWindow.instanceOf();
            winMenu = new WindowMenu( mw.getResourceString
                                      ( "window"+labelSuffix ),
                                      (MultiDeskTopPane)dtp );
            mb.add( winMenu );
            mb.add(createMenu("help"));
            setJMenuBar( mb );

            // Create a pane inside the main one. This will hold the
            // toolbar, which can float and occupy any border of the
            // inner pane. The desktop pane will go in the middle.
            Container pane = getContentPane();
            content = new JPanel( new BorderLayout() );
            pane.add( content, "Center" );

            // Add the toolbar to the window.
            JToolBar tbar = createToolbar();
            tbar.setOrientation(SwingConstants.HORIZONTAL);
            content.add( tbar, "North" );
            toolMan = new ToolManager( tbar,toolMenu,toolWin );

            // Create the desktop pane that will manage the multiple
            // internal windows. Put it in the center of the inner
            // pane, so the toolbar floats around it.
            sPane = new JScrollPane(dtp);
            propPane = new JPanel();
            createProps();
            sideBar = new SideBar(this);

            content.add( sPane, "Center" );
            content.add(sideBar,"East");


            // create the statusbar
            sb = new StatusBar();
            pane.add(sb,"South");

            // Create the mouse position indicator.
            mpi = new MousePositionIndicator(sb);

            // create the application icon

            setIconImage(Toolkit.getDefaultToolkit().createImage(getClass().getResource("App" + imageSuffix)));

            URL url = mw.getResource( "App" + imageSuffix );
            if(url != null)
                this.setIconImage((new ImageIcon(url)).getImage());

            dropTargets = new ArrayList();

        } catch( Exception exc ) {
            // Catch any and all exceptions. This allows our
            // caller to still invoke "show" on us. That way,
            // even if we failed to build out the UI the user
            // can close the program via the window close button.
            System.err.println( exc );
            exc.printStackTrace();
        }
    } // Visualizer

    /**
     * Takes a component and centers it within the main window.
     * Usually used for displaying child dialogs.
     *
     * @param  c  component to be centered
     */
    public void centerComponent( Component c ) {
        Rectangle winBounds = getBounds();
        Rectangle cBounds = c.getBounds();
        c.setLocation( ( winBounds.width - cBounds.width ) / 2 +
                       winBounds.x,
                       ( winBounds.height - cBounds.height ) / 2 +
                       winBounds.y );
    } // centerComponent

    /**
     * Closes the current window. This will remove the window from
     * the desktop pane.
     */
    public void closeWindow() {
        JInternalFrame fr = findSelected();
        if ( fr == null ) {
            System.err.println( "Can't close window if not selected." );
            return;
        }
        // we don't want further events being sent out
        fr.removeInternalFrameListener( this );
        fr.dispose();
        // set the frame's layer to non-default so it is not
        // included in the window menu when it is rebuilt
        fr.setLayer( JLayeredPane.DRAG_LAYER );
        winMenu.buildList( dtp.getAllFramesInLayer
                           ( JLayeredPane.DEFAULT_LAYER.intValue() ) );
    } // closeWindow

    public void closeWindow(JInternalFrame fr) {
        if ( fr == null ) {
            return;
        }
        // we don't want further events being sent out
        fr.removeInternalFrameListener( this );
        fr.dispose();
        // set the frame's layer to non-default so it is not
        // included in the window menu when it is rebuilt
        fr.setLayer( JLayeredPane.DRAG_LAYER );
        winMenu.buildList( dtp.getAllFramesInLayer
                           ( JLayeredPane.DEFAULT_LAYER.intValue() ) );
    } // closeWindow

    /**
     * Create a menu for the app. By default this pulls the
     * definition of the menu from the associated resource file.
     *
     * @param  key  menu key in resource bundle
     * @return  menu built out
     */
    protected JMenu createMenu( String key ) {
        MainWindow mw = MainWindow.instanceOf();
        String[] itemKeys = MainWindow.tokenize
            ( mw.getResourceString( key ) );
        JMenu menu = new JMenu( mw.getResourceString( key + labelSuffix ) );
        URL url = mw.getResource( key + imageSuffix );
        if ( url != null ) {
            menu.setHorizontalTextPosition( JButton.RIGHT );
            menu.setIcon( new ImageIcon( url ) );
        }
	String accel = mw.getResourceString(key+accelSuffix);
	if (accel != null) {
	  try {
	    int mask = 0;
	    if (accel.startsWith("CTRL")) {
	      mask += ActionEvent.CTRL_MASK;
	      accel = accel.substring(5);
	    }
	    if (accel.startsWith("SHIFT")) {
	      mask += ActionEvent.SHIFT_MASK;
	      accel = accel.substring(6);
	    }
	    if (accel.startsWith("ALT")) {
	      mask += ActionEvent.ALT_MASK;
	      accel = accel.substring(4);
	    }
	    int keyS = KeyEvent.class.getField("VK_"+accel).getInt(null);
	    menu.setAccelerator(KeyStroke.getKeyStroke(keyS, mask));
	  } catch (Exception e) {
	    // ignore
	  }
	}

	String mnemonic = mw.getResourceString(key+memSuffix);
	if (mnemonic != null && mnemonic.length() > 0)
	  menu.setMnemonic(mnemonic.toCharArray()[0]);

        int i = 0;
        while ( i < itemKeys.length ) {
            if ( itemKeys[i].equals( "-" ) ) {
                // A "-" means insert a separator.
                menu.addSeparator();
            } else if ( itemKeys[i].equals( ">" ) ) {
                // A ">" signals that the next key is a submenu name.
                menu.add( createMenu( itemKeys[++i] ) );
            } else if ( itemKeys[i].startsWith( "@" ) ) {
                // Item is a special menu.
                itemKeys[i] = itemKeys[i].substring( 1 );
                if ( itemKeys[i].equals( "lookAndFeel" ) ) {
                    // User wants the look & feel menu.
                    menu.add( new LookAndFeelMenu
                              ( mw.getResourceString( "lookAndFeelLabel" ),
                                this ) );
                }

                if ( itemKeys[i].equals( "layout" ) )
                {
                    // User wants the look & feel menu.
                    menu.add( createCheckBoxMenuItem( itemKeys[i]));
                }
                if(itemKeys[i].equals("viewList"))
                {
                    viewMenu = new ViewMenu
                              ( mw.getResourceString( "viewList"+ labelSuffix));
                    // User wants the view menu.
                    menu.add( viewMenu);
                }
                if((itemKeys[i].equals("MoveTool"))||
                   (itemKeys[i].equals("ZoomTool"))||
                   (itemKeys[i].equals("BuildTool")))
                {
                    // User wants the tool menu.
                    menu.add( createRadioButtonMenuItem( itemKeys[i]));
                }

            } else {
                // A normal menu item...
                menu.add( createMenuItem( itemKeys[i] ) );
            }
            i++;
        }
        return menu;
    } // createMenu

    /**
     * Create the menubar for the app. By default this pulls the
     * definition of the menu from the associated resource file.
     *
     * @return  menu bar with menus built out
     */
    protected JMenuBar createMenubar() {
        MainWindow mw = MainWindow.instanceOf();
        JMenuBar mb = new JMenuBar();
        String[] menuKeys = MainWindow.tokenize
            ( mw.getResourceString( "menubar" ) );
        for ( int i = 0; i < menuKeys.length; i++ ) {
            JMenu m = createMenu( menuKeys[i] );
            if ( m != null ) {
                mb.add( m );
            }
        }
        return mb;
    } // createMenubar

    /**
     * This is the hook through which all menu items are
     * created. Using the <code>cmd</code> string it finds
     * the menu item label and image (if any) in the resource
     * bundle.
     *
     * @param  cmd  action command string for this menu item;
     *              used to get the label and image
     * @return  new menu item
     * @see #createMenu
     */
    protected JMenuItem createMenuItem( String cmd ) {
        MainWindow mw = MainWindow.instanceOf();
        // create menu item and set image and text
        JMenuItem mi = new JMenuItem
            ( mw.getResourceString( cmd + labelSuffix ) );
        URL url = mw.getResource( cmd + imageSuffix );
        if ( url != null ) {
            mi.setHorizontalTextPosition( JButton.RIGHT );
            mi.setIcon( new ImageIcon( url ) );
        }
	String accel = mw.getResourceString(cmd+accelSuffix);
	if (accel != null) {
	  try {
	    int mask = 0;
	    if (accel.startsWith("CTRL")) {
	      mask += ActionEvent.CTRL_MASK;
	      accel = accel.substring(5);
	    }
	    if (accel.startsWith("SHIFT")) {
	      mask += ActionEvent.SHIFT_MASK;
	      accel = accel.substring(6);
	    }
	    if (accel.startsWith("ALT")) {
	      mask += ActionEvent.ALT_MASK;
	      accel = accel.substring(4);
	    }
	    int key = KeyEvent.class.getField("VK_"+accel).getInt(null);
	    mi.setAccelerator(KeyStroke.getKeyStroke(key, mask));
	  } catch (Exception e) {
	    // ignore
	  }
	}
	String mnemonic = mw.getResourceString(cmd+memSuffix);
	if (mnemonic != null && mnemonic.length() > 0)
	  mi.setMnemonic(mnemonic.toCharArray()[0]);

        // set menu action command
        mi.setActionCommand( cmd );
        Action a = mw.getAction( cmd );
        if ( a != null ) {
            // set up the action to listen for events
            mi.addActionListener( a );
            a.addPropertyChangeListener( new ActionChangedListener( mi ) );
            mi.setEnabled( a.isEnabled() );
        } else {
            // no action! disable menu item
            mi.setEnabled( false );
        }
        return mi;
    } // createMenuItem

    protected JMenuItem createCheckBoxMenuItem( String cmd ) {
    MainWindow mw = MainWindow.instanceOf();
      // Layout
      JCheckBoxMenuItem mi = new JCheckBoxMenuItem(mw.getResourceString( cmd + labelSuffix ));
        URL url = mw.getResource( cmd + imageSuffix );
        if ( url != null ) {
            mi.setHorizontalTextPosition( JButton.RIGHT );
            mi.setIcon( new ImageIcon( url ) );
        }
        // set menu action command
        mi.setActionCommand( cmd );
        Action a = mw.getAction( cmd );
        if ( a != null ) {
            // set up the action to listen for events
            mi.addActionListener( a );
            a.addPropertyChangeListener( new ActionChangedListener( mi ) );
            mi.setEnabled( a.isEnabled() );
        } else {
            // no action! disable menu item
            mi.setEnabled( false );
        }
        return mi;
    }

    protected JRadioButtonMenuItem createRadioButtonMenuItem( String cmd ) {
    MainWindow mw = MainWindow.instanceOf();
      // Layout
      JRadioButtonMenuItem mi = new JRadioButtonMenuItem(mw.getResourceString( cmd + labelSuffix ));
        URL url = mw.getResource( cmd + imageSuffix );
        if ( url != null ) {
            mi.setHorizontalTextPosition( JButton.RIGHT );
            mi.setIcon( new ImageIcon( url ) );
        }
        // set menu action command
        mi.setActionCommand( cmd );
        mi.setEnabled(true);
      toolMenu.put(cmd,mi);
      toolGroup.add(mi);
        return mi;
    }

    /**
     * Create the toolbar. By default this reads the
     * resource file for the definition of the toolbar.
     *
     * @return  new toolbar built out
     */
    protected JToolBar createToolbar() {
        MainWindow mw = MainWindow.instanceOf();
        JToolBar toolbar = new JToolBar();
        String[] toolKeys = MainWindow.tokenize
            ( mw.getResourceString( "toolbar" ) );
        for ( int i = 0; i < toolKeys.length; i++ ) {
            if ( toolKeys[i].equals( "-" ) ) {
                toolbar.add( Box.createHorizontalStrut( 5 ) );
            } else {
                toolbar.add( createToolbarButton( toolKeys[i] ) );
            }
        }
        toolbar.add( Box.createHorizontalStrut( 10 ) );
        return toolbar;
    } // createToolbar



    /**
     * Create a button to go inside of the toolbar. By default this
     * will load an image resource. The image filename is relative to
     * the classpath (including the '.' directory if its a part of the
     * classpath), and may either be in a JAR file or a separate file.
     *
     * @param  key  key in resource bundle for tool
     * @return  new toolbar button
     */
    protected JButton createToolbarButton( String key ) {
        MainWindow mw = MainWindow.instanceOf();
        // create the button
        URL url = mw.getResource( key + imageSuffix );
        JButton b = url != null ?
            new JButton( new ImageIcon( url ) ) {
                public float getAlignmentY() {
                    return 0.5f;
                }
            }
            : new JButton( key );
        b.setRequestFocusEnabled( false );
        b.setMargin( new Insets( 1, 1, 1, 1 ) );

        // get action and attach it to button as a listener
        b.setActionCommand( key );
        Action a = mw.getAction( key );
        if ( a != null ) {
                     b.addActionListener( a );
            a.addPropertyChangeListener( new ActionChangedListener( b ) );
            b.setEnabled( a.isEnabled() );
        } else {
            b.setEnabled( false );
        }

        // attach tooltip to button
        String tip = mw.getResourceString( key + tipSuffix );
        if ( tip != null ) {
            b.setToolTipText( tip );
        }
        return b;
    } // createToolbarButton

    /**
     * Remove ourselves (and all our components) from any lists we
     * might be one.
     */
    public void detach() {
        toolMan.detach();
        toolMan = null;
        mpi = null;
        dtp = null;
        winMenu.buildList( null );
        winMenu = null;
    } // detach

    /**
     * Finds the selected frame and returns a reference to it.
     * <p>
     * NOTE: We use this instead of simply tracking the active window
     * via the internal frame listener because renameWindow()
     * might be called just after a new window is created and
     * that new window is not set active yet because the listener
     * hasn't yet been notified.
     *
     * @return  reference to currently selected frame, or null
     */
    protected JInternalFrame findSelected() {
        CreekDocument currDoc = CreekDocumentGroup.instanceOf().getCurrentDocument();
        if(currDoc != null)
        {
            Container par = ((Component)currDoc.getView()).getParent();
            JInternalFrame[] frames = dtp.getAllFrames();
            for ( int i = 0; i < frames.length; i++ ) {
                if(frames[i].getContentPane()==par)
                    return frames[i];
            }
        }
        else
        {
            JInternalFrame[] frames = dtp.getAllFrames();
            for ( int i = 0; i < frames.length; i++ ) {
                // look for the frame that is selected
                if ( frames[i].isSelected() ) {
                    return frames[i];
                }
            }

        }
        return null;

    } // findSelected

    /**
     * Returns the currently selected graph tool for this window.
     *
     * @return  currently selected GraphTool object
     */
    public CreekTool getCurrentTool() {
        return toolMan.getCurrentTool();
    } // getCurrentTool


    /**
     * Retrieves the document attached to the given frame.
     *
     * @param  fr  internal frame
     * @return  document attached to internal frame
     */
    protected CreekDocument getDocument( JInternalFrame fr ) {
      try
      {
        ScrollView sp = (ScrollView)fr.getContentPane();
        ConceptView view = (ConceptView)sp.getDrawingSurface();
        return CreekDocumentGroup.instanceOf().getDocument( view );
      }
      catch(ClassCastException e)
      {

      }
      return null;
    } // getDocument

    /**
     * Retrieves the InternalFrame attached to the given document.
     *
     * @param  doc  CreekDocument attached to the frame
     * @return  JInternalFrame attached to the document
     */
     public JInternalFrame getWin(CreekDocument  doc)
     {
        JInternalFrame[] frames = dtp.getAllFrames();
        for ( int i = 0; i < frames.length; i++ ) {
            if(frames[i].getLayer()==0)
               if(doc != null && doc == getDocument( frames[i] ))
               {
                    return frames[i];
               }
        }
        return null;
     }

    public void activateWin( CreekDocument doc ) {
      System.out.println("activateWin");
        try
        {
          JInternalFrame attFrm =getWin(doc);
          if(attFrm!=null)
            attFrm.setSelected(true);
        } catch ( java.beans.PropertyVetoException pve ) { }
    } // getDocument

    /**
     * Retrieves all the open documents attached to this
     * main window.
     *
     * @return  array of graph document objects
     */
    public CreekDocument[] getDocuments() {
        JInternalFrame[] frames = dtp.getAllFrames();
        CreekDocument[] docs = new CreekDocument[frames.length];
        for ( int i = 0; i < frames.length; i++ ) {
            docs[i] = getDocument( frames[i] );
        }
        return docs;
    } // getDocuments

    /**
     * Find the hosting frame for this component. Often used
     * when displaying dialogs which require a host frame.
     *
     * @param  child  component to find parent frame for
     * @return hosting frame or null if none
     */
    public static Visualizer getFrame( Component child ) {
        Container p = child.getParent();
        while ( p != null ) {
            if ( p instanceof JPopupMenu ) {
                // special case for popup menus which do not
                // have parents but have invokers instead
                p = ((JPopupMenu)p).getInvoker().getParent();
            }
            if ( p instanceof Visualizer ) {
                return (Visualizer)p;
            }
            p = p.getParent();
        }
        // last ditch effort to find frame
        if ( child instanceof Visualizer ) {
            return (Visualizer)child;
        }
        return null;
    } // getFrame


    public  static JInternalFrame getInternalFrame( Component child ) {
        Container p = child.getParent();
        while ( p != null ) {
            if ( p instanceof JPopupMenu ) {
                // special case for popup menus which do not
                // have parents but have invokers instead
                p = ((JPopupMenu)p).getInvoker().getParent();
            }
            if ( p instanceof JInternalFrame ) {
                return (JInternalFrame)p;
            }
            p = p.getParent();
        }
        // last ditch effort to find frame
        if ( child instanceof JInternalFrame ) {
            return (JInternalFrame)child;
        }
        return null;
    } // getFrame
    /**
     * Use this notification to tell the document group which
     * view is currently active.
     *
     * @param  e  internal frame event
     */
    public void internalFrameActivated( InternalFrameEvent e ) {
        //needed to check not a ToolWindow
        if(((JInternalFrame)e.getSource()).getLayer() != 0) return;
        // find document attached to internal frame
        CreekDocument doc = getDocument( (JInternalFrame)e.getSource() );
        // found document attached to this window, set it as current
        CreekDocumentGroup.instanceOf().setCurrentDocument( doc );
    } // internalFrameActivated

    /**
     * We ignore this event.
     *
     * @param  e  internal frame event
     */
    public void internalFrameClosed( InternalFrameEvent e ) {
    } // internalFrameClosed

    /**
     * Use this opportunity to save the document associated with
     * this window. We can receive this message for a window that
     * is not currently selected. Because of this, we need to get
     * the document associated with this window and save it, not
     * just save the current document in the document group. And
     * by going through the file menu, we give the user the chance
     * to not save the file.
     *
     * @param  e  internal frame event
     */
    public void internalFrameClosing( InternalFrameEvent e ) {
        //needed to check not a ToolWindow
        if(((JInternalFrame)e.getSource()).getLayer() != 0) return;

        System.out.println("Closing internal window");

        // get reference to document attached to this window
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        JInternalFrame fr = (JInternalFrame)e.getSource();
        CreekDocument doc = getDocument( fr );
        // set that as current document
        dg.setCurrentDocument( doc );
        // set this window as current, since clicking the close
        // button on a window that isn't active doesn't make it so
        try {
            fr.setSelected( true );
        } catch ( java.beans.PropertyVetoException pve ) { }

        ConceptView view = ((MultiInternalFrame)fr).getConceptView();

        DropTargetListener dtl = (DropTargetListener)toolMan.getTool("MoveTool");
        if(view != null)
            if(Component.class.isInstance( view ))
            {
                    Component c = (Component) view;
                    DropTarget dt = c.getDropTarget();
                    if(dt != null ) {
                            dt.setComponent( null );
                            dt.removeDropTargetListener(dtl);
                    }
            }

        // close document by firing close action
        Action a = MainWindow.instanceOf().getAction( "close" );
        a.actionPerformed( new ActionEvent( this,
                                            ActionEvent.ACTION_PERFORMED,
                                            "close" ) );
    } // internalFrameClosing

    /**
     * Use this notification to tell the document group that
     * there's no current document. Hopefully another window
     * will be activated soon and the current document will
     * be set to that.
     *
     * @param  e  internal frame event
     */
    public void internalFrameDeactivated( InternalFrameEvent e ) {
        //needed to check not a ToolWindow
        if(((JInternalFrame)e.getSource()).getLayer() != 0) return;

        // tell doc group no one is active now
//        CreekDocumentGroup.instanceOf().setCurrentDocument( null );
    } // internalFrameDeactivated

    /**
     * We ignore this event.
     *
     * @param  e  internal frame event
     */
    public void internalFrameDeiconified( InternalFrameEvent e ) {
    } // internalFrameDeiconified

    /**
     * We ignore this event.
     *
     * @param  e  internal frame event
     */
    public void internalFrameIconified( InternalFrameEvent e ) {
    } // internalFrameIconified

    /**
     * We ignore this event.
     *
     * @param  e  internal frame event
     */
    public void internalFrameOpened( InternalFrameEvent e ) {
    } // internalFrameOpened

    /**
     * Create a new internal window and set its title to the passed
     * argument. The window is given a default size and position.
     * The window is also set selected. This also causes the window
     * menu to update itself.
     *
     * @param  title  title to give new internal window
     * @return  reference to new view object inside window
     */
    public ConceptView newWindow( String title ) {
        // create the internal frame and set its bounds
        MultiInternalFrame win = new MultiInternalFrame
            ( title, true, true, true, true );
        win.setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
        win.setBounds( 10, 10, 600, 400 );
        // we want to know of changes to this window
        win.addInternalFrameListener( this );
        // create the scrollpane and view for this window
        DrawingSurface    view = new DrawingSurface();
        ScrollView sView = new ScrollView(view);
        // add our mouse position indicator as a listener of view
        view.addMouseMotionListener( mpi );

        win.setContentPane( sView );
        win.setConceptView(view);
        // add window to default layer in desktop pane
        dtp.add( win, JLayeredPane.DEFAULT_LAYER );
        win.setVisible(true);

        DropTargetListener dtl = (DropTargetListener)toolMan.getTool("MoveTool");
        //make it a drop target and add it to the drop target list
        if(Component.class.isInstance( view ))
         {
                Component c = (Component) view;
                try {
                                DropTarget dt = new DropTarget(c ,DnDConstants.ACTION_COPY_OR_MOVE ,dtl);
                                dropTargets.add( dt );
                                //System.out.println("View " + view.getID() + " Initialized to DND.");
                }
                catch (java.lang.NullPointerException npe) {
                        System.err.println("View Failed to initialize to DND.");
                        System.err.println("Container likely did not have peer before the DropTarget was added");
                        System.err.println(npe);
                }
        }

        // set this window as selected
        try {
            win.setSelected( true );
        } catch ( java.beans.PropertyVetoException pve ) { }
        // rebuild the window menu
        winMenu.buildList( dtp.getAllFrames() );
        return view;
    } // newWindow

    public Container getPropertyWindow() {
/*
        PropertyPopup win = new PropertyPopup(title, 250, 250, 150, 150, 4, new JButton());
        // add window to default layer in desktop pane
        dtp.add( win, JLayeredPane.PALETTE_LAYER );
        return win;
*/
        if(propWin== null)
        {
              propWin = new JInternalFrame
                  ( MainWindow.instanceOf().instanceOf().getResourceString("permanentFrame"+titleSuffix),
                   true, true /*, true, true */ );
              propWin.setResizable(true);
              propWin.setIconifiable(false);
              propWin.setDefaultCloseOperation( WindowConstants.HIDE_ON_CLOSE );
              propWin.setBounds( 300, 10, 200, 200 );
              // add window to default layer in desktop pane
              dtp.add( propWin, JLayeredPane.PALETTE_LAYER );
        }
        propWin.setVisible(true);
        // set this window as selected
        try {
            propWin.setSelected( true );
        } catch ( java.beans.PropertyVetoException pve ) { }
        return propWin;
    } // getPropertyWindow

    public void registerWindow(JInternalFrame frame)
    {

        dtp.add( frame, JLayeredPane.DEFAULT_LAYER );
        try {
            frame.setSelected( true );
        } catch ( java.beans.PropertyVetoException pve ) { }
        if(viewMenu != null)
          viewMenu.addList(frame);

    } // getPropertyWindow


    public PropertyControl getPropertyControl()
    {
        return pc;
    }

    public FrameView getFrameView()
    {
//      System.out.println("Trying to find frame view, is "+props.get("FrameView"));
      try
      {
        return (FrameView) props.get("FrameView");
      }
      catch(ClassCastException e)
      {
        return null;
      }
    }

    public void displayPropertyWindow(Entity selEntity)
    {
          if(selEntity!= null)
          {
              frameProperty.updateModelView(selEntity);
              JInternalFrame jWin = (JInternalFrame)getPropertyWindow();
              if(jWin!=null)
              {
                jWin.setContentPane(frameProperty);
                jWin.setTitle(selEntity.getName()+" "+
                MainWindow.instanceOf().getResourceString("permanentFrame"+Visualizer.titleSuffix));
              }
          }

    }

    public void closePropertyWindow()
    {
          JInternalFrame jWin = (JInternalFrame)getPropertyWindow();
          if(jWin!=null)
          {
              if(!jWin.isClosed())
                  jWin.dispose();
          }
    }

    public void newViewWindow( String title )
    {
      System.out.println("NewViewWindow");
        JInternalFrame win = new JInternalFrame
            ( title /*, true, true, true, true */ );
        win.setResizable(false);
        win.setIconifiable(true);
        win.setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
        win.setBounds( 400, 10, 200, 200 );
        treeView = new TreeView();
        treeView.setVisible(true);
        win.setContentPane( treeView );
        // add window to default layer in desktop pane
        dtp.add( win, JLayeredPane.DEFAULT_LAYER );
        win.setVisible(true);
        // set this window as selected
        try {
            win.setSelected( true );
        } catch ( java.beans.PropertyVetoException pve ) { }
        if(viewMenu != null)
          viewMenu.addList(win);

    }


    public CreekDocument getSelContext()
    {
        return treeView.getSelDoc();
    }

    public void newEntityWindow(String title)
    {
        JInternalFrame win = new JInternalFrame
            ( title /*, true, true, true, true */);
        win.setResizable(true);
        win.setIconifiable(true);
        win.setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
        win.setBounds(400, 220, 420, 200  );
        findEnt = new EntityList();

        win.setContentPane( findEnt );
        // add window to default layer in desktop pane
        dtp.add( win, JLayeredPane.DEFAULT_LAYER);
        win.setVisible(true);
        // set this window as selected
        try {
            win.setSelected( true );
        } catch ( java.beans.PropertyVetoException pve ) { }

        if(viewMenu != null)
          viewMenu.addList(win);
    }

    public void newToolWindow(String title)
    {
        toolWin = new JInternalFrame
            ( title /*, true, true, true, true */);
        toolWin.setResizable(true);
        toolWin.setIconifiable(true);
        toolWin.setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
        toolWin.setBounds(620, 10, 200, 200  );

        toolWin.setContentPane( new JPanel() );
        // add window to default layer in desktop pane
        dtp.add( toolWin, JLayeredPane.PALETTE_LAYER);
        toolWin.setVisible(true);
        // set this window as selected
        try {
            toolWin.setSelected( true );
        } catch ( java.beans.PropertyVetoException pve ) { }


    }

    public void newFrameWin( String title )
    {
        JInternalFrame win = new JInternalFrame
            ( title /*, true, true, true, true */ );
        win.setResizable(true);
        win.setIconifiable(true);
        win.setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
        win.setBounds( 400, 440, 420, 400 );
        fView = new FrameView();
        fView.setBackground(Color.lightGray);
        fView.setVisible(true);
        win.setContentPane( fView );
        // add window to default layer in desktop pane
        dtp.add( win, JLayeredPane.DEFAULT_LAYER );
        win.setVisible(true);
        // set this window as selected
        try {
            win.setSelected( true );
        } catch ( java.beans.PropertyVetoException pve ) { }
        if(viewMenu != null)
          viewMenu.addList(win);

    }
    /**
     * Renames the currently selected window to the given name.
     *
     * @param  name  new name to give to current window
     */
    public void renameWindow( String name ) {
        JInternalFrame fr = findSelected();
        if ( fr != null ) {
            fr.setTitle( name );
            winMenu.buildList( dtp.getAllFrames() );
        }
    } // renameWindow

    /**
     * Invoked when the window is set to be the user's active
     * window, which means the window (or one of its subcomponents)
     * will receive keyboard events.
     * <p>
     * We use this event to make sure that our selected internal
     * frame's document is set as the current document. Otherwise
     * unpredictable bad things could happen (with having multiple
     * current documents, there could be two graph tools attached
     * to the same document).
     *
     * @param  e  window event
     */
    public void windowActivated( WindowEvent e ) {
        toolMan.setActive( true );
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        // find selected internal frame
        JInternalFrame fr = findSelected();
        if ( fr != null ) {
            //needed to check not a ToolWindow
            if(fr.getLayer() != 0) return;
            // make corresponding document the current one
            CreekDocument doc = getDocument( fr );
            if ( doc != null ) {
                if ( doc != dg.getCurrentDocument() ) {
                    // set current only if not current already
                    // (this avoids firing useless change events)
                    dg.setCurrentDocument( doc );
                }
                // else do nothing
            } else {
                // else, set current document null
                dg.setCurrentDocument( null );
            }
        } else {
            // else, set current document null
            dg.setCurrentDocument( null );
        }
    } // windowActivated

    /**
     * Invoked when a window has been closed.
     *
     * @param  e  window event
     */
    public void windowClosed( WindowEvent e ) {
    } // windowClosed

    /**
     * Invoked when a window is in the process of being closed.
     * The close operation can be overridden at this point.
     *
     * @param  e  window event
     */
    public void windowClosing( WindowEvent e ) {
        // invoke exit action which will save files if needed
        Action a = MainWindow.instanceOf().getAction( "exit" );
        a.actionPerformed( new ActionEvent
                           ( (Component)e.getSource(),
                             ActionEvent.ACTION_PERFORMED,
                             "exit" ) );
    } // windowClosing

    /**
     * Invoked when a window is de-activated.
     * <p>
     * We use this event to deactivate the tool manager which
     * avoids a serious problem. If there are multiple tool
     * managers active then they will register multiple tools
     * with the current document, which leads to havoc.
     *
     * @param  e  window event
     */
    public void windowDeactivated( WindowEvent e ) {
        toolMan.setActive( false );
    } // windowDeactivated

    /**
     * Invoked when a window is de-iconified.
     *
     * @param  e  window event
     */
    public void windowDeiconified( WindowEvent e ) {
    } // windowDeiconified

    /**
     * Invoked when a window is iconified.
     *
     * @param  e  window event
     */
    public void windowIconified( WindowEvent e ) {
    } // windowIconified

    /**
     * Invoked when a window has been opened.
     *
     * @param  e  window event
     */
    public void windowOpened( WindowEvent e ) {
    } // windowOpened

    protected void createProps()
    {
        props = new Hashtable();
        // Create the props and add them to the hashtable
        MainWindow mw = MainWindow.instanceOf();
        String[] allProps = MainWindow.tokenize
            ( mw.getResourceString( "sidebar" ) );


        for ( int i = 0; i < allProps.length; i++ ) {
            try {
                Class prop = Class.forName
                    ( "jcreek.cke.visualizer." + allProps[i] );
                props.put( allProps[i],
                           prop.newInstance() );
            } catch ( Exception e ) { e.printStackTrace();}
        }
    }

    public boolean isShowing(String key)
    {
        if(isPropVisible)
        {
            if(propKey!=null)
              if(propKey.equals(key))
                return true;
        }
        return false;
    }//isShowing

    public void show(String key)
    {
        if(key==null)
        {
            isPropVisible = false;
            content.remove(splitter);
            content.add(sPane,"Center");
            this.validate();
            return;
        }

        if(isPropVisible)
        {
            divLocation = splitter.getDividerLocation();
            Component rightComp = (Component) props.get(key);

            splitter.setRightComponent(rightComp);
            propKey = key;
            divLocation = Math.min(splitter.getMaximumDividerLocation(), divLocation);
            splitter.setDividerLocation(divLocation);
            this.validate();
        }
        else
        {
            content.remove(sPane);
            if(splitter==null)
            {
              splitter = new JSplitPane();
              splitter.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
            }

            splitter.setLeftComponent(sPane);
            Component rightComp = (Component) props.get(key);
            splitter.setRightComponent(rightComp);
            content.add(splitter,"Center");
            splitter.validate();
            propKey = key;
            isPropVisible = true;
            this.validate();
            int max = splitter.getWidth();
            if(divLocation == 0)
              divLocation = max-200;
            divLocation = Math.min(splitter.getMaximumDividerLocation(), divLocation);
            splitter.setDividerLocation(divLocation);
            /* if(divLocation==0)
              splitter.setDividerLocation(rightComp.getPreferredSize().width);
            else
              splitter.setDividerLocation(divLocation);
            divLocation = splitter.getDividerLocation(); */
        }
    }


    /**
     * Watches for changes in actions and deals with them by
     * changing the corresponding menu items or toolbar buttons.
     */
    protected class ActionChangedListener implements PropertyChangeListener {
        /**
         * Menu item or toolbar button we are associated with.
         */
        JComponent component;

        /**
         * Constructor for our action change listener.
         *
         * @param  mi  menu item we are to associate with
         */
        public ActionChangedListener( JMenuItem mi ) {
            super();
            component = (JComponent)mi;
        } // ActionChangedListener

        /**
         * Constructor for our action change listener.
         *
         * @param  b  button we are to associate with
         */
        public ActionChangedListener( JButton b ) {
            super();
            component = (JComponent)b;
        } // ActionChangedListener

        /**
         * Handles changes in the action. If the action name
         * changed we change our menu name. If the action changed
         * it's enabled state, we change our component's state.
         *
         * @param  e  property change event
         */
        public void propertyChange( PropertyChangeEvent e ) {
            String propertyName = e.getPropertyName();
            if ( e.getPropertyName().equals( Action.NAME ) ) {
                if ( component instanceof JMenuItem ) {
                    String text = (String)e.getNewValue();
                    ((JMenuItem)component).setText( text );
                }
            } else if ( propertyName.equals( "enabled" ) ) {
                Boolean enabledState = (Boolean)e.getNewValue();
                component.setEnabled( enabledState.booleanValue() );
            }
        } // propertyChange
    } // ActionChangedListener

}//Visualizer

/**
 * Class for listening to the mouse motion on the ConceptView.
 * When the mouse moves this object updates the x,y position
 * of itself. This allows the user to know the x,y position of
 * the mouse pointer. A single object of this class can listen
 * to several components simultaneously, as in the case when the
 * user opens multiple graph files within one MainWindow object.
 *
 * @author
 * @version
 */
class MousePositionIndicator extends JLabel implements MouseMotionListener {

    private StatusBar sb;

    /**
     * No-arg constructor for this class. It creates the label
     * and sets the text to an initial string.
     */
    public MousePositionIndicator(StatusBar sb) {
        // If we don't give some text here, the layout manager will
        // make us zero in size and we don't want that.
        super( "X: 0, Y: 0             Scale: 100%" );
        this.sb = sb;
    } // MousePositionIndicator

    /**
     * Mouse was dragged over the canvas. Let's track the mouse position.
     * We display the mouse pointer location in the label.
     *
     * @param  me  the mouse event for the mouse movement
     */
    public void mouseDragged( MouseEvent me ) {
        // Set our label to be current x,y position.
        setText( "X: " + me.getX() + ", Y: " + me.getY()
                +"             "+"Scale: "+
                (int)(((ConceptView)me.getSource()).getScale()*100)+"%" );
        sb.setMessage( "X: " + me.getX() + ", Y: " + me.getY()
                +"             "+"Scale: "+
                (int)(((ConceptView)me.getSource()).getScale()*100)+"%");
    } // mouseDragged

    /**
     * Mouse was moved over the canvas. Let's track the mouse position.
     * We display the mouse pointer location in the label.
     *
     * @param  me  the mouse event for the mouse movement
     */
    public void mouseMoved( MouseEvent me ) {
        // Set our label to be current x,y position.
        setText( "X: " + me.getX() + ", Y: " + me.getY()
                +"             "+"Scale: "+
                (int)(((ConceptView)me.getSource()).getScale()*100)+"%" );
        sb.setMessage( "X: " + me.getX() + ", Y: " + me.getY()
                +"             "+"Scale: "+
                (int)(((ConceptView)me.getSource()).getScale()*100)+"%");
    } // mouseMoved
} // MousePositionIndicator

/**
 * Specialized menu class that implements the look & feel menu for
 * Visualizer. It automatically adds builds out the menu for
 * selecting the available look and feels.
 * <p>
 * This is one of the available special menus. It is requested in
 * the resources file using the @ special menu tag, with the
 * keyword "lookAndFeel".
 *
 * @author
 * @version
 */
class LookAndFeelMenu extends JMenu implements ItemListener {
    /**
     * Parent window, used to set the busy cursor.
     */
    protected Frame win;
    /**
     * Array of LookAndFeel information objects.
     */
    protected UIManager.LookAndFeelInfo[] lafInfo;

    /**
     * One-arg constructor which creates new look & feel menu.
     *
     * @param  name  title for this menu
     * @param  win  parent window for displaying dialogs
     */
    public LookAndFeelMenu( String name, Frame win ) {
        super( name, true );
        this.win = win;

        // Create menu items for changing Look & Feel.
        ButtonGroup group = new ButtonGroup();
        lafInfo = UIManager.getInstalledLookAndFeels();
        String curLAFName = UIManager.getLookAndFeel().getName();
        for ( int i = 0; i < lafInfo.length; i++ ) {
            String lafName = lafInfo[i].getName();
            JRadioButtonMenuItem lafMenuItem =
                new JRadioButtonMenuItem( lafName );
            if ( curLAFName.equals( lafName ) ) {
                lafMenuItem.setSelected( true );
            }
            add( lafMenuItem );
            group.add( lafMenuItem );
            lafMenuItem.addItemListener( this );
        }
    } // LookAndFeelMenu

    /**
     * One of the look & feels was selected. See which one it was
     * and switch the entire user interface to that look & feel.
     *
     * @param  e  Indicates which item was selected.
     */
    public void itemStateChanged( ItemEvent e ) {

        JRadioButtonMenuItem rb = (JRadioButtonMenuItem)e.getSource();
        if ( rb.isSelected() ) {
            win.setCursor( Cursor.getPredefinedCursor
                           ( Cursor.WAIT_CURSOR ) );
            try {
                for ( int i = 0; i < lafInfo.length; i++ ) {
                    if ( rb.getText().equals( lafInfo[i].getName() ) ) {
                        // Switch to the selected look & feel.
                        UIManager.setLookAndFeel
                            ( lafInfo[i].getClassName() );
                        // Save the setting in the preferences.
                        AppSettings props = AppSettings.instanceOf();
                        props.setString( AppSettings.LOOK_FEEL,
                                         lafInfo[i].getClassName() );
                        break;
                    }
                }
                SwingUtilities.updateComponentTreeUI( win );
            } catch ( Exception exc ) {
                rb.setEnabled( false );
                System.err.println( "Error loading look & feel \"" +
                                    rb.getText() + "\": " + exc );
                exc.printStackTrace();
            }
            win.setCursor( Cursor.getPredefinedCursor
                           ( Cursor.DEFAULT_CURSOR ) );
        }

    } // itemStateChanged
} // LookAndFeelMenu

/**
 * This is a specialized menu class that implements the Window menu
 * for Visualizer. This menu will hold a list of titles representing
 * the internal windows contained in the main window. By selecting a
 * menu item you will cause the corresponding window to move to the front.
 * <p>
 * Use the buildList() method to cause this menu to create the
 * necessary menu items.
 *
 * @author
 * @version
 */
class WindowMenu extends JMenu implements ActionListener {
    /**
     * List of internal frames we represent in our menu.
     */
    protected JInternalFrame[] list;
    /**
     * Label for the Minimize All menu item.
     */
    protected String minLabel;
    /**
     * Label for the Maximize All menu item.
     */
    protected String maxLabel;
    /**
     * Label for the Maximize All menu item.
     */
    protected String cascadeLabel;
    /**
     * Label for the Maximize All menu item.
     */
    protected String tileLabel;
    /**
     * DeskTop Manager
     */
    protected MultiDeskTopPane deskTopManager;

    /**
     * Constructor for this class, creates the menu and adds some
     * basic window-management functions. In addition to these
     * items, buildList() will add a list of the currently opened
     * windows, when called upon.
     *
     * @param  name  title for this menu
     */
    public WindowMenu( String name,MultiDeskTopPane dMan ) {
        super( name, true );
        deskTopManager = dMan;
        MainWindow mw = MainWindow.instanceOf();
        minLabel = mw.getResourceString( "minimizeAllLabel" );
        JMenuItem mi = new JMenuItem( minLabel );
        add( mi );
        mi.addActionListener( this );
        maxLabel = mw.getResourceString( "maximizeAllLabel" );
        mi = new JMenuItem( maxLabel );
        add( mi );
        mi.addActionListener( this );
        cascadeLabel = mw.getResourceString( "cascadeAllLabel" );
        mi = new JMenuItem( cascadeLabel );
        add( mi );
        mi.addActionListener( this );
        tileLabel = mw.getResourceString( "tileAllLabel" );
        mi = new JMenuItem( tileLabel );
        add( mi );
        mi.addActionListener( this );
        addSeparator();
    } // WindowMenu

    /**
     * A menu item was selected, we should bring the corresponding
     * window to the front.
     *
     * @param  e  the event that was triggered
     */
    public void actionPerformed( ActionEvent e ) {
        // if list is null, nothing to do
        if ( list == null ) {
            return;
        }
        // get the name of the menu item
        String title = e.getActionCommand();

        // see if it's one of the action menu items
        if ( title.equals( minLabel ) ) {
            // minimize all the windows
            deskTopManager.minimizeAll();
        } else if ( title.equals( maxLabel ) ) {
            // maximize all the windows
            deskTopManager.maximizeAll();
        } else if ( title.equals( cascadeLabel ) ) {
            // cascade all the windows
            deskTopManager.cascadeFrames();
        } else if ( title.equals( tileLabel ) ) {
            // tile all the windows
            deskTopManager.tileFrames();
        } else {
            // find the matching window and bring it to the front
            for ( int i = 0; i < list.length; i++ ) {
                if ( title.equals( list[i].getTitle() ) ) {
                    // setting the window selected brings it forward
                    try {
                        list[i].setSelected( true );
                    } catch ( java.beans.PropertyVetoException pve ) { }
                    break;
                }
            }
        }
    } // actionPerformed

    /**
     * This builds the menu items for this menu, using the passed
     * array of internal frames to get the menu item titles.
     */
    public void buildList( JInternalFrame[] list ) {
        // remove all the current "window" menu items
        if ( this.list != null ) {
            int count = getItemCount() - 1;
            int min = count - this.list.length;
            for ( int i = count; i > min; i-- ) {
                remove( i );
            }
        }

        JInternalFrame[] newList = null;
        if(list != null)
        {
            //rebuild the list
             Vector intFrames = new Vector();
             for(int i=0;i<list.length;i++)
             {
                if(list[i].getLayer()==0)
                    intFrames.add(list[i]);
             }
             newList = new JInternalFrame[intFrames.size()];
             intFrames.copyInto(newList);
         }

        // save the list for use in actionPerformed()
        this.list = newList;
        // if list is empty, there's nothing to do
        if ( newList == null ) {
            return;
        }
        // add the new menu items, one for each internal frame
        for ( int i = 0; i < newList.length; i++ ) {
            JMenuItem mi = new JMenuItem( newList[i].getTitle() );
                mi.addActionListener( this );
                add( mi );
        }
    } // buildList
} // WindowMenu

/**
 * Manages the graph tools used to manipulate the graph. Builds
 * the toolbar buttons and creates the graph tool objects. Keeps
 * track of the current tool and always makes sure it is attached
 * to the current document.
 *
 * @author
 * @version
 */
class ToolManager implements ItemListener, PropertyChangeListener {
    /**
     * Currently selected tool.
     */
    protected CreekTool currentTool;
    /**
     * Table of available tools.
     */
    protected Hashtable tools;
    /**
     * table of Available Tool Menus
     */
    protected Hashtable toolMenu;
    /**
     * table of Available Tool Buttons
     */
    protected Hashtable toolButton;

    /**
     * True if this tool manager is active and can attach
     * tools to the current document.
     */
    protected boolean isActive;

    public JInternalFrame win;

    /**
     * Creates a new ToolManager object and adds tools to the
     * given toolbar.
     *
     * @param  toolbar  toolbar to add tool buttons to
     */
    public ToolManager( JToolBar toolbar,Hashtable toolMenu,JInternalFrame win ) {
        this.win = win;
        ButtonGroup group = new ButtonGroup();
        tools = new Hashtable();
        toolButton = new Hashtable();
        this.toolMenu = toolMenu;

        // Create the tools and add them to the toolbar
        // and the button group.
        MainWindow mw = MainWindow.instanceOf();
        String[] allTools = MainWindow.tokenize
            ( mw.getResourceString( "tools" ) );


        for ( int i = 0; i < allTools.length; i++ ) {
            JToggleButton button = createToolbarButton( group, allTools[i] );
            toolbar.add( button );
            group.add( button );
            try {
                Class toolType = Class.forName
                    ( "jcreek.cke.tool." + allTools[i] );
                tools.put( button.getActionCommand(),
                           toolType.newInstance() );
                toolButton.put(button.getActionCommand(),
                           button );
            } catch ( Exception e ) { e.printStackTrace();}
            AbstractButton menuButton = (AbstractButton)toolMenu.get(allTools[i]);
            menuButton.addActionListener(new ActionListener()
            {
                public void actionPerformed(ActionEvent event)
                {
                    JRadioButtonMenuItem menuItem= (JRadioButtonMenuItem)event.getSource();
                    if(menuItem.isSelected())
                        ((JToggleButton)toolButton.get(menuItem.getActionCommand())).setSelected(true);
                }
            });
            if ( i == 0 ) {
                button.setSelected( true );
            }
        }

        // Add some glue to the end of the toolbar.
        toolbar.add( Box.createHorizontalGlue() );

        // Add ourselves as a document group listener so we will
        // know when to attach/detach tools from the documents.
        CreekDocumentGroup.instanceOf().addPropertyChangeListener
            ( "current", this );
    } // ToolManager

    /**
     * Create a button to go inside of the toolbar. By default this
     * will load an image resource. The image filename is relative to
     * the classpath (including the '.' directory if its a part of the
     * classpath), and may either be in a JAR file or a separate file.
     *
     * @param  key  key in resource bundle for tool
     * @return  new toolbar button
     */
    protected JToggleButton createToolbarButton( ButtonGroup group,
                                                String key ) {
        MainWindow mw = MainWindow.instanceOf();
        // create the button
        URL url = mw.getResource( key + Visualizer.imageSuffix );
        JToggleButton b = url != null ?
            new JToggleButton( new ImageIcon( url ) ) {
                public float getAlignmentY() {
                    return 0.5f;
                }
            }
            : new JToggleButton( key );
        b.setRequestFocusEnabled( false );
        b.setBorderPainted( true );
        b.setMargin( new Insets( 1, 1, 1, 1 ) );
        // add ourselves as an item listener
        b.addItemListener( this );

        // set action string for button
        b.setActionCommand( key );

        // attach tooltip to button
        String tip = mw.getResourceString( key + Visualizer.tipSuffix );
        if ( tip != null ) {
            b.setToolTipText( tip );
        }
        group.add( b );
        return b;
    } // createToolbarButton


    /**
     * Remove ourselves from any lists we might be on.
     */
    public void detach() {
        CreekDocumentGroup.instanceOf().removePropertyChangeListener
            ( "current", this );
        currentTool = null;
        tools = null;
    } // detach

    /**
     * Returns the currently selected graph tool.
     *
     * @return  current creek tool
     */
    public CreekTool getCurrentTool() {
        return currentTool;
    } // getCurrentTool

    public CreekTool getTool(String command)
    {
        return (CreekTool)tools.get(command);
    }
    /**
     * See what tool the user has selected and attach it to
     * the current model and view objects. This will detach
     * the current tool first, before attaching the new one.
     *
     * @param  e  item event
     */
    public void itemStateChanged( ItemEvent e ) {
        if ( e.getStateChange() == ItemEvent.SELECTED ) {
            Object src = e.getSource();
            if ( src instanceof AbstractButton ) {
                // Detach the current tool in preparation for
                // attaching a new one.
                if ( currentTool != null ) {
                    currentTool.detach();
                }
                // Find the desired tool and attach it to the current
                // document, if any.
                currentTool = (CreekTool)tools.get
                    ( ((AbstractButton)src).getActionCommand() );
                ((JRadioButtonMenuItem)toolMenu.get( ((AbstractButton)src).getActionCommand() ))
                  .setSelected(true);
                CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
                CreekDocument doc = dg.getCurrentDocument();
                if ( ( doc != null ) && ( currentTool != null ) ) {
                    currentTool.attach( doc.getModel(), doc.getView(),win );
                }
            }
        }
    } // itemStateChanged


    /**
     * See if the current document has been switched. If so,
     * detach and re-attach the current tool.
     *
     * @param  e  property change event
     */
    public void propertyChange( PropertyChangeEvent e ) {
        if ( currentTool == null ) {
            // no current tool, do nothing
            return;
        }
        // re-attach current tool (let the tool detach itself)
        CreekDocument doc = (CreekDocument)e.getNewValue();
        if(!isActive && (doc == null))
            currentTool.detach();
        if ( isActive && ( doc != null ) ) {
            currentTool.attach( doc.getModel(), doc.getView(),win );
        }
    } // propertyChange

    /**
     * Sets this tool manager active or not. When the tool
     * manager is active it will attach/detach its tools when
     * the current documents are opened, closed, or switched.
     * When inactive, the tool manager will do nothing.
     *
     * @param  active  true to set tool manager active
     */
    public void setActive( boolean active ) {
        isActive = active;
        if ( active ) {
            CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
            CreekDocument doc = dg.getCurrentDocument();
            if ( ( doc != null ) && ( currentTool != null ) ) {
                // re-attach the current tool (let the tool detach itself)
                currentTool.attach( doc.getModel(), doc.getView(),win );
            }
        }
    } // setActive


} // ToolManager
class ViewMenu extends JMenu implements ActionListener {

    /**
     * List of internal frames we represent in our menu.
     */
    protected Vector list;

    public ViewMenu( String name ) {
        super( name, true );
        list = new Vector();
    } // ViewMenu

    /**
     * A menu item was selected, we should bring the corresponding
     * window to the front.
     *
     * @param  e  the event that was triggered
     */
    public void actionPerformed( ActionEvent e ) {
        // get the name of the menu item
        String title = e.getActionCommand();
        // find the matching window and bring it to the front
        for ( int i = 0; i < list.size(); i++ ) {
            if ( title.equals( ((JInternalFrame)list.get(i)).getTitle() ) ) {
                // setting the window selected brings it forward
                try {
                      JInternalFrame selFrame = (JInternalFrame)list.get(i);
                      boolean state = ((JCheckBoxMenuItem)e.getSource()).getState();
                      if(state)
                      {
                          if(selFrame.isIcon())
                              selFrame.setIcon(false);
                      }
                      selFrame.setVisible(state);
                } catch ( java.beans.PropertyVetoException pve )
                { pve.printStackTrace(); }
                break;
            }
        }

    } // actionPerformed

    public void addList(JInternalFrame frm)
    {
        list.add(frm);
        JCheckBoxMenuItem mi = new JCheckBoxMenuItem(frm.getTitle(),frm.isVisible());
        mi.addActionListener( this );
        add( mi );
    }

} // ViewMenu








