package jcreek.cke.command;


import jcreek.cke.document.CreekDocument;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.graph.VertexList;
import jcreek.cke.conceptview.VisualEntity;
import jcreek.representation.Entity;


import javax.swing.undo.UndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import java.awt.Point;


/**
 * Title:        JavaCreek
 * Description:  A Case Based Reasoning approach developed in Java
 * Copyright:    Copyright (c) 1999
 * Company:      Phil Systems
 * @author Victor Dias
 * @version
 */

public class ActivateEntityCommand extends CreekCommand
{

    /**
     * Entity added to model.
     */
      VisualEntity addEntity;

//    protected VertexList entities;



    /**
     * Constructor for this command. Saves the entity and document
     * for use with do, undo, and redo operations.
     *
     * @param  doc  document we're performed on
     * @param  v  entity to be added
     */
  public ActivateEntityCommand(CreekDocument doc,VisualEntity ve)
  {

      super(doc);
      addEntity = ve;
      canUndoRedo = true;

  }


    /**
     * Return the human-readable name for undoing this command.
     *
     * @return  human-readable name for undoable form of this command
     */
  public String getUndoPresentationName()
  {
    return "Undo Activate Entity";
  }


    /**
     * Perform the action for the first time.
     *
     * @return  true if command executed, false if error
     */
  public boolean perform()
  {
        if ( ( getDocument() != null ) && ( addEntity != null ) ) {
            CreekModel model = getDocument().getModel();
//            return model.addEntityList( entities );
            return model.addEntity(addEntity);
        } else {
            return false;
        }
  }

  public void die()
  {
        canUndoRedo = false;
        addEntity = null;
  }


    /**
     * Return true if this command is not a side-effect of some
     * other change. activating a Entity is always significant.
     *
     * @return  true if change is a side-effect of other changes
     */
  public boolean isSignificant()
  {
        return true;
  }

  public void redo()
  {
        if ( ( canUndoRedo ) && ( addEntity != null ) ) {
            CreekModel model = getDocument().getModel();
            model.addEntity(addEntity);
        } else {
            throw new CannotUndoException();
        }
  }
  public void undo()
  {
        if ( ( canUndoRedo ) && ( addEntity != null ) ) {
            CreekModel model = getDocument().getModel();
            model.removeEntity(addEntity);
        } else {
            throw new CannotUndoException();
        }
  }


    /**
     * Try to merge anEdit with this edit.
     *
     * @param  anEdit  edit to absorb
     * @return  true if anEdit was absorbed
     */

  public boolean addEdit(UndoableEdit anEdit)
  {
	// For now we don't support merging.
        return false;
  }


    /**
     * Return the human-readable name for redoing this command.
     *
     * @return  human-readable name for redoable form of this command
     */
  public String getRedoPresentationName()
  {
    return "Redo Activate Entity";
  }

    /**
     * See if we can be the given edit, thus replacing it in the
     * undo queue.
     *
     * @param  anEdit  edit command to replace
     * @return  true if anEdit replaced
     */

  public boolean replaceEdit(UndoableEdit anEdit)
  {
        // Assume that we can't for now.
        return false;
  }


    /**
     * Return the human-readable name for this action.
     *
     * @return  human-readable name for this command
     */
  public String getPresentationName()
  {
    return "Activate Entity";
  }

}


