package jcreek.cke.command;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.MainWindow;
import jcreek.cke.document.CreekDocument;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.Context;
import jcreek.cke.visualizer.SaveDialog;

import java.awt.Frame;
import java.awt.FileDialog;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.Action;
import java.io.File;
import javax.swing.JOptionPane;
import javax.swing.JInternalFrame;

public class CloseModelAction extends SaveModelAction {

  public CloseModelAction() {
        super("closeModel");
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }

  public CloseModelAction(String actionName) {
        super(actionName);
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }

    /**
     * Performs the save action.
     * @param  event  action event
     */
  public void actionPerformed(ActionEvent event) {
        Visualizer win = getWindow( event );
        // save the file without special options
        if(saveFile( win, true, false ))
        {
            //closes the document windows
            closeDocuments(win);
            //clean up all view files created
            finish();
            //close permanentProperty Window
            win.closeWindow((JInternalFrame)win.getPropertyWindow());
            //closes the model
            CreekDocumentGroup.instanceOf().closeModel();
            win.setTitle(MainWindow.instanceOf().getResourceString("AppTitle"));
        }

  }//actionPerformed

    /**
     * Saves all the documents to disk.
     * This are just temp files for a given
     * session.The data will then be copied to
     * the KnowledgeModel
     *
     */ /*
    public void saveDocuments(Visualizer win) {
        CreekDocumentGroup cdg = CreekDocumentGroup.instanceOf();
        CreekDocument[] docs = cdg.getDocuments();
        SaveViewAction saveView =(SaveViewAction) MainWindow.instanceOf().getAction("saveView");
        for(int i=0;i<docs.length;i++)
        {
            if(docs[i].isDirty() || !(docs[i].isTitled()))
            {
                JInternalFrame holder = win.getWin(docs[i]);
                cdg.setCurrentDocument(docs[i]);
                saveView.saveFile(win,true,false);
                cdg.closeDocument();
                win.closeWindow(holder);
            }
        }
    } // saveDocument
*/
    protected void closeDocuments(Visualizer win) {
        CreekDocumentGroup cdg = CreekDocumentGroup.instanceOf();
        CreekDocument[] docs = cdg.getDocuments();
        for(int i=0;i<docs.length;i++)
        {
              JInternalFrame holder = win.getWin(docs[i]);
                cdg.setCurrentDocument(docs[i]);
                cdg.closeDocument();
                win.closeWindow(holder);
        }
    } // saveDocument

    protected void finish()
    {
        CreekDocumentGroup cdg = CreekDocumentGroup.instanceOf();
        KnowledgeModel km = cdg.getKM();
        Context[] conts = km.getContexts();
        /*
        for(int i=0;i<conts.length;i++)
        {

            File tFile = conts[i].getFile();
            if(tFile.exists())
                tFile.delete();
        }
        */
    }
/*    protected boolean saveFile( Visualizer win,
                                boolean askUser,
                                boolean forceSaveAs ) {
        // first see if there is a KnowledgeModel and if it is dirty
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        KnowledgeModel km = dg.getKM();
        if ( km == null ) {
	    noCurrentKM( win );
            return false;
        }
        if ( ( dg.isDirty() == false ) && !forceSaveAs ) {
            return true;
        }

        boolean okay = true;
        if(sd == null)
            sd = new SaveDialog
                ( win, getString( "saveAs"+Visualizer.titleSuffix ), true );
        sd.initTree();
        centerComponent(sd,win);
        sd.show();

        if(sd.isOk())
        {
              if(sd.isModelSelected())
              {
                  CreekDocument[] nDocs = sd.getNonSelectedDocuments();
                  for(int k=0;k<nDocs.length;k++)
                  {
                        JInternalFrame holder = win.getWin(nDocs[k]);
                      if(!nDocs[k].isTitled())
                      {
                        CreekDocumentGroup.instanceOf().deleteContext(nDocs[k]);
                        if(holder!= null)
                            win.closeWindow(holder);
                      }
                  }/*
                  CreekDocument[] docs = sd.getSelectedDocuments();
                  for(int i=0;i<docs.length;i++)
                  {
                      if(!docs[i].isTitled())
                      {
                            docs[i].setTitle(docs[i].getFile().getName());
                            docs[i].setTitled(true);
                      }
                      dg.setCurrentDocument(docs[i]);
                      if(!(okay=dg.saveDocument()))
                          break;
                  }
                  if(okay)
                      okay = dg.saveKM();
              }
        }
        else
            return false;

        if ( !okay ) {
            JOptionPane.showMessageDialog
                ( win,
		  getString( "couldntSaveMsg" ),
                  getString( "operFailedTitle" ),
                  JOptionPane.ERROR_MESSAGE );
        }
        return okay;
    } // saveFile */



}