package jcreek.cke.command;

/**
 * Class to implement a FileFilter to allow only supported
 * graph files in the JFileChooser. Note that this FileFilter
 * is the com.sun.java.swing.filechooser.FileFilter and not
 * the java.io.FileFilter.
 *
 * @author Victor Dias
 * @version 1.0
 */

import javax.swing.filechooser.FileFilter;
import java.io.File;
import java.util.Hashtable;

public class CreekFileFilter extends FileFilter {

    /**
     * List of document types supported by Creek.
     */
    protected Hashtable docTypes;

    /**
     * Creates a new CreekFileFilter object.
     *
     * @param  types  supported document types
     */
    public CreekFileFilter(Hashtable types) {
        super();
        docTypes = types;
    } // CreekFileFilter

    /**
     * Tests whether or not the specified abstract pathname
     * should be included in a pathname list.
     *
     * @param  pathname  The pathname to be tested.
     * @return  true if the pathname should be included.
     */
    public boolean accept( File pathname ) {
        if ( pathname.isDirectory() ) {
            return true;
        } else {
            String name = pathname.getName();
            int i = name.lastIndexOf( '.' );
            String tempName =  name.substring( i + 1 );
            // See if the filename extension is in our list.
            if ( docTypes.get( name.substring( i + 1 ) ) != null ) {
                // It is, accept this file.
                return true;
            }
        }
        return false;
    } // accept

    /**
     * Returns the description of this file type.
     *
     * @return  string representation of file type
     */
    public String getDescription() {
        return "Creek Models";
    } // getDescription
    } // CreekFileFilter
