package jcreek.cke.command;

import javax.swing.*;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.*;
import java.awt.Toolkit;
import java.util.Iterator;
import java.lang.reflect.Array;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.browser.Visualizer;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.util.EntitySelection;
import jcreek.cke.graph.VertexList;
import jcreek.cke.util.SelectionSet;
import jcreek.cke.graph.RelationList;
import jcreek.cke.util.DPoint;
import jcreek.cke.conceptview.*;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class CutAction extends CreekAction implements PropertyChangeListener{

  public static final DataFlavor dataFlavor =
          new DataFlavor(SelectionSet.class,"Array of VisualEntities");

  public CutAction() {
        super( "cut" );
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }

    public void actionPerformed(ActionEvent event) {
        Visualizer win = getWindow( event );
         // first see if there is a current file and if it is dirty
        CreekDocument doc = getDocument();
        if ( doc == null ) {
	    noCurrentDocument( win );
            return;
        }
        cut();
  }


  public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "selection" ) ) {
            CreekDocument doc = (CreekDocument)e.getOldValue();
              if ( doc != null ) {
                      if(doc.getModel().getSelectedEntities().length>0)
                            setEnabled(true);
                      else
                            setEnabled(false);
              } else {
                      setEnabled( false );
              }
        }
  }

  public void cut()
  {
/*
      // Get System Clipboard
      Clipboard c = Toolkit.getDefaultToolkit().getSystemClipboard();
      //Copy and Save the Selected Entities in a Transferranle object
      CreekDocument doc = getDocument();
        if ( doc != null )
        {
            try
            {
            VisualEntity ve[] = doc.getModel().getSelectedEntities();
            VertexList vl = new VertexList(ve);
            CutCommand cc = new CutCommand(getDocument(),vl);
            if(!getDocument().performCommand(cc))
              throw new Exception();
            if(ve.length>0)
            {
                  EntitySelection s = new EntitySelection(vl.copy(),dataFlavor);
                  c.setContents(s,s);
            }
            }
            catch(Exception e)
            {
                Toolkit.getDefaultToolkit().beep();
            }
        }
*/
      // Get System Clipboard
      Clipboard c = Toolkit.getDefaultToolkit().getSystemClipboard();
      //Copy and Save the Selected Entities in a Transferranle object
      CreekDocument doc = getDocument();
        if ( doc != null )
        {
            VisualEntity ve[] = doc.getModel().getSelectedEntities();
            VertexList vl   = new  VertexList();
            RelationList rl = new RelationList();
            for(int k=0;k<ve.length;k++)
            {
                DPoint pt = ve[k].getPosition();
                vl.add(new VisualEntity(ve[k].getEntity(),(int)pt.x,(int)pt.y));
            }

            for(int i =0;i<ve.length;i++)
            {
                            Iterator iterator = ve[i].getEdgeIterator();
                            VisualRelation visualRelation ;
                            while(iterator.hasNext())
                            {
                                visualRelation = (VisualRelation)iterator.next();
                                if(isSelected(visualRelation,ve))
                                {
                                      rl.add(new VisualRelation(getVisualEntity(visualRelation.getFrom(),vl)
                                            ,getVisualEntity(visualRelation.getTo(),vl),
                                            visualRelation.getRelation()));
                                }

                            }
            }
            if(ve.length>0)
            {
                  try
                  {
                      EntitySelection s = new EntitySelection(new SelectionSet(vl,rl)
                          ,dataFlavor);
                      c.setContents(s,s);
                  }
                  catch(Exception ex)
                  {
                      ex.printStackTrace();
                  }
            }
            CutCommand cc = new CutCommand(getDocument(),new VertexList(ve));
            try
            {
                  if(!getDocument().performCommand(cc))
                    throw new Exception();
            }
            catch(Exception e)
            {
                e.printStackTrace();
            }
        }

  }


  public boolean isSelected(VisualRelation vr,VisualEntity[] ves)
  {
      boolean from = false, to=false;
      for(int i=0;i<ves.length;i++)
      {
          if(ves[i]==vr.from)
              from=true;
          if(ves[i]==vr.to)
              to=true;
          if(from && to)
            return true;
      }

      return false;
  }

  public VisualEntity getVisualEntity(VisualEntity ve,VertexList vl)
  {
      for(int i=0;i<vl.size();i++)
      {
          if(ve.getEntity()==vl.get(i).getEntity())
              return vl.get(i);
      }

      return null;
  }


}