package jcreek.cke.command;

import javax.swing.undo.UndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;

import jcreek.cke.graph.CreekModel;
import jcreek.representation.Relation;
import jcreek.cke.conceptview.VisualRelation;
import jcreek.cke.document.CreekDocument;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class DeactivateRelationCommand extends CreekCommand {

    /**
     * Relation added to model.
     */
    protected VisualRelation remRelation;

  public DeactivateRelationCommand(CreekDocument doc,VisualRelation vr) {
      super(doc);
      remRelation = vr;
      canUndoRedo = true;
  }

  public String getUndoPresentationName() {
    return "Undo Deactivate Relation";
  }

  public boolean perform() {
        if ( ( getDocument() != null ) && ( remRelation != null ) ) {
            CreekModel model = getDocument().getModel();
            return model.removeRelation(remRelation);
        } else {
            return false;
        }
  }//perform

  public void die() {
        canUndoRedo = false;
        remRelation = null;
  }

  public boolean isSignificant() {
    return true;
  }

  public void redo() throws javax.swing.undo.CannotRedoException {
        if ( ( canUndoRedo ) && ( remRelation != null ) ) {
            CreekModel model = getDocument().getModel();
            model.removeRelation(remRelation);
        } else {
            throw new CannotUndoException();
        }
  }

  public void undo() throws javax.swing.undo.CannotUndoException {
        if ( ( canUndoRedo ) && ( remRelation != null ) ) {
            CreekModel model = getDocument().getModel();
            model.addRelation(remRelation);
        } else {
            throw new CannotUndoException();
        }
  }

  public boolean addEdit(UndoableEdit parm1) {
	// For now we don't support merging.
        return false;
  }

  public String getRedoPresentationName() {
    return "Redo Deactivate Relation";
  }

  public boolean replaceEdit(UndoableEdit parm1) {
        // Assume that we can't for now.
        return false;
  }

  public String getPresentationName() {
    return "Deactivate Relation";
  }
}


