package jcreek.cke.command;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.graph.VertexList;
import jcreek.cke.conceptview.VisualEntity;

import jcreek.representation.Relation;
import jcreek.representation.RelationType;
import jcreek.representation.EntityTypeNotMatchedException;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JOptionPane;
import javax.swing.JDialog;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class DeleteAction extends CreekAction implements PropertyChangeListener {

  public DeleteAction() {
        super("delete");
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );

  }

  public void actionPerformed(ActionEvent event) {
        Visualizer win = getWindow( event );
         // first see if there is a current file and if it is dirty
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        CreekDocument doc = dg.getCurrentDocument();
        if ( doc == null ) {
	    noCurrentDocument( win );
            return;
        }
        CreekModel model = doc.getModel();
        VertexList vl = new VertexList(model.getSelectedEntities());
        if ( askToDelete(vl, win ) ) {
          vl = new VertexList(model.getSelectedEntities());
          DeleteEntityCommand de = new DeleteEntityCommand(doc,vl);
          doc.performCommand(de);
        }
  }

  public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "selection" ) ) {
            CreekDocument doc = (CreekDocument)e.getOldValue();
              if ( doc != null ) {
                      if(doc.getModel().getSelectedEntities().length>0)
                            setEnabled(true);
                      else
                            setEnabled(false);
              } else {
                      setEnabled( false );
              }
        }
  }

    /**
     * Ask the user if we should save the current dirty document.
     *
     * @param  win  parent window to attach dialog to
     * @return  true if we should save
     */
    protected boolean askToDelete( VertexList remEntity, Visualizer win ) {

        // See if the entities to remove represent relation types which have
        // active instances.

        boolean abort = false;
        boolean askedAll = true;
        for(int i=0;!abort && i<remEntity.size();i++)
        {
          VisualEntity ve = remEntity.get(i);
          try
          {
            RelationType relationType = new RelationType(ve.getEntity());
            Relation[] instances = relationType.getInstances();
            if(instances.length > 0)
            {
              String instList = "";
              for(int j=0;j<instances.length;j++)
                instList += "\n\t"+instances[j].toString();
              int result = JOptionPane.showConfirmDialog
                  ( win,
                    "Are you sure you want to delete the relation type '"+relationType.getName()+"'?\n"+
                    "This will also remove these instances of this relation type:"+instList,
                    getString( "warningTitle" ),
                    JOptionPane.YES_NO_OPTION,
                    JOptionPane.QUESTION_MESSAGE );
              abort = (result != JOptionPane.YES_OPTION);
            }
            else
              askedAll = false;
          }
          catch(EntityTypeNotMatchedException e)
          {
            // If the entity type is not a relation type, it can't have instances,
            // (all entities with instances are recognized as relation types)
            // and as such this exception will not cause a problem.
            askedAll = false;
          }
        }

        if(abort)
          return false;

        if(!askedAll)
        {
          int result = JOptionPane.showConfirmDialog
              (win,
               getString("confDelMsg"),
               getString("warningTitle"),
               JOptionPane.YES_NO_OPTION,
               JOptionPane.QUESTION_MESSAGE);
          return result == JOptionPane.YES_OPTION;
        }

        return true;
    } // askToSave



}


