package jcreek.cke.command;


import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;

import java.awt.event.ActionEvent;
import java.awt.Frame;
import javax.swing.JInternalFrame;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import javax.swing.JOptionPane;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class DeleteViewAction extends CreekAction implements PropertyChangeListener {

  public DeleteViewAction() {
      super("deleteView");
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }

  public void actionPerformed(ActionEvent event) {
        // get graph window that contains our invoker
      Visualizer win = (Visualizer)getWindow( event );
      // save the file, asking the user if they want to save or not
      CreekDocument doc = (CreekDocument)getValue("selDoc" );//win.getSelContext();
        if ( doc == null ) {
            doc = getDocument();
            if(doc==null)
            {
                noCurrentDocument( win );
                return;
            }
        }
          if ( askToDelete( win ) ) {
              JInternalFrame holder = win.getWin(doc);
              CreekDocumentGroup.instanceOf().deleteContext(doc);
              if(holder!= null)
                  win.closeWindow(holder);
          }
        putValue("selDoc",null);
  }

    /**
     * Ask the user if we should save the current dirty document.
     *
     * @param  win  parent window to attach dialog to
     * @return  true if we should save
     */
    protected boolean askToDelete( Visualizer win ) {
        int result = JOptionPane.showConfirmDialog
            ( win,
              getString( "confDelMsg" ),
              getString( "warningTitle" ),
              JOptionPane.YES_NO_OPTION,
              JOptionPane.QUESTION_MESSAGE );
        return result == JOptionPane.YES_OPTION;
    } // askToSave


    public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "opened" ) )
        {
            if(e.getNewValue()==null)
                  setEnabled(false);
            else
                  setEnabled(true);
        }
        else if(e.getPropertyName().equals( "closed" ))
            setEnabled(false);

    }


}