package jcreek.cke.command;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;

import jcreek.representation.*; import jcreek.representation.cbr.*;

import java.awt.event.ActionEvent;

/**
 * Example of a new action in the TrollCreek application. To add this action to
 * the application, edit the jcreek/cke/resources/Creek.properties file. An
 * example of adding this action to the "Model" menu:
 * <pre>
 *   #
 *   # Model Menu definition
 *   #
 *   # Each of the strings that follow form a key to be
 *   # used as the basis of a menu item definition.
 *   #
 *   model =  modelProperty - example newRelation newDomainRelation newEntity newCase - delete - @BuildTool
 *   exampleLabel = Import From XML
 *   exampleImage = resources/blank.gif
 *   exampleAction = ExampleAction
 * </pre>
 *
 * In the above definition, the exampleLabel specify the name of the action, the
 * exampleImage the image associated with it (drawn from jcreek/cke/resources/)
 * and exampleAction is the class that will be invoked when the action is performed.
 * This class must be present in the jcreek.cke.command package.
 * When the "example" is added to the "model" property like in the example above,
 * it is automatically included in the "model" menu. Toolbars and context menus
 * can also be modified similarly elsewhere.
 *
 * @author Frode Srmo
 * @version 1.0
 */

public class ExampleAction extends CreekAction
{
  public ExampleAction()
  {
    // The name of the action should be supplied to the superclass' constructor.
    super("example");
  }

  /**
   * This method is invoked whenever the action is performed.
   */
  public void actionPerformed(ActionEvent event)
  {
    // This is the application frame. Visualizer is a subclass of JFrame.
    Visualizer win = getWindow( event );

    // The reference to the current KnowledgeModel.
    KnowledgeModel km = CreekDocumentGroup.instanceOf().getKM();

    // Example of creating a dialog in the frame.
    javax.swing.JOptionPane.showMessageDialog (win, "Hi", "This is a message", javax.swing.JOptionPane.INFORMATION_MESSAGE);
  }

}
