/*********************************************************************
 *
 * DESCRIPTION:
 *      This file defines the file exit action.
 *
 ********************************************************************/

package jcreek.cke.command;

import java.awt.Frame;
import java.awt.event.ActionEvent;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.MainWindow;
import jcreek.cke.CreekDocumentGroup;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * Implements the exit program action. This ensures that all
 * opened documents are saved before exiting the Java VM.
 *
 * @author
 * @version
 */
public class ExitAction extends CloseModelAction {

    /**
     * Creates a new ExitAction object with the default action
     * command string of "exit".
     */
    public ExitAction() {
        super( "exit" );
        setEnabled(true);
    } // ExitAction

    /**
     * Performs the exit action. This saves all the open documents
     * and then closes the main window associated with the invoker
     * of this event.
     *
     * @param  event  action event
     */
    public void actionPerformed( ActionEvent event ) {
        // get window that contains our invoker
        Visualizer win = getWindow( event );
        boolean oktoexit = true;
        if(CreekDocumentGroup.instanceOf().getKM()!=null)
        {
            // save the file without special options
            if((oktoexit=saveFile( win, true, false )))
            {
                //closes the document windows
                closeDocuments(win);
                //closes the model
                CreekDocumentGroup.instanceOf().closeModel();
            }
        }

        if(oktoexit)
        {
            MainWindow.instanceOf().removeWindow(win);
        }
    } // actionPerformed

  public void propertyChange( PropertyChangeEvent e ) {
      setEnabled(true);
  }
} // ExitAction
