/*********************************************************************
 *
 * DESCRIPTION:
 *      This file defines the file open action. It also defines the
 *      filter class used to select only supported graph files.
 *
 ********************************************************************/

package jcreek.cke.command;

import jcreek.cke.browser.*;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;

import java.awt.event.ActionEvent;
import java.io.*;
import java.util.Hashtable;
import java.awt.image.BufferedImage;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import com.sun.image.codec.jpeg.*;

/**
 * Implements the open file action used to open an existing
 * graph document. This class also defines a file filter used
 * with the file selector to limit the user to choosing only
 * supported graph files.
 *
 * @author
 * @version
 */
public class ExportAction extends CreekAction implements PropertyChangeListener {

    /**
     * Creates a new OpenAction object with the default action
     * command string of "open".
     */
    public ExportAction() {
        super( "export" );
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
    } // OpenAction

    /**
     * Performs the open action. This presents the user with
     * a file selector and opens the selected file as a graph
     * document. Causes a new window to be opened.
     *
     * @param  event  action event
     */
    public void actionPerformed( ActionEvent event ) {
        String[] ext = new String[]{"jpg"};
        String extLabel = "JPG File (*.jpg)";
        saveFile( getWindow( event ), ext,extLabel);
    } // actionPerformed


    public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "current" ) ) {
            CreekDocument doc = (CreekDocument)e.getNewValue();
            if ( doc != null ) {
                setEnabled( true );
            }
        }
    } // propertyChange

    protected boolean saveFile( Visualizer win,String[] ext,String extlabel) {
        // first see if there is a current file and if it is dirty
        CreekDocument doc = getDocument();
        if ( doc == null ) {
	    noCurrentDocument( win );
            return true;
        }
            // get name and path for new file
        JFileChooser save=new JFileChooser(System.getProperty( "user.dir" ));
        save.setMultiSelectionEnabled(false);
        save.addChoosableFileFilter(new SimpleFileFilter(ext,extlabel));
        int option=save.showSaveDialog(win);
        if(option != JFileChooser.APPROVE_OPTION)
              return true;
          return exportView(win,save.getSelectedFile());
    } // saveFile

    protected boolean exportView(Visualizer win,File saveFile)
    {
        boolean okay = true;
        if(saveFile !=null)
        {
            String fn =saveFile.toString();
            if(!fn.endsWith(".jpg"))
              {
                fn = fn+".jpg";
              }
             try
               {
//                    FileOutputStream file = new FileOutputStream(fn);
                    BufferedOutputStream out = new BufferedOutputStream(
                      new FileOutputStream(fn));
                    BufferedImage buf = getView().createPrintImage();
                    if(buf != null)
                    {
//                      JPEGImageEncoder jpgEncode = JPEGCodec.createJPEGEncoder(file);
//                        jpgEncode.encode(buf);
                        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
                        JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(buf);
                        param.setQuality(0.75f, false);
                        encoder.setJPEGEncodeParam(param);
                        encoder.encode(buf);
                    }

//                    file.flush();
//                    file.close();
                    out.close();
                }
             catch(IOException ex)
                {
                   System.out.println("Failed to save: "+ex.getMessage());
                   return false;
                }
        }
        else
          okay = false;
        if ( !okay ) {
            JOptionPane.showMessageDialog
                ( win,
		  getString( "couldntSaveMsg" ),
                  getString( "operFailedTitle" ),
                  JOptionPane.ERROR_MESSAGE );
        }
        return okay;
    }

} // OpenAction
