package jcreek.cke.command;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.MainWindow;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.conceptview.ConceptView;
import jcreek.cke.visualizer.AppSettings;

import jcreek.representation.LocalKnowledgeModel;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.DefaultBoundedRangeModel;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.Cursor;
import java.awt.Component;

/**
 * Opens an existing knowledge model and merges it with the current model.
 *
 * @author Frode Srmo
 * @version 1.0
 */
public class MergeKnowledgeModelAction extends CreekAction implements PropertyChangeListener
{
    /**
     * Creates a new MergeAction object with the default action
     * command string of "merge".
     */
    public MergeKnowledgeModelAction()
    {
        super( "mergeKnowledgeModel" );

        // This action is disabled until a knowledge model is opened.
        setEnabled( false );

        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
    } //

    /**
     * Performs the open action. This presents the user with
     * a file selector and opens the selected file as a graph
     * document. Causes a new window to be opened.
     *
     * @param  event  action event
     */
    public void actionPerformed( ActionEvent event )
    {
        // get graph window that contains our invoker
        Visualizer win = getWindow( event );
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();

        JFileChooser fc = new JFileChooser(AppSettings.instanceOf().getCurrentDirectory());
        fc.setFileFilter( new CreekFileFilter( dg.getKMTypes() ) );
        if ( fc.showOpenDialog( win ) == JFileChooser.CANCEL_OPTION ) {
            // user hit cancel
            return;
        }
        File f = fc.getSelectedFile();

        AppSettings.instanceOf().setCurrentDirectory(f.getAbsolutePath());
        //User hit escape key
        if(f==null)
          return;

        try {
            if ( !f.canRead() ) {
                JOptionPane.showMessageDialog
                    ( win,
                      getString( "cantReadFileMsg" ),
                      getString( "operFailedTitle" ),
                      JOptionPane.ERROR_MESSAGE );
                return;
            }
        } catch ( SecurityException se ) {
            JOptionPane.showMessageDialog
                ( win,
                  getString( "noFileAccessMsg" ),
                  getString( "operFailedTitle" ),
                  JOptionPane.ERROR_MESSAGE );
            return;
        }

        ((Component)win).setCursor(new Cursor(Cursor.WAIT_CURSOR));
        // have document group create a new document for this file
//        ConceptView view = win.newWindow( f.getName() );
        String error = null;
        try
        {
          int confirm = JOptionPane.showConfirmDialog(win,
                                        getString("mergeConfirmMsg"),
                                        getString("mergeConfirmTitle"),
                                        JOptionPane.YES_NO_OPTION);

          if(confirm == JOptionPane.YES_OPTION)
          {
            dg.getKM().mergeModel(new LocalKnowledgeModel(f));
          }
        }
        catch(IOException e)
        {
          error = e.toString();
        }
        catch(ClassNotFoundException e)
        {
          error = e.toString();
        }
        if (error != null ) {
          ((Component)win).setCursor(Cursor.getDefaultCursor());
            // failed, close window we just created
            win.closeWindow();
            // chances are we couldn't understand the file format
            JOptionPane.showMessageDialog
                ( win,
                  getString( "couldntOpenFileMsg" )+" "+error,
                  getString( "operFailedTitle" ),
                  JOptionPane.ERROR_MESSAGE );
        }
        ((Component)win).setCursor(Cursor.getDefaultCursor());
    } // actionPerformed

  public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "opened" ) )
        {
            if(e.getNewValue()==null)
                  setEnabled(false);
            else
                  setEnabled(true);
        }
        else if(e.getPropertyName().equals( "closed" ))
            setEnabled(false);
  }

} // MergeAction
