package jcreek.cke.command;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.visualizer.DomainRelationDlg;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.RelationType;

import java.util.Vector;
import java.awt.event.ActionEvent;
import java.awt.Frame;
import java.awt.Component;
import java.awt.Rectangle;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JOptionPane;


/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class NewDomainRelationAction extends CreekAction implements PropertyChangeListener {

  public NewDomainRelationAction() {
        super("newDomainRelation");
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }
  public void actionPerformed(ActionEvent event) {
        Visualizer win = getWindow( event );
        // first see if there is a current file and if it is dirty
        CreekDocument doc = getDocument();
        if ( doc == null ) {
	    noCurrentDocument( win );
            return;
        }
/*        try
        {
            Class compileTool = Class.forName("sun.tools.javac.Main");
        }
        catch(ClassNotFoundException ce)
        {
            JOptionPane.showMessageDialog
                ( win,
                  getString( "classNotFoundMsg" ),
                  getString( "usrErrorTitle" ),
                  JOptionPane.ERROR_MESSAGE );
            return;
        } */
        CreekModel model = doc.getModel();
        displayRelationDlg(win);
  }

    public void displayRelationDlg(Visualizer win)
    {
        DomainRelationDlg newreldg = new DomainRelationDlg(win);
        centerComponent(newreldg,win);
        newreldg.setVisible(true);
        if(newreldg.isOK())
        {
              KnowledgeModel km  = CreekDocumentGroup.instanceOf().getKM();
              try
              {
//                System.err.println("Creating realtion with name '"+newreldg.getClassName()+"' and inverse name '"+newreldg.getClassInverseName()+"'");
                new RelationType(
                                       km,
                                       newreldg.getSelectedRelation(),
                                       newreldg.getClassName(),
                                       newreldg.getClassDescription(),
                                       newreldg.getClassInverseName(),
                                       newreldg.getClassDescription(),
                                       newreldg.getRelationStrength()
                                       );
              }
              catch(Exception e)
              {
                  e.printStackTrace();
              }

        }

    }//displaySubModel

    public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "current" ) ) {
            if(e.getNewValue()!=null)
                  setEnabled(true);
            else
                  setEnabled(false);
        }
    }

  private void centerComponent( Component c ,Frame mainFrame) {
        Rectangle winBounds = mainFrame.getBounds();
        Rectangle cBounds = c.getBounds();
        c.setLocation( ( winBounds.width - cBounds.width ) / 2 +
                       winBounds.x,
                       ( winBounds.height - cBounds.height ) / 2 +
                       winBounds.y );
    } // centerComponent


}
