package jcreek.cke.command;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.MainWindow;
import jcreek.cke.visualizer.ModelPropertyDlg;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.BasicModel;
import jcreek.representation.cbr.CBRModel;
import jcreek.representation.LocalKnowledgeModel;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.FileDialog;
import java.awt.Component;
import java.awt.Rectangle;
import javax.swing.JOptionPane;
import java.io.*;
import javax.swing.Action;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class NewKnowledgeModelAction extends CreekAction implements PropertyChangeListener {

  public NewKnowledgeModelAction() {
        super("newModel");
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( true );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }

  public void actionPerformed(ActionEvent event) {
        displayModelDlg( getWindow( event ));
  }

  public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "opened" ) )
        {
            if(e.getNewValue()==null)
                  setEnabled(true);
            else
                  setEnabled(false);
        }
        else if(e.getPropertyName().equals( "closed" ))
            setEnabled(true);
  }


    public void displayModelDlg(Visualizer win)
    {
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        ModelPropertyDlg newKMdg = new ModelPropertyDlg(win,
                                            getString("modelProp"+Visualizer.titleSuffix),
                                            ModelPropertyDlg.NEW_MODE);
        if(newKMdg.isOK())
        {
          LocalKnowledgeModel km = null;
          if(newKMdg.getModelType() == 2)
          {
            try
            {
              InputStream input = getClass().getResourceAsStream(
                "/jcreek/resources/IsoPodModel.km");
              km = new LocalKnowledgeModel(input);
            }
            catch(IOException e)
            {
              e.printStackTrace();
            }
            catch(ClassNotFoundException e)
            {
              e.printStackTrace();
            }
          }
          else if(newKMdg.getModelType() == 1)
          {
            km = new LocalKnowledgeModel(new CBRModel());
          }
          else
            km = new LocalKnowledgeModel(new BasicModel());

          dg.newKM(km);

          km.setFile(newKMdg.getFile());
          km.setAuthor(newKMdg.getAuthor());
          km.setName(newKMdg.getName());
          km.setAuthor(newKMdg.getAuthor());
          win.setTitle(getString("AppTitle")
            + " " +km.getName());
        // open new Map View by firing close action
        Action a = MainWindow.instanceOf().getAction( "newConceptView" );
        a.actionPerformed( new ActionEvent( win,
                                            ActionEvent.ACTION_PERFORMED,
                                            "newConceptView" ) );

/*
                else
                    JOptionPane.showMessageDialog
                        ( win,
                          getString( "couldntSaveMsg" ),
                          getString( "operFailed"+Visualizer.titleSuffix ),
                          JOptionPane.ERROR_MESSAGE );
*/
        }

    }//displaySubModel


}




