package jcreek.cke.command;

import javax.swing.undo.UndoableEdit;

import jcreek.cke.conceptview.*;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.util.DPoint;
import jcreek.representation.Relation;
import jcreek.representation.Entity;
import jcreek.representation.RelationType;
import jcreek.representation.KnowledgeModel;


public class NewRelationCommand extends CreekCommand {

  protected RelationType rcd;
  protected VisualEntity head;
  protected VisualEntity tail;

  public NewRelationCommand(CreekDocument doc,
                    RelationType rcd,
                    VisualEntity head,
                    VisualEntity tail) {
      super(doc);
      this.rcd = rcd;
      this.head = head;
      this.tail = tail;
      canUndoRedo = false;
  }

  public String getUndoPresentationName() {
    return "Undo New Relation";
  }

  public boolean perform() {
        if ( rcd != null )
        {
            Relation[] relations  = head.getEntity().getRelations(rcd);
            for(int i=0;i<relations.length;i++)
            {
                if(relations[i].getValue().equals(tail.getEntity()))
                    return false;
            }
            Relation newRel = rcd.newInstance(head.getEntity(), tail.getEntity());
            if(newRel!=null)
            {
//                  if(rcd.isInverseDescriptor())
//                    newRel = newRel.getInverse();
                  VisualRelation vr = new VisualRelation(
                        document.getModel().findVisualEntity(newRel.getFrom()),
                        document.getModel().findVisualEntity((Entity)newRel.getValue()),
                        newRel);
                  ActivateRelationCommand action = new ActivateRelationCommand(document,vr);
                  if (document.performCommand( action ))
                      return true;
            }
        }
        return false;
  }

  public void die() {
        canUndoRedo = false;
        rcd = null;
        head = null;
        tail = null;
  }

  public boolean isSignificant() {
        return true;
  }


  public void redo() throws javax.swing.undo.CannotRedoException {
  }

  public void undo() throws javax.swing.undo.CannotUndoException {
  }

  public boolean addEdit(UndoableEdit parm1) {
	// For now we don't support merging.
        return false;
  }

  public String getRedoPresentationName() {
      return "Redo New Relation";
  }

  public boolean replaceEdit(UndoableEdit parm1) {
        // Assume that we can't for now.
        return false;
  }

  public String getPresentationName() {
    return "New Relation";
  }

}