package jcreek.cke.command;

import javax.swing.undo.UndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.cke.visualizer.AppSettings;
import jcreek.cke.conceptview.*;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.command.ActivateEntityCommand;
import jcreek.cke.command.ActivateRelationCommand;
import jcreek.cke.util.DPoint;
import jcreek.cke.MainWindow;
import jcreek.cke.graph.VertexList;
import jcreek.cke.graph.RelationList;
import jcreek.cke.graph.layout.TreeLayout;

import java.util.Vector;
import java.awt.event.ActionEvent;
import java.awt.Frame;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JOptionPane;

public class OverviewCommand extends CreekCommand {
/*
  TreeLayout talgl = new TreeLayout('r');
  TreeLayout talgd = new TreeLayout('u');
*/
  CreekModel model;

    /**
     * List of entities to be moved by the LayoutAlgorithm.
     */
    protected VertexList entities;


  public OverviewCommand(CreekDocument doc) {
      super(doc);
      model = doc.getModel();
      canUndoRedo = true;
  }

  public String getUndoPresentationName() {
    return "Undo Overview";
  }

  public boolean perform() {
        if ( ( getDocument() != null ) ) {
            model = getDocument().getModel();
            return createTree();
        } else {
            return false;
        }
  }

  public void die() {
        canUndoRedo = false;
  }

  public boolean isSignificant() {
    return true;
  }

  public void redo() throws javax.swing.undo.CannotRedoException {
        if ( canUndoRedo ) {
            CreekModel model = getDocument().getModel();
            resetEntities();
            getDocument().getView().repaint();
        } else {
            throw new CannotUndoException();
        }
  }

  public void undo() throws javax.swing.undo.CannotUndoException {
        if (canUndoRedo) {
            CreekModel model = getDocument().getModel();
            resetEntities();
            getDocument().getView().repaint();
        } else {
            throw new CannotUndoException();
        }
  }

  public boolean addEdit(UndoableEdit parm1) {
	// For now we don't support merging.
        return false;
  }

  public String getRedoPresentationName() {
    return "Redo Overview";
  }

  public boolean replaceEdit(UndoableEdit parm1) {
        // Assume that we can't for now.
        return false;
  }

  public String getPresentationName() {
    return "Overview";
  }

  protected boolean createTree()
  {

      VisualEntity[] ves = model.getSelectedEntities();
      entities = (model.getEntities()).copy();
      if(ves.length>0)
      {
          AppSettings props = AppSettings.instanceOf();
          TreeLayout layout;
          if (props.getBoolean(props.HORIZONTAL_ALLIGNMENT))
          {
            layout = new TreeLayout(TreeLayout.LAYOUT_DIRECTION_RIGHT);
          }
          else
          {
            layout = new TreeLayout(TreeLayout.LAYOUT_DIRECTION_UP);
          }
          layout.setRoot(ves[0]);
          layout.layout(model.getGraph(), document.getView().getBounds());
          model.fireChange();
          return true;
      }
      else
          return false;

  }

  protected void resetEntities()
  {
        VertexList currentList = model.getEntities();
        VertexList tempList = currentList.copy();
        int size = currentList.size();
        for ( int i = 0; i < size; i++ ) {
            VisualEntity selV = currentList.get( i );
            VisualEntity origV = entities.get( i );
            selV.setPosition( origV.getPosition().x, origV.getPosition().y );
        }
        entities = tempList.copy();
  }

}