package jcreek.cke.command;

import java.awt.event.ActionEvent;
import java.awt.print.PageFormat;
import java.awt.print.PrinterJob;
import java.awt.Frame;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JOptionPane;

import jcreek.cke.document.CreekDocument;
import jcreek.cke.CreekDocumentGroup;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class PageLayoutAction extends CreekAction implements PropertyChangeListener{

  public PageLayoutAction() {
      super("pagelayout");
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }


  public void actionPerformed(ActionEvent event) {
        // get window that contains our invoker
        Frame win = getWindow( event );
        // test for a current document
        CreekDocument doc = getDocument();

        if ( doc == null ) {
	    noCurrentDocument( win );
	    return;
        }

        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();

	  try {
                PrinterJob pj = PrinterJob.getPrinterJob ();
                PageFormat pf = pj.pageDialog (dg.getPageFormat());
                dg.setPageFormat(pf);
	  } catch (Exception ex) {
            JOptionPane.showMessageDialog
                ( win,
                  ex,
                  getString( "operFailedTitle" ),
                  JOptionPane.ERROR_MESSAGE );
	  }
  }

    public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "opened" ) )
        {
            if(e.getNewValue()==null)
                  setEnabled(false);
            else
                  setEnabled(true);
        }
        else if(e.getPropertyName().equals( "closed" ))
            setEnabled(false);
    }

}


