package jcreek.cke.command;

import javax.swing.*;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.*;
import java.awt.Toolkit;
import java.lang.reflect.Array;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.browser.Visualizer;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.conceptview.VisualEntity;
import jcreek.cke.util.EntitySelection;
import jcreek.cke.graph.VertexList;
import jcreek.cke.tool.CreekTool;
import jcreek.cke.util.SelectionSet;


/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class PasteAction extends CreekAction implements PropertyChangeListener{

  public static final DataFlavor dataFlavor =
          new DataFlavor(SelectionSet.class,"Array of VisualEntities");

  public PasteAction() {
        super( "paste" );
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }
    public void actionPerformed(ActionEvent event) {
        Visualizer win = getWindow( event );
        CreekTool tool = win.getCurrentTool();
         // first see if there is a current file and if it is dirty
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        CreekDocument doc = dg.getCurrentDocument();
        if ( doc == null ) {
	    noCurrentDocument( win );
            return;
        }
        paste(tool.getMousePosition(),win);
  }


  /**
   * Get the Transferable Objects from the Clipboard
   * Represent them on the View
   * Beep on failure
   */
  public void paste(Point loc,Visualizer win)
  {
      Clipboard c = Toolkit.getDefaultToolkit().getSystemClipboard();
      Transferable t = c.getContents(this);
      if(t == null)
      {
        Toolkit.getDefaultToolkit().beep();
        return;
      }

      try
      {
/*
          //ask for clipboard contents to be converted to our dataflavor
          VertexList vl = (VertexList)t.getTransferData(dataFlavor);
          PasteCommand pc = new PasteCommand(getDocument(),vl,loc);
          if(!getDocument().performCommand(pc))
            throw new Exception();
*/
          SelectionSet s = (SelectionSet)t.getTransferData(dataFlavor);
          if(s==null)
            return;
          PasteCommand pc = new PasteCommand(getDocument(),s.getVertexList(),
                            s.getRelationList(),loc);
          if(!getDocument().performCommand(pc))
            throw new Exception();

      }
      catch(UnsupportedFlavorException e)
      {
          JOptionPane.showMessageDialog
              ( win,
                getString( "pasteFailedMsg" ),
                getString( "operFailedTitle" ),
                JOptionPane.ERROR_MESSAGE );
      }
      catch(Exception e)
      {
          JOptionPane.showMessageDialog
              ( win,
                getString( "pasteFailedMsg" ),
                getString( "operFailedTitle" ),
                JOptionPane.ERROR_MESSAGE );
      }
  }

    public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "opened" ) )
        {
            if(e.getNewValue()==null)
                  setEnabled(false);
            else
                  setEnabled(true);
        }
        else if(e.getPropertyName().equals( "closed" ))
            setEnabled(false);
    }

}