package jcreek.cke.command;

import javax.swing.undo.UndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;

import javax.swing.*;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.datatransfer.*;
import java.awt.Toolkit;
import java.util.Iterator;
import java.lang.reflect.Array;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.browser.Visualizer;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.conceptview.*;
import jcreek.cke.util.EntitySelection;
import jcreek.cke.graph.VertexList;
import jcreek.cke.util.Matrix33;
import jcreek.cke.util.DPoint;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.cke.graph.RelationList;
import jcreek.cke.visualizer.AppSettings;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class PasteCommand extends CreekCommand {

    /**
     * List of Entities to be Pasted
     */
    protected VertexList nodes;


    protected RelationList relations;

    /**
     * Location to which the Entities
     * have to be pasted
     */
    protected Point loc;

  public PasteCommand(CreekDocument doc,VertexList ve,RelationList rl,Point loc) {
      super(doc);
      this.nodes = ve;
      this.relations = rl;
      this.loc = loc;
      canUndoRedo = true;
  }

  public String getUndoPresentationName() {
    return "Undo Paste";
  }

  public boolean perform() {
        if ( ( getDocument() != null ) && ( nodes != null ) ) {
            CreekModel model = getDocument().getModel();
//            initialiseEntities();
            resetEntities(nodes);
            resetRelations(relations);
            trimList();
            translateEntities();
            if(model.addEntityList(nodes)&&model.addRelationList(relations))
            {
                for(int i=0;i<nodes.size();i++)
                {
                    model.setSelected(nodes.get(i),true);
                }
                return true;
            }
            else
                return false;
        } else {
            return false;
        }
  }

  public void die() {
        canUndoRedo = false;
        nodes.clear();
        nodes = null;
        loc = null;
  }

  public boolean isSignificant() {
    return true;
  }

  public void redo() throws javax.swing.undo.CannotRedoException {
        if ( ( canUndoRedo ) && ( nodes != null ) ) {
            CreekModel model = getDocument().getModel();
            model.addEntityList(nodes);
            model.addRelationList(relations);
        } else {
            throw new CannotUndoException();
        }
  }

  public void undo() throws javax.swing.undo.CannotUndoException {
        if ( ( canUndoRedo ) && ( nodes != null ) ) {
            CreekModel model = getDocument().getModel();
            model.removeRelationList(relations);
            model.removeEntityList(nodes);

        } else {
            throw new CannotUndoException();
        }
  }

  public boolean addEdit(UndoableEdit parm1) {
	// For now we don't support merging.
        return false;
  }

  public String getRedoPresentationName() {
    return "Redo Paste";
  }

  public boolean replaceEdit(UndoableEdit parm1) {
        // Assume that we can't for now.
        return false;
  }

  public String getPresentationName() {
    return "Paste Selection";
  }

  public void translateEntities()
  {
      ConceptView view =  getDocument().getView();
      Matrix33 mTrans = view.getMoveTransform();
      Matrix33 vTrans = view.getViewTransform();

      double diffX=0,diffY=0;
      double prevX=0,prevY=0;
      if ( ( nodes == null ) ||
           ( nodes.size() == 0 ) ) {
          return;
      }
      else
      {

          prevX  = nodes.get(0).getPosition().x;
          prevY  = nodes.get(0).getPosition().y;
          DPoint tpt = new DPoint(loc.x,loc.y);
          tpt.transform(mTrans);
          nodes.get(0).setPosition(tpt);
//          diffX = (nodes.get(0).getPosition().x -tpt.x);
//          diffY = (nodes.get(0).getPosition().y -tpt.y);
      }

      // Loop through the given list and translate each vertex.
      int size = nodes.size();
      double tX=0,tY=0;
      for ( int i = 1; i < size; i++ ) {

          tX  = nodes.get(i).getPosition().x;
          tY  = nodes.get(i).getPosition().y;
          diffX = prevX-tX;
          diffY = prevY-tY;

          nodes.get(i).setPosition(nodes.get(i-1).getPosition().x-diffX,
                                  nodes.get(i-1).getPosition().y-diffY);
          prevX = tX;
          prevY = tY;
      }
  }

  /**
   * Removes VisualEntity and VisualRelation that
   * already exist in view
   *
   */
  public void trimList()
  {
        VertexList tVertexList = new VertexList();
        RelationList tRelationList =new RelationList();

        CreekModel model = getDocument().getModel();
        for(int i=0;i<nodes.size();i++)
        {
            VisualEntity ve = model.findVisualEntity(nodes.get(i).getEntity());
            if(ve==null)
                tVertexList.add(nodes.get(i));
        }
        nodes = tVertexList;

        for(int j=0;j<relations.size();j++)
        {
            if((model.findVisualRelation(relations.get(j).getRelation()))==null)
                tRelationList.add(relations.get(j));
        }
        relations = tRelationList;
  }

    private void initialiseEntities()
    {
        int j,relationSize = 0;
        VisualRelation ver;
        VisualEntity node;
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        KnowledgeModel km = dg.getKM();
        /*
        for(int i=0;i<nodes.size();i++)
        {
            node = nodes.get(i);
            node.getEntity() = (Entity)km.getEntity((node.getEntity().getName()));
            Relation[] currentRelations =  node.getEntity().getRelations( );
            Iterator iterator = node.getEdgeIterator();
            while(iterator.hasNext())
            {
                ver = (VisualRelation)iterator.next();
                for(int k=0;k<currentRelations.length;k++)
                    if((ver.getRelation().getName().equalsIgnoreCase(currentRelations[k].getName()))
                    &&(ver.getTo().getEntity().getName().equalsIgnoreCase(((Entity)currentRelations[k].getValue()).getName()))
                    &&(ver.getFrom().getEntity().getName().equalsIgnoreCase(((Entity)currentRelations[k].getFrom()).getName())))
                    {
                        ver.getRelation() = currentRelations[k];
                        break;
                    }
            }

        }
        */

    }


  protected void resetEntities(VertexList vl)
  {
      VisualEntity node;
      CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
      KnowledgeModel km = dg.getKM();
/*      for(int i=0;i<vl.size();i++)
      {
          node = vl.get(i);
          node.getEntity() = (Entity)km.getEntity((node.getEntity().getName()));
      }*/
  }

  protected void resetRelations(RelationList rl)
  {
      VisualRelation edge;
      CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
      KnowledgeModel km = dg.getKM();
      AppSettings appSettings = AppSettings.instanceOf();

      for(int i=0;i<rl.size();i++)
      {
          edge = rl.get(i);
          Relation[] currentRelations =  edge.getFrom().getEntity().getRelations( );
          for(int k=0;k<currentRelations.length;k++)
              if((edge.getRelation().getName().equalsIgnoreCase(currentRelations[k].getName()))
              &&(edge.getTo().getEntity().getName().equalsIgnoreCase(((Entity)currentRelations[k].getValue()).getName()))
              &&(edge.getFrom().getEntity().getName().equalsIgnoreCase(((Entity)currentRelations[k].getFrom()).getName())))
              {
//                  edge.getRelation() = currentRelations[k];
//                  appSettings.addChangeListener(edge);
                  break;
              }
      }
  }


}
