/*********************************************************************
 *
 * DESCRIPTION:
 *      This file defines the file print action.
 *
 ********************************************************************/

package jcreek.cke.command;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.MainWindow;
import jcreek.cke.conceptview.ConceptView;
import jcreek.cke.CreekDocumentGroup;

import java.awt.Frame;
import java.awt.Graphics;
import java.awt.PrintJob;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import java.awt.print.*;
import javax.swing.JOptionPane;

/**
 * Implements the print file action used to print the current
 * graph document to a hardcopy printer.
 *
 * @author
 * @version
 */
public class PrintAction extends CreekAction implements PropertyChangeListener{

  //mrg 9/11
  PrinterJob pJob;
  PageFormat pf;

    /**
     * Creates a new PrintAction object with the default action
     * command string of "print".
     */
    public PrintAction() {
        super( "print" );
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
    } // OpenAction

    /**
     * Performs the print action. This asks the current document
     * to discard any changes that have been made. Possibly
     * displays an error dialog if the print fails.
     *
     * @param  event  action event
     */
    public void actionPerformed( ActionEvent event ) {
        // get window that contains our invoker
        Frame win = getWindow( event );
        // test for a current document
        CreekDocument doc = getDocument();
        if ( doc == null ) {
	    noCurrentDocument( win );
	    return;
        }
        // get the toolkit so we can get a print job object
        Toolkit tk = Toolkit.getDefaultToolkit();
        if ( tk == null ) {
            System.err.println( "ERROR: couldn't get default toolkit" );
            return;
        }
        MainWindow mw = MainWindow.instanceOf();
/*
        PrintJob p = tk.getPrintJob
            ( win, mw.getResourceString( "AppTitle" ) + " - " +
              doc.getTitle(), null );
        // p will be null if user cancelled print dialog
        if ( p != null ) {
            Graphics g = p.getGraphics();
            if ( g != null ) {
                // get the view  so we print the
                // entire view
                ConceptView view = doc.getView();
                // draw the view to the graphics context
                view.print( g );
                g.dispose();
            } else {
                System.err.println( "ERROR: PrintJob returned null" );
            }
            p.end();
        }
*/

        if(pJob == null)
          pJob = PrinterJob.getPrinterJob();
        if(pJob != null)
        {
            if(pf == null)
                pf = CreekDocumentGroup.instanceOf().getPageFormat();
              pf = pJob.validatePage(pf);
        }
        else
        {
            JOptionPane.showMessageDialog
                ( win,
                  getString( "noPrinterMsg" ),
                  getString( "operFailedTitle" ),
                  JOptionPane.ERROR_MESSAGE );
            return;
        }

        if(pJob.printDialog())
        {
//          initPrint();
          pJob.setPrintable(doc.getView(),pf);
          try
          {
            pJob.print();
          }
          catch(PrinterException pEx)
          {
            JOptionPane.showMessageDialog
                ( win,
                  pEx,
                  getString( "operFailedTitle" ),
                  JOptionPane.ERROR_MESSAGE );
          }
        }

    } // actionPerformed

    public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "opened" ) )
        {
            if(e.getNewValue()==null)
                  setEnabled(false);
            else
                  setEnabled(true);
        }
        else if(e.getPropertyName().equals( "closed" ))
            setEnabled(false);
    }

} // OpenAction
