/*********************************************************************
 *
 * DESCRIPTION:
 *      This file defines the redo action.
 *
 ********************************************************************/
package jcreek.cke.command;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.undo.*;

import jcreek.cke.MainWindow;

/**
 * Implements the redo action used to backout a recent change
 * made to the graph.
 *
 * @author
 * @version
 */

public class RedoAction extends AbstractAction
{
    /**
     * The undo manager we belong to.
     * @serial
     */
    protected UndoManager undo;

    /**
     * Creates a new RedoAction object with the default action
     * command string of "redo". Sets the action initially
     * disabled.
     */
    public RedoAction() {
	super( "redo" );
	setEnabled( false );
    } // RedoAction

    /**
     * Creates a new RedoAction object with the default action
     * command string of "redo". Sets the action initially
     * disabled.
     *
     * @param  undo  undo manager that tells us if we can
     *               be performed or not
     */
    public RedoAction( UndoManager undo ) {
        this();
	setManager( undo );
    } // RedoAction

    /**
     * Performs the redo action.
     *
     * @param  e  action event
     */
    public void actionPerformed( ActionEvent e ) {
	try {
	    undo.redo();
	} catch ( CannotRedoException ex ) {
	    System.out.println( "Unable to redo: " + ex );
	}
	update();
	UndoAction undoAction = (UndoAction)MainWindow.instanceOf
	    ().getAction( "undo" );
	undoAction.update();
    } // actionPerformed

    /**
     * Sets the undo manager for this action to the one given.
     *
     * @param  undo  undo manager that tells us if we can
     *               be performed or not
     */
    public void setManager( UndoManager undo ) {
        this.undo = undo;
    } // setManager

    /**
     * Checks if we can redo an event or not and sets this
     * action enabled appropriately.
     */
    public void update() {
	if ( undo.canRedo() ) {
	    setEnabled( true );
	    putValue( Action.NAME, undo.getRedoPresentationName() );
	}
	else {
	    setEnabled( false );
	    putValue( Action.NAME, "Redo" );
	}
    } // update

}//RedoAction