package jcreek.cke.command;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.visualizer.RenameViewDialog;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JOptionPane;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class RenameViewAction extends CreekAction implements PropertyChangeListener{

    /**
     * Creates a new RenameViewAction object with the default action
     * command string of "renameView". Note that the action will be
     * disabled by default and will modify it's enabled state
     * automatically as documents become dirty.
     */

  public RenameViewAction() {
        super( "renameView" );
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }

    /**
     * Performs the Rename action. This will possibly ask the
     * user to input a title to save the View with
     *
     * @param  event  action event
     */
  public void actionPerformed(ActionEvent event) {
        //get the window to which it will be attached
        Visualizer win = getWindow( event );
        // first see if there is a current file
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        CreekDocument doc = (CreekDocument)getValue("selDoc" );
        if ( doc == null ) {
            doc = getDocument();
            if(doc==null)
            {
                noCurrentDocument( win );
                return;
            }
        }
//        String title = JOptionPane.showInputDialog(getString( "renameViewMsg" ));
        RenameViewDialog viewName = new RenameViewDialog(win,
                                getString( "renameViewTitle" ),
                                doc.getTitle());
        String title = null;

        if(viewName.isOK())
          title = viewName.getViewTitle();
        if(title ==null)
          return;
//        if(title.length()==0)
//          doc.setTitle(doc.getFile().getName());
//        else
          doc.setTitle(title);
        doc.setTitled(true);
        if(dg.isActive(doc))
        {
            win.getWin(doc).setTitle(doc.getTitle());
        }
        putValue("selDoc",null);
  }

    public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "opened" ) )
        {
            if(e.getNewValue()==null)
                  setEnabled(false);
            else
                  setEnabled(true);
        }
        else if(e.getPropertyName().equals( "closed" ))
            setEnabled(false);
    }

}