package jcreek.cke.command;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.MainWindow;
import jcreek.cke.document.CreekDocument;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.LocalKnowledgeModel;
import jcreek.cke.visualizer.SaveDialog;
import jcreek.cke.visualizer.AppSettings;

import java.awt.Frame;
//import java.awt.FileDialog;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.Action;
import javax.swing.JInternalFrame;
import javax.swing.JFileChooser;
import java.io.File;
import javax.swing.JOptionPane;
import java.awt.Component;
import java.awt.Rectangle;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class SaveModelAction extends CreekAction implements PropertyChangeListener {

  // dont want to recreate the dialog
  protected SaveDialog sd;

  public SaveModelAction() {
        super("saveModel");
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }

  public SaveModelAction(String actionString) {
        super(actionString);
  }

    /**
     * Performs the save action.
     * @param  event  action event
     */
  public void actionPerformed(ActionEvent event) {
        // save the file without special options
        saveFile( getWindow( event ), false, false );
  }//actionPerformed



    protected boolean saveFile( Visualizer win,
                                boolean askUser,
                                boolean forceSaveAs ) {
        // first see if there is a KnowledgeModel and if it is dirty
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        KnowledgeModel km = dg.getKM();
        if ( km == null ) {
	    noCurrentKM( win );
            return false;
        }

        if (!dg.isDirty() && !forceSaveAs ) {
            return true;
        }

        if (askUser && !askToSave(win))
          return true;

        if(forceSaveAs || !((LocalKnowledgeModel) km).hasFile())
        {
          JFileChooser chooser = new JFileChooser(AppSettings.instanceOf().getCurrentDirectory());
          chooser.setFileFilter( new CreekFileFilter( dg.getKMTypes() ) );

          if ( chooser.showSaveDialog( win ) == JFileChooser.CANCEL_OPTION ) {
              // user hit cancel
              return false;
          }
          File f = chooser.getSelectedFile();
          AppSettings.instanceOf().setCurrentDirectory(f.getAbsolutePath());
          if(!f.getName().endsWith(".km"))
            f = new File(f.getAbsolutePath()+".km");

          /*
          FileDialog fd = new FileDialog
              ( win,
                getString( "saveAs"+Visualizer.titleSuffix ),
                FileDialog.SAVE );
          fd.show();

          if ( ( fd.getFile() != null ) ||
               ( fd.getDirectory() != null ) ) {
                km.setFile(new File(fd.getDirectory(), fd.getFile()));
            }
          */
          ((LocalKnowledgeModel) km).setFile(f);
        }

        String error = null;
        if(((LocalKnowledgeModel) km).hasFile())
        {
          try
          {
            dg.saveKM();
          }
          catch(java.io.IOException e)
          {
            error = e.toString();
          }
        }
        else
          error = getString( "couldntSaveMsg" );
/*
        boolean okay = true;
        if(sd == null)
            sd = new SaveDialog
                ( win, getString( "saveAs"+Visualizer.titleSuffix ), true );
        sd.initTree();
        centerComponent(sd,win);
        sd.show();

        if(sd.isOk())
        {
              if(sd.isModelSelected())
              {
                  CreekDocument[] docs = sd.getSelectedDocuments();
                  for(int i=0;i<docs.length;i++)
                  {
                      if(!docs[i].isTitled())
                      {
                            docs[i].setTitle(docs[i].getFile().getName());
                            docs[i].setTitled(true);
                      }
                      dg.setCurrentDocument(docs[i]);
                      if(!(okay=dg.saveDocument()))
                          break;
                  }
                  if(okay)
                      okay = dg.saveKM();
              }
        }
        else
            return false;
      */
        if (error != null) {
            JOptionPane.showMessageDialog
                ( win,
		  error,
                  getString( "operFailedTitle" ),
                  JOptionPane.ERROR_MESSAGE );
            return false;
        }
        return true;
    } // saveFile



  public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "opened" ) )
        {
            if(e.getNewValue()==null)
                  setEnabled(false);
            else
                  setEnabled(true);
        }
        else if(e.getPropertyName().equals( "closed" ))
            setEnabled(false);
  }

    /**
     * Displays a dialog indicating that there's no current
     * KnowledgeModel to perform the desired operation.
     *
     * @param  win  frame to attach dialog to
     */
    public void noCurrentKM( Frame win ) {
	JOptionPane.showMessageDialog
	    ( win,
	      getString( "noCurKmMsg" ),
	      getString( "usrErrorTitle" ),
	      JOptionPane.ERROR_MESSAGE );
    } // noCurrentDocument

    /**
     * Saves all the documents to disk.
     *
     *
     */
    /*
    public void saveDocuments(Visualizer win) {
        CreekDocumentGroup cdg = CreekDocumentGroup.instanceOf();
        CreekDocument[] docs = cdg.getDocuments();
        SaveViewAction saveView =(SaveViewAction) MainWindow.instanceOf().getAction("saveView");
        for(int i=0;i<docs.length;i++)
        {
            if(docs[i].isDirty() || !(docs[i].isTitled()))
            {
                cdg.setCurrentDocument(docs[i]);
                saveView.saveFile(win,true,false);
            }
        }
    } // saveDocument
*/
    /**
     *
     * Ask the user if we should save the current dirty document.
     *
     * @param  win  parent window to attach dialog to
     * @return  true if we should save
     */
    protected boolean askToSave( Frame win ) {
        int result = JOptionPane.showConfirmDialog
            ( win,
              getString( "modelNotSavedMsg" ),
              getString( "saveFileLabel" ),
              JOptionPane.YES_NO_OPTION,
              JOptionPane.QUESTION_MESSAGE );
        return result == JOptionPane.YES_OPTION;
    } // askToSave


  protected void centerComponent( Component c ,Frame mainFrame) {
        Rectangle winBounds = mainFrame.getBounds();
        Rectangle cBounds = c.getBounds();
        c.setLocation( ( winBounds.width - cBounds.width ) / 2 +
                       winBounds.x,
                       ( winBounds.height - cBounds.height ) / 2 +
                       winBounds.y );
    } // centerComponent


}



