package jcreek.cke.command;

import java.util.Vector;
import java.awt.event.ActionEvent;
import java.awt.Frame;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JOptionPane;
import java.awt.Component;
import java.awt.Rectangle;


import jcreek.cke.browser.Visualizer;
import jcreek.cke.visualizer.SubModelDialog;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.cke.conceptview.*;
import jcreek.representation.RelationType;
import jcreek.cke.util.DPoint;
import jcreek.cke.graph.CreekModel;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class SubModelAction extends CreekAction implements PropertyChangeListener {


  public SubModelAction() {
        super("submodel");
        // we start life disabled, until a document is opened
        // and made dirty (which we'll know about by being a
        // change listener for the document group
        setEnabled( false );
        // we want to be notified of changes in the documents
        CreekDocumentGroup.instanceOf().addPropertyChangeListener( this );
  }


  public void actionPerformed(ActionEvent event) {
        Visualizer win = getWindow( event );
        // first see if there is a current file and if it is dirty
        CreekDocument doc = getDocument();
        if ( doc == null ) {
	    noCurrentDocument( win );
            return;
        }
        CreekModel model = doc.getModel();
        if(model != null)
        {
            if((model.getSelectedEntities()).length <=0)
            {
                  JOptionPane.showMessageDialog
                      ( win,
                        getString( "noCurSelMsg" ),
                        getString( "usrErrorTitle" ),
                        JOptionPane.ERROR_MESSAGE );
                  return;
            }
        }
        else
          return;
        displaySubModel(win,model.getSelectedEntities()[0]);
  }

  public void propertyChange( PropertyChangeEvent e ) {
        if ( e.getPropertyName().equals( "opened" ) )
        {
            if(e.getNewValue()==null)
                  setEnabled(false);
            else
                  setEnabled(true);
        }
        else if(e.getPropertyName().equals( "closed" ))
            setEnabled(false);
  }

    public void displaySubModel(Visualizer win,VisualEntity selEnt)
    {
        SubModelDialog smdg = new SubModelDialog(win,selEnt);
        centerComponent(smdg,win);
        smdg.setVisible(smdg.getCanShow());
        if(smdg.isOk())
        {
            SubModelCommand sc;
            CreekDocument currentDoc = getDocument();
            if(smdg.isNewView())
            {
                  ConceptView view = win.newWindow( "new window" );
                  CreekDocumentGroup docgroup = CreekDocumentGroup.instanceOf();

                    if ( !docgroup.newDocument( view ) ) {
                        // failed, close window we just created
                        win.closeWindow();
                        JOptionPane.showMessageDialog
                            ( win,
                              getString( "couldntCreateViewMsg" ),
                              getString( "operFailedTitle" ),
                              JOptionPane.ERROR_MESSAGE );
                              return;

                    } else {
                        // get document untitled name, set window title
                        currentDoc = docgroup.getCurrentDocument();
                        win.renameWindow( currentDoc.getTitle() );
                    }

            }
               sc = new SubModelCommand(currentDoc,
                  selEnt.getEntity(),smdg.getActivatedRelations());
            if(!currentDoc.performCommand(sc))
                JOptionPane.showMessageDialog
                    ( win,
                      getString( "invalidKMMsg" ),
                      getString( "operFailedTitle" ),
                      JOptionPane.ERROR_MESSAGE );
            smdg.dispose();
            return;
        }
    }//displaySubModel

  private void centerComponent( Component c ,Frame mainFrame) {
        Rectangle winBounds = mainFrame.getBounds();
        Rectangle cBounds = c.getBounds();
        c.setLocation( ( winBounds.width - cBounds.width ) / 2 +
                       winBounds.x,
                       ( winBounds.height - cBounds.height ) / 2 +
                       winBounds.y );
    } // centerComponent


}






