package jcreek.cke.command;

import java.util.Vector;
import java.awt.event.ActionEvent;
import java.awt.Frame;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JOptionPane;
import java.awt.Component;
import java.awt.Rectangle;
import javax.swing.undo.UndoableEdit;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;


import jcreek.cke.browser.Visualizer;
import jcreek.cke.visualizer.SubModelDialog;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.SubModel;
import jcreek.representation.EntityTypeNotMatchedException;
import jcreek.cke.conceptview.*;
import jcreek.representation.RelationType;

import jcreek.cke.util.DPoint;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.graph.VertexList;
import jcreek.cke.graph.RelationList;
import jcreek.cke.visualizer.AppSettings;
import jcreek.cke.graph.layout.TreeLayout;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

public class SubModelCommand extends CreekCommand {

  protected Entity ed;
  protected Entity cd[];
  protected SubModel activateSubModel;
  protected SubModel activeSubModel;
  protected TreeLayout talgl = new TreeLayout('r');
  protected TreeLayout talgd = new TreeLayout('u');


  public SubModelCommand(CreekDocument doc,Entity ed,
                    Entity cd[])
  {
      super(doc);
      this.ed = ed;
      this.cd = cd ;
      canUndoRedo = true;
  }

  public String getUndoPresentationName() {
    return "Undo SubModel";
  }

  public boolean perform() {
        if ( ( canUndoRedo ) && ( cd != null ) && ( ed != null )) {
            createSubModel();
            return true;
        } else {
            return false;
        }
  }

  public void die() {
        canUndoRedo = false;
        ed = null;
        cd = null;
        activateSubModel = null;
        activeSubModel = null;
  }

  public boolean isSignificant() {
    return true;
  }

  public void redo() throws javax.swing.undo.CannotRedoException {
        if ( ( canUndoRedo ) && ( cd != null ) && ( ed != null )) {
            deActivate(activeSubModel);
            activateEntity(activateSubModel);
            activateRelation(activateSubModel);

        } else {
            throw new CannotUndoException();
        }
  }

  public void undo() throws javax.swing.undo.CannotUndoException {
        if ( ( canUndoRedo ) && ( activateSubModel != null )&& ( activeSubModel != null ) ) {
          deActivate(activateSubModel);
          activateEntity(activeSubModel);
          activateRelation(activeSubModel);

        } else {
            throw new CannotUndoException();
        }
  }

  public boolean addEdit(UndoableEdit parm1) {
	// For now we don't support merging.
        return false;
  }

  public String getRedoPresentationName() {
    return "Redo SubModel";
  }

  public boolean replaceEdit(UndoableEdit parm1) {
        // Assume that we can't for now.
        return false;
  }

  public String getPresentationName() {
    return "SubModel";
  }

  protected void createSubModel()
  {

      Entity root=null;
      AppSettings props = AppSettings.instanceOf();

      CreekModel model = getDocument().getModel();

      boolean first = true;
      activateSubModel = new SubModel();
      activeSubModel   = new SubModel();

      Vector entitySize    = new Vector();
      Vector entityChecked = new Vector();

      VisualEntity ved = null;

      do
      {
          if(entitySize.size() != 0)
          {
            ed = (Entity)entitySize.lastElement();
            entitySize.remove(entitySize.lastElement());
            entityChecked.add((Object)ed);
            activateSubModel.add(ed);
          }

          ved = model.findVisualEntity(ed);
           if(ved!=null)
           {
                activeSubModel.add(ed);
                activeSubModel.add(model.getActiveRelations(ved));
           }
          if(first)
          {
              entityChecked.add((Object)ed);
              activateSubModel.add(ed);
              first = false;
//              document.getView().setRoot(ved);
              root = ed;
          }

          if(cd != null)
          {
              for(int i=0;i<cd.length;i++)
              {
                try
                {
                Relation[] relationClasses = ed.getLocalRelations((RelationType) new RelationType(cd[i]));
                activateSubModel.add(relationClasses);
                for(int j=0;j<relationClasses.length;j++)
                  if(!entityChecked.contains((Object)relationClasses[j].getValue()))
                    entitySize.add((Object)relationClasses[j].getValue());
                }
                catch(EntityTypeNotMatchedException e)
                {
                  e.printStackTrace();
                }
/*
                Relation[] invRelationClasses = ed.getLocalRelations(((RelationType)cd[i]).getInverseDescriptor());
                activateSubModel.add(invRelationClasses);
                for(int k=0;k<invRelationClasses.length;k++)
                    if(!entityChecked.contains((Object)invRelationClasses[k].getValue()))
                        entitySize.add((Object)invRelationClasses[k].getValue());
*/
              }//for

          }//if

      }
      while(entitySize.size()!=0);

      deActivate(activeSubModel);
      activateEntity(activateSubModel);
      activateRelation(activateSubModel);

      TreeLayout layout;

      if (props.getBoolean(props.HORIZONTAL_ALLIGNMENT))
      {
        layout = new TreeLayout(TreeLayout.LAYOUT_DIRECTION_RIGHT);
      }
      else
      {
        layout = new TreeLayout(TreeLayout.LAYOUT_DIRECTION_UP);
      }
      layout.setRoot(document.getModel().findVisualEntity(root));
      layout.layout(model.getGraph(), document.getView().getBounds());
      model.fireChange();
  }


  public void deActivate(SubModel sb)
  {
      CreekModel model = getDocument().getModel();

      model.removeRelationList(sb.getRelations());
      model.removeEntityList(sb.getEntities());
  }


  public void activateEntity(SubModel sb)
  {
      CreekModel model = getDocument().getModel();
      DPoint tempPoint = new DPoint();
      Entity[] ents = sb.getEntities();
      VisualEntity[] vents = new VisualEntity[ents.length];
      VisualEntity vent = null;
      for(int i=0;i<ents.length;i++)
      {
            tempPoint = getDocument().getView().activateEntity(vent);
            vents[i] = new VisualEntity(ents[i],(int)tempPoint.x,(int)tempPoint.y);
            vent = vents[i];
       }
       model.addEntityList(new VertexList(vents));
  }

  public void activateRelation(SubModel sb)
  {
      CreekModel model = getDocument().getModel();
      Relation[] rels = sb.getRelations();
      VisualRelation[] vrels = new VisualRelation[rels.length];
      for(int i=0;i<rels.length;i++)
      {
            vrels[i] = new VisualRelation(model.findVisualEntity(rels[i].getFrom()),
                      model.findVisualEntity((Entity)rels[i].getValue()),rels[i]);
       }

       model.addRelationList(new RelationList(vrels));
  }
}