/*********************************************************************
 *
 * DESCRIPTION:
 *      This file defines the undo action.
 *
 ********************************************************************/
package jcreek.cke.command;

import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.undo.*;

import jcreek.cke.MainWindow;

/**
 * Implements the undo action used to backout a recent change
 * made to the graph.
 *
 * @author
 * @version
 */

public class UndoAction  extends AbstractAction
{
    /**
     * The undo manager we belong to.
     * @serial
     */
    protected UndoManager undo;

    /**
     * Creates a new UndoAction object with the default action
     * command string of "undo". Sets the action initially
     * disabled.
     */
    public UndoAction() {
	super( "undo" );
	setEnabled( false );
    } // UndoAction

    /**
     * Creates a new UndoAction object with the default action
     * command string of "undo". Sets the action initially
     * disabled.
     *
     * @param  undo  undo manager that tells us if we can
     *               be performed or not
     */
    public UndoAction( UndoManager undo ) {
        this();
	setManager( undo );
    } // UndoAction

    /**
     * Performs the undo action.
     *
     * @param  e  action event
     */
    public void actionPerformed( ActionEvent e ) {
	try {
	    undo.undo();
	} catch ( CannotUndoException ex ) {
	    System.out.println( "Unable to undo: " + ex );
	}
	update();
	RedoAction redoAction = (RedoAction)MainWindow.instanceOf
	    ().getAction( "redo" );
	redoAction.update();
    } // actionPerformed

    /**
     * Sets the undo manager for this action to the one given.
     *
     * @param  undo  undo manager that tells us if we can
     *               be performed or not
     */
    public void setManager( UndoManager undo ) {
        this.undo = undo;
    } // setManager

    /**
     * Checks if we can undo an event or not and sets this
     * action enabled appropriately.
     */
    public void update() {
	if ( undo.canUndo() ) {
	    setEnabled( true );
	    putValue( Action.NAME, undo.getUndoPresentationName() );
	} else {
	    setEnabled( false );
	    putValue( Action.NAME, "Undo" );
	}
    } // update

}//UndoAction