/*********************************************************************
 *
 * DESCRIPTION:
 *      This file contains the ConceptViewPaintEvent class which defines
 *      the event object sent to ConceptViewPaintListeners when they are
 *      to draw to the view.
 *
 ********************************************************************/

package jcreek.cke.conceptview;

import java.awt.Graphics2D;
import java.util.EventObject;

/**
 * Event object used with ConceptViewPaintListener interface. Provides
 * necessary information for the paint listeners so they can
 * draw to the correct view object with the appropriate graphics
 * context.
 *
 * @author
 * @version
 */

public class ConceptViewPaintEvent extends EventObject
{
    /**
     * Graphics context the view paint listener is supposed to
     * draw to.
     * @serial
     */
    protected Graphics2D g;

    /**
     * Creates a new ViewPaintEvent object, using the given
     * view as the source of the event.
     */
    public ConceptViewPaintEvent(DrawingSurface  view, Graphics2D g ) {
	super( view );
	this.g = g;
    } // ViewPaintEvent

    /**
     * Returns the graphics context to which the view paint
     * listener can draw to the view. The returned graphics
     * context object is actually a copy of the original,
     * so the caller may modify this context as they please.
     *
     * @return  graphics context associated with this view
     */
    public Graphics2D getGraphics() {
	return (Graphics2D)g.create();
    } // getGraphics

    /**
     * Returns the ConceptView object associated with this
     * view paint event. In the event that the source of
     * the event is not a ConceptView object then this method
     * will return null. Use <code>EventObject.getSource()</code>
     * instead.
     *
     * @return  ConceptView object associated with this event, or
     *          null if source is not a view object
     */
    public DrawingSurface getView() {
	Object src = getSource();
	if ( src instanceof DrawingSurface ) {
	    return (DrawingSurface)src;
	} else {
	    return null;
	}
    } // getView

} //ConceptViewPaintEvent