/*********************************************************************
 *
 * DESCRIPTION:
 *      This file contains the ConceptViewPaintListener interface which defines
 *      the API for view-related painting activities.
 *
 ********************************************************************/

package jcreek.cke.conceptview;

import java.util.EventListener;

/**
 * Defines the interface for being a view paint listener. This
 * gives classes a chance to draw to the view before and/or
 * after the view performs the usual paint operation. This can
 * be used to add additional decorations to the view or to
 * allow unique user interaction not supported by the view.
 *
 * @author
 * @version
 */

public interface ConceptViewPaintListener extends EventListener
{

    /**
     * Called after the view draws the graph. Gives the
     * listener a chance to draw something to the view
     * after everything else has been drawn. The order in
     * which listeners are called is non-deterministic.
     *
     * @param  event  view paint event
     */
    public void paintAfter( ConceptViewPaintEvent event );

    /**
     * Called before the view draws the graph. Gives the
     * listener a chance to draw something to the view
     * before anything else appears. The order in which
     * listeners are called is non-deterministic.
     *
     * @param  event  view paint event
     */
    public void paintBefore( ConceptViewPaintEvent event );

} //ConceptViewPaintListener