package jcreek.cke.conceptview;

import javax.swing.JPanel;
import java.awt.*;

import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.EventListenerList;
import javax.swing.RepaintManager;
import java.awt.image.BufferedImage;
import java.awt.geom.Rectangle2D;
import java.awt.print.PageFormat;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.Vector;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import jcreek.cke.util.Matrix33;
import jcreek.cke.util.DDimension;
import jcreek.cke.util.DPoint;
import jcreek.cke.util.DPoint;
import jcreek.cke.command.ResizeListener;
import jcreek.cke.command.ResizeEvent;
import jcreek.cke.graph.*;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.graph.Graph;
import jcreek.cke.graph.layout.Layout;

import java.io.*;
import java.awt.print.PrinterException;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:
 * @author
 * @version 1.0
 */

public class DrawingSurface extends JPanel implements ConceptView, PropertyChangeListener
{

    /**
     * List of event listeners.
     * @serial
     */
    protected EventListenerList listeners;

    /**
     * List of entities from the model.
     * @serial
     */
//    protected VertexList el;

    /**
     * List of relations from the model.
     * @serial
     */
//    protected RelationList rl;

    protected Graph graph;
    protected CreekModel model;

    private BufferedImage backImage_ = null;
    private BufferedImage printImage = null;

    private Dimension	windowSize_ =
    new Dimension(-1, -1);	// window viewable size, -1 values
   				// will force a change_size at start

   // the scroll offsets - screen position + offset = physical position
   private double		offsetx_ = 0, offsety_ = 0;

   private double		minx_, miny_, maxx_, maxy_;

   private double		scale_ = 1.0;

   private Font font_;

   private double	width_,		// drawable width
   height_;	// drawable height


   private	DPoint	offset_ = new DPoint(0, 0);

   private Matrix33	viewTransform_, moveTransform_;
   private Matrix33	scaleMatrix_, shiftMatrix_;

   private Vector resizeListeners_ = new Vector();

   protected double hSpacing = 60, vSpacing = 80;

   protected VisualEntity rootEnt;


   protected  int MAX_NUM_PAGE = 100;

    public DrawingSurface()
    {

        font_ = new Font("Helvetica", Font.PLAIN, 12);
        computeBounds_();
         scaleMatrix_ = new Matrix33();

         scaleMatrix_.matrix[0][0] = scaleMatrix_.matrix[1][1] =
                              scaleMatrix_.matrix[2][2] = 1.0;

         shiftMatrix_ = new Matrix33(scaleMatrix_);


          listeners = new EventListenerList();

         updateViewTransform_();
         graph = new DefaultGraph();
//         el = new VertexList();
//         rl = new RelationList();
         setOpaque(true);
    }


   // This will give the initial window size.
      public Dimension getPreferredSize()
      {
         return new Dimension(400, 400);
      }

    protected void paintComponent(Graphics g)
    {
        super.paintComponent(g);
        paintOver(g);
    }

/*
    public synchronized void paint( Graphics g ) {
        update(g);
    }

    public synchronized void update( Graphics grap ) {
*/
    public synchronized void paintOver(Graphics g)
    {
        Dimension tmpdim = getSize();


//        System.out.println("Size: "+getSize().getHeight()+","+getWidth());
//        System.out.println("Offsets: "+getOffset().x+","+getOffset().y);
//        System.out.println("width: "+getWinWidth()+" height: "+getWinHeight());

         if(tmpdim.width != windowSize_.width || tmpdim.height != windowSize_.height)
         {
 /*        	// Maintain the center point.
            if(windowSize_.width > 0) // not first time
            {
               setOffsets_(offset_.x + (tmpdim.width - windowSize_.width) / 2,
                           offset_.y + (tmpdim.height - windowSize_.height) / 2);
            }
            else // initialize to centered
            {
               setOffsets_(.5 * (tmpdim.width - (minx_ + maxx_) * scale_ ),
                           .5 * (tmpdim.height - (miny_ + maxy_) * scale_));
            } */

            setOffsets_(0.0, 0.0);

            windowSize_.width = tmpdim.width;
            windowSize_.height = tmpdim.height;

            notifyResize(0);

            // Force recreation of back buffer.
            backImage_ = null;
        }


        if(backImage_ == null)
          backImage_ = (BufferedImage)createImage(windowSize_.width, windowSize_.height);


	firePaintBefore( (Graphics2D)g );


         Graphics2D back_graphics = getBackGraphics_();
         back_graphics.setColor(Color.white);
         back_graphics.setPaintMode();
         back_graphics.fillRect(0,0,windowSize_.width,windowSize_.height);


         back_graphics.setColor(Color.black);
         drawObjects(back_graphics);

         g.setPaintMode();
         g.drawImage(backImage_, 0, 0, null);


	firePaintAfter( (Graphics2D)g );
        back_graphics.dispose();
    }

      public void drawSelectedNodes()
      {
         Graphics2D graphics = (Graphics2D) getGraphicsInternal_();
         firePaintBefore( (Graphics2D)graphics );
         graphics.setColor(Color.black);
         graphics.setPaintMode();
         graphics.drawImage(backImage_, 0, 0, null);
         drawObjects(graphics);
         graphics.dispose();
	firePaintAfter( (Graphics2D)graphics );
      }

     // Get the graphics with the font set.
      private Graphics getGraphicsInternal_()
      {

         Graphics graphics = getGraphics();
         graphics.setFont(font_);
         return graphics;
      }



    private Graphics2D getBackGraphics_()
    {
         Graphics2D graphics = (Graphics2D)backImage_.createGraphics();
         graphics.setFont(font_);
         graphics.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                                  RenderingHints.VALUE_ANTIALIAS_ON );

         return graphics;
    }


     // Draw selected or unselected objects.
    public synchronized void drawObjects(Graphics2D graphics)
    {
        VisualEntity v;
        VisualRelation vr;
        // Then draw all of the vertices.
        if ( model != null && model.getGraph() != null )
        {

          for(Iterator it = model.getGraph().getNodeIterator(); it.hasNext(); )
          {
            Object node = it.next();
            if(node instanceof VisualEntity)
            {
              v = (VisualEntity) node;

              v.draw(graphics, viewTransform_);
//              System.out.println("Node "+v.creekLEntity+" pos "+v.getPosition().x+","+v.getPosition().y);
            }
          }
          /*
            int count = el.size();
            for ( int i = 0; i < count; i++ ) {
                v = el.get(i);
                v.draw(graphics, viewTransform_);
            } */
        }
        // Then draw all of the relations.


        for(Iterator it = model.getGraph().getEdgeIterator(); it.hasNext(); )
        {
          Object rel = it.next();
          if(rel instanceof VisualRelation)
          {
            vr = (VisualRelation) rel;
            vr.draw(graphics, viewTransform_);
          }
          /*
          }
        if ( rl != null ) {
            int count = rl.size();
            for ( int i = 0; i < count; i++ ) {
                vr = rl.get(i);
                vr.draw(graphics, viewTransform_);
            } */
        }

    }

      public DDimension contentsSize()
      {
         double w = width_ * scale_ + (double)windowSize_.width * 2.0;
         double h = height_ * scale_ + (double)windowSize_.height * 2.0;

         return new DDimension(w,h);
      }


      public DPoint getOffset()
      {
         DPoint val = new DPoint(0, 0);

         val.x = -(offset_.x - (double)windowSize_.width) - minx_ * scale_;
         val.y = -(offset_.y - (double)windowSize_.height) - miny_ * scale_;

         return val;
      }

      private void setOffsets_(double offx, double offy)
      {
         offset_.x = offx;
         offset_.y = offy;

         shiftMatrix_.matrix[0][2] = offx;
         shiftMatrix_.matrix[1][2] = offy;

         updateViewTransform_();
      }

      public void setOffsets(double xoffset, double yoffset, boolean redraw)
      {
         offsetx_ = xoffset / scale_;
         offsety_ = yoffset / scale_;
         computeDrawOffset_();
         if(redraw)
            repaint();

      }

      private void computeDrawOffset_()
      {
         setOffsets_((-offsetx_ - minx_) * scale_ + windowSize_.width,
                        (-offsety_ - miny_) * scale_ + windowSize_.height);
      }

      private void computeBounds_()
      {
         double	oldminx = minx_;
         double	oldminy = miny_;

         minx_ = miny_ = maxx_ = maxy_ = 0;
        if(model != null && model.getGraph() != null && model.getGraph().getNodeIterator().hasNext())
        {
             Iterator it = model.getGraph().getNodeIterator();
             VisualEntity tmpnode = (VisualEntity) it.next();
             DPoint tmppoint;
             DDimension tmpdim;

             if(tmpnode != null)
             {
                tmppoint = tmpnode.getPosition();
                tmpdim = tmpnode.getBoundingBox();

                minx_ = tmppoint.x - tmpdim.width / 2.0;
                maxx_ = tmppoint.x + tmpdim.width / 2.0;
                miny_ = tmppoint.y - tmpdim.height / 2.0;
                maxy_ = tmppoint.y + tmpdim.height / 2.0;

                while(it.hasNext())
                {
                   tmpnode = (VisualEntity) it.next();
                   tmppoint = tmpnode.getPosition();
                   tmpdim = tmpnode.getBoundingBox();

                   double w = tmpdim.width / 2.0;
                   double h = tmpdim.height / 2.0;

                   if(tmppoint.x - w < minx_)
                      minx_ = tmppoint.x - w;
                   if(tmppoint.x + w > maxx_)
                      maxx_ = tmppoint.x + w;
                   if(tmppoint.y - h < miny_)
                      miny_ = tmppoint.y - h;
                   if(tmppoint.y + h > maxy_)
                      maxy_ = tmppoint.y + h;
                }
             }
          }

      	// 3D approach.
         double maxdim = Math.abs(maxy_);
         if(Math.abs(miny_) > maxdim)
            maxdim = Math.abs(miny_);
         if(Math.abs(minx_) > maxdim)
            maxdim = Math.abs(minx_);
         if(Math.abs(maxx_) > maxdim)
            maxdim = Math.abs(maxx_);

         maxx_ = maxy_ = maxdim;
         minx_ = miny_ = -maxdim;

         width_ = height_ = 2.0 * maxdim;
      }

      private synchronized void updateViewTransform_()
      {
         viewTransform_ = new Matrix33(shiftMatrix_);
         viewTransform_.mult(scaleMatrix_);

         moveTransform_ = new Matrix33();
         moveTransform_.matrix[0][0] = moveTransform_.matrix[1][1] =
                              moveTransform_.matrix[2][2] = 1.0;

         Matrix33 tmp_matrix = new Matrix33();
         tmp_matrix.setTo(scaleMatrix_);
         tmp_matrix.matrix[0][0] = 1.0 / tmp_matrix.matrix[0][0];
         tmp_matrix.matrix[1][1] = 1.0 / tmp_matrix.matrix[1][1];
         moveTransform_.mult(tmp_matrix);

         tmp_matrix.setTo(shiftMatrix_);
         tmp_matrix.matrix[0][2] = - tmp_matrix.matrix[0][2];
         tmp_matrix.matrix[1][2] = - tmp_matrix.matrix[1][2];
         moveTransform_.mult(tmp_matrix);

         viewTransform_.scale = scale_;
      }

      public void center()
      {
         computeBounds_();

         setOffsets_(.5 * (windowSize_.width - (minx_ + maxx_) * scale_),
                     .5 * (windowSize_.height - (miny_ + maxy_) * scale_));
         notifyResize(0);
         repaint();
      }

      public synchronized void addResizeListener(ResizeListener rl)
      {
          resizeListeners_.addElement(rl);
      }

      public synchronized void removeResizeListeners(ResizeListener rl)
      {
          resizeListeners_.removeElement(rl);
      }

      public void fireContentsResize()
      {
          notifyResize(0);
      }

      private synchronized void notifyResize(int stats)
      {
          ResizeEvent re = new ResizeEvent(this,stats);
          Enumeration em = resizeListeners_.elements();
          while(em.hasMoreElements())
          {
              ((ResizeListener)em.nextElement()).resizePerformed(re);
          }
      }



    /**
     * Adds a view listener to this object's listener list.
     *
     * @param  l  listener to add to view's list
     */
    public void addViewListener( ConceptViewListener l ) {
        addMouseListener( l );
        addMouseMotionListener( l );
    } // addViewListener

    /**
     * Adds a view paint listener to this object's listener
     * list.
     *
     * @param  l  listener to add to view's list
     */
    public void addViewPaintListener( ConceptViewPaintListener l ) {
	listeners.add( ConceptViewPaintListener.class, l );
    } // addViewPaintListener


    /**
     * Give all the paint listeners a chance to draw after
     * the view has drawn the graph.
     *
     * @param  g  graphics context to draw to
     */
    protected void firePaintAfter( Graphics2D g ) {
        // Get the listener list as class/instance pairs.
        Object[] list = listeners.getListenerList();
	ConceptViewPaintEvent e = null;
        // Process the listeners last to first.
        // List is in pairs: class, instance
        for ( int i = list.length - 2; i >= 0; i -= 2 ) {
            if ( list[i] == ConceptViewPaintListener.class ) {
		if ( e == null ) {
		    e = new ConceptViewPaintEvent( this, g );
		}
                ConceptViewPaintListener vpl = (ConceptViewPaintListener)list[i+1];
                vpl.paintAfter( e );
            }
        }
    } // firePaintAfter

    /**
     * Give all the paint listeners a chance to draw before
     * the view draws.
     *
     * @param  g  graphics context to draw to
     */
    protected void firePaintBefore( Graphics2D g ) {
        // Get the listener list as class/instance pairs.
        Object[] list = listeners.getListenerList();
	ConceptViewPaintEvent e = null;
        // Process the listeners last to first.
        // List is in pairs: class, instance
        for ( int i = list.length - 2; i >= 0; i -= 2 ) {
            if ( list[i] == ConceptViewPaintListener.class ) {
		if ( e == null ) {
		    e = new ConceptViewPaintEvent( this, g );
		}
                ConceptViewPaintListener vpl = (ConceptViewPaintListener)list[i+1];
                vpl.paintBefore( e );
            }
        }
    } // firePaintBefore

    /**
     * Removes a view listener from this object's listener list.
     *
     * @param  l  listener to remove from view's list
     */
    public void removeViewListener( ConceptViewListener l ) {
        removeMouseListener( l );
        removeMouseMotionListener( l );
    } // removeViewListener

    /**
     * Removes a view paint listener from this object's listener
     * list.
     *
     * @param  l  listener to remove from view's list
     */
    public void removeViewPaintListener( ConceptViewPaintListener l ) {
	listeners.remove( ConceptViewPaintListener.class, l );
    } // removeViewPaintListener


    /**
     * The settings object we're listening to has changed.
     * Check if we have to change our drawing options.
     *
     * @param  event  change event
     */
    public void stateChanged( ChangeEvent event ) {
    } // stateChanged


    /**
     * The view is being shut down. This gives the view a chance
     * to remove itself from any listener lists it may be on.
     */
    public void detach() {
//        AppSettings.instanceOf().removeChangeListener( this );
      model = null;
        if(resizeListeners_ != null)
            resizeListeners_.removeAllElements();
    } // detach


    /**
     * Deprectaed, use setGraph()
     */
    public void dataChanged(VertexList vl, RelationList rl)
    {
      setGraph(new DefaultGraph(vl, rl));
    }

    /**
     * Updates the display to match the current state of the model.
     *
     * @param  el  list of entities
     * @param  rl  list of entities
     */
    public void setGraph(Graph graph) {
/*        for(int i= 0;i<vel.size();i++)
        {
            if(el.indexOf(vel.get(i))!= -1)
              layout.entityActivated(vel.get(i));
        }
*/
        this.graph = graph;
        computeBounds_();
        notifyResize(0); //check this
        repaint(0,0,windowSize_.width,windowSize_.height);
/*        for(int i= 0;i<el.size();i++)
            layout.entityActivated(el.get(i));
*/
    } // dataChanged

    /**
     * We're being attached to a document for the first time.
     */
    public void attach(CreekModel model) {
    this.model = model;
    model.addChangeListener(this);
//        detach();
//        AppSettings.instanceOf().addChangeListener( this );
    } // attach

    public Matrix33 getViewTransform()
    {
        return viewTransform_;
    }

    public Matrix33 getMoveTransform()
    {
        return moveTransform_;
    }
    public double getWinWidth()
    {
        return width_;
    }

    public double getWinHeight()
    {
        return height_;
    }

      public Point getMidPoint()
      {
         Point retval = new Point((int)(windowSize_.width / 2.0),
                                      (int)(windowSize_.height / 2.0));
//         retval.transform(moveTransform_);

         return retval;
      }


      public String toString()
      {
          return (CreekDocumentGroup.instanceOf().getDocument((ConceptView)this)).getTitle();
      }

      public void setScale(double new_scale)
      {
      	// Scale about the center - compute new offset to keep centered
         setOffsets_(windowSize_.width / 2.0 - (new_scale / scale_) * (windowSize_.width / 2.0 - offset_.x),
                     windowSize_.height / 2.0 - (new_scale / scale_) * (windowSize_.height / 2.0 - offset_.y));

         scale_ = new_scale;

         scaleMatrix_.matrix[0][0] = scaleMatrix_.matrix[1][1] = scale_;

         updateViewTransform_();

         notifyResize(0);

         repaint(0,0,windowSize_.width,windowSize_.height);
      }

    public double getScale()
    {
        return scale_;
    }

    public Rectangle getBounds()
    {
      return new Rectangle(0,0, 300, 300);
    }

      public VisualEntity findVisualEntity(double x, double y)
      {
         VisualEntity node = null;

         VisualEntity tmpnode;
         DPoint pos;
         DDimension bbox;

         double closest = (width_ + height_) * scale_;
         if(model != null && model.getGraph() != null)
         {
             for(Iterator it = model.getGraph().getNodeIterator(); it.hasNext();)
             {
               tmpnode = (VisualEntity) it.next();
                if(tmpnode != null)
                {
                    pos = tmpnode.getPosition();
                    bbox = tmpnode.getBoundingBox();

                    pos.transform(viewTransform_);

                    pos.x = Math.abs(pos.x - x);
                    pos.y = Math.abs(pos.y - y);


                    if(pos.x < bbox.width / 2.0 * scale_ + 1 && pos.y < bbox.height / 2.0 * scale_ + 1)
                    {
                       if(pos.x + pos.y < closest)  // rough estimate of closeness
                       {
                          closest = pos.x + pos.y;
                          node = tmpnode;
                       }
                    }
                }
             }
        }
         return node;
      }//findVisualEntity

  /** changed
   */
  public double getHSpacing()
  {
    return 1/scale_ * hSpacing;
  }

  /** changed
   */
  public double getVSpacing()
  {
    return 1/scale_ * vSpacing;
  }

/*  public void layout(Layout layout,Graph graph)
  {
/*
        if(graph.numberOfNodes() > 1)
        {
            graph.pack();  // Pack the indexes.
            alg.compute(graph, this);
        }
        notifyResize(0); //check this
        repaint(0,0,windowSize_.width,windowSize_.height);
*
        graph.setRoot(getRoot());
        layout.layout(graph, new Rectangle(0,0, (int) Math.round(width_), (int) Math.round(height_)));
        notifyResize(0); //check this
        repaint(0,0,windowSize_.width,windowSize_.height);
  }  */

  public VisualEntity getRoot()
  {
      return rootEnt;
  }

  public void setRoot(VisualEntity vent)
  {
      rootEnt = vent;
  }

  public int entitySize()
  {
    return model.getGraph().nodeSize();
  }

  public Graph getGraph()
  {
    return model.getGraph();
  }
/*
  public VisualEntity entityAt(int i)
  {
    return (VisualEntity) el.get(i);
  }
  */
  public DPoint activateEntity( VisualEntity activator)
  {
    if(activator == null)
    {
      Point p = new Point(getMidPoint());
      VisualEntity closest = findVisualEntity(p.x,p.y);
      if(closest == null)
      {
          DPoint pos = new DPoint(p.x,p.y);
          pos.transform(getMoveTransform());
          return pos;
      }
      else
        return activateEntity(closest);
    }
    else
      return new DPoint(Math.round(Math.random()*200)+activator.x_+100,(int) Math.round(Math.random()*200)-100+activator.y_);

  }



    public BufferedImage createPrintImage()
    {
       BufferedImage printImage;
/*
       // Get graph drawn boundaries.
       DPoint graphwidth = new DPoint(), graphheight = new DPoint();
       getDrawBounds_(graphwidth, graphheight);
       printImage = (BufferedImage)createImage((int)(graphwidth.y - graphwidth.x),(int)(graphheight.y - graphheight.x));
       Graphics2D printGraphics = (Graphics2D)printImage.getGraphics();
       printGraphics.translate(-graphwidth.x,-graphheight.x);
*/

/*
      computeBounds_();
       printImage = (BufferedImage)createImage((int)( maxx_ - minx_),(int)( maxy_ - miny_));
       Graphics2D printGraphics = (Graphics2D)printImage.getGraphics();
       printGraphics.translate(minx_,miny_);
*/

        Rectangle2D.Double bounds = getContentRect(getGraphics());
        if(bounds==null)
          return new BufferedImage(1,1,BufferedImage.TYPE_INT_RGB);

        printImage = (BufferedImage)createImage((int)((bounds.width+50.0)*getScale()),
                                                (int)((bounds.height+50.0)*getScale()));
/*
        printImage = new BufferedImage((int)((bounds.width+50.0)*getScale()),
                                                (int)((bounds.height+50.0)*getScale()),
                                                BufferedImage.TYPE_INT_RGB);
*/
        Graphics2D printGraphics = (Graphics2D)printImage.getGraphics();

        DPoint tPoint = new DPoint(bounds.x,bounds.y);
        tPoint.transform(getViewTransform());
        printGraphics.translate(-(tPoint.x-25.0),-(tPoint.y-25.0));


//       printGraphics.setColor(Color.white);
//       printGraphics.setPaintMode();
//       printGraphics.fillRect(0,0,);
//       printGraphics.setColor(Color.black);

       drawObjects(printGraphics);
//       printGraphics.dispose();
       return printImage;
    }//createPrintImage

/*
         // Get bounds of drawing with current transform.
      public void getDrawBounds_(DPoint width, DPoint height)
      {
         boolean first = true;
         DPoint tmpwidth = new DPoint(), tmpheight = new DPoint();
         DPoint pos = new DPoint();
         VisualEntity tmpnode;
         for(int i=0;i<el.size();i++)
         {
            tmpnode = el.get(i);
            tmpnode.getDrawBounds_(scale_, viewTransform_,
                                   tmpwidth, tmpheight);
            if(first)
            {
               first = false;
               width.move(tmpwidth);
               height.move(tmpheight);
            }
            else
            {
               width.x = Math.min(width.x, tmpwidth.x);
               width.y = Math.max(width.y, tmpwidth.y);
               height.x = Math.min(height.x, tmpheight.x);
               height.y = Math.max(height.y, tmpheight.y);
            }


         }

         if(first)
         {
            width.move(0.0, 1.0);
            height.move(0.0, 1.0);
         }

      }//getDrawBounds_

*/

  public Rectangle2D.Double getContentRect(Graphics g)
  {
      Rectangle2D.Double bounds = null;
      VisualEntity tmpnode;
      boolean first = true;

      for(Iterator it = model.getGraph().getNodeIterator(); it.hasNext(); )
      {
        tmpnode = (VisualEntity) it.next();
        if(first)
          bounds = tmpnode.getDrawBounds(g);
        else
          bounds.add(tmpnode.getDrawBounds(g));
        first = false;
      }
      return bounds;

  }

  public int print(Graphics pg, PageFormat pageFormat , int pageIndex)
            throws PrinterException
  {

      if(printImage==null)
          printImage = createPrintImage();

      if (pageIndex >= MAX_NUM_PAGE || printImage == null)
        return NO_SUCH_PAGE;


      pg.translate((int)pageFormat.getImageableX(),
      (int)pageFormat.getImageableY());

      int wPage = (int)pageFormat.getImageableWidth();
      int hPage = (int)pageFormat.getImageableHeight();
      int w = printImage.getWidth(this);
      int h = printImage.getHeight(this);

      if (w == 0 || h == 0)
      {
        printImage = null;
        return NO_SUCH_PAGE;
      }
      int nCol = Math.max((int)Math.ceil((double)w/wPage), 1);
      int nRow = Math.max((int)Math.ceil((double)h/hPage), 1);
      MAX_NUM_PAGE = nCol*nRow;
      int iCol = pageIndex % nCol;
      int iRow = pageIndex / nCol;
      int x = iCol*wPage;
      int y = iRow*hPage;
      int wImage = Math.min(wPage, w-x);
      int hImage = Math.min(hPage, h-y);

      pg.drawImage(((Image)printImage), 0, 0, wImage, hImage,
      x, y, x+wImage, y+hImage, this);

      System.gc();

      return PAGE_EXISTS;
  }

  public void propertyChange(PropertyChangeEvent e)
  {
//    System.out.println("PropertyChangeEvent. Size of graph: "+model.getGraph().nodeSize());
    computeBounds_();
    notifyResize(0); //check this
        repaint(0,0,windowSize_.width,windowSize_.height);
  }

}


