package jcreek.cke.conceptview;

import java.io.*;
import java.util.Vector;
import java.awt.Graphics2D;
import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.Rectangle;
import java.lang.Math;
import java.awt.Graphics;
import java.awt.FontMetrics;
import java.awt.geom.Rectangle2D;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.util.Iterator;
import java.util.ArrayList;

import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;
import jcreek.cke.util.DPoint;
import jcreek.cke.util.DDimension;
import jcreek.cke.util.Matrix33;

import jcreek.cke.graph.Node;
import jcreek.cke.graph.Edge;

import jcreek.cke.CreekDocumentGroup;


public class VisualEntity  implements Node
{
    public Object entityID;

    protected double scale=1.0;
    protected Font entityFont = new Font("Arial",Font.PLAIN,(int) Math.round(14*scale));

    protected boolean	selected;

    protected double  width, height, depth_;

    protected double	x_, y_, z_;



//    protected Color DEFAULT_COLOR = new Color(134,134,174);
    protected Color DEFAULT_COLOR = new Color(149,149,255);
    protected Color FONT_COLOR = new Color(0,0,0);

  private static final long serialVersionUID=6680498738713073216L;

  public VisualEntity(Entity myEntity, Point pos)
  {
    width  =20.0;
    height =20.0;
    depth_  = 20.0;
    x_ = pos.x;
    y_ = pos.y;
    z_ = 0;
    entityID = myEntity.getID();
    edgeList = new ArrayList(5);
    childList = new ArrayList(5);
  }

  public VisualEntity( Entity myEntity, int startX, int startY)
  {
    width  =20.0;
    height =20.0;
    depth_  = 20.0;
    x_ = startX;
    y_ = startY;
    z_ = 0;
    entityID = myEntity.getID();
    edgeList = new ArrayList(5);
    childList = new ArrayList(5);
  }



  public boolean represents(Entity e)
  {
    return e.getID().equals(entityID);
  }


  public void draw(Graphics2D graphics, Matrix33 transform)
  {
    draw(graphics, transform, transform.scale);
  }
    public void draw(Graphics2D graphics, Matrix33 transform, double currentScale)
//  public void draw(Graphics2D graphics, double currentScale)
  {
       Color defColor = graphics.getColor();

//       double currentScale = transform.scale;
       if(scale != currentScale)
       {
           scale = currentScale;
           entityFont = new Font("Arial",Font.PLAIN,(int) Math.round(14*scale));
       }

       graphics.setFont(entityFont);

       double w = width * scale;
       double h = height * scale;

       if(w < 1)
          w = 1;
       if(h < 1)
          h = 1;

        DPoint pos = getPosition();
        if(transform != null)
        pos.transform(transform);

       int x = (int)pos.x;
       int y = (int)pos.y;

       graphics.setColor(DEFAULT_COLOR);

       graphics.fillOval((int)(x - w / 2), (int)(y - h / 2),
                               (int)w, (int)h);

       //draw label
       graphics.setColor(FONT_COLOR);
       double widest = 0.0;
       double lx, ly;
       FontMetrics fm = graphics.getFontMetrics();
       widest = fm.stringWidth(getEntity().getName());
       lx = x - widest / 2.0;
       ly = y + h / 2.0 + fm.getAscent() + 1.0;
       graphics.drawString(getEntity().getName(), (int)lx, (int)ly);

       //draw selection
       if(!selected)
          return;

       graphics.setColor(Color.red);

      // Draw selection components.
       DPoint tr = new DPoint(pos.x + width *  scale / 2.0 + 1,
                              pos.y - height * scale / 2.0 - 1);
       DPoint bl = new DPoint(pos.x - width *  scale / 2.0 - 1,
                              pos.y + height * scale / 2.0 + 1);

       graphics.drawRect((int) Math.round(pos.x - 4), (int) Math.round(pos.y - 1), (int) Math.round(8), (int) Math.round(2));
       graphics.drawRect((int) Math.round(pos.x - 1), (int) Math.round(pos.y - 4), (int) Math.round(2) , (int) Math.round(8));
       graphics.setColor(Color.white);
       graphics.drawLine((int) Math.round(pos.x - 3), (int)pos.y,
                            (int) Math.round(pos.x + 3), (int)pos.y);
       graphics.drawLine((int)pos.x, (int) Math.round(pos.y - 3),
                            (int)pos.x, (int) Math.round(pos.y + 3));

        graphics.setColor(defColor);

    }



      public void setPosition(DPoint new_position)
      {
         x_ = new_position.x;
         y_ = new_position.y;
      }


      public void setPosition(double new_x, double new_y, double new_z)
      {
         x_ = new_x;
         y_ = new_y;
         z_ = new_z;
      }


      public void setBoundingBox(double new_width, double new_height)
      {
         width = new_width;
         height = new_height;
      }

      public void setBoundingBox(DDimension new_bbox)
      {
         width = new_bbox.width;
         height = new_bbox.height;
      }



      public DDimension getBoundingBox()
      {
         DDimension bbox = new DDimension(width, height);
         return bbox;
      }

      public void slide(int xoffs, int yoffs)
      {
          x_ += xoffs;
          y_ += yoffs;
      }

      public DPoint intersectWithLineTo(DPoint to)
      {

         double dy = to.y - y_;
         double dx = to.x - x_;

      	// If point is inside the node, return the centerpoint.
         if(dx * dx / (width * width) + dy * dy / (height * height)
            <= .25)
            return new DPoint(x_, y_);



         if(dx == 0)
         {
            if(dy > 0)
               return new DPoint(x_, y_ + height / 2);
            else
               return new DPoint(x_, y_ - height / 2);
         }
         double slope = dy / dx;

         double x, y;

//          double tempx = (width/2)*Math.cos(Math.atan(slope));

            x = width * height / (2 * Math.sqrt(height * height
                                                  + slope * slope * width * width));
            if(to.x < x_)
               x = -x;
            y = slope * x;

            return new DPoint(x_ + x, y_ + y);
      }


/*
      // Get the drawn bounds of the Node.
      public void getDrawBounds_(double scale, Matrix33 transform,
                                 DPoint width, DPoint height)
      {
         double w = width * scale;
         double h = height * scale;

         if(w < 1)
            w = 1;
         if(h < 1)
            h = 1;


         DPoint position = new DPoint(x_, y_);
         position.transform(transform);

         width.x = position.x - w / 2.0;
         width.y = position.x + w / 2.0;
         height.x = position.y - h / 2.0;
         height.y = position.y + h / 2.0;
      }

*/

      public Rectangle2D.Double getDrawBounds(Graphics g)
      {
         double w = width;
         double h = height;

         if(w < 1)
            w = 1;
         if(h < 1)
            h = 1;
         Rectangle2D.Double bounds = new Rectangle2D.Double(
                                    getPosition().x-w/2,
                                    getPosition().y-h/2,
                                    w,
                                    h);
        bounds.add(getTextBounds(g));
        return bounds;
      }

      protected Rectangle2D.Double getTextBounds(Graphics g)
      {
         double widest = 0.0;
         double lx, ly;
         double h = height;
         if(h < 1)
            h = 1;
         FontMetrics fm = g.getFontMetrics();
         widest = fm.stringWidth(getEntity().getName());
         lx = getPosition().x - widest / 2.0;
         ly = getPosition().y + h / 2.0 + fm.getAscent() + 1.0;
         return new Rectangle2D.Double(lx,ly,widest,fm.getHeight());
      }


      public Object clone() throws java.lang.CloneNotSupportedException
      {
         VisualEntity copy;
         copy = (VisualEntity)super.clone();
         return copy;
      }


/**
 * Node Methods
 */

      private ArrayList edgeList;
      private ArrayList childList;


      public DPoint getPosition()
      {
         DPoint position = new DPoint(x_, y_);
         return position;
      }


       public void setPosition(double new_x, double new_y)
      {
         x_ = new_x;
         y_ = new_y;
      }

      public void setSelected(boolean selected)
      {
         this.selected = selected;
      }


      public boolean isSelected()
      {
         return selected;
      }

      public boolean addEdge(Edge edge)
      {
          if(!edgeList.contains(edge))
          {
              if(edge.getHead()==this)
                addChild(edge.getTail());
              else
                addChild(edge.getHead());
              edgeList.add(edge);
              return true;
          }
          return false;
      }

      public void removeEdge(Edge edge)
      {
          int index = edgeList.indexOf(edge);
          if(index>=0)
          {
              childList.remove(index);
              edgeList.remove(index);
          }
      }

        public boolean containsEdge(Edge edge)
        {
            return edgeList.contains(edge);
        }

        public void removeChild(Node child)
        {
            Node node;
            Edge edge;
            int index;
            Iterator iterator = childList.iterator();
            while(iterator.hasNext())
            {
                node = (Node)iterator.next();
                if(node==child)
                {
                    index = childList.indexOf(child);
                    edge  = (Edge)edgeList.get(index);
                    removeEdge(edge);
                    iterator = childList.iterator();
                }
            }
        }

        public boolean containsChild(Node child)
        {
            return childList.contains(child);
        }

        public Iterator getChildIterator()
        {
            return childList.iterator();
        }

        public int getNumChildren()
        {
            return childList.size();
        }

        public Iterator getEdgeIterator()
        {
            return edgeList.iterator();
        }

        protected void addChild(Node child)
        {
            childList.add(child);
        }


        public Entity getEntity()
        {
            return CreekDocumentGroup.instanceOf().getKM().getEntity(entityID);
        }

        public boolean relationIsActive(RelationType rcd)
        {
              Relation[] rels = getEntity().getLocalRelations(rcd);
              if(rels.length==0)
                  return false;
              boolean active = true;
              for(int i=rels.length-1;i>=0;i--)
              {
                if(!relationIsActive(rels[i]))
                {
                  active = false;
                }
              }
              return active;
        }

        public boolean relationIsActive(Relation relation)
        {
            boolean active = false;
            Relation activeRelation;
            Iterator iterator = getEdgeIterator();
            while(iterator.hasNext())
            {
              activeRelation = ((Edge)iterator.next()).getRelation();
              if(activeRelation == relation || activeRelation == relation.getInverse())
              {
                active = true;
                break;
              }
            }

            return active;
        }

        public boolean relationIsActive(String relationName)
        {
            Relation[] rels = getEntity().getRelations(relationName);
            if(rels.length==0)
                return false;
            boolean active = true;
            for(int i=rels.length-1;i>=0;i--)
            {
              if(!rels[i].isInherited() && !relationIsActive(rels[i]))
              {
                active = false;
              }
            }
            return active;
      }

        public int getCount(Relation relation)
        {
              int count =0;
              return count;
        }

        public int hashCode()
        {
          return 7+getEntity().hashCode();
        }

        public boolean equals(Object ob)
        {
          if(ob instanceof VisualEntity)
            return ((VisualEntity) ob).getEntity().equals(getEntity());
          else
            return false;
        }
    public String toString()
    {
      return "VisualEntity for "+getEntity().getName()+" at "+x_+", "+y_;
    }
}


