package jcreek.cke.controls;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.font.*;
import java.awt.geom.*;

import jcreek.cke.MainWindow;
import jcreek.cke.browser.Visualizer;


public class SideBar extends JPanel implements ActionListener
{

  protected JToggleButton hiddenButton;
  protected Visualizer win;

  public SideBar(Visualizer win)
  {
        this.win = win;
        this.setLayout( new BoxLayout( this, BoxLayout.Y_AXIS ) );
        createButtons();
  }


  protected void createButtons()
  {
        ButtonGroup group = new ButtonGroup();
        hiddenButton = new JToggleButton();
        group.add(hiddenButton);
        hiddenButton.setSelected(true);


        // Create the tools and add them to the toolbar
        // and the button group.
        MainWindow mw = MainWindow.instanceOf();
        String[] allTools = MainWindow.tokenize
            ( mw.getResourceString( "sidebar" ) );


        for ( int i = 0; i < allTools.length; i++ ) {
            JToggleButton button = createToolbarButton( group, allTools[i] );
            add( button );
        }

  }

     protected JToggleButton createToolbarButton( ButtonGroup group,
                                                String key ) {
        MainWindow mw = MainWindow.instanceOf();
        String label = mw.getResourceString( key + Visualizer.labelSuffix );
        if(label==null)
            label = key;
        // create the button
        JToggleButton b = new JToggleButton();
        b.setMargin(new Insets(0,0,0,0));
        b.setRequestFocusEnabled(false);
        b.setIcon(new RotatedTextIcon(b.getFont(),
			label));

        // add ourselves as an action listener
        b.addActionListener( this );

        // set action string for button
        b.setActionCommand( key );

        // attach tooltip to button
        String tip = mw.getResourceString( key + Visualizer.tipSuffix );
        if ( tip != null ) {
            b.setToolTipText( tip );
        }
        group.add(b);
        return b;
    } // createToolbarButton


    public void actionPerformed( ActionEvent e ) {
            Object src = e.getSource();
            if ( src instanceof JToggleButton ) {
                show(e.getActionCommand());
            }
    } // actionPerformed


    protected void show(String key)
    {
        if(win == null)
          return;
        if(!win.isShowing(key))
        {
            win.show(key);
        }
        else
        {
            win.show(null);
            hiddenButton.setSelected(true);
        }
    }


	public static class RotatedTextIcon implements Icon
	{

		public RotatedTextIcon(Font font, String text)
		{
			this.font = font;

			FontRenderContext fontRenderContext
				= new FontRenderContext(null,true,true);
			this.text = text;
			glyphs = font.createGlyphVector(fontRenderContext,text);
			width = (int)glyphs.getLogicalBounds().getWidth() + 4;
			//height = (int)glyphs.getLogicalBounds().getHeight();

			LineMetrics lineMetrics = font.getLineMetrics(text,fontRenderContext);
			ascent = lineMetrics.getAscent();
			height = (int)lineMetrics.getHeight();

			renderHints = new RenderingHints(
				RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
			renderHints.put(RenderingHints.KEY_FRACTIONALMETRICS,
				RenderingHints.VALUE_FRACTIONALMETRICS_ON);
			renderHints.put(RenderingHints.KEY_RENDERING,
				RenderingHints.VALUE_RENDER_QUALITY);
		}

		public int getIconWidth()
		{
			return (int)( height );
		}

		public int getIconHeight()
		{
			return (int)( width );
		}

		public void paintIcon(Component c, Graphics g, int x, int y)
		{
			Graphics2D g2d = (Graphics2D)g;
			g2d.setFont(font);
			AffineTransform oldTransform = g2d.getTransform();
			RenderingHints oldHints = g2d.getRenderingHints();

			g2d.setRenderingHints(renderHints);
			g2d.setColor(c.getForeground());

                        AffineTransform trans = new AffineTransform();
                        trans.concatenate(oldTransform);
                        trans.translate(x,y + 2);
                        trans.rotate(Math.PI / 2,
                                height / 2, width / 2);
                        g2d.setTransform(trans);
                        g2d.drawGlyphVector(glyphs,(height - width) / 2,
                                (width - height) / 2
                                + ascent);

			g2d.setTransform(oldTransform);
			g2d.setRenderingHints(oldHints);
		}

		private int rotate;
		private Font font;
		private String text;
		private GlyphVector glyphs;
		private float width;
		private float height;
		private float ascent;
		private RenderingHints renderHints;
	}



}