package jcreek.cke.document;

import jcreek.cke.graph.Graph;

import java.io.FileInputStream;
import java.io.FileOutputStream;

/**
 * Defines the interface for classes that wish to act as a file format
 * for the JavaCreek program. By implementing this interface, a class
 * can support a new file format, allowing JavaCreek to read and write
 * in that format.
 * <p>
 * Because the read/write methods use generic input/output streams
 * the caller can read or write to anything, not just files on disk.
 *
 * @author
 * @version
 */
public interface CreekFormat {

    /**
     * Read the given input file and fill in the given
     * Graph. If the read failed at any point then false is
     * returned.
     *
     * @param  is  input stream to read from
     * @return  true if successful, false otherwise
     */
    public boolean read( FileInputStream is,Graph grp,StringBuffer  title);

    /**
     * Writes the graph to the given output stream.
     * If the read failed at any point then false is
     * returned.
     *
     * @param  os  output stream to write to
     * @return  true if successful, false if error
     */
    public boolean write( FileOutputStream os,Graph graph,String title);
} // GraphFormat
