package jcreek.cke.graph;

import java.util.*;

public class DefaultGraph implements Graph{

  protected  ArrayList nodeList;
  protected  ArrayList edgeList;
  protected  Node root;

  public DefaultGraph() {
      nodeList  = new ArrayList();
      edgeList = new ArrayList();
  }

  public DefaultGraph(VertexList vList, RelationList rList)
  {
    nodeList = new ArrayList(vList.size());
    edgeList = new ArrayList(rList.size());
    for(int i=0;i<vList.size();i++)
    {
      addNode(vList.get(i));
    }
    for(int i=0;i<rList.size();i++)
    {
      addEdge(rList.get(i));
    }
  }

  public boolean addNode(Node node)
  {
	if(!nodeList.contains(node))
	{
		nodeList.add(node);
                return true;
	}
	return false;
  }

  public boolean addEdge(Edge edge)
  {
	if(!nodeList.contains(edge.getHead()))
		return false;
	if(!nodeList.contains(edge.getTail()))
		return false;
        if(edgeList.contains(edge))
          return false;

        if(edge.getHead().addEdge(edge))
        {
            if(!edge.getTail().addEdge(edge))
            {
                edge.getHead().removeEdge(edge);
                return false;
            }
            edgeList.add(edge);
            return true;
        }
        return false;
  }


  public boolean removeNode(Node node)
  {
	if(nodeList.contains(node)&& node!=null)
	{
		removeEdges(node);
		nodeList.remove(node);
		return true;
	}
	return false;
  }

  public boolean removeEdge(Edge edge)
  {
	if(edge==null)
          return false;
	edge.getHead().removeEdge(edge);
        edge.getTail().removeEdge(edge);
        edgeList.remove(edge);

	return true;

  }

  public boolean setRoot(Node node)
  {
	if(nodeList.contains(node))
	{
		this.root = node;
		return true;
	}
	return false;

  }

  public boolean isRoot(Node node)
  {
	return root == node;
  }

  public Node getRoot()
  {
	return this.root;
  }


  public int getNumChildren(Node node)
  {
      return node.getNumChildren();
  }

  protected boolean removeEdges( Node child ) {
    Iterator  iterator;
    Node parent;

    iterator = nodeList.iterator();
    while( iterator.hasNext( )) {
      parent = (Node) iterator.next();
      parent.removeChild(child);
    }

    return true;
  }

  public int edgeSize()
  {
    return edgeList.size();
  }

  public int nodeSize()
  {
    return nodeList.size();
  }

  public Iterator getNodeIterator()
  {
      return nodeList.iterator();
  }

  public Iterator getEdgeIterator(Node node)
  {
      return node.getEdgeIterator();
  }

  public Iterator getEdgeIterator()
  {
      return edgeList.iterator();
  }

  public Iterator getChildIterator(Node node)
  {
      return node.getChildIterator();
  }

  public void copy(Graph newGraph)
  {
          Iterator nodeIterator,edgeIterator;
          Node node;
          Edge edge;
          nodeIterator = newGraph.getNodeIterator();
          while(nodeIterator.hasNext())
          {
              node = (Node)nodeIterator.next();
              addNode(node);
          }
          nodeIterator = newGraph.getNodeIterator();
          while(nodeIterator.hasNext())
          {
              node = (Node)nodeIterator.next();
              edgeIterator = newGraph.getEdgeIterator(node);
              while(edgeIterator.hasNext())
              {
                  edge = (Edge)edgeIterator.next();
                  addEdge(edge);
              }
          }
          setRoot(newGraph.getRoot());

  }

  public String toString()
  {
    String ret = "DefaultGraph - nodelist:\n";
    Iterator it = getNodeIterator();
    while(it.hasNext())
      ret +="\t"+it.next()+"\n";
    ret +="\nedgelist:\n";
    it = getEdgeIterator();
    while(it.hasNext())
      ret += "\t"+it.next();
    return ret;

  }

}