package jcreek.cke.graph;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Random;
import java.util.Vector;
import java.util.Iterator;

import java.io.*;

import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.graph.*;
//import jcreek.cke.algorithm.*;
//import jcreek.cke.algorithm.tree.*;
import jcreek.cke.conceptview.*;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.Entity;
import jcreek.cke.util.Matrix33;
import jcreek.cke.util.DPoint;
import jcreek.cke.util.DDimension;
import jcreek.representation.RelationType;
import jcreek.representation.Relation;
import jcreek.cke.visualizer.AppSettings;

public class GraphModel implements CreekModel
{
    /**
     * List of change listeners. Each time the model is changed
     * these listeners are notified via the PropertyChangeEvent. The
     * listeners must implement the PropertyChangeListener interface.
     */
    protected transient PropertyChangeSupport changeListeners;
    /**
     * True if we are to not send out change notifications. This
     * is used when we're performing repetitive actions that would
     * normally fire change events.
     */
    protected transient boolean suspendChangeNotices;

    private Graph	graph;

    public double hSpacing = 60, vSpacing = 80;

    public GraphModel()
    {
        graph = new DefaultGraph();
        changeListeners = new PropertyChangeSupport( this );
        suspendChangeNotices = false;
    }

    public synchronized boolean addEntity( VisualEntity node ) {
        boolean success = false;
        if ( node != null )
        {
            if(findVisualEntity(node.getEntity()) == null)
              if(graph.addNode(node))
              {
                  fireChange();
                  success= true;
              }

        }
        return success;
    } // addentity


    /**
     * Add the given list of entities to the graph.
     *
     * @param  nodes list of entities to add
     * @return  true if entities successfully added to graph
     */
    public boolean addEntityList( VertexList nodes )
    {

        boolean success = true;
        if ( ( nodes == null ) ||
             ( nodes.size() == 0 ) ) {
            return true;
        }
        boolean oldState = suspendChanges( true );
        // Loop through the given list and add each vertex.
        int size = nodes.size();
        for ( int i = 0; i < size; i++ ) {
            if ( addEntity( nodes.get(i) ) == false ) {
                success = false;
                break;
            }
        }
        suspendChanges( oldState );
        if ( success ) {
            fireChange();
        }
        return success;

    }

    /**
     * Remove a  node from the model.
     *
     * @param  node  node to remove
     * @return  true if successful, false if error
     */
    public synchronized void removeEntity( VisualEntity node ) {
        // Now remove the entity itself.
        Iterator it = node.getEdgeIterator();
        Vector v = new Vector();
        while(it.hasNext())
          v.add((VisualRelation) it.next());
        for(int i=0;i<v.size();i++)
        {
//          System.out.println("Removing rel");
          removeRelation((VisualRelation) v.get(i));
        }
        graph.removeNode(node);
//        System.out.println(graph);
        fireChange();

    } // removeentity

    public boolean removeEntity(Entity e)
    {
        boolean oldState = suspendChanges( true );
        VisualEntity vEntity  = findVisualEntity(e);
        if(vEntity == null)
          return false;
        removeEntity(vEntity);
        suspendChanges( oldState );
        fireChange();
	return true;
    }


    /**
     * Remove a list of nodes from the model.
     *
     * @param  nodes list of nodes to remove
     * @return  true if successful, false if error
     */
    public boolean removeEntityList( VertexList nodes )
    {
         if ( ( nodes == null ) /*||
             ( nodes.size() == 0 )*/ ) {
            return false;
        }
        boolean oldState = suspendChanges( true );
        unselectAllVertices();
        // Loop through the given list and remove each vertex.
        int size = nodes.size();
        for ( int i = 0; i < size; i++ ) {
            removeEntity( nodes.get( i ) );
        }
        suspendChanges( oldState );
        fireChange();
	return true;
   }


    public boolean removeEntityList( Entity[] nodes )
    {
         if (nodes == null) {
            return false;
        }
        boolean oldState = suspendChanges( true );
        unselectAllVertices();
        // Loop through the given list and remove each vertex.
        for ( int i = 0; i < nodes.length; i++ ) {
            removeEntity( nodes[i] );
        }
        suspendChanges( oldState );
        fireChange();
	return true;
   }



    /**
     * Delete the given list of entities from the graph.
     *
     * @param  nodes list of entities to remove
     * @return  true if successful, false if error
     */
    public boolean deleteEntityList( VertexList nodes )
    {
         if ( ( nodes == null ) ||
             ( nodes.size() == 0 ) ) {
            return false;
        }
        boolean oldState = suspendChanges( true );
        unselectAllVertices();
        // Loop through the given list and remove each vertex.
        int size = nodes.size();
        for ( int i = 0; i < size; i++ ) {
            removeEntity( nodes.get( i ) );
            nodes.get(i).getEntity().remove();
        }
        suspendChanges( oldState );
        fireChange();
	return true;
    }


    /**
     * Return the reference to the entity list contained in this
     * graph.
     *
     * @return  list of vertices in graph
     */
    public VertexList getEntities() {
        VertexList vl = new VertexList();
        VisualEntity ve = null;
        Iterator iterator;
        iterator = graph.getNodeIterator();
        while( iterator.hasNext() )
        {
            vl.add((VisualEntity)iterator.next());
        }
        return vl;
    } // getEntities


    /**
     * Gets the list of the selected entities in the graph.
     *
     * @return list of selected entities; null if none selected
     */
    public VisualEntity[] getSelectedEntities()
    {
        Vector selEntities = new Vector();
          VisualEntity ve = null;
          Iterator iterator;
          iterator = graph.getNodeIterator();
          while( iterator.hasNext() )
          {
              ve = (VisualEntity)iterator.next();
              if(ve.isSelected())
                selEntities.add(ve);
          }
        if ( selEntities.size() == 0 ) {
            VisualEntity[] selEnt = new VisualEntity[0];
            return selEnt;
        }

        VisualEntity[] selEnt = new VisualEntity[selEntities.size()];
        selEntities.copyInto(selEnt);
        return selEnt;
    }//getSelectedEntities

    /**
     * Mark all the entities in the List as selected. This
     * will cause a change event to be fired and sent to any
     * selection change listeners.
     */
    public synchronized void selectVertices(boolean state,VertexList vList) {
        int size = vList.size();
        if ( size == 0 ) {
            return;
        }
        // Go through all of the entities and mark
        // each one as selected.
        for ( int i = 0; i < size; i++ ) {
            vList.get( i ).setSelected( state );
        }
        fireSelection();

    } // selectVertices


    /**
     * Set the given entity to the selected state given. This
     * will cause a change event to be fired and sent to any
     * selection change listeners.
     *
     * @param  v  entity to select or unselect
     * @param  s  true to mark entity selected, false unselected
     */
    public void setSelected( VisualEntity v, boolean s ) {
        v.setSelected( s );
        fireSelection();
    } // setSelected


    /**
     * Mark all the entities in the graph as unselected. This
     * will cause a change event to be fired and sent to any
     * selection change listeners.
     */
    public synchronized void unselectAllVertices() {
          Iterator iterator;
          iterator = graph.getNodeIterator();
          while( iterator.hasNext() )
          {
              VisualEntity vNode = (VisualEntity)iterator.next();
              vNode.setSelected(false);
          }
        fireSelection();

    } // unselectAllVertices

    public VisualEntity findVisualEntity( Entity e)
    {
          VisualEntity ve = null;
          Iterator iterator;
          if(e != null)
          {
              iterator = graph.getNodeIterator();
              while( iterator.hasNext() )
              {
                  VisualEntity vNode = (VisualEntity)iterator.next();
                  if(vNode.getEntity().equals(e))
                  {
                      ve = vNode;
                      break;
                  }
              }
          }
          return ve;
    }

    /**
     * Moves a list of entities to a new location, given by the
     * dx, dy, dz coordinates. Will fire a change notification so
     * listeners will know of the move.
     *
     * @param  vl  list of entities to be moved
     * @param  dx  x position change
     * @param  dy  y position change
     * @param  dz  z position change
     * @return  true if successful, false if error
     */
    public boolean moveEntityList( VertexList el, int dx, int dy) {
        if ( ( el == null ) || ( el.size() == 0 ) ) {
            return false;
        }
        int size = el.size();
        VisualEntity v;
        for ( int i = 0; i < size; i++ ) {
            v = el.get( i );
            v.slide(dx,dy);
        }
        fireChange();
        return true;
    } // moveEntityList






/**
 * Edge Methods
 */







    /**
     * Add a relation to the graph.
     *
     * @param  relation relation to add to matrix
     * @return  true if relation successfully added to graph
     */
    public synchronized boolean addRelation( VisualRelation relation ) {
        boolean success = false;
        if ( relation != null ) {
            VisualEntity vFrom = findVisualEntity(relation.getFrom().getEntity());
            VisualEntity vTo = findVisualEntity(relation.getTo().getEntity());
//            System.out.println("vFrom: "+vFrom+" vTo: "+vTo);
            if((vFrom!=null)&&(vTo!=null))
            {
                relation.from = vFrom;
                relation.to = vTo;
            }
            else
              return success;
            if(!relationIsActive(relation))
            {
                graph.addEdge(relation);
                fireChange();
            }
            success = true;
        }
        return success;
    } // addentity

    /**
     * Add the given list of relations to the graph.
     *
     * @param  relations list of relations to add
     * @return  true if relations successfully added to graph
     */
    public boolean addRelationList( RelationList relations )
    {
        boolean success = true;
        if ( ( relations == null ) ||
             ( relations.size() == 0 ) ) {
            return true;
        }
        boolean oldState = suspendChanges( true );
        // Loop through the given list and add each vertex.
        int size = relations.size();
        for ( int i = 0; i < size; i++ ) {
            if ( addRelation( relations.get(i) ) == false ) {
                success = false;
                break;
            }
        }
        suspendChanges( oldState );
        if ( success ) {
            fireChange();
        }
        return success;
    }//addRelationList

    public RelationList getRelations() {
        VertexList vl = getEntities();
        VisualRelation[] activeRelations;
        RelationList rl = new RelationList();
        for(int i=0;i<vl.size();i++)
        {
            activeRelations = getActiveVisualRelations(vl.get(i));
            if(activeRelations.length>0)
              for(int j=0;j<activeRelations.length;j++)
                  rl.add(activeRelations[j]);
        }
        return rl;
    } // getRelations

    /**
     * Remove an relation from the graph.
     *
     * @param  edgeToRemove  the edge to remove from the graph
     * @return  true if successful, false if error
     */
    public synchronized boolean removeRelation( VisualRelation edgeToRemove ) {
        if ( edgeToRemove == null ) {
            return false;
        }
        graph.removeEdge(edgeToRemove);
        edgeToRemove.remove();
        fireChange();
	return true;
    } // removeEdge

    public boolean removeRelation(Relation rel)
    {
        VisualRelation vrel = findVisualRelation(rel);
        return removeRelation(vrel);
    }

    public boolean removeRelationList( RelationList edges )
    {
         if ( ( edges == null ) /*||
             ( edges.size() == 0 ) */) {
            return false;
        }
        unselectAllEdges();
        boolean oldState = suspendChanges( true );
        // Loop through the given list and remove each vertex.
        int size = edges.size();
        for ( int i = 0; i < size; i++ ) {
            removeRelation( edges.get( i ) );
        }
        suspendChanges( oldState );
        fireChange();
	return true;
   }

    public boolean removeRelationList( Relation[] edges )
    {
         if (edges == null) {
            return false;
        }
        unselectAllEdges();
        boolean oldState = suspendChanges( true );
        // Loop through the given list and remove each vertex.
        for ( int i = 0; i < edges.length; i++ ) {
            removeRelation( edges[i] );
        }
        suspendChanges( oldState );
        fireChange();
	return true;
   }

    public boolean deleteRelationList( RelationList edges )
    {
         if ( ( edges == null ) ||
             ( edges.size() == 0 ) ) {
            return false;
        }
        boolean oldState = suspendChanges( true );
        unselectAllEdges();
        // Loop through the given list and remove each vertex.
        int size = edges.size();
        for ( int i = 0; i < size; i++ ) {
            removeRelation( edges.get( i ) );
            edges.get(i).getRelation().remove();
        }
        suspendChanges( oldState );
        fireChange();
	return true;
    }

    /**
     * Set the given edge to the selected state given. This
     * will cause a change event to be fired and sent to any
     * selection change listeners.
     *
     * @param  e  edge to select or unselect
     * @param  s  true to mark edge selected, false unselected
     */
    public void setSelected( VisualRelation e, boolean s ) {
        e.setSelected( s );
        fireSelection();
    } // setSelected

    /**
     * Mark all of the edges in the graph as selected. This
     * will cause a change event to be fired and sent to any
     * selection change listeners.
     */
    public synchronized void selectRelations(boolean state,RelationList rList) {
        int size = rList.size();
        if ( size == 0 ) {
            return;
        }
        // Loop through all the edges marking them
        // all selected.
        for ( int i = 0; i < size; i++ ) {
            rList.get( i ).setSelected( state );
        }
        fireSelection();
    } // selectAllEdges

    /**
     * Mark all of the edges in the graph as unselected. This
     * will cause a change event to be fired and sent to any
     * selection change listeners.
     */
    public synchronized void unselectAllEdges() {
    /*
        int size = relations.size();
        if ( size == 0 ) {
            return;
        }
        // Loop through all the edges marking them
        // all not selected.
        for ( int i = 0; i < size; i++ ) {
            relations.get( i ).setSelected( false );
        }
        fireSelection();
    */
    } // unselectAllEdges



    public VisualRelation findVisualRelation(Relation rel)
    {
          VisualRelation vrel = null;
          VisualEntity from = findVisualEntity(rel.getFrom());
          if(from!=null)
          {
              Iterator iterator;
              iterator = graph.getEdgeIterator(from);
              while( iterator.hasNext() )
              {
                  VisualRelation tmpRel = (VisualRelation)iterator.next();
                  if((tmpRel.getRelation().equals(rel))
                    ||(tmpRel.getRelation().equals(rel.getInverse())))
                  {
                        vrel = tmpRel;
                        break;
                  }
              }
          }
          return vrel;
    }

    public boolean relationIsActive(VisualRelation vRelation)
    {
        Iterator iterator;
        Node headNode;
        boolean active = false;
        Relation r = vRelation.getRelation();
        headNode = vRelation.getHead();
        iterator = graph.getEdgeIterator(headNode);
        if(iterator!=null)
        {
            while(iterator.hasNext())
            {
                VisualRelation tRelation = (VisualRelation)iterator.next();
                if(tRelation.getRelation().equals(r) ||
                   tRelation.getRelation().equals(r.getInverse()))
                {
                  active = true;
                  break;
                }

            }
        }
        return active;
    }



    /**
     * Return the reference to the entity list contained in this
     * graph. This returns a clone of the actual entity list.
     *
     * @return  list of vertices in graph
     */
    public Relation[] getActiveRelations(VisualEntity ved) {
          Vector tempRelations = new Vector();
          Relation[] activeRelations;
          if(ved!=null)
          {
              Iterator iterator;
              iterator = graph.getEdgeIterator(ved);
              while( iterator.hasNext() )
              {
                  tempRelations.add(((VisualRelation)iterator.next()).getRelation());
              }
          }
          activeRelations = new Relation[tempRelations.size()];
          tempRelations.copyInto(activeRelations);
          return activeRelations;
    } // getActiveRelations


    public VisualRelation[] getActiveVisualRelations(VisualEntity ved) {
          Vector tempRelations = new Vector();
          VisualRelation[] activeRelations;
          if(ved!=null)
          {
              Iterator iterator;
              iterator = graph.getEdgeIterator(ved);
              while( iterator.hasNext() )
              {
                  tempRelations.add((VisualRelation) iterator.next());
              }
          }
          activeRelations = new VisualRelation[tempRelations.size()];
          tempRelations.copyInto(activeRelations);
          return activeRelations;
    } // getActiveRelations

    public Graph getGraph()
    {
        return graph;
    }

    /**
     * Utility method to cause a change event to be fired off to
     * all the listeners. Used when someone has changed the graph
     * data at a low level (i.e. by manipulating a single entity)
     * and they want others to know of the change.
     */
    public void markDirty() {
        fireChange();
    } // markDirty

    /**
     * Add a property change listener to this model. Any time a entity
     * or edge is added, removed or changed in any way via the graph,
     * a "change" event is sent to the change listeners. If an element
     * is selected or deselected, "selection" events are sent.
     *
     * @param  listener  new listener to add notification list
     * @see #removeChangeListener
     */
    public void addChangeListener( PropertyChangeListener listener ) {
        changeListeners.addPropertyChangeListener( listener );
    } // addChangeListener

    /**
     * Let all the change listeners know of a recent change
     * in the model data. This fires off a "change" event.
     */
    public void fireChange() {
        if ( !suspendChangeNotices ) {
            changeListeners.firePropertyChange( "change", 0, 1 );
        }
    } // fireChange

    /**
     * Let all the selection change listeners know of a recent change
     * in the selection of a data object. This fires off a "selection"
     * change event.
     */
    protected void fireSelection() {
        if ( !suspendChangeNotices )
          changeListeners.firePropertyChange( "selection", 0, 1 );
    } // fireSelection



    /**
     * Remove a change listener from the listener list.
     *
     * @param  listener  listener to remove from notification list
     * @see #addChangeListener
     */
    public void removeChangeListener( PropertyChangeListener listener ) {
        changeListeners.removePropertyChangeListener( listener );
    } // removeChangeListener



    /**
     * Tells us to suspend sending out change notifications. Be sure
     * to enable change notifications after you've performed the work
     * that would have otherwise fired many changes. It returns the
     * previous state of the suspend. This can be used like so:
     *
     * <pre>
     * boolean oldState = suspendChanges( true );
     * ...
     * suspendChanges( oldState );
     * </pre>
     *
     * @param  suspend  true to suspend notifications
     * @return  previous suspend state
     */
    protected boolean suspendChanges( boolean suspend ) {
        boolean old = suspendChangeNotices;
        suspendChangeNotices = suspend;
        return old;
    } // suspendChanges


    public void replace( Graph grp ) {
        this.graph.copy(grp);
        initialiseEntities();
        fireChange();
    }

    private void initialiseEntities()
    {
        int j,relationSize = 0;
        VisualRelation ver;
        VisualEntity node;
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        AppSettings appSettings = AppSettings.instanceOf();
        KnowledgeModel km = dg.getKM();
        Iterator nodeIterator = graph.getNodeIterator();
/*
        while(nodeIterator.hasNext())
        {
            node = (VisualEntity)nodeIterator.next();

            node.getEntity() = (Entity)km.getEntity((node.getEntity().getName()));
            Relation[] currentRelations =  node.getEntity().getRelations( );
            Iterator iterator = node.getEdgeIterator();
            while(iterator.hasNext())
            {
                ver = (VisualRelation)iterator.next();
                for(int k=0;k<currentRelations.length;k++)
                    if((ver.getRelation().getName().equalsIgnoreCase(currentRelations[k].getName()))
                    &&(ver.getTo().getEntity().getName().equalsIgnoreCase(((Entity)currentRelations[k].getValue()).getName()))
                    &&(ver.getFrom().getEntity().getName().equalsIgnoreCase(((Entity)currentRelations[k].getFrom()).getName())))
                    {
                        ver.getRelation() = currentRelations[k];
                        break;
                    }
            }
          }
        */
    }

    private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException
    {
      in.defaultReadObject();
      changeListeners = new PropertyChangeSupport( this );
       suspendChangeNotices = false;
    }
}


