package jcreek.cke.graph.layout;

import jcreek.cke.graph.*;
import jcreek.cke.util.DPoint;

import java.util.*;
import java.awt.Rectangle;
import java.lang.Thread;
import javax.swing.SwingUtilities;

public class TreeLayout implements Layout
{
      private char rootOrient = 'u';

      private double levelDistance;
      private double siblingDistance;
      private double subTreeDistance;
      private  Node root;

      private DPoint rootPos;

      private Graph graph;

//      private ConceptView view;

      private HashMap nodeInfo;

      private int depth;
      private Node prevNodeAtLevel[];
      private double levelHeight[];
      private double levelPosition[];

      volatile private Thread layouter;

      public static final char LAYOUT_DIRECTION_RIGHT = 'r';
      public static final char LAYOUT_DIRECTION_LEFT = 'l';
      public static final char LAYOUT_DIRECTION_UP = 'u';
      public static final char LAYOUT_DIRECTION_DOWN = 'd';


      public TreeLayout(char orient)
      {
          this.rootOrient = orient;
      }

      public void setRoot(Node node)
      {
        this.root = node;
      }

      public Node getRoot()
      {
        return root;
      }

      public void layout(Graph graph, Rectangle bounds)
      {
          this.graph = createTree(graph);
//          this.view  = view;
          nodeInfo = new HashMap();

          if(root==null)
              return;

         levelDistance = 80;//view.getVSpacing();
         subTreeDistance = siblingDistance = 80;//view.getHSpacing();

         rootPos = root.getPosition();

         Iterator iterator;
         iterator = graph.getNodeIterator();
         while(iterator.hasNext())
         {
            Node node = (Node)iterator.next();
            nodeInfo.put(node,new NodeInfo());
         }


         depth = 0;
         initializeData(root, 0);

         prevNodeAtLevel = new Node[depth + 1];
         int i;
         for(i = 0; i < depth + 1; i++)
            prevNodeAtLevel[i] = null;

         setInitialPositions(root);
         levelHeight = new double[depth + 1];

         for(i = 0; i < depth + 1; i++)
            levelHeight[i] = 0.0;
         setLevelHeight(root);

         levelPosition = new double[depth + 1];
         levelPosition[0] = 0;

         for(i = 1; i < depth + 1; i++)
            levelPosition[i] = levelPosition[i - 1] +
                                   (levelHeight[i - 1] + levelHeight[i]) / 2.0;
         setFinalPositions(root, 0.0);


      	// Shift root to original position.
          Node tmpnode;
         double xoffs = rootPos.x - root.getPosition().x;
         double yoffs = rootPos.y - root.getPosition().y;
         iterator = graph.getNodeIterator();
          while(iterator.hasNext())
          {
            tmpnode = (Node)iterator.next();
            if(((NodeInfo)(nodeInfo.get(tmpnode))).level != -1)
            {
               DPoint pos = tmpnode.getPosition();
               tmpnode.setPosition(pos.x + xoffs, pos.y + yoffs);
               tmpnode.setPosition((int)(pos.x + xoffs),(int)(pos.y + yoffs));
//               view.repaint();
            }
          }


      }

      protected Graph createTree(Graph graph)
      {
          Iterator nodeIterator,edgeIterator;
          Node node;
          Edge edge;
          TreeGraph tGraph = new TreeGraph();
          nodeIterator = graph.getNodeIterator();
          while(nodeIterator.hasNext())
          {
              node = (Node)nodeIterator.next();
              tGraph.addNode(node);
//              System.out.println("Adding entity "+node.getEntity());
          }
          nodeIterator = graph.getNodeIterator();
          while(nodeIterator.hasNext())
          {
              node = (Node)nodeIterator.next();
              edgeIterator = graph.getEdgeIterator(node);
              while(edgeIterator.hasNext())
              {
                  edge = (Edge)edgeIterator.next();
                  tGraph.addEdge(edge);
//                  System.out.println("Adding edge "+edge.getRelation());
              }
          }
          tGraph.setRoot(graph.getRoot());
//          System.out.println("TreeGraph: "+tGraph.toString());
          return tGraph;
      }


      protected void initializeData(Node node,int level)
      {
         String pre = "";
         for(int i=0;i<level;i++)
           pre += "\t";

//         System.out.println(pre+node.getEntity()+" ("+node.getNumChildren()+" children)");

         ((NodeInfo)(nodeInfo.get(node))).level = level;

         if(level > depth)
            depth = level;

//         Node children[] = new Node[graph.getNumChildren(node)];
         Node child;
         int num_children = 0;

        Iterator iterator;
        iterator = graph.getChildIterator(node);
        Vector childVect = new Vector();
        while(iterator.hasNext())
        {
            child = (Node)iterator.next();
            if(!childVect.contains(child) && ((NodeInfo)(nodeInfo.get(child))).level == -1)
              childVect.add(child);
  //             children[num_children++] = child;
        }

        Node[] children = (Node[]) childVect.toArray(new Node[0]);
        num_children = childVect.size();
         if(num_children == 0)
         {
            (((NodeInfo)nodeInfo.get(node))).isLeaf = true;
            return;
         }

         int i, j;

      	// Sort the children by "left" coordinate;
         Node tmpnode;
         for(i = 0; i < num_children - 1; i++)
            for(j = i + 1; j < num_children; j++)
               if(((rootOrient == 'd' || rootOrient == 'u') &&
                     children[j].getPosition().x < children[i].getPosition().x) ||
                     ((rootOrient == 'l' || rootOrient == 'r') &&
                      children[j].getPosition().y < children[i].getPosition().y))
               {
                  tmpnode = children[i];
                  children[i] = children[j];
                  children[j] = tmpnode;
               }

      	// Initialize links.
        NodeInfo nodeData = ((NodeInfo)nodeInfo.get(node));
         nodeData.leftChild = children[0];
         nodeData.rightChild = children[num_children - 1];
         for(i = 0; i < num_children; i++)
            (((NodeInfo)nodeInfo.get(children[i]))).parent = node;
         for(i = 0; i < num_children - 1; i++)
           if(i < num_children-1)
            (((NodeInfo)nodeInfo.get(children[i]))).rightSibling = children[i + 1];
           else
            (((NodeInfo)nodeInfo.get(children[i]))).rightSibling = null;
         for(i = 1; i < num_children; i++)
           if(i > 0)
            (((NodeInfo)nodeInfo.get(children[i]))).leftSibling = children[i - 1];
           else
            (((NodeInfo)nodeInfo.get(children[i]))).leftSibling = null;
         for(i = 0; i < num_children; i++)
            (((NodeInfo)nodeInfo.get(children[i]))).level = level + 1;


      	// Initialize data for the children.
         for(i = 0; i < num_children; i++)
         {
//            System.out.println(pre+" :"+nodeInfo.get(children[i]));
            initializeData(children[i], level + 1);
         }

      }

      private void setInitialPositions(Node node)
      {
         NodeInfo data = (NodeInfo)nodeInfo.get(node);

      	// Initialize neighbor links.
         data.leftNeighbor = prevNodeAtLevel[data.level];
         if(data.leftNeighbor != null)
            ((NodeInfo)nodeInfo.get(data.leftNeighbor)).rightNeighbor = node;
         prevNodeAtLevel[data.level] = node;

         Node tmpnode;
         if(data.leftSibling != null)
         {
            if(rootOrient == 'd' || rootOrient == 'u')
               data.prelim = ((NodeInfo)nodeInfo.get(data.leftSibling)).prelim +
                             siblingDistance +
                                (node.getBoundingBox().width + data.leftSibling.getBoundingBox().width) / 2.0;
            else
               data.prelim = ((NodeInfo)nodeInfo.get(data.leftSibling)).prelim +
                             siblingDistance +
                                (node.getBoundingBox().height + data.leftSibling.getBoundingBox().height) / 2.0;
         }

         if(!data.isLeaf)
         	// Bottom-up, left-to-right recurse.
         {
            for(tmpnode = data.leftChild; tmpnode != null; // && tmpnode != ((NodeInfo)nodeInfo.get(tmpnode)).rightSibling;
                tmpnode = ((NodeInfo)nodeInfo.get(tmpnode)).rightSibling)
            {
//               System.out.println("tempnode: "+tmpnode.getEntity());
               setInitialPositions(tmpnode);
            }

            double mid_point = (((NodeInfo)nodeInfo.get(data.leftChild)).prelim +
                                   ((NodeInfo)nodeInfo.get(data.rightChild)).prelim) / 2.0;

            if(data.leftSibling != null)
            {
               data.modifier = data.prelim - mid_point;
               evenOut(node);
            }
            else
               data.prelim = mid_point;
         }

      }


      private void evenOut (Node node)
      {
         NodeInfo data = (NodeInfo)nodeInfo.get(node);


         Node left_most = data.leftChild;
         Node neighbor = ((NodeInfo)nodeInfo.get(left_most)).leftNeighbor;
         int compare_depth = 1;
         while(left_most != null && neighbor != null)
         {
            double left_mod_sum = 0.0;
            double right_mod_sum = 0.0;

            Node ancestor_leftmost = left_most;
            Node ancestor_neighbor = neighbor;

            int i;
            for(i = 0; i < compare_depth; i++)
            {
               ancestor_leftmost = ((NodeInfo)nodeInfo.get(ancestor_leftmost)).parent;
               ancestor_neighbor = ((NodeInfo)nodeInfo.get(ancestor_neighbor)).parent;

               right_mod_sum += ((NodeInfo)nodeInfo.get(ancestor_leftmost)).modifier;
               left_mod_sum += ((NodeInfo)nodeInfo.get(ancestor_leftmost)).modifier;
            }

            double move_distance;
            if(rootOrient == 'd' || rootOrient == 'u')
               move_distance = ((NodeInfo)nodeInfo.get(neighbor)).prelim + left_mod_sum + subTreeDistance +
                                  (left_most.getBoundingBox().width + neighbor.getBoundingBox().width) / 2.0 -
                                  ((NodeInfo)nodeInfo.get(left_most)).prelim - right_mod_sum;
            else
               move_distance = ((NodeInfo)nodeInfo.get(neighbor)).prelim + left_mod_sum + subTreeDistance +
                                  (left_most.getBoundingBox().height + neighbor.getBoundingBox().height) / 2.0 -
                                  ((NodeInfo)nodeInfo.get(left_most)).prelim - right_mod_sum;

            if(move_distance > 0.0)
            {
               Node tmpnode;
               int left_sibling;
               for(tmpnode = node, left_sibling = 0; tmpnode != null && tmpnode != ancestor_neighbor;
                   tmpnode = ((NodeInfo)nodeInfo.get(tmpnode)).leftSibling)
                  left_sibling++;

               if(tmpnode != null)
               {
                  double slide_value = move_distance / (double)left_sibling;
                  for(tmpnode = node; tmpnode != ancestor_neighbor;
                      tmpnode = ((NodeInfo)nodeInfo.get(tmpnode)).leftSibling)
                  {
                     ((NodeInfo)nodeInfo.get(tmpnode)).prelim += move_distance;
                     ((NodeInfo)nodeInfo.get(tmpnode)).modifier += move_distance;
                     move_distance -= slide_value;
                  }
               }
               else
                  return;
            }

            compare_depth++;
            if(((NodeInfo)nodeInfo.get(left_most)).leftChild == null)
               left_most = leftMost(node, 0, compare_depth);
            else
               left_most = ((NodeInfo)nodeInfo.get(left_most)).leftChild;
            if(left_most != null)
               neighbor = ((NodeInfo)nodeInfo.get(left_most)).leftNeighbor;
         }
      }




      private Node leftMost(Node node, int level, int depth)
      {
         if(level >= depth)
            return node;
         if(((NodeInfo)nodeInfo.get(node)).leftChild == null)
            return null;

         Node left_most, right_most;
         right_most = ((NodeInfo)nodeInfo.get(node)).leftChild;
         left_most = leftMost(right_most, level + 1, depth);
         while(left_most == null && ((NodeInfo)nodeInfo.get(right_most)).rightSibling != null)
         {
            right_most = ((NodeInfo)nodeInfo.get(right_most)).rightSibling;
            left_most = leftMost(right_most, level + 1, depth);
         }
         return left_most;
      }

      private void setLevelHeight(Node node)
      {
         NodeInfo data = (NodeInfo)nodeInfo.get(node);

         if(rootOrient == 'd' || rootOrient == 'u')
         {
            if(node.getBoundingBox().height > levelHeight[data.level])
               levelHeight[data.level] = node.getBoundingBox().height;
         }
         else
         {
            if(node.getBoundingBox().width > levelHeight[data.level])
               levelHeight[data.level] = node.getBoundingBox().width;
         }

         if(data.leftChild != null)
            setLevelHeight(data.leftChild);
         if(data.rightSibling != null)
            setLevelHeight(data.rightSibling);
      }

      private void setFinalPositions(Node node, double mod_sum)
      {
         double x = 0.0, y = 0.0;
         Node tmpnode;
         NodeInfo data = (NodeInfo)nodeInfo.get(node);

         x = data.prelim + mod_sum;
         y = -data.level * levelDistance - levelPosition[data.level];

         if(data.leftChild != null)
            setFinalPositions(data.leftChild, mod_sum + data.modifier);

         if(data.rightSibling != null)
            setFinalPositions(data.rightSibling, mod_sum);

         if(rootOrient == 'd')
            node.setPosition(x, y);
         else if(rootOrient == 'u')
            node.setPosition(x, -y);
         else if(rootOrient == 'l')
            node.setPosition(y, x);
         else if(rootOrient == 'r')
            node.setPosition(-y, x);
      }



}
class NodeInfo
{
      public int level;
      public Node parent;
      public Node leftChild;
      public Node rightChild;
      public Node leftSibling;
      public Node rightSibling;
      public Node leftNeighbor;
      public Node rightNeighbor;
      public boolean isLeaf;
     //mrg 9/11
      public boolean isInitialized;
      //
      public double modifier;
      public double prelim;
//      public VisualEntity group;

      public NodeInfo()
      {
         level = -1;
         parent = leftChild = rightChild = leftSibling = rightSibling =
            leftNeighbor = rightNeighbor = null;
         isLeaf = false;
         //mrg 9/11
         isInitialized=false;
         //
         modifier = prelim = 0;
//         group = null;
      }

      public String toString()
      {
        return "level "+level+"  left "+(leftSibling==null?"null":""+leftSibling.getEntity())+"  right "+(rightSibling==null?"null":""+rightSibling.getEntity());
      }

}