package jcreek.cke.importexport;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import jcreek.cke.importexport.jcxml.JCXMLexport;
import jcreek.cke.importexport.jcxml.JCXMLimport;
import jcreek.cke.importexport.jcxml.JCXMLpreferences;
import jcreek.cke.importexport.owl.OWLexport;
import jcreek.cke.importexport.owl.OWLimport;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.LocalKnowledgeModel;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

/**
 * A general import or export wizards window. The window contains a list
 * of possible wizards, and has also the ability to set the preferences
 * for each wizard as well. When doing the import or export (the next step
 * in the process) the jcreek.cke.plugins.importexport.ProcessStatusWindow
 * should be used as a feedback window for the user so that the user may supervise
 * the import or export process.
 *
 * @author Stein L. Tomassen, NTNU 2002
 *
 * @see ProcessStatusWindow
 * @see KnowledgeModelImportUtillities
 * @see KnowledgeModelExportUtillities
 */
public class ImportExportsWizards extends JDialog implements ActionListener {
        /** Specifies that this dialog should be for export of data. */
        public static int EXPORT = 0;
        /** Specifies that this dialog should be for import of data. */
        public static int IMPORT = 1;
        //
        private final String JCXML = "JavaCreek XML";
        private final String OWL = "OWL/RDF";
        private final String NOEMIE = "NOEMIE Model";
        private final String importItems[] = { JCXML, OWL };
        private final String exportItems[] = { JCXML, OWL };
        private int dialogType;
        private KnowledgeModel km;
        private boolean aborted = false;

        // visual components
        JPanel mainPanel = new JPanel();
        JPanel lowerButtonPanel = new JPanel();
        JScrollPane scrollPane = new JScrollPane();
        JList choiceList = new JList();
        JLabel descriptionLabel = new JLabel();
  JPanel furtherPanel = new JPanel();
  JPanel prefsPanel = new JPanel();
  JButton prefsButton = new JButton();
  JButton actionButton = new JButton();
  JButton cancelButton = new JButton();

        /**
         * Initializes the GUI.
         *
         * @param parent the parent frame calling this window.
         * @param km the Knowledge Model to either import a knowledge model into
         * or export a knowledge model from.
         * @param dialogType the type of dialog this should be. Might Either be
         * ImportExportsWizards.EXPORT or ImportExportsWizards.IMPORT.
         */
        public ImportExportsWizards(Frame parent, KnowledgeModel km, int dialogType) {
                super(parent, true);
                this.km = km;
                this.dialogType = dialogType;
                try {
                        jbInit();
                }
                catch (Exception e) {
                        e.printStackTrace();
                }
        } // ImportExportsWizards

        /**
         * Initializes the GUI
         *
         * @throws Exception throws and exception if an error in initialization
   * of the GUI happened
         */
        private void jbInit() throws Exception {
                setSize(new Dimension(500, 400));
                mainPanel.setLayout(new BorderLayout());
                getContentPane().setLayout(new BorderLayout());
                mainPanel.setBorder(BorderFactory.createEmptyBorder(12, 12, 6, 12));
                choiceList.setBorder(BorderFactory.createEtchedBorder());
                choiceList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
                descriptionLabel.setText("?");
    prefsButton.setText("Preferences");
    prefsButton.addActionListener(this);
    actionButton.setPreferredSize(new Dimension(73, 27));
    actionButton.setToolTipText("");
    actionButton.setText("Next >");
    actionButton.addActionListener(this);
    cancelButton.setText("Cancel");
    cancelButton.addActionListener(this);
    lowerButtonPanel.setLayout(new BorderLayout());
    getContentPane().add(mainPanel, BorderLayout.CENTER);
                setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
                addWindowListener(new WindowAdapter() {
                        public void windowClosing(WindowEvent we) {
                                aborted = true;
                                dispose();
                        }
                });
                mainPanel.add(lowerButtonPanel, BorderLayout.SOUTH);
                lowerButtonPanel.add(prefsPanel, BorderLayout.WEST);
    prefsPanel.add(prefsButton, null);
    lowerButtonPanel.add(furtherPanel,  BorderLayout.EAST);
    furtherPanel.add(actionButton, null);
    furtherPanel.add(cancelButton, null);
                mainPanel.add(scrollPane, BorderLayout.CENTER);
                mainPanel.add(descriptionLabel, BorderLayout.NORTH);
                scrollPane.getViewport().add(choiceList, null);
                if (dialogType == EXPORT) {
                        setTitle("Export");
                        choiceList.setListData(exportItems);
                        descriptionLabel.setText("Select an export destination:");
                        actionButton.setActionCommand("Export");
                }
                else {
                        setTitle("Import");
                        choiceList.setListData(importItems);
                        descriptionLabel.setText("Select an import source:");
                        actionButton.setActionCommand("Import");
                }
                choiceList.setSelectedIndex(0);

                //Center the window
                Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
                Dimension frameSize = getSize();
                if (frameSize.height > screenSize.height) {
                        frameSize.height = screenSize.height;
                }
                if (frameSize.width > screenSize.width) {
                        frameSize.width = screenSize.width;
                }
                setLocation(
                        (screenSize.width - frameSize.width) / 2,
                        (screenSize.height - frameSize.height) / 2);
        } // jbInit

        /**
         * Checks whether the process was aborted or not. The process might get
         * aborted either by the user or if an fatal error did occur and the
         * 'Abort of Fatal Error' in the preferences is set.
         *
         * @return true if the process was aborted, does otherwise return false.
         */
        public boolean isAborted() {
                return aborted;
        } // isAborted

        /**
         * Does the handling of the actions performed by the user
         * on the GUI.
         *
         * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
         */
        public void actionPerformed(ActionEvent e) {
                String command = e.getActionCommand();
                if (command.equalsIgnoreCase("Import")) {
                        if (((String) choiceList.getSelectedValue()).equalsIgnoreCase(JCXML)) {
                            JCXMLimport jcxmlImp = new JCXMLimport((Frame) getParent(), km);
                            hide();
                            jcxmlImp.startImport();
                            if (jcxmlImp.isAborted()) {
                                km = new LocalKnowledgeModel();
                                aborted = true;
                            }
                            dispose();
                        }
                        else if (((String) choiceList.getSelectedValue()).equalsIgnoreCase(OWL)) {
                            OWLimport owlImp = new OWLimport((Frame) getParent(), km);
                            hide();
                            owlImp.startImport();
                            if (owlImp.isAborted()) {
                                km = new LocalKnowledgeModel();
                                aborted = true;
                            }
                            dispose();
                        }
                        else if (((String) choiceList.getSelectedValue()).equalsIgnoreCase(NOEMIE)) {
                                showNotImplMsg();
                        }
                }
                else if (command.equalsIgnoreCase("Export")) {
                    if (((String) choiceList.getSelectedValue()).equalsIgnoreCase(JCXML)) {
                        JCXMLexport jcxmlExp = new JCXMLexport((Frame) getParent(), km);
                        hide();
                        jcxmlExp.startExport();
                        if (jcxmlExp.isAborted()) {
                            km = new LocalKnowledgeModel();
                            aborted = true;
                        }
                        this.dispose();
                    }
                    else if (((String) choiceList.getSelectedValue()).equalsIgnoreCase(OWL)) {
                        OWLexport owlExp = new OWLexport((Frame) getParent(), km);
                        hide();
                        owlExp.startExport();
                        if (owlExp.isAborted()) {
                            km = new LocalKnowledgeModel();
                            aborted = true;
                        }
                        this.dispose();
                    }
                }
                else if (command.equalsIgnoreCase("Cancel")) {
                        aborted = true;
                        dispose();
                }
                else if (command.equalsIgnoreCase("Preferences")) {
                        if (((String) choiceList.getSelectedValue()).equalsIgnoreCase(JCXML)) {
                                JCXMLpreferences jcprefs = new JCXMLpreferences(this);
                                jcprefs.show();
                        }
                        else if (((String) choiceList.getSelectedValue()).equalsIgnoreCase(NOEMIE)) {
                                showNotImplMsg();
                        }
                }
        } // actionPerformed

        /**
         * Shows a 'Not Implemented' information message for the features that
   * are not implemented (yet..).
         */
        private void showNotImplMsg() {
                JOptionPane.showMessageDialog(
                        this,
                        "This feature is not implemented...",
                        "Alert",
                        JOptionPane.ERROR_MESSAGE);
        } // showNotImplMsg
}
