package jcreek.cke.importexport;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.WindowConstants;

/**
 * The intention of this class is to have a common graphical
 * interface for all the import and export situations. This
 * interface should be used to keep the user up to date of
 * what is happening and what is not during such a process.
 * This will enable the user to feel that more controll of
 * the situation, since the user sees what is going on.
 *
 * @author Stein L. Tomassen, NTNU 2002
 */
public class ProcessStatusWindow
	extends JDialog
	implements ActionListener, ProcessStatusWindowInterface {
	JPanel buttonPanel = new JPanel();
	JScrollPane scrollPane = new JScrollPane();
	JTextArea logTextArea = new JTextArea();
	JButton actionButton = new JButton();
	ProcessControlInterface processControl;
	JPanel mainPanel = new JPanel();

	/**
	 * Initializes the window with its graphical components.
	 *
	 * @param parent the parent frame window, needed to have modal window
	 * @param title the title of the window
	 * @param processControl the object containing actions to controll the process
	 */
	public ProcessStatusWindow(
		Frame parent,
		String title,
		ProcessControlInterface processControl) {
		super(parent, title, true);
		this.processControl = processControl;
		try {
			jbInit();
		} catch (Exception e) {
			e.printStackTrace();
		}
	} // JCXMLstatusWindow

	/**
	 * Initializes the window with its graphical components.
	 *
	 * @param parent the parent dialog window, needed to have modal window
	 * @param title the title of the window
	 * @param processControl the object containing actions to controll the process
	 */
	public ProcessStatusWindow(
		Dialog parent,
		String title,
		ProcessControlInterface processControl) {
		super(parent, title, true);
		this.processControl = processControl;
		try {
			jbInit();
		} catch (Exception e) {
			e.printStackTrace();
		}
	} // JCXMLstatusWindow

	/**
	 * Initialises the graphical components of the window.
	 *
	 * @throws Exception thros an exception in case an error occurs
	 */
	private void jbInit() throws Exception {
		this.setSize(new Dimension(600, 400));
		this.getContentPane().setLayout(new BorderLayout());
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBorder(BorderFactory.createEmptyBorder(12, 12, 6, 12));
		buttonPanel.setBorder(BorderFactory.createEmptyBorder(6, 0, 0, 0));
		this.getContentPane().add(mainPanel, BorderLayout.CENTER);
		actionButton.setPreferredSize(new Dimension(73, 27));
		actionButton.setText("?");
		actionButton.addActionListener(this);
		logTextArea.setEditable(false);
		logTextArea.setAutoscrolls(true);
		this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				if (actionButton.getActionCommand().equalsIgnoreCase("Close"))
					dispose();
				else
					processControl.abortProcess();
			}
		});
		mainPanel.add(scrollPane, BorderLayout.CENTER);
		mainPanel.add(buttonPanel, BorderLayout.SOUTH);
		scrollPane.getViewport().add(logTextArea, null);
		buttonPanel.add(actionButton, null);

		//Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = this.getSize();
		if (frameSize.height > screenSize.height) {
			frameSize.height = screenSize.height;
		}
		if (frameSize.width > screenSize.width) {
			frameSize.width = screenSize.width;
		}
		this.setLocation(
			(screenSize.width - frameSize.width) / 2,
			(screenSize.height - frameSize.height) / 2);
		// default is Cancel button
		setCancelButton();
	} // jbInit

	/**
	 * @see jcreek.cke.plugins.importexport.ProcessStatusWindowInterface#setCloseButton()
	 */
	public void setCloseButton() {
		actionButton.setText("Close");
		actionButton.setActionCommand("Close");
		logTextArea.setCursor(null);
	} // setCloseButton

	/**
	 * @see jcreek.cke.plugins.importexport.ProcessStatusWindowInterface#setCancelButton()
	 */
	public void setCancelButton() {
		actionButton.setText("Cancel");
		actionButton.setActionCommand("Cancel");
		logTextArea.setCursor(new Cursor(Cursor.WAIT_CURSOR));
	} // setCancelButton

	/**
	 * Does the handling of actions, that is when the Close and Cancel
	 * buttons are pressed.
	 *
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equalsIgnoreCase("Close")) {
			dispose();
		} else if (command.equalsIgnoreCase("Cancel")) {
			processControl.abortProcess();
                        dispose();
		}
	} // actionPerformed

	/**
	 * @see jcreek.cke.plugins.importexport.ProcessStatusWindowInterface#appendText(String)
	 */
	public void appendText(String text) {
		logTextArea.append(text + '\n');
	} // appendText
}