package jcreek.cke.importexport.jcxml;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import jcreek.cke.browser.SimpleFileFilter;
import jcreek.cke.visualizer.AppSettings;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * A dialog box presenting the preferences for the JCXML import/export
 * process.
 *
 * @author Stein L. Tomassen, NTNU 2002
 */
public class JCXMLpreferences extends JDialog implements ActionListener {
  /** The Schema key */
	public static final String SCHEMA_KEY = "jcxml_schema";
  /** The abort on fata error key */
	public static final String ABORT_KEY = "jcxml_abortOnFatalError";
  /** The JCXML directory key */
	public static final String LAST_DIRECTORY = "jcxml_directory";
  /** The JCXML include default values on export key */
	public static final String INCLUDE_DEFAULT = "jcxml_include_default";
	private AppSettings props;
	JLabel label = new JLabel();
	JTextField schemaFileTextField = new JTextField();
	JButton loadButton = new JButton();
	JButton okButton = new JButton();
	JButton cancelButton = new JButton();
	JPanel buttonPanel = new JPanel();
	JPanel mainPanel = new JPanel();
	JCheckBox abortCheckBox = new JCheckBox();
  JCheckBox includeDefaultCheckBox = new JCheckBox();

	/**
   * Initialises the GUI of the preferences dialog box.
   *
	 * @see java.awt.Dialog#Dialog(Dialog)
	 */
	public JCXMLpreferences(Dialog parent) {
		super(parent, "JavaCreek XML Preferences", true);
		try {
			jbInit();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	} // JCXMLpreferences

	/**
   * Initialises the GUI of the preferences dialog box.
   *
	 * @see java.awt.Window#Window(Frame)
	 */
	public JCXMLpreferences(Frame parent) {
		super(parent, "JavaCreek XML Preferences", true);
		try {
			jbInit();
		}
		catch (Exception e) {
			e.printStackTrace();
		}
	} // JCXMLpreferences

	/**
	 * Initialises the graphical components of the preferences dialog box.
   *
	 * @throws Exception thros an exception in case an error occurs
	 */
	private void jbInit() throws Exception {
		this.setSize(new Dimension(469, 192));
		props = AppSettings.instanceOf();
		props.load("Creek.ini");
		label.setText("JavaCreek Schema location:");
		label.setBounds(new Rectangle(10, 13, 173, 17));
		this.getContentPane().setLayout(new BorderLayout());
		schemaFileTextField.setToolTipText(
			"The full path to the JavaCreek XML (jcxml) Schema.");
		schemaFileTextField.setBounds(new Rectangle(21, 35, 392, 21));
		schemaFileTextField.setText(props.getString(SCHEMA_KEY));
		loadButton.setBounds(new Rectangle(422, 35, 28, 21));
		loadButton.setText("...");
		loadButton.setActionCommand("load");
		loadButton.addActionListener(this);
		okButton.setPreferredSize(new Dimension(73, 27));
		okButton.setText("OK");
		okButton.addActionListener(this);
		cancelButton.setText("Cancel");
		cancelButton.addActionListener(this);
		this.setResizable(false);
		mainPanel.setLayout(null);
		buttonPanel.setBorder(BorderFactory.createEtchedBorder());
		abortCheckBox.setText("Abort on Fatal Error");
		abortCheckBox.setBounds(new Rectangle(10, 66, 140, 25));
		abortCheckBox.setSelected(props.getBoolean(ABORT_KEY));
		includeDefaultCheckBox.setSelected(props.getBoolean(INCLUDE_DEFAULT));
    includeDefaultCheckBox.setBounds(new Rectangle(10, 95, 211, 25));
    includeDefaultCheckBox.setText("Include default values on export");
    buttonPanel.add(okButton, null);
		buttonPanel.add(cancelButton, null);
		this.getContentPane().add(mainPanel, BorderLayout.CENTER);
		mainPanel.add(schemaFileTextField, null);
		mainPanel.add(label, null);
		mainPanel.add(loadButton, null);
		mainPanel.add(abortCheckBox, null);
    mainPanel.add(includeDefaultCheckBox, null);
		this.getContentPane().add(buttonPanel, BorderLayout.SOUTH);

		//Center the window
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = this.getSize();
		if (frameSize.height > screenSize.height) {
			frameSize.height = screenSize.height;
		}
		if (frameSize.width > screenSize.width) {
			frameSize.width = screenSize.width;
		}
		this.setLocation(
			(screenSize.width - frameSize.width) / 2,
			(screenSize.height - frameSize.height) / 2);
	} // jbInit

	/**
   * Does the handling of actions, that is when the OK, Cancel or Load
   * buttons are pressed.
   *
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		if (command.equalsIgnoreCase("OK")) {
			props.setString(SCHEMA_KEY, schemaFileTextField.getText());
			props.setBoolean(ABORT_KEY, abortCheckBox.isSelected());
			props.setBoolean(INCLUDE_DEFAULT, includeDefaultCheckBox.isSelected());
			props.commit();
			dispose();
		}
		else if (command.equalsIgnoreCase("Cancel")) {
			dispose();
		}
		else if (command.equalsIgnoreCase("load")) {
			JFileChooser fc = new JFileChooser(schemaFileTextField.getText());
			fc.setMultiSelectionEnabled(false);
			fc.addChoosableFileFilter(
				new SimpleFileFilter(new String[] { "jcreekmodel.xsd" }, "JCXML Schema Files (jcreekmodel.xsd)"));
			if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
				schemaFileTextField.setText(fc.getSelectedFile().getAbsolutePath());
			}
		}
	} // actionPerformed
}