package jcreek.cke.importexport.owl;

import java.awt.Frame;

import jcreek.cke.browser.SimpleFileFilter;
import jcreek.cke.util.FileUtilities;
import jcreek.cke.importexport.ProcessControlInterface;
import jcreek.cke.importexport.ProcessStatusWindow;
import jcreek.representation.KnowledgeModel;
import jcreek.cke.visualizer.AppSettings;
import jcreek.representation.sw.OWLimportParser;
import javax.swing.JOptionPane;
import java.io.StringReader;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.UnknownHostException;

/**
 * This class is the one having the overall control of the import process of
 * OWL files to the Knowledge Model of Creek.
 *
 * Copied from JCXMLimport, Stein L. Tomassen, NTNU 2002
 *
 * @author Mikael K. Fidjeland, NTNU 2006
 */
public class OWLimport extends Thread implements ProcessControlInterface {
    private ImportThread importThread = null;
    private Frame parent = null;
    private KnowledgeModel km = null;
    private boolean importDone = false;
    private boolean aborted = false;
    private String importURI = null;
    private ProcessStatusWindow statusWindow = null;

    /**
     * Constructor for OWLimport, which sets some basic settings.
     *
     * @param parent the parent frame window.
     * @param km the JavaCreek knowledge model to import the model to.
     */
    public OWLimport(Frame parent, KnowledgeModel km) {
        this.parent = parent;
        this.km = km;
        this.statusWindow =
            new ProcessStatusWindow(parent, "OWL Import Status", this);
    } // OWLimport

    /**
     * Checks if the process was aborted;
     *
     * @return true if the porcess was aborted, otherwise false.
     */
    public boolean isAborted() {
        return aborted;
    }

    /**
     * Starts the process by first showing the URI input dialog
     * then starts the import process, by first starting the thread
     * and gives feedback of the process to the OWL status dialog.
     */
    public void startImport() {
        JOptionPane inputBox = new JOptionPane();
        importURI = inputBox.showInputDialog(parent, "Import URI",
                                             "http://creek.idi.ntnu.no/owl/example/car-model.owl");

        if (importURI != null && importURI != "") {
            // now start the thread
            importThread = new ImportThread();
            importThread.start();
            // wait until done or aborted, update the status dialog while waiting
            // open the status dialog in the process
            statusWindow.show();
        }
    }

    /**
     * Aborts the import process. This is the implementation of the
     * method from 'ProcessControlInterface'. This method could be used
     * by other classes to abort a process.
     */
    public void abortProcess() {
        // check if not already been aborted
        if (importThread != null) {
            // interrupt the thread
            importThread = null;
            aborted = true;
        }

    } // abortProcess

    //****************************************
    // Inner classes
    //****************************************
    protected class ImportThread extends Thread {
        /**
         * Does a check if aborted before every new task
         *
         * @see java.lang.Runnable#run()
         */
        public void run() {
            statusWindow.setCancelButton();
            if (importURI != null && importURI != "") {
                // satt slik foreløpig
                if (aborted) {
                    abortedMessage();
                    return;
                }

                statusWindow.appendText("Starts import of the knowledge model...");
                OWLimportParser trav =  new OWLimportParser(importURI, km);
                try
                    {
                        trav.importOntology();
                        statusWindow.appendText("Import of OWL successful...");
                    }
                catch (Exception e)
                    {
                        e.printStackTrace();
                        statusWindow.appendText("Error: " + e.getMessage());
                    }
            }
            // when finished
            statusWindow.setCloseButton();
            importDone = true;
        } // run

        /**
         * The default message given if the process was aborted.
         */
        public void abortedMessage() {
            statusWindow.appendText("Import of OWL was aborted...");
            statusWindow.setCloseButton();
        } // abortedMessage
    } // ImportThread
}
