package jcreek.cke.importexport.xml;

import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.StringWriter;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;

/**
 * The intention of this class is to gather methods for
 * serialization of DOM documents to various output formats.
 *
 * @author Stein L. Tomassen, NTNU 2002
 */
public class DOMdocumentSerializer extends XMLSerializer {
  //********************************************
  // Private variables
  //********************************************
  private String output = "XML";
  private String encoding = "ISO-8859-1";
  private boolean indenting = true;

  //********************************************
  // Public methods
  //********************************************

  /**
   * Serializes the specified DOM document to a specified file.
   *
   * @param doc the DOM document to serialize.
   * @param filename the filename to serialize to.
   */
  public void serializeToFile(Document doc, String filename) {
    try {
      FileOutputStream outfile = new FileOutputStream(filename);
      setOutputFormat(new OutputFormat(output, encoding, indenting));
      setOutputByteStream(outfile);
      serialize(doc);
      outfile.flush();
      outfile.close();
    } catch (FileNotFoundException ex) {
      System.err.println(ex.toString());
    } catch (IOException ex) {
      System.err.println(ex.toString());
    }
  } // saveToFile

  /**
   * Saves the specified document to a specified file.
   *
   * @param doc the document to save.
   * @param filename the filename to serialize to.
   */
  public void saveToFile(String doc, String filename) {
    try {
      // create output stream
      BufferedWriter out = new BufferedWriter(
        new OutputStreamWriter(
          new FileOutputStream(filename)));
      // now write to file
      out.write(doc);
      // now close the stream
      out.flush();
      out.close();
    } catch (FileNotFoundException ex) {
      System.err.println(ex.toString());
    } catch (IOException ex) {
      System.err.println(ex.toString());
    }
  } // saveToFile

  /**
   * Serializes the specified DOM document to a String Reader.
   *
   * @param doc the DOM document to serialize.
   * @return the serialized DOM document as a String Reader if everything goes well,
   * does otherwise return null.
   * @see java.io.StringReader
   */
  public StringReader serializeToStringReader(Document doc) {
    try {
      // first set the writer which writes to a StringBuffer
      StringWriter out = new StringWriter();
      // not happy about encoding to string... (don't know why, seems like a IE problem)
      // set OutputFormat
      setOutputFormat(new OutputFormat(output, encoding, indenting));
      setOutputCharStream(out);
      // then serialize the document
      serialize(doc);
      out.flush();
      // then return the StringBuffer as a StringReader
      return new StringReader(out.getBuffer().toString());
    } catch (FileNotFoundException ex) {
      System.err.println(ex.toString());
    } catch (IOException ex) {
      System.err.println(ex.toString());
    }
    // if this far something wrong happened
    return null;
  } // serializeToStringReader

  /**
   * Serializes the specified DOM document to a String.
   *
   * @param doc the DOM document to serialize.
   * @return the serialized DOM document as a String if everything goes well,
   * does otherwise return null.
   */
  public String serializeToString(Document doc) {
    try {
      // first set the writer which writes to a StringBuffer
      StringWriter out = new StringWriter();
      // not happy about encoding to string... (don't know why, seems like a IE problem)
      // set OutputFormat
      setOutputFormat(new OutputFormat(output, encoding, indenting));
      setOutputCharStream(out);
      // then serialize the document
      serialize(doc);
      out.flush();
      // then return the StringBuffer as a StringReader
      return out.getBuffer().toString();
    } catch (FileNotFoundException ex) {
      System.err.println(ex.toString());
    } catch (IOException ex) {
      System.err.println(ex.toString());
    }
    // if this far something wrong happened
    return null;
  } // serializeToString
} // DOMdocumentSerializer