/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999-2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package jcreek.cke.importexport.xml;

import java.io.InputStream;
import java.io.Reader;

import org.w3c.dom.Document;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;

/**
 * Encapsulates a DOM parser.
 *
 * <p>This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <a href="http://www.apache.org/">www.apache.org</a>.</p>
 *
 * @author Stein L. Tomassen, NTNU 2002
 * @author Apache org.
 */
public interface DOMparserWrapper {
	/**
	 * Parses the specified URL and returns it as a DOM document.
	 *
	 * @param url the URL of the document to parse
	 * @return the parsed document as a DOM object
	 * @throws Exception an expection if an parser error occurs
	 */
	public Document parse(String url) throws Exception;

	/**
	 * Parses the specified InputStream and returns the document.
	 *
	 * @param istream the input stream being the source of the document to parse
	 * @return the parsed document as a DOM object
	 * @throws Exception an expection if an parser error occurs
	 */
	public Document parse(InputStream istream) throws Exception;

	/**
	 * Parses the specified Reader and returns it as a DOM document.
	 *
	 * @param reader the input reader being the source of the document to parse
	 * @return the parsed document as a DOM object
	 * @throws Exception an expection if an parser error occurs
	 */
	public Document parse(Reader reader) throws Exception;

	/**
	 * Parses the specified Document and returns it as a DOM document.
	 * This method makes it easier to revalidate a document. In a future
	 * version of Xerces, maybe revalidation will a feature...
	 *
	 * @param document the DOM object to parse
	 * @return the parsed document as a DOM object
	 * @throws Exception an expection if an parser error occurs
	 */
	public Document parse(Document document) throws Exception;

	/**
	 * Set the state of a feature.
	 *
	 * Set the state of any feature in a SAX2 parser.  The parser
	 * might not recognize the feature, and if it does recognize
	 * it, it might not be able to fulfill the request.
	 *
	 * @param featureId The unique identifier (URI) of the feature.
	 * @param state The requested state of the feature (true or false).
	 *
	 * @throws org.xml.sax.SAXNotRecognizedException If the
	 *            requested feature is not known.
	 * @throws org.xml.sax.SAXNotSupportedException If the
	 *            requested feature is known, but the requested
	 *            state is not supported.
	 * @throws org.xml.sax.SAXException If there is any other
	 *            problem fulfilling the request.
	 */
	public void setFeature(String featureId, boolean state)
		throws SAXNotRecognizedException, SAXNotSupportedException;

	/**
	 * Returns a string with all the error messages the parser reports when
	 * parsing. The messages is separeted with a new line character.
	 *
	 * @return a string with all the error messages
	 */
	public String getErrorsMsg();

	/**
	 * Returns a string with all the warning messages the parser reports when
	 * parsing. The messages is separeted with a new line character.
	 *
	 * @return a string with all the warning messages
	 */
	public String getWarningsMsg();

	/**
	 * Returns a string with all the fatal error messages the parser reports when
	 * parsing. The messages is separeted with a new line character.
	 *
	 * @return a string with all the fatal error messages
	 */
	public String getFatalErrorsMsg();

	/**
	 * Returns the number of error messages the parser reports when parsing
	 *
	 * @return the number of errors
	 */
	public int getNumberOfErrors();

	/**
	 * Returns the number of warning messages the parser reports when parsing
	 *
	 * @return the number of warnings
	 */
	public int getNumberOfWarnings();

	/**
	 * Returns the number of fatal error messages the parser reports when parsing
	 *
	 * @return the number of fatal errors
	 */
	public int getNumberOfFatalErrors();
} // interface DOMParserWrapper