package jcreek.cke.importexport.xml;

import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * The intention of this class is to gather utilities that may be
 * useful when working with DOM documents.
 *
 * @author Stein L. Tomassen, NTNU 2002
 */
public class DOMutilities {
	/**
	 * Get's a value without it's namespace. The assumed delimiter
	 * is the "#" character.
	 *
	 * @param valueStr the string containing the value
	 * @return the value without the namespace. If the value
	 * didn't have any namespace, then the original value is returned.
	 */
	public static String getValueWithoutNS(String valueStr) {
		String delim = "#";
		return valueStr.substring(valueStr.lastIndexOf(delim) + 1);
	} // getValueWithoutNS

	/**
	 * Gets the value of an attribute from an attribute list
	 * by giving the name of the attribute. Does not use namespaces,
	 *
	 * @param attribName the name of the attribute looking for
	 * @param attribList the list of attributes
	 * @return the value of the attribute if found else it will
	 * return null
	 */
	public static String getAttributeValueByName(
		String attribName,
		NamedNodeMap attribList) {

		if (attribList != null) {
			try {
				Node attrib;
				for (int i = 0; i < attribList.getLength(); i++) {
					attrib = attribList.item(i);
					if (attribName.equalsIgnoreCase(attrib.getLocalName()))
						return attrib.getNodeValue();
				}
			} catch (Exception e) {
			}
		}
		return null;
	} // getAttributeValueByName

	/**
	 * Gets the node in from an node list by giving the name of
	 * the node. Does not use namespaces,
	 *
	 * @param nodeName the name of the node looking for
	 * @param nodeType the type of node looking for. The node types
	 * are declared in the Node class.
	 * @param nodeList the list of nodes
	 * @return the node if found else it will return null
	 */
	public static Node getNodeByName(
		String nodeName,
		short nodeType,
		NodeList nodeList) {
		if (nodeList != null) {
			try {
				Node node;
				for (int i = 0; i < nodeList.getLength(); i++) {
					node = nodeList.item(i);
					if (node.getNodeType() == nodeType
						&& nodeName.equalsIgnoreCase(node.getLocalName()))
						return node;
				}
			} catch (Exception e) {
			}
		}
		return null;
	} // getNodeByName
}
