/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999, 2000 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package jcreek.cke.importexport.xml;

import java.io.InputStream;
import java.io.Reader;

import org.w3c.dom.Document;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.SAXNotRecognizedException;
import org.xml.sax.SAXNotSupportedException;
import org.xml.sax.SAXParseException;

/**
 * Wraps the Xerces DOM parser.
 *
 * <p>This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <a href="http://www.apache.org/">www.apache.org</a>.</p>
 *
 * @author Stein L. Tomassen, NTNU 2002
 * @author Apache org.
 */
public class XercesParserWrapper implements DOMparserWrapper, ErrorHandler {
	//********************************************
	// Data
	//********************************************
	private StringBuffer _errorsMsg;
	private StringBuffer _warningsMsg;
	private StringBuffer _fatalErrorsMsg;
	private int _nbrOfErrors = 0;
	private int _nbrOfWarnings = 0;
	private int _nbrOfFatalErrors = 0;

	//********************************************
	// The parser
	//********************************************
	private org.apache.xerces.parsers.DOMParser parser =
		new org.apache.xerces.parsers.DOMParser();

	//********************************************
	// Constructors
	//********************************************

	/**
	 * The constructor that initializes the parser and sets the basic features.
	 * In case of an initialization fault, a fatal error message will be created, and
	 * must therefore be checked for after intialization.
	 */
	public XercesParserWrapper() {
		try {
			parser.setFeature("http://xml.org/sax/features/validation", true);
			_errorsMsg = new StringBuffer();
			_warningsMsg = new StringBuffer();
			_fatalErrorsMsg = new StringBuffer();
		} catch (Exception ex) {
			_nbrOfFatalErrors = 1;
			_fatalErrorsMsg.append("Unable To Set Parser Features");
		}
		parser.setErrorHandler(this);
	} // XercesParserWrapper

	//********************************************
	// ParserWrapper methods
	//********************************************

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#parse(String)
	 */
	public Document parse(String url) throws Exception {
		parser.parse(url);
		return parser.getDocument();
	} // parse(String):Document

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#parse(InputStream)
	 */
	public Document parse(InputStream istream) throws Exception {
		parser.parse(new InputSource(istream));
		return parser.getDocument();
	}

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#parse(Reader)
	 */
	public Document parse(Reader reader) throws Exception {
		parser.parse(new InputSource(reader));
		return parser.getDocument();
	}

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#parse(Document)
	 */
	public Document parse(Document document) throws Exception {
		DOMdocumentSerializer serializer = new DOMdocumentSerializer();
		parser.parse(new InputSource(serializer.serializeToStringReader(document)));
		return parser.getDocument();
	}

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#setFeature(String, boolean)
	 */
	public void setFeature(String featureId, boolean state)
		throws SAXNotRecognizedException, SAXNotSupportedException {
		parser.setFeature(featureId, state);
	}

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#getErrorsMsg()
	 */
	public String getErrorsMsg() {
		return _errorsMsg.toString();
	}

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#getWarningsMsg()
	 */
	public String getWarningsMsg() {
		return _warningsMsg.toString();
	}

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#getFatalErrorsMsg()
	 */
	public String getFatalErrorsMsg() {
		return _fatalErrorsMsg.toString();
	}

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#getNumberOfErrors()
	 */
	public int getNumberOfErrors() {
		return _nbrOfErrors;
	}

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#getNumberOfWarnings()
	 */
	public int getNumberOfWarnings() {
		return _nbrOfWarnings;
	}

	/**
	 * @see jcreek.cke.importexport.xml.DOMparserWrapper#getNumberOfFatalErrors()
	 */
	public int getNumberOfFatalErrors() {
		return _nbrOfFatalErrors;
	}

	//********************************************
	// Protected ErrorHandler methods that are overriding those
	// located in the super class
	//********************************************

	/**
	 * In case the parser encounters a warning while parsing
	 * the document, then the warning should be logged.
	 *
	 * @param ex the exception that occured
	 *
	 * @see org.xml.sax.ErrorHandler#warning(SAXParseException)
	 */
	public void warning(SAXParseException ex) {
		_warningsMsg.append(
			"[Warning] " + getLocationString(ex) + ": " + ex.getMessage() + '\n');
		_nbrOfWarnings++;
	}

	/**
	 * In case the parser encounters an error while parsing
	 * the document, then the error should be logged.
	 *
	 * @param ex the exception that occured
	 *
	 * @see org.xml.sax.ErrorHandler#error(SAXParseException)
	 */
	public void error(SAXParseException ex) {
		_errorsMsg.append(
			"[Error] " + getLocationString(ex) + ": " + ex.getMessage() + '\n');
		_nbrOfErrors++;
	}

	/**
	 * In case the parser encounters a fatal error while parsing
	 * the document, then the fatal error should be logged. In addition
	 * an exception should be thrown, because in case of a fatal error
	 * the parsing cannot continue.
	 *
	 * @param ex the exception that occured
	 * @throws org.xml.sax.SAXException
	 *
	 * @see org.xml.sax.ErrorHandler#fatalError(SAXParseException)
	 */
	public void fatalError(SAXParseException ex) throws SAXException {
		_fatalErrorsMsg.append(
			"[Fatal Error] " + getLocationString(ex) + ": " + ex.getMessage() + '\n');
		_nbrOfFatalErrors++;
		throw ex;
	}

	//********************************************
	// Protected methods
	//********************************************

	/**
	 * Finds the location of an error in the DOM document and returns
	 * this location as a string.
	 *
	 * @param ex the exception that occured
	 * @return the location string
	 */
	protected String getLocationString(SAXParseException ex) {
		StringBuffer str = new StringBuffer();

		String systemId = ex.getSystemId();
		if (systemId != null) {
			int index = systemId.lastIndexOf('/');
			if (index != -1)
				systemId = systemId.substring(index + 1);
			str.append(systemId);
		}
		str.append(':');
		str.append(ex.getLineNumber());
		str.append(':');
		str.append(ex.getColumnNumber());

		return str.toString();
	} // getLocationString(SAXParseException):String
} // class XercesParserWrapper