package jcreek.cke.tool;

import jcreek.cke.graph.CreekModel;
import jcreek.cke.conceptview.*;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.command.ZoomCommand;
import jcreek.cke.command.EntityMoveCommand;
import jcreek.cke.graph.VertexList;
import jcreek.cke.util.DPoint;
import jcreek.cke.util.Matrix33;
import jcreek.cke.graph.RelationList;
import jcreek.representation.Relation;
import jcreek.representation.Entity;
import jcreek.cke.browser.Visualizer;
import jcreek.cke.visualizer.FrameView;
import jcreek.cke.MainWindow;
import jcreek.representation.RelationType;
import jcreek.cke.command.DeactivateRelationCommand;
import jcreek.cke.command.ActivateEntityCommand;
import jcreek.cke.command.ActivateRelationCommand;

import javax.swing.JInternalFrame;
import javax.swing.SwingUtilities;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenuItem;
import javax.swing.JMenu;
import javax.swing.JButton;
import javax.swing.JPopupMenu;
import javax.swing.JOptionPane;
import javax.swing.ImageIcon;
import java.awt.event.MouseEvent;
import java.awt.Point;
import java.awt.Component;
import java.net.URL;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import javax.swing.Action;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.util.Iterator;

import jcreek.cke.command.NewRelationCommand;
import jcreek.cke.command.NewEntityCommand;



import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.JInternalFrame;
import javax.swing.SwingUtilities;

import java.beans.PropertyVetoException;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class BuildTool implements CreekTool, ConceptViewPaintListener, PropertyChangeListener {

    /**
     * Model that we're associated with.
     */
    protected CreekModel model;
    /**
     * View that we're associated with.
     */
    protected ConceptView view;
    /**
     * Document this mediator is associated with.
     */
    protected CreekDocument document;
    /**
     * Vertex the user pressed on in mousePressed(). Could be
     * null if the user didn't press on a entity.
     */
    protected VisualEntity pressVertex;
    /**
     * Vertex the rightClicked. Could be
     * null if the user didn't rightClick on a entity.
     */
    protected VisualEntity propVertex;
    /**
     * List of the selected entities in the order they were
     * selected by the user.
     */
    protected VertexList selectedList;
    /**
     * Vertex the mouse is currently over during the adding of
     * an edge. Could be null if mouse isn't over a vertex or
     * if user is not adding an edge.
     */
    protected VisualEntity targetVertex;
    /**
     * True if mouse was pressed on the view before being released.
     * Helps avoid some mis-firings of the mouse buttons.
     */
    protected boolean wasPressed;
    /**
     * The list of the entities being moved around. These
     * entities are a copy of the originals and can be modified.
     */
    protected VertexList movingVertices;
    /**
     * True if we're in the middle of adding a new edge.
     */
    protected boolean isAddEdge;
    /**
     * First mouse press position when dragging started.
     */
    protected Point firstMousePos;
    /**
     * Most recent mouse position while dragging.
     */
    protected Point lastMousePos;
    /**
     * A dashed line stroke for drawing.
     */
    protected BasicStroke dashedStroke;

    public PropertyWindow propWin;
    protected JPopupMenu relationList = new JPopupMenu("test");

    ItemListener  relationListHandler = new ItemListener() {
        public void itemStateChanged(ItemEvent e) {
            toggleRelation(propVertex, ((JCheckBoxMenuItem) e.getItem()).getText());
      }
    } ;

  public BuildTool() {
        selectedList = new VertexList();
        firstMousePos = new Point();
        lastMousePos = new Point();
        float dash[] = { 4.0f };
        dashedStroke = new BasicStroke( 1.0f, BasicStroke.CAP_BUTT,
                                        BasicStroke.JOIN_MITER,
                                        10.0f, dash, 0.0f );
        propWin = (PropertyWindow)new BuildProperty();
  }//BuildTool


  public void attach(CreekModel model, ConceptView view, JInternalFrame win) {
        if ( ( model == this.model ) && ( view == this.view ) ) {
            // Nothing has changed, nothing to do.
            return;
        }
        // Detach in case our manager forgot to.
        detach();

        // Now attach to new model and view.
        this.model = model;
//        this.model.unselectAllVertices();
        this.view = view;
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        dg.setKnowledgeModelEditable(true);
        document = dg.getDocument( view );
        view.addViewListener( this );
        view.addViewPaintListener( this );
        model.addChangeListener( this );
        propWin.attach(model,view,win);
  }//attach


  public void detach() {
        if ( view != null ) {
            view.removeViewPaintListener( this );
            view.removeViewListener( this );
            view = null;
        }
        if ( model != null ) {
            model.removeChangeListener( this );
            model = null;
        }
        document = null;
        CreekDocumentGroup.instanceOf().setKnowledgeModelEditable(false);
        propWin.detach();
  }//detach


  public VertexList getSelected() {
      return null;
  }//getSelected


  public void mouseClicked(MouseEvent me)
  {

    if(SwingUtilities.isLeftMouseButton(me))
    {
      ((Component)view).requestFocus();

        // Remember the mouse position for when we have to
        // move entities around the view.
/*        firstMousePos.x = me.getX();
        firstMousePos.y = me.getY();
        lastMousePos.x = firstMousePos.x;
        lastMousePos.y = firstMousePos.y; */

        pressVertex = findNode(me.getX(), me.getY());

        if ( selectedList.size() > 0 ) {
            selectedList.clear();
        }
        model.unselectAllVertices();

        if(pressVertex != null)
        {
          Visualizer viz = Visualizer.getFrame((Component)view);
          model.setSelected( pressVertex, true );
          selectedList.add( pressVertex );

          FrameView fView = viz.getFrameView();
          if(fView != null)
          {
            fView.setForInput();
          }
        }
        else
        {
          if(!me.isControlDown())
          {
            // Create a new entity here
              NewEntityCommand action = new NewEntityCommand(document,
                  new Point(me.getX(),me.getY()));
              if(document.performCommand( action ))
              {
                Visualizer viz = Visualizer.getFrame((Component)view);
                FrameView fView = viz.getFrameView();
                if(fView != null)
                {
                  viz.show("FrameView");
                  fView.setForInput();
                }
              }
          }
        }
    }

    if (me.isPopupTrigger() || SwingUtilities.isRightMouseButton(me))
    {
      ((Component)view).requestFocus();

      propVertex = findNode(me.getX(), me.getY());

        if(propVertex!=null)
           if(!propVertex.isSelected())
            {
                  if ( selectedList.size() > 0 ) {
                      selectedList.clear();
                  }
                  model.unselectAllVertices();
                  model.setSelected( propVertex, true );
                  selectedList.add( propVertex );
            }
        formPopup(me.getX(),me.getY());
    }
  }//mouseClicked


    /**
     * User has pressed a mouse button on us. Interpret the click
     * as clicking a vertex and save the vertex reference. May be
     * null if the user didn't click on a vertex.
     *
     * @param  me  mouse event for this canvas
     */
  public void mousePressed(MouseEvent me) {
        ((Component)view).requestFocus();
        if(SwingUtilities.isLeftMouseButton(me))
        {
            // Remember the mouse position for when we have to
            // move entities around the view.
            firstMousePos.x = me.getX();
            firstMousePos.y = me.getY();
            lastMousePos.x = firstMousePos.x;
            lastMousePos.y = firstMousePos.y;
            pressVertex = findNode(firstMousePos.x, firstMousePos.y);
            wasPressed = true;

            if(pressVertex == null)
            {

            }
        }//left mouse button loop

/*        if (me.isPopupTrigger())
        {
            firstMousePos.x = me.getX();
            firstMousePos.y = me.getY();
            propVertex = findNode(firstMousePos.x, firstMousePos.y);
            if(propVertex!=null)
                if(!propVertex.isSelected())
                {
                      if ( selectedList.size() > 0 ) {
                          selectedList.clear();
                      }
                      model.unselectAllVertices();
                      model.setSelected( propVertex, true );
                      selectedList.add( propVertex );
                }
            formPopup(me.getX(),me.getY());
        } */
  }//mousePressed


    /**
     * User has released a mouse button on us. Now we check if the
     * user released the mouse on a vertex. This could mean several
     * things, as follows:
     *
     * pressVertex = targetVertex = null, add vertex
     * pressVertex != targetVertex and no edge exists, add one
     *
     * @param  me  mouse event for this canvas
     */
  public void mouseReleased(MouseEvent me) {
        if(SwingUtilities.isLeftMouseButton(me))
        {
            if ( wasPressed == false ) {
                return;
            }
            isAddEdge = false;
            wasPressed = false;
            view.repaint();
            // See if user released on a vertex.
            targetVertex = view.findVisualEntity( me.getX(), me.getY() );

            wasPressed = false;
            int mX = me.getX();
            int mY = me.getY();
            if(me.isControlDown())
            {
                if ( movingVertices != null ) {
                    // We were moving vertices around the screen.
                    resetMovingVertices();
                    double deltaX = mX - firstMousePos.x;
                    double deltaY = mY - firstMousePos.y;
                    EntityMoveCommand action = new EntityMoveCommand
                        ( document, selectedList, ((int)deltaX),((int) deltaY));
                    document.performCommand( action );
                }
            }
            else
            {
                // If yes, then check if both pressVertex and targetVertex
                // not null and different.
                if ( targetVertex != null ) {
                    if ( ( pressVertex != null ) &&
                         ( pressVertex != targetVertex ) ) {
                        RelationType rcd = ((BuildProperty)propWin).getSelectedRelation();
                        if ( rcd != null ) {
                            NewRelationCommand action = new NewRelationCommand
                                                  (document,
                                                   rcd,
                                                   pressVertex,
                                                   targetVertex);
                            document.performCommand( action );
                        }
                    }
                    else
                    {


                            if ( selectedList.size() > 0 ) {
                                selectedList.clear();
                            }
                            model.unselectAllVertices();

                            model.setSelected( targetVertex, true );
                            selectedList.add( targetVertex );
                    }
                }
                else
                {
                    // Else user did not release on a vertex.
                    if ( pressVertex == null )
                    {
                          // User didn't press on a vertex either, so let's add
                          // a new vertex where they clicked.

                    }
                }
            }
            finishMoving();
            pressVertex = null;
            targetVertex = null;
        }
  }//mouseReleased


  public void mouseEntered(MouseEvent e) {
  }//mouseEntered

  public void mouseExited(MouseEvent e) {
  }//mouseExited

    /**
     * The user is dragging the mouse. Check if we are possibly
     * creating a new edge or not.
     *
     * @param  me  mouse event
     */
  public void mouseDragged(MouseEvent me) {
        if(SwingUtilities.isLeftMouseButton(me))
        {
            if ( !wasPressed ) {
                return;
            }
            int deltaX = me.getX()-lastMousePos.x;
            int deltaY = me.getY()-lastMousePos.y ;
            lastMousePos.x =  me.getX();
            lastMousePos.y =  me.getY();

            if(me.isControlDown())
            {
                // First make sure the vertex the user clicked on is
                // selected (putting it in the selected list). This
                // makes it possible to move several vertices at once.
                if ( ( pressVertex != null ) &&
                     ( pressVertex.isSelected() == false ) ) {
                  if ( selectedList.size() > 0 ) {
                      selectedList.clear();
                  }
                  model.unselectAllVertices();
                  model.setSelected( pressVertex, true );
                  selectedList.add( pressVertex );
                }

                // If there are some vertices selected, deal with moving
                // them around the view.
                if ( ( pressVertex != null ) && ( selectedList.size() > 0 ) ) {
                    if ( movingVertices == null ) {
                        // Make a copy of the selected vertices so we can
                        // modify their positions based off the originals.
                        // We'll use the copy in resetMovingVertices() to
                        // restore the original positions.
                        movingVertices = selectedList.copy();
                    }
                    VisualEntity v1;
                    int size = movingVertices.size();
                    for ( int i = 0; i < size; i++ ) {
                        // Move the original vertices based off the
                        // position of the copies.
                        v1 = selectedList.get( i );
                        v1.slide(deltaX,deltaY);
                    }
    //                view.drawSelectedNodes();
                }
            }
            else
            {
                  if ( pressVertex != null ) {
                      // cause view to repaint so we can show edge being
                      // added from the start vertex
                      isAddEdge = true;
                      lastMousePos.setLocation( me.getX(), me.getY() );
                      targetVertex = view.findVisualEntity( lastMousePos.x,
                                                       lastMousePos.y);
                  }
            }
            view.repaint();
        }

  }//mouseDragged


  public void mouseMoved(MouseEvent e) {
  }//mouseMoved

    /**
     * Called after the view draws the graph. Gives the
     * listener a chance to draw something to the view
     * after everything else has been drawn. The order in
     * which listeners are called is non-deterministic.
     *
     * @param  event  view paint event
     */
    public void paintAfter( ConceptViewPaintEvent event ) {
        if ( isAddEdge && ( pressVertex != null ) ) {
            Graphics2D g = event.getGraphics();
            g.setColor( Color.black );
            g.setStroke( dashedStroke );
            Matrix33 viewTransform = view.getViewTransform();

            DPoint pos = pressVertex.getPosition();
            pos.transform(viewTransform);
            // draw dashed line where new edge will go
            g.drawLine( (int)pos.x, (int)pos.y,lastMousePos.x, lastMousePos.y );
        }
    } // paintAfter

    /**
     * Called before the view draws the graph. Gives the
     * listener a chance to draw something to the view
     * before anything else appears. The order in which
     * listeners are called is non-deterministic.
     *
     * @param  event  view paint event
     */
    public void paintBefore( ConceptViewPaintEvent event ) {
        // We have nothing to do for now.
    } // paintBefore

    /**
     * The state of the object we're listening to has changed.
     * This means we should check if any of the selected entities
     * have been removed.
     *
     * @param  e  change event
     */
    public void propertyChange( PropertyChangeEvent e ) {
    }

    /**
     * Returns the position of the mouse,this will return the
     * first Mouse Position or last Mouse Position depending
     * on the current Tool and the state of the tool
     *
     * @return Point position of the mouse
     */
     public Point getMousePosition()
     {
        return firstMousePos;
     }

    /**
     * Called when user was defining a rectangular region with
     * which to select entities. Determines which entities are
     * within the rectangle and marks them selected.
     */
    protected void selectRegion() {
        // determine the bounding rectangle based on mouse positions
        int x1 = (int)Math.min( firstMousePos.x, lastMousePos.x );
        int y1 = (int)Math.min( firstMousePos.y, lastMousePos.y );
        int x2 = (int)Math.max( firstMousePos.x, lastMousePos.x );
        int y2 = (int)Math.max( firstMousePos.y, lastMousePos.y );
        // for each vertex in the model...
        VertexList vl = model.getEntities();
        int size = vl.size();
        Matrix33 viewTransform = view.getViewTransform();
        for ( int i = 0; i < size; i++ ) {
            VisualEntity v = vl.get( i );
               DPoint pos = v.getPosition();
               pos.transform(viewTransform);
            // ...if the vertex is within the region, and it is
            // not currently selected, mark it selected now
               if(pos.x >= x1 && pos.x <= x2 && pos.y >= y1 && pos.y <= y2)
               {
                  model.setSelected( v, true );
                  selectedList.add( v );
              }
        }
    } // selectRegion


    /**
     * Moves all the moving entities back to their original
     * positions by using the original positions stored in
     * the copy of the entities.
     */
    protected void resetMovingVertices() {
        int size = selectedList.size();
        for ( int i = 0; i < size; i++ ) {
            VisualEntity selV = selectedList.get( i );
            VisualEntity origV = movingVertices.get( i );
            selV.setPosition( origV.getPosition().x, origV.getPosition().y );
        }
    } // resetMovingVertices

    /**
     * Find an entity close to the given x,y position.
     *
     * @param  x  mouse x position
     * @param  y  mouse y position
     */
    protected VisualEntity findNode( int x, int y ) {
        return view.findVisualEntity(x,y);
   } // findEdge
    /**
     * If we were moving entities, clean up now.
     */
    protected void finishMoving() {
        if ( movingVertices != null ) {
            movingVertices.clear();
            movingVertices = null;
            //got to add a recal method
            view.fireContentsResize();
        }
        else
            view.repaint();

    } // finishMoving


  protected void formPopup(int x,int y)
  {

      relationList.removeAll();
      relationList.add(createMenuItem("paste"));
      if(propVertex != null)
      {
        relationList.add(createMenuItem("cut"));
        relationList.add(createMenuItem("copy"));
        relationList.addSeparator();
        relationList.add(createMenuItem("openEntity"));
        relationList.add(createRelationsMenu());
        relationList.add(createMenuItem("entityProperty"));
        relationList.add(createMenuItem("openCaseView"));
        relationList.add(createMenuItem("openCaseMatch"));
        relationList.addSeparator();
        relationList.add(createMenuItem("deactivate"));
        relationList.addSeparator();
        relationList.add(createMenuItem("delete"));
      }
      Visualizer viz = Visualizer.getFrame((Component)view);
      relationList.show(viz.getWin(document).getContentPane(), x, y);
  }

    protected JMenuItem createMenuItem( String cmd ) {
        MainWindow mw = MainWindow.instanceOf();
        // create menu item and set image and text
        JMenuItem mi = new JMenuItem
            ( mw.getResourceString( cmd + Visualizer.labelSuffix ) );
        URL url = mw.getResource( cmd + Visualizer.imageSuffix );
        if ( url != null ) {
            mi.setHorizontalTextPosition( JButton.RIGHT );
            mi.setIcon( new ImageIcon( url ) );
        }

        // set menu action command
        mi.setActionCommand( cmd );
        Action a = mw.getAction( cmd );
        if ( a != null ) {
            // set up the action to listen for events
            mi.addActionListener( a );
            mi.setEnabled( a.isEnabled() );
        } else {
            // no action! disable menu item
            mi.setEnabled( false );
        }
        return mi;
    } // createMenuItem

    protected JMenu createRelationsMenu()
    {
          JMenu rel = new JMenu(
            MainWindow.instanceOf().getResourceString("relations"+Visualizer.labelSuffix));
          URL url = MainWindow.instanceOf().getResource("relations"+ Visualizer.imageSuffix);
          if ( url != null ) {
              rel.setHorizontalTextPosition( JButton.RIGHT );
              rel.setIcon( new ImageIcon( url ) );
          }
          RelationType[] rels = propVertex.getEntity().getRelationTypes();

          for(int i = rels.length-1;i >= 0;i--)
          {
            Relation[] r = propVertex.getEntity().getLocalRelations(rels[i]);



//            actualRelations = (Entity.class.isAssignableFrom(r[j].getFrom().getClass()) && Entity.class.isAssignableFrom(r[j].getValue().getObject().getClass()));

            if(r.length > 0)
            {
              JCheckBoxMenuItem mi = new JCheckBoxMenuItem(rels[i].getName(), propVertex.relationIsActive(rels[i]));
              mi.addItemListener(relationListHandler);
              rel.add(mi);
            }
            else
            {
              JMenuItem mi = new JMenuItem(rels[i].getName());
              mi.setEnabled(false);
              rel.add(mi);
            }
          }
          return rel;
    }

    protected void toggleRelation(VisualEntity e, String relationName)
    {
        if(e.relationIsActive(relationName))
          deactivateRelation(relationName);
        else
          activateRelation(relationName);
    }

      public void deactivateRelation(String relationName)
      {
        Iterator iterator = propVertex.getEdgeIterator();
        Relation relation;
        VisualRelation visualRelation;
        while(iterator.hasNext())
        {
              visualRelation = (VisualRelation)iterator.next();
              relation = visualRelation.getRelation();
              if(!relation.isInherited() && (relation.getRelationType().getName().equals(relationName) && relation.getFrom() == propVertex.getEntity()) ||
                (relation.getRelationType().getInverseName().equals(relationName) && relation.getValue() == propVertex.getEntity()))
              {
                  DeactivateRelationCommand delRelCmd = new DeactivateRelationCommand(document,visualRelation);
                  if(document.performCommand(delRelCmd)==false)
                      JOptionPane.showMessageDialog
                          ( Visualizer.getFrame((Component)view),
                            MainWindow.instanceOf().getResourceString( "edgeCloseMsg" ),
                            MainWindow.instanceOf().getResourceString( "userError"+Visualizer.titleSuffix ),
                            JOptionPane.ERROR_MESSAGE );

              }
        }
      }

      protected void activateRelation(String relationType)
      {
        Relation[] rels = propVertex.getEntity().getRelations(relationType);
        for(int i=rels.length-1;i>=0;i--)
          if(!rels[i].isInherited())
            activateRelation(rels[i]);
      }

      protected void activateRelation(Relation r)
      {
        if(r.getFrom().equals(propVertex.getEntity()))
        {
            VisualEntity veto = activateEntity((Entity)r.getValue(),propVertex);
            ActivateRelationCommand arAction = new ActivateRelationCommand( document,new VisualRelation(propVertex, veto, r) );
            if ( document.performCommand( arAction ) == false )
            {
                  JOptionPane.showMessageDialog
                      ( Visualizer.getFrame((Component)view),
                        MainWindow.instanceOf().getResourceString( "edgeAddMsg" ),
                        MainWindow.instanceOf().getResourceString("userError"+Visualizer.titleSuffix),
                        JOptionPane.ERROR_MESSAGE );
            }
        }
        else if(r.getValue().equals(propVertex.getEntity()))
        {
            VisualEntity vef = activateEntity((Entity)r.getFrom(),propVertex);
            ActivateRelationCommand arAction = new ActivateRelationCommand( document,new VisualRelation(vef, propVertex, r) );
            if ( document.performCommand( arAction ) == false )
            {
                  JOptionPane.showMessageDialog
                      ( Visualizer.getFrame((Component)view),
                        MainWindow.instanceOf().getResourceString( "edgeAddMsg" ),
                        MainWindow.instanceOf().getResourceString("userError"+Visualizer.titleSuffix),
                        JOptionPane.ERROR_MESSAGE );
            }
        }
      }


      public VisualEntity activateEntity(Entity ed,VisualEntity activator)
      {
          VisualEntity ved = model.findVisualEntity(ed);
          if(ved == null)
          {
              DPoint tempPoint = view.activateEntity(activator);
              ved = new VisualEntity(ed,(int)tempPoint.x,(int)tempPoint.y);
              ActivateEntityCommand aed = new ActivateEntityCommand(document,ved);
              if(!document.performCommand(aed))
                  return null;
          }
          return ved;
      }




}//BuildTool


