package jcreek.cke.tool;

import java.util.Vector;

import javax.swing.JInternalFrame;
import java.awt.Point;

import jcreek.cke.conceptview.*;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.graph.VertexList;

/**
 * Defines the interface for the tool objects in Creek.
 * Since all tools in Creek will interact with the user
 * (via the view) this interface extends the view listener
 * interface.
 *
 */

public interface CreekTool extends ConceptViewListener
{

    /**
     * Attach to the given model and view objects. This begins
     * user interaction and will imply that this tool is the
     * currently active tool. The tool should add itself as a
     * listener to the view object to begin interaction with
     * the user.
     *
     * @param  model  GraphModel to attach to
     * @param  view   GraphView to attach to
     */
    public void attach( CreekModel model, ConceptView view,JInternalFrame win );

    /**
     * Detach from the model and view and prepare to be
     * deactivated. This does not imply that this object is
     * about to be destroyed. The tool should remove itself
     * from the view listener list and cease interaction
     * with the user.
     * <p>
     * Note that this method may be called more than once
     * between calls to <code>attach()</code>
     */
    public void detach();

    /**
     * Returns the list of selected vertices, in the order they
     * were selected by the user. Often called by another class
     * that is going to perform some operation on the graph,
     * such as a search.
     *
     * @return  selected vertex list
     */
    public VertexList getSelected();


    /**
     * Returns the position of the mouse,this will return the
     * first Mouse Position or last Mouse Position depending
     * on the current Tool and the state of the tool
     *
     * @return Point position of the mouse
     */
     public Point getMousePosition();
}
