/**
 * Why isn't there a general sorting algorithm implemented in JAVA 1? :-((
 *
 * This implementation of C.A.R Hoare's Quick Sort Algorithm is based on the
 * example sorting algorithm provided by Sun in JDK 1.1.8 .
 *
 * @author Frode Srmo
 * @author James Gosling
 * @author Kevin A. Smith
 */

package jcreek.cke.util;

abstract public class QuickSort
{
  public QuickSort()
  {
  }

   /** This is a generic version of C.A.R Hoare's Quick Sort
    * algorithm.  This will handle arrays that are already
    * sorted, and arrays with duplicate keys.<BR>
    *
    * If you think of a one dimensional array as going from
    * the lowest index on the left to the highest index on the right
    * then the parameters to this function are lowest index or
    * left and highest index or right.  The first time you call
    * this function it will be with the parameters 0, a.length - 1.
    *
    * @param a       an Object array
    * @param lo0     left boundary of array partition
    * @param hi0     right boundary of array partition
    */
   void qSort(Object a[], int lo0, int hi0)
   {
      int lo = lo0;
      int hi = hi0;
      Object mid;

      if ( hi0 > lo0)
      {

         /* Arbitrarily establishing partition element as the midpoint of
          * the array.
          */
         mid = a[ ( lo0 + hi0 ) / 2 ];

         // loop through the array until indices cross
         while( lo <= hi )
         {
            /* find the first element that is greater than or equal to
             * the partition element starting from the left Index.
             */
	     while( ( lo < hi0 ) && ( compare(a[lo], mid) ))
		 ++lo;

            /* find an element that is smaller than or equal to
             * the partition element starting from the right Index.
             */
	     while( ( hi > lo0 ) && ( compare(mid, a[hi]) ))
		 --hi;

            // if the indexes have not crossed, swap
            if( lo <= hi )
            {
               swap(a, lo, hi);
               ++lo;
               --hi;
            }
         }

         /* If the right index has not reached the left side of array
          * must now sort the left partition.
          */
         if( lo0 < hi )
            qSort( a, lo0, hi );

         /* If the left index has not reached the right side of array
          * must now sort the right partition.
          */
         if( lo < hi0 )
            qSort( a, lo, hi0 );

      }
   }

   private void swap(Object a[], int i, int j)
   {
      Object T;
      T = a[i];
      a[i] = a[j];
      a[j] = T;

   }

   public void sort(Object a[])
   {
      qSort(a, 0, a.length - 1);
   }

   public abstract boolean compare(Object ob1, Object ob2);
}