package jcreek.cke.visualizer;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.text.JTextComponent;
import jcreek.cke.MainWindow;



public class AboutDialog extends JDialog {

    protected static final String versionHeader[] = { "\n[ Version Info ]\n\n", "* Program File Name:", "* Class Name:\t", "* Extends From:", "* Java Version:", "* Operating System:", "* Revision History:", "* Class Purpose:", "* Appox. Stability:", "* Uploaded on:\t" };
    static String versionData[]= { "TrollCreek", "Visualizer", "JFrame", "1.0.0", "All", "05-Jan-03 - Initial version", "Class To Create KnowledgeModel", "100%", "16-Jan-03" };
    String header;
    Dimension screenSize,size;
    Date date;
    JTextArea textArea;
    JScrollPane scrollPane;
    JLabel northLabel;
    JLabel southLabel;
    Font smallFont;
    FontMetrics fm;
    JPanel contentHolder;

    protected KeyHandler keyHandler;

  public AboutDialog() {
        contentHolder =  new JPanel();

        smallFont = new Font("Arial", 1, 10);
        fm = getFontMetrics(smallFont);
        header = "TrollCreek";
        screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        size = new Dimension(400,400);
        contentHolder.setLayout(new BorderLayout(10, 0));
        contentHolder.setBackground(Color.white);
        setSize(size);
        this.setResizable(false);

        textArea = new JTextArea(5, 20);
        textArea.setMargin(new Insets(5, 5, 5, 5));
        textArea.setEditable(false);
        textArea.setFont(smallFont);
        scrollPane = new JScrollPane(textArea, 20, 30);

        MainWindow mWin = MainWindow.instanceOf();
        ImageIcon imageIcon1 = new ImageIcon(mWin.getResource("topImage"));
        ImageIcon imageIcon2 = new ImageIcon(mWin.getResource("bottomImage"));
        ImageIcon imageIcon3 = new ImageIcon(mWin.getResource("rightImage"));
        ImageIcon imageIcon4 = new ImageIcon(mWin.getResource("leftImage"));

        northLabel = new JLabel("TrollHetta INC ", imageIcon1, 2);
        northLabel.setFont(smallFont);

        southLabel = new JLabel(" ", imageIcon2, 4);
        southLabel.setFont(smallFont);
        southLabel.setIconTextGap(25);
        southLabel.setHorizontalTextPosition(2);

        JLabel jLabel1 = new JLabel(imageIcon4);
        JLabel jLabel2 = new JLabel(imageIcon3);
        jLabel1.setVerticalAlignment(1);
        jLabel2.setVerticalAlignment(3);
        contentHolder.add(northLabel, "North");
        contentHolder.add(southLabel, "South");
        contentHolder.add(jLabel2, "East");
        contentHolder.add(jLabel1, "West");
        contentHolder.add(scrollPane, "Center");
        textArea.append(new StringBuffer(String.valueOf("JavaCreek")).append("\n\n").toString());

        this.setContentPane(contentHolder);
        setLocation((screenSize.width - size.width) / 2,
                    (screenSize.height - size.height) / 2);

        println("TrollCreek Knowledge Editor");
        println("(C) Copyright 2001-2004 Trollhetta\n");


        Properties versionProperties = new Properties();
        try {
          versionProperties.load(getClass().getResourceAsStream(
              "/jcreek/resources/version.properties"));
          String version = versionProperties.getProperty("name") + " " +
              versionProperties.getProperty("branch") + " version " +
              versionProperties.getProperty("version") + " build " +
              versionProperties.getProperty("build.number");
          String time = versionProperties.getProperty("build.time");

          println("Version\t"+version);
          println("Created at\t"+time);
        }
        catch (java.io.IOException e)
        {
          println("Version information not available.");
          e.printStackTrace();
        }


//        println("- TrollCreek v1.0 -");
//        println("Copyright (C) 2001 TrollHetta");
//        println("Created in May 2001");
//        printVersion(versionData);
        keyHandler = new KeyHandler();
        addKeyListener(keyHandler);
        addWindowListener(new WindowHandler());

        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.validate();
        this.show();
  }




    public void printVersion(String astring[])

    {

        for (int i = 0; i < versionHeader.length; i++)

        {

            textArea.append(versionHeader[i]);

            if (i > 0)

            {

                textArea.append("\t");

                textArea.append(new StringBuffer(String.valueOf(astring[i - 1])).append("\n").toString());

            }

        }

        textArea.append("\n");

        updateTextArea();

    }



    public void println(String string)

    {

        textArea.append(new StringBuffer(String.valueOf(string)).append("\n").toString());

        updateTextArea();

    }




    public void updateTextArea()

    {

        JScrollBar jScrollBar = scrollPane.getVerticalScrollBar();

        jScrollBar.setValue(jScrollBar.getMaximum());

    }

	public void ok()
	{
		dispose();
	}

	public void cancel()
	{
		dispose();
	}


	class KeyHandler extends KeyAdapter
	{
		public void keyPressed(KeyEvent evt)
		{
			if(evt.isConsumed())
				return;

			if(evt.getKeyCode() == KeyEvent.VK_ENTER)
			{
				Component comp = getFocusOwner();
				while(comp != null)
				{
					if(comp instanceof JComboBox)
					{
						JComboBox combo = (JComboBox)comp;
						if(combo.isEditable())
						{
							Object selected = combo.getEditor().getItem();
							if(selected != null)
								combo.setSelectedItem(selected);
						}
						break;
					}

					comp = comp.getParent();
				}

				ok();
				evt.consume();
			}
			else if(evt.getKeyCode() == KeyEvent.VK_ESCAPE)
			{
				cancel();
				evt.consume();
			}
		}
	}

	class WindowHandler extends WindowAdapter
	{
		public void windowClosing(WindowEvent evt)
		{
			cancel();
		}
	}


}
