package jcreek.cke.visualizer;

import java.awt.*;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.Collection;
import javax.swing.*;
import javax.swing.border.*;
import java.util.Vector;

import jcreek.representation.KnowledgeModel;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.visualizer.EntityComboBoxModel;
import jcreek.representation.*; import jcreek.representation.cbr.*;
import jcreek.cke.controls.*;
import jcreek.cke.MainWindow;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class DomainRelationDlg extends JDialog implements ActionListener {
  BorderLayout borderLayout1 = new BorderLayout();
  JPanel contentPanel = new JPanel();
  JPanel relationInput = new JPanel();
  JPanel userInput = new JPanel();
  GridLayout gridLayout1 = new GridLayout();
  JLabel clsNameLbl = new JLabel();
  JLabel clsInvNameLbl = new JLabel();
  JLabel expStngthLbl = new JLabel();
  JLabel clsDescLbl = new JLabel();
  JTextField clsName = new JTextField();
  JTextField clsDesc = new JTextField();
  JTextField clsInvName = new JTextField();
  protected JSpinner expStrength = new JSpinner(new SpinnerNumberModel(0.5,0.0,1.0,0.1));
  TitledBorder titledBorder1;
  TitledBorder titledBorder2;
  JPanel buttonPanel = new JPanel();
  FlowLayout flowLayout1 = new FlowLayout();
  FlowLayout flowLayout2 = new FlowLayout();
  JComboBox relationClassList = new JComboBox();
  JButton cancelButton = new JButton();
  JButton okButton = new JButton();
  BoxLayout boxLayout21;
  Hashtable valueTable = new Hashtable();
  boolean isOk = false;
  KnowledgeModel km = null;

  public DomainRelationDlg(Frame owner) {
    super(owner,"New Domain Relation",true);
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  private void jbInit() throws Exception {
    km = CreekDocumentGroup.instanceOf().getKM();

    MainWindow mw = MainWindow.instanceOf();

    updateModelView();
    titledBorder1 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Relation");
    titledBorder2 = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Relation");
    this.getContentPane().setLayout(borderLayout1);
    boxLayout21 = new BoxLayout(contentPanel,BoxLayout.Y_AXIS);
    contentPanel.setLayout(boxLayout21);
    relationInput.setLayout(gridLayout1);
    gridLayout1.setRows(4);
    gridLayout1.setColumns(2);
    gridLayout1.setHgap(2);
    gridLayout1.setVgap(2);
    clsNameLbl.setToolTipText("");
    clsNameLbl.setText("Class Name");
    clsName.setInputVerifier(
        new InputVerifier()
        {
            public boolean verify(JComponent comp)
            {
                String val = ((JTextField)comp).getText();
                if(val.length()==0)
                    return false;
                else if(val.startsWith(" ") || km.hasEntity(val))
                    return false;
                valueTable.put(clsName,new Boolean(true));
                return true;
            }
        }
    );

    valueTable.put(clsName,new Boolean(false));
    clsInvNameLbl.setText("Class Inverse Name");
    clsInvName.setInputVerifier(
        new InputVerifier()
        {
            public boolean verify(JComponent comp)
            {
                String val = ((JTextField)comp).getText();
                if(val.length()==0)
                    return false;
                else if(val.startsWith(" ") || km.hasEntity(val))
                    return false;
                valueTable.put(clsInvName,new Boolean(true));
                return true;
            }
        }
    );
    valueTable.put(clsInvName,new Boolean(false));
    expStngthLbl.setText("Explanation Strength");
    clsDescLbl.setText("Class Description");
    clsDesc.setInputVerifier(
        new InputVerifier()
        {
            public boolean verify(JComponent comp)
            {
                valueTable.put(clsDesc,new Boolean(true));
                return true;
            }
        }
    );
    valueTable.put(clsDesc,new Boolean(false));
    relationInput.setBorder(titledBorder1);
    userInput.setBorder(titledBorder2);
    userInput.setLayout(flowLayout2);
    userInput.add(relationClassList);
    buttonPanel.setLayout(flowLayout1);
    cancelButton.setText(mw.getResourceString("cancelLabel"));
    cancelButton.addActionListener(this);

    getRootPane().setDefaultButton(cancelButton);

    okButton.setText(mw.getResourceString("okLabel"));
    okButton.addActionListener(this);
    this.setModal(true);
    this.setSize(275,275);
    this.setResizable(false);
    this.getContentPane().add(contentPanel, BorderLayout.CENTER);
    contentPanel.add(relationInput, null);
    contentPanel.add(userInput, null);
    this.getContentPane().add(buttonPanel,  BorderLayout.SOUTH);
    relationInput.add(clsNameLbl, null);
    relationInput.add(clsName, null);
    relationInput.add(clsInvNameLbl, null);
    relationInput.add(clsInvName, null);
    relationInput.add(clsDescLbl, null);
    relationInput.add(clsDesc, null);
    relationInput.add(expStngthLbl, null);
    relationInput.add(expStrength, null);
    buttonPanel.add(okButton, null);
    buttonPanel.add(cancelButton, null);
  }


  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      cancel();
    }
    super.processWindowEvent(e);
  }

  void cancel() {
    hide();
    dispose();
  }

    /**
    * This methods handles the two buttons of this panel, to save all the information
    *   in this panel if the user click on OK button no matter there is change or not.
    *   I do watch the name changes of this panel, because name is used as key for the
    *   object. <P>
    */
    public void actionPerformed(ActionEvent event) {
        String command = ((JButton)event.getSource()).getText();
        if (command.equals(
            MainWindow.instanceOf().getResourceString("cancelLabel"))) {
            isOk = false;
        }
        else{
            for(Enumeration e = valueTable.elements();e.hasMoreElements();)
                if(!((Boolean)e.nextElement()).booleanValue())
                    return;
            isOk = true;
        }
        hide();
        dispose();

    }

    public boolean isOK()
    {
        return isOk;
    }


  public void updateModelView()
  {
    if(km != null)
    {
        Object selectedRelation = relationClassList.getSelectedItem();
        relationClassList.setModel(new RelationComboBoxModel(km));
        relationClassList.setSelectedItem(selectedRelation);
    }
  }

    public RelationType getSelectedRelation()
    {
      try
      {
        RelationType rcd = null;
        if (relationClassList.getSelectedIndex() != -1) {
          RelationComboBoxModel rcdModel = (RelationComboBoxModel)
              relationClassList.getModel();
          rcd = (RelationType) new RelationType( (Entity) rcdModel.getConceptAt(
              relationClassList.getSelectedIndex()));
        }
        return rcd;
      }
      catch(EntityTypeNotMatchedException ex)
      {
        ex.printStackTrace();
        return null;
      }
    }//getSelectedRelation

    public String getClassName()
    {
        return clsName.getText();
    }

    public String getClassInverseName()
    {
        return clsInvName.getText();
    }
    public String getClassDescription()
    {
        return clsDesc.getText();
    }
    public double getRelationStrength()
    {
        return ((Double) expStrength.getValue()).doubleValue();
    }

    public class RelationComboBoxModel extends EntityListModel implements ComboBoxModel
    {
      Object  selectedObject;
      public RelationComboBoxModel(KnowledgeModel km)
      {
        super(km);
      }

      public Object getSelectedItem()
      {
        return selectedObject;
      }

      public void setSelectedItem(Object ob)
      {
        selectedObject = ob;
      }

      protected Entity[] getConceptList()
      {
        Collection col = this.km.getRelationType(CBRModel.RELATION).getAllSubclassesCollection();
        col.add(this.km.getRelationType(CBRModel.RELATION));
        return (Entity[]) col.toArray(new Entity[col.size()]);
      }
  }//EntityComboBoxModel
}


