package jcreek.cke.visualizer;

import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JLabel;
import javax.swing.JTextField;
import java.awt.Color;
import javax.swing.BoxLayout;
import java.awt.BorderLayout;
import javax.swing.ListModel;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.JOptionPane;
import javax.swing.DefaultListModel;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.Frame;
import java.util.Vector;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.document.CreekDocument;
import jcreek.representation.Entity;
import jcreek.cke.command.ActivateEntityCommand;
import jcreek.cke.MainWindow;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.util.QuickSort;
import jcreek.cke.util.DPoint;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.ModelChangeListener;
import jcreek.representation.ModelChangeEvent;
import jcreek.cke.conceptview.*;



/**
 * Title:        JavaCreek
 * Description:  A Case Based Reasoning approach developed in Java
 * Copyright:    Copyright (c) 1999
 * Company:      Phil Systems
 * @author Victor Dias
 * @version
 */

public class EntityList extends JPanel
           implements PropertyChangeListener,
                      ModelChangeListener
{
       JTextField        search;//to type the entity name to be searched
       JPanel findEntity;
       JList             entityList;

        public EntityList()
        {
          init();
        }//EntityList

        public void init()
        {
              setBackground(Color.white);
              setLayout(new BorderLayout());

              search = new JTextField();
              search.addActionListener(new ActionListener ()
              {
                public void actionPerformed(ActionEvent e)
                {
                  searching();
                }
              }); //ActionListener

              search.getDocument().addDocumentListener(new DocumentListener (){
                  public void changedUpdate(DocumentEvent e)
                    {  }
                  public void removeUpdate(DocumentEvent e)
                    {
                        searching();
                    }
                   public void insertUpdate(DocumentEvent e)
                    {
                        searching();
                    }
              });

              findEntity = new JPanel();
              findEntity.setLayout(new BoxLayout(findEntity,BoxLayout.Y_AXIS));
              findEntity.add(new JLabel(" Entity list"));
              findEntity.add(search);

              entityList = new JList();
              entityList.addMouseListener(new MouseAdapter()
              {
                 public void mouseClicked(MouseEvent e)
                 {
                    if(e.getClickCount() == 2)
                    {
                      findEntityButtonPressed();
                    }
                    //rh 30/10
                    //to set text as per selection in the list
                    if(e.getClickCount()==1)
                    {
                      if(!entityList.isSelectionEmpty())

                      search.setText(entityList.getSelectedValue().toString());
                    }
                 }
              }); //mouseAdapter


              add(findEntity,BorderLayout.NORTH);
              add(new JScrollPane(entityList),BorderLayout.CENTER);
              CreekDocumentGroup.instanceOf().addPropertyChangeListener(this);

        }//init

        /**
         * this function takes care of the search in the entityList the list
         * that displays the entities present in a particular model
         **/
        public void searching()
        {
            int k;
            String name=search.getText();
            if(name.length()==0)return;
            try
            {
                for(int i=0;i<entityList.getModel().getSize();i++)
                {
                    k=0;

                    if(name.charAt(k)==entityList.getModel().getElementAt(i).toString().charAt(k))
                    {
                          if(entityList.getModel().getElementAt(i).toString().startsWith(name))
                          {
                            entityList.setSelectedIndex(i);
                            entityList.ensureIndexIsVisible(i);
                            break;
                          }
                    }
                    else
                    {
                        continue;
                    }
                }
              }
              catch(StringIndexOutOfBoundsException ex)
              {
                  ex.printStackTrace();
              }
          }//searching

        public void updateModelView(KnowledgeModel km)
        {
          if(km != null)
            entityList.setModel(new EntityListModel(km));
          else
          {
            search.setText("");
            entityList.clearSelection();
            ((EntityListModel)entityList.getModel()).clear();
            repaint();
          }
        }//updateModelView

        void findEntityButtonPressed()
        {
            if(entityList.getModel().getSize()==0)
                return;
            CreekDocument doc = CreekDocumentGroup.instanceOf().getCurrentDocument();
            Frame win = Visualizer.getFrame(this);
            if ( doc == null ) {
                noCurrentDocument( win );
                return;
            }
            Entity ent = (Entity) ((EntityListModel)entityList.getModel()).getConceptAt(entityList.getSelectedIndex());
            DPoint tempPoint = doc.getView().activateEntity(null);
            VisualEntity ve = new VisualEntity(ent,(int)tempPoint.x,(int)tempPoint.y);
            ActivateEntityCommand action = new ActivateEntityCommand( doc, ve );
            if ( doc.performCommand( action ) == false ) {
                // If vertex add failed then complain to user.
                JOptionPane.showMessageDialog
                    ( win,
                      MainWindow.instanceOf().getResourceString( "nodeAddMsg" ),
                      MainWindow.instanceOf().getResourceString( "userErrorTitle" ),
                      JOptionPane.ERROR_MESSAGE );
          }
        }//findEntityButtonPressed


          /**
           * Displays a dialog indicating that there's no current
           * document to perform the desired operation.
           *
           * @param  win  frame to attach dialog to
           */
          public void noCurrentDocument( Frame win ) {
              JOptionPane.showMessageDialog
                  ( win,
                    MainWindow.instanceOf().getResourceString( "noCurDocMsg" ),
                    MainWindow.instanceOf().getResourceString( "usrErrorTitle" ),
                    JOptionPane.ERROR_MESSAGE );
          } // noCurrentDocument

          public void propertyChange( PropertyChangeEvent e )
          {
              if ( e.getPropertyName().equals( "opened" ))
              {
                  if(e.getNewValue() !=null)
                  {
                    ((KnowledgeModel)e.getNewValue()).addModelChangeListener(this);
                    updateModelView((KnowledgeModel)e.getNewValue());
                  }
                  else
                    updateModelView(null);
              }
              else if(e.getPropertyName().equals( "closed" ))
              {
                    if(e.getOldValue()!=null)
                      ((KnowledgeModel)e.getOldValue()).removeModelChangeListener(this);
                    updateModelView(null);
              }
          }//propertyChange

        public void modelChanged(ModelChangeEvent e)
        {
          if(e.getType() == ModelChangeEvent.ENTITY_ADDED ||
             e.getType() == ModelChangeEvent.ENTITY_REMOVED)
            updateModelView(CreekDocumentGroup.instanceOf().getKM());
        }
}//EntityList

