package jcreek.cke.visualizer;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.net.URL;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import jcreek.cke.browser.Visualizer;
import jcreek.cke.MainWindow;
import jcreek.representation.Entity;
import jcreek.representation.NameAlreadyExistException;



public class EntityProperties extends JPanel implements ActionListener{

    // Setting up entityView panel

  protected JComboBox relationClassList = new JComboBox() ;
  protected JComboBox entityClassList   = new JComboBox();
  protected JLabel            frameLabel = new JLabel("Frame View");
  protected JLabel            relationLabel = new JLabel("Relations");
  protected JTextField        entityNameText = new JTextField("<No active entity>");
  protected JTextArea         entityDescriptionText = new JTextArea("<No active entity>");
  protected EntityTableModel  entityTableModel = new EntityTableModel(frameLabel, entityNameText,
            entityDescriptionText,relationClassList,entityClassList);
  protected JPanel            buildView = new JPanel(new BorderLayout());
  protected JPanel            findEnt = new JPanel(new BorderLayout());
  protected JTable            entityTable;
  protected HeaderPanel headerPanel;


  public EntityProperties() {
        setLayout(new BorderLayout());
        JPanel    entityInfoPanel = new JPanel(new GridLayout(3,0));
        entityInfoPanel.setBackground(Color.white);
        JPanel    entityInfoPanel2 = new JPanel(new BorderLayout());
        entityInfoPanel2.setBackground(Color.white);
        JPanel    entityInfoPanel3 = new JPanel(new BorderLayout());

        JLabel  nameTag = new JLabel(
          MainWindow.instanceOf().getResourceString("nameProp"+Visualizer.labelSuffix));
        JLabel  descTag = new JLabel(
          MainWindow.instanceOf().getResourceString("descProp"+Visualizer.labelSuffix));
        nameTag.setForeground(Color.black);
        descTag.setForeground(Color.black);
        entityInfoPanel.add(nameTag);
        entityInfoPanel.add(entityNameText);
        entityInfoPanel.add(descTag);

        entityInfoPanel2.add(entityInfoPanel, BorderLayout.NORTH);

        JScrollPane entityDescriptionPane = new JScrollPane(entityDescriptionText, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        entityDescriptionPane.setPreferredSize(new Dimension(100,60));
        entityDescriptionPane.setBackground(Color.white);

        entityInfoPanel2.add(entityDescriptionPane, BorderLayout.CENTER);

        headerPanel = new HeaderPanel();
        headerPanel.addComponent(relationLabel,createActionButton("newRelation"));

        entityInfoPanel2.add(headerPanel,BorderLayout.SOUTH);

        entityNameText.setEditable(false);
        entityDescriptionText.setEditable(false);



        findEnt.setBackground(Color.white);
        setBackground(Color.white);

        entityTable = new JTable(entityTableModel)
        {
          EntityTableCellRenderer myRenderer = new EntityTableCellRenderer();

          public TableCellRenderer getCellRenderer(int column, int row)
          {
            return myRenderer;
         }
        };
        entityTable.setPreferredScrollableViewportSize(new Dimension(100,80));


        entityTable.getTableHeader().setReorderingAllowed(false);

        entityInfoPanel3.add(entityInfoPanel2, BorderLayout.NORTH);
        entityInfoPanel3.add(new JScrollPane(entityTable), BorderLayout.CENTER);
        entityInfoPanel3.setBackground(Color.white);

        entityInfoPanel3.setBorder(javax.swing.border.LineBorder.createGrayLineBorder());

        headerPanel = new HeaderPanel();

        headerPanel.addComponent(frameLabel,createActionButton("entityProperty"));
        add(headerPanel, BorderLayout.NORTH);
        add(entityInfoPanel3, BorderLayout.CENTER);

  }


  protected void setUpRelationColumn(TableColumn relationColumn)
  {
        DefaultCellEditor cellEditor = new DefaultCellEditor(relationClassList);
        cellEditor.setClickCountToStart(3);
        relationColumn.setCellEditor(cellEditor);

  }

  protected void setUpStrengthColumn(TableColumn relationColumn)
  {
    relationColumn.setCellEditor(new DefaultCellEditor(new JTextField())
    {
      public Component getTableCellEditorComponent(JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   int row,
                                                   int column)
      {
        return new JSpinner(new SpinnerNumberModel(0.1,0.0,1.0,0.1));
      }

    });
  }
  protected void setUpEntityColumn(TableColumn entityColumn)
  {

        DefaultCellEditor cellEditor = new DefaultCellEditor(entityClassList);
        cellEditor.setClickCountToStart(3);
        entityColumn.setCellEditor(cellEditor);

  }

  public boolean updateEntityName()
  {

      //added 02/01/02
      if(entityNameText.getText().length()==0)
      {
        entityNameText.setText(entityTableModel.getCurrentEntity().getName());
        JOptionPane.showMessageDialog(
                        this,
                        MainWindow.instanceOf().getResourceString("blankNameMsg"),
                        MainWindow.instanceOf().getResourceString("invalidName"+Visualizer.titleSuffix),
                        JOptionPane.ERROR_MESSAGE);
        return false;
      }
      if(entityNameText.getText().startsWith(" "))
      {
        entityNameText.setText(entityTableModel.getCurrentEntity().getName());
        JOptionPane.showMessageDialog(
                        this,
                        MainWindow.instanceOf().getResourceString("invalidNameMsg"),
                        MainWindow.instanceOf().getResourceString("invalidName"+Visualizer.titleSuffix),
                        JOptionPane.ERROR_MESSAGE);
        return false;
      }
      if((entityTableModel.getCurrentEntity() != null) &&
        !(entityNameText.getText().equals(entityTableModel.getCurrentEntity().getName())))
      {
        try
        {
          entityTableModel.getCurrentEntity().setName(entityNameText.getText());
        }
        catch(NameAlreadyExistException e)
        {
          JOptionPane.showMessageDialog(
                        this,
                        MainWindow.instanceOf().getResourceString("duplicateNameMsg"),
                        MainWindow.instanceOf().getResourceString("invalidName"+Visualizer.titleSuffix),
                        JOptionPane.ERROR_MESSAGE);
/*
          if(!entityNameText.hasFocus())
            entityNameText.requestFocus();
*/
          return false;
        }
      }
/*
      entityNameText.setSelectionStart(0);
      entityNameText.setSelectionEnd(entityNameText.getText().length());
*/
      return true;
  }//updateEntityName


  /**
   * Method used to update the entity description
   *
   * rh 11/09
   */
  protected void updateEntityDescription()
  {
    if((entityTableModel.getCurrentEntity() != null) &&
      (!entityNameText.getText().equals(entityTableModel.getCurrentEntity().getDescription())))
    {
        entityTableModel.getCurrentEntity().setDescription(entityDescriptionText.getText());
    }
  }//updateEntityDescription


  public JTextField getEntityNameText()
  {
      return entityNameText;
  }

  public JTextArea getEntityDescriptionText()
  {
      return entityDescriptionText;
  }

  public JTable getEntityTable()
  {
      return entityTable;
  }

  public EntityTableModel getEntityTableModel()
  {
      return entityTableModel;
  }

  protected JButton createActionButton(String key)
  {
        JButton button;
        URL url = MainWindow.instanceOf().getResource( key +
           Visualizer.imageSuffix );
        if ( url != null ) {
          button = new JButton(new ImageIcon(url));
        }
        else
        button = new JButton(
          MainWindow.instanceOf().getResourceString( key +
                 Visualizer.labelSuffix ));

        button.setActionCommand(key);
/*
        Action a = MainWindow.instanceOf().getAction( key );
        if ( a != null ) {
            // set up the action to listen for events
            button.addActionListener( a );
            button.setEnabled( a.isEnabled() );
            a.addPropertyChangeListener(
                new StateChangedListener(button));
        }
         else {
            button.setEnabled( false );
        }
*/
        button.addActionListener(this);
        return button;
  }

  public void actionPerformed(ActionEvent event) {
      Visualizer win = Visualizer.getFrame(this);
      Entity selEntity = entityTableModel.getEntity();
      if(event.getActionCommand().equals("newRelation"))
      {
          if(selEntity!=null)
          {
              NewRelationDialog newRel = new NewRelationDialog(win,
                        "New Relation For "+selEntity.getName(),
                        selEntity);
          }

      }
      else if(event.getActionCommand().equals("entityProperty"))
      {
          if(selEntity!=null)
              win.displayPropertyWindow(selEntity);
      }
  }
  protected class StateChangedListener implements PropertyChangeListener
  {
        JComponent component;


        /**
         * Constructor for our state change listener.
         *
         * @param  b  button we are to associate with
         */
        public StateChangedListener( JButton b ) {
            super();
            component = (JComponent)b;
        } // StateChangedListener

        public void propertyChange( PropertyChangeEvent e ) {
            String propertyName = e.getPropertyName();
            if ( propertyName.equals( "enabled" ) ) {
                Boolean enabledState = (Boolean)e.getNewValue();
                component.setEnabled( enabledState.booleanValue() );
            }
        } // propertyChange
  }

}
class HeaderPanel extends JPanel
{
	protected GridBagLayout gridBag;

        public HeaderPanel()
        {
            setLayout(gridBag=new GridBagLayout());
        }

	public void addComponent(Component comp1, Component comp2)
	{
		GridBagConstraints cons = new GridBagConstraints();
                cons.anchor = GridBagConstraints.WEST;
                cons.fill = GridBagConstraints.BOTH;
                cons.weightx = 1.0f;

                gridBag.setConstraints(comp1,cons);
                add(comp1);

                cons.anchor = GridBagConstraints.EAST;
                cons.fill = GridBagConstraints.NONE;
                cons.weightx = 0.0f;

		gridBag.setConstraints(comp2,cons);
		add(comp2);

        }
}



