package jcreek.cke.visualizer;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.awt.*;

import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;
import jcreek.representation.KnowledgeModel;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.command.ChangeRelationCommand;


public class EntityTableModel extends AbstractTableModel
{
  private Entity currentEntity;
  private Relation[] currentRelations;
  private JLabel frameLabel;
  private JTextField entityNameText;
  private JTextArea entityDescriptionText;
  private JComboBox relationClassList;
  private JComboBox entityClassList;
  private boolean suspendChanges = false;

  EntityTableModel(JLabel frameLabel, JTextField entityNameText
  , JTextArea entityDescriptionText,JComboBox relationClassList,JComboBox entityClassList)
  {
    this.frameLabel = frameLabel;
    this.entityNameText = entityNameText;
    this.entityDescriptionText = entityDescriptionText;
    this.relationClassList = relationClassList;
    this.entityClassList = entityClassList;
    setEntity(null);
  }

  public void setEntity(Entity e)
  {
    if(suspendChanges)
        return;
    this.currentEntity = e;
    if(e != null)
    {
      currentRelations = e.getRelations();
      frameLabel.setText(" Frame view of '"+e.getName()+"'");
      entityNameText.setText(e.getName());
      entityDescriptionText.setText(e.getDescription());
      entityNameText.setEditable(true);
      entityDescriptionText.setEditable(true);
      entityDescriptionText.setBackground(entityNameText.getBackground());

      relationClassList.setModel(new EntityComboBoxModel(e.getKnowledgeModel()));
      entityClassList.setModel(new RelationComboBoxModel(e.getKnowledgeModel()));

    }
    else
    {
      currentRelations = new Relation[0];
      frameLabel.setText(" Frame view");
      entityNameText.setText("<No active entity>");
      entityDescriptionText.setText("<No active entity>");
      entityNameText.setEditable(false);
      entityDescriptionText.setEditable(false);
      entityDescriptionText.setBackground(entityNameText.getBackground());
//      relationClassList.removeAll();
    }
    fireTableDataChanged();
  }

  public Entity getEntity()
  {
      return currentEntity;
  }

  public int getRowCount()
  {
    return currentRelations == null?0:currentRelations.length;
  }

  public int getColumnCount()
  {
    return 3;
  }

  public Object getValueAt(int row, int column)
  {
    if(currentEntity == null || currentRelations == null || currentRelations.length == 0)
      return " ";
    if(currentEntity == currentRelations[row].getFrom())
    {
      if(column == 0)
        return currentRelations[row].getName();
      else if(column == 1)
        return currentRelations[row].getValue().getValueString();
      else
        return String.valueOf(currentRelations[row].getExplanationStrength());
    }
    else
    {
      if(column == 0)
        return currentRelations[row].getInverseName();
      else if(column == 1)
        return currentRelations[row].getFrom().getValueString();
      else
        return String.valueOf(currentRelations[row].getExplanationStrength());
    }
  }

  public void setValueAt(Object value,int row, int column)
  {
      if(row>=currentRelations.length)
      {
          return;
      }
      suspendChanges = true;
      KnowledgeModel km = CreekDocumentGroup.instanceOf().getKM();
      Relation remove = getRelationAt(row);
      Entity tEntity = (Entity)remove.getValue();
      if(column == 0)
      {
          if((String)getValueAt(row,column)== (String)value)
          {
              suspendChanges=false;
              return;
          }
          if(relationClassList.getSelectedIndex()!=-1)
          {
                EntityComboBoxModel rcdModel = (EntityComboBoxModel) relationClassList.getModel();
                RelationType rcd = (RelationType) rcdModel.getConcept((String)value);
                Relation newRel = rcd.newInstance(currentEntity,tEntity);
                CreekDocument doc = CreekDocumentGroup.instanceOf().getCurrentDocument();
                if(doc!=null)
                {
                  ChangeRelationCommand chgRel = new ChangeRelationCommand(
                          doc,
                          currentRelations[row],
                          newRel);
                  doc.performCommand(chgRel);
                }
          }
          suspendChanges = false;
          remove.remove();

      }
      else if(column==1)
      {
          if((String)getValueAt(row,column)== (String)value)
          {
              suspendChanges=false;
              return;
          }
          if(entityClassList.getSelectedIndex()!=-1)
          {
                Entity ent = (Entity) ((EntityListModel)entityClassList.getModel()).getConceptAt(entityClassList.getSelectedIndex());
                EntityComboBoxModel rcdModel = (EntityComboBoxModel) relationClassList.getModel();
                RelationType rcd = (RelationType) rcdModel.getConcept((String)getValueAt(row,0));
                Relation newRel = rcd.newInstance(currentEntity,ent);
                CreekDocument doc = CreekDocumentGroup.instanceOf().getCurrentDocument();
                if(doc!=null)
                {
                  ChangeRelationCommand chgRel = new ChangeRelationCommand(
                          doc,
                          currentRelations[row],
                          newRel);
                  doc.performCommand(chgRel);
                }
          }
          suspendChanges = false;
          remove.remove();
      }
      else if(column==2)
      {
          suspendChanges = false;
          remove.setExplanationStrength(((Double)value).doubleValue());
      }
  }


  public Relation getRelationAt(int row)
  {
    if(row>=currentRelations.length)
        return null;
    else
        return currentRelations[row];
  }

  public Entity getCurrentEntity()
  {
    return currentEntity;
  }

  public String getColumnName(int column)
  {
    if(column == 0)
      return "Relation";
    else if (column == 1)
      return "Value";
    else
      return "Strength";
  }

  public boolean isCellEditable(int row,int column)
  {
          return true;
  }


  public Class getColumnClass(int column) {

      if (column == 2) {
          return Number.class;
      }

      return Object.class;
  }




}//EntityTableModel
