package jcreek.cke.visualizer;

import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.io.File;

import jcreek.cke.MainWindow;
import jcreek.cke.browser.Visualizer;
import jcreek.cke.command.CreekFileFilter;
import jcreek.cke.CreekDocumentGroup;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.LocalKnowledgeModel;

public class ModelPropertyDlg extends JDialog implements ActionListener
{


  private JButton okButton = new JButton();
  private JButton cancelButton = new JButton();
  private JTextField authorText = new JTextField();
  private JTextArea descText = new JTextArea();
  private JTextField nameText = new JTextField();
  private JTextField fileText = new JTextField();
  private JButton filebutt = new JButton();

  private JComboBox modelType = new JComboBox(new String[] { "Basic Model (no reasoning)", "Simple CBR Model", "Creek IsoPod Model" });

  public static int EDIT_MODE = 0;
  public static int NEW_MODE = 1;

  private int mode = 0;

  protected File file;
  protected boolean isOk = false;


  public ModelPropertyDlg(Frame owner,String title,int mode) {
    super(owner,title,true);
    this.mode = mode;
    enableEvents(AWTEvent.WINDOW_EVENT_MASK);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  private void jbInit() throws Exception {

    MainWindow mw = MainWindow.instanceOf();

    JPanel content = new JPanel(new BorderLayout());
    content.setBorder(new EmptyBorder(12,12,12,12));
    setContentPane(content);

    ControlPanel propPane = new ControlPanel();



    propPane.addComponent(mw.getResourceString("modelPropNameLabel"),
                          nameText);

    if(mode == NEW_MODE)
    {
      modelType.setEditable(false);
      propPane.addComponent(mw.getResourceString("modelPropTypeLabel"),
                            modelType);
    }

    descText.setLineWrap(true);
    descText.setWrapStyleWord(true);
    descText.setPreferredSize(new Dimension(150,100));
    JScrollPane sPane = new JScrollPane(descText,
                                        JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                        JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);

    propPane.addComponent(mw.getResourceString("modelPropDescLabel"),
                          sPane);

    propPane.addComponent(mw.getResourceString("modelPropAuthorLabel"),
                          authorText);

    JPanel chooser =new JPanel();
    chooser.setLayout(new BoxLayout(chooser,BoxLayout.X_AXIS));
    chooser.setBorder(new EmptyBorder(12,0,0,0));

    filebutt.setPreferredSize(new Dimension(19,24));
    filebutt.addActionListener(this);
    filebutt.setText("::");
    fileText.setEditable(false);
    chooser.add(fileText);
    chooser.add(filebutt);

    propPane.addComponent(mw.getResourceString("modelPropFileLabel"),
                          chooser);

    content.add(BorderLayout.NORTH,propPane);


    JPanel buttons = new JPanel();
    buttons.setLayout(new BoxLayout(buttons,BoxLayout.X_AXIS));
    buttons.setBorder(new EmptyBorder(12,0,0,0));
    buttons.add(Box.createGlue());
    okButton.setText(mw.getResourceString("okLabel"));
    okButton.addActionListener( this );
    buttons.add(okButton);
    buttons.add(Box.createHorizontalStrut(6));
    cancelButton.setText(mw.getResourceString("cancelLabel"));
    cancelButton.addActionListener( this );
    buttons.add(cancelButton);
    buttons.add(Box.createGlue());

    getRootPane().setDefaultButton(cancelButton);

    content.add(BorderLayout.SOUTH,buttons);

    if(mode == EDIT_MODE)
        initFields();

    pack();
    setLocationRelativeTo(getOwner());
    show();

  }

  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      cancel();
    }
    super.processWindowEvent(e);
  }

  void cancel() {
    hide();
    dispose();
  }
  public boolean isOK()
  {
      return isOk;
  }

    public void actionPerformed( ActionEvent e )
   {
        String command = e.getActionCommand();
        if ( command.equals(MainWindow.instanceOf().getResourceString("okLabel")))
        {
            if((fileText.getText()!=null)&&(fileText.getText().length()>0))
            {
                isOk = true;
                dispose();
            }
        }
        else if(command.equals(MainWindow.instanceOf().getResourceString("cancelLabel")))
        {
          isOk = false;
          dispose();
        }
        else
        {
            if(mode == NEW_MODE)
                displayFileDialog();
        }
    }

    public boolean displayFileDialog()
    {
        boolean okay = true;
        Visualizer win = (Visualizer)getOwner();
        MainWindow mw = MainWindow.instanceOf();
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();

        JFileChooser chooser = new JFileChooser(AppSettings.instanceOf().getCurrentDirectory());
        chooser.setFileFilter( new CreekFileFilter( dg.getKMTypes() ) );

        if ( chooser.showSaveDialog( win ) == JFileChooser.CANCEL_OPTION ) {
          // user hit cancel
          return false;
        }
        file = chooser.getSelectedFile();
        AppSettings.instanceOf().setCurrentDirectory(file.getAbsolutePath());
        fileText.setText(file.getAbsolutePath());

        return true;
    }

    public void initFields()
    {
        KnowledgeModel km = CreekDocumentGroup.instanceOf().getKM();
        if(km!=null)
        {
              nameText.setText(km.getName());
              descText.setText(km.getDescription());
              authorText.setText(km.getAuthor());
              File kmFile = ((LocalKnowledgeModel) km).getFile();
              if(kmFile != null)
                fileText.setText(kmFile.getAbsolutePath());
        }
    }

    public int getModelType()
    {
      if(modelType.getSelectedItem() != null)
        return modelType.getSelectedIndex();
      else
        return 0;
    }

    public File getFile()
    {
      return file;
    }

    public String getDesc()
    {
        return descText.getText();
    }

    public String getName()
    {
        return nameText.getText();
    }

    public String getAuthor()
    {
        return authorText.getText();
    }


}
class ControlPanel extends JPanel
{
	/**
	 * The layout manager.
	 */
	protected GridBagLayout gridBag;

	/**
	 * The number of components already added to the layout manager.
	 */
	protected int y;

        public ControlPanel()
        {
            setLayout(gridBag=new GridBagLayout());
        }

	/**
	 * Adds a labeled component to the option pane. Components are
	 * added in a vertical fashion, one per row. The label is
	 * displayed to the left of the component.
	 * @param label The label
	 * @param comp The component
	 */
	public void addComponent(String label, Component comp)
	{
		JLabel l = new JLabel(label);
		l.setBorder(new EmptyBorder(0,0,0,12));
		addComponent(l,comp,GridBagConstraints.BOTH);
	}

	/**
	 * Adds a labeled component to the option pane. Components are
	 * added in a vertical fashion, one per row. The label is
	 * displayed to the left of the component.
	 * @param label The label
	 * @param comp The component
	 * @param fill Fill parameter to GridBagConstraints for the right
	 * component
	 */
	public void addComponent(String label, Component comp, int fill)
	{
		JLabel l = new JLabel(label);
		l.setBorder(new EmptyBorder(0,0,0,12));
		addComponent(l,comp,fill);
	}

	/**
	 * Adds a labeled component to the option pane. Components are
	 * added in a vertical fashion, one per row. The label is
	 * displayed to the left of the component.
	 * @param comp1 The label
	 * @param comp2 The component
	 *
	 */
	public void addComponent(Component comp1, Component comp2)
	{
		addComponent(comp1,comp2,GridBagConstraints.BOTH);
	}

	/**
	 * Adds a labeled component to the option pane. Components are
	 * added in a vertical fashion, one per row. The label is
	 * displayed to the left of the component.
	 * @param comp1 The label
	 * @param comp2 The component
	 * @param fill Fill parameter to GridBagConstraints for the right
	 * component
	 *
	 */
	public void addComponent(Component comp1, Component comp2, int fill)
	{
		GridBagConstraints cons = new GridBagConstraints();
		cons.gridy = y++;
		cons.gridheight = 1;
		cons.gridwidth = 1;
		cons.weightx = 0.0f;
		cons.insets = new Insets(1,0,1,0);
		cons.fill = GridBagConstraints.BOTH;

		gridBag.setConstraints(comp1,cons);
		add(comp1);

		cons.fill = fill;
		cons.gridx = 1;
		cons.weightx = 1.0f;
		gridBag.setConstraints(comp2,cons);
		add(comp2);
	}

}



