package jcreek.cke.visualizer;

import javax.swing.*;
import javax.swing.border.EmptyBorder;

import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;

import jcreek.cke.MainWindow;
import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.representation.KnowledgeModel;
import jcreek.cke.controls.*;
import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:
 * @author
 * @version 1.0
 */

public class NewRelationDialog extends JDialog implements ActionListener {

  private JButton okButton      = new JButton();
  private JButton cancelButton  = new JButton();

  protected JComboBox relationClassList = new JComboBox() ;
  protected JComboBox entityClassList   = new JComboBox();

  protected JSpinner relStrength = new JSpinner(new SpinnerNumberModel(0.5,0.0,1.0,0.1));

  protected boolean isOk = false;

  protected Entity selEntity;

  public NewRelationDialog(Frame owner,String title,Entity selEntity)
  {
      super(owner,title,true);
      this.selEntity = selEntity;
      enableEvents(AWTEvent.WINDOW_EVENT_MASK);
      try {
        jbInit();
      }
      catch(Exception e) {
        e.printStackTrace();
      }
  }

  private void jbInit() throws Exception {

    MainWindow mw = MainWindow.instanceOf();
    JPanel content = new JPanel(new BorderLayout());
    content.setBorder(new EmptyBorder(12,12,12,12));
    setContentPane(content);

    JPanel buttons = new JPanel();
    buttons.setLayout(new BoxLayout(buttons,BoxLayout.X_AXIS));
    buttons.setBorder(new EmptyBorder(12,0,0,0));
    buttons.add(Box.createGlue());
    okButton.setText(mw.getResourceString("okLabel"));
    okButton.addActionListener( this );
    buttons.add(okButton);
    buttons.add(Box.createHorizontalStrut(6));
    cancelButton.setText(mw.getResourceString("cancelLabel"));
    cancelButton.addActionListener( this );
    buttons.add(cancelButton);
    buttons.add(Box.createGlue());

    getRootPane().setDefaultButton(cancelButton);

    content.add(BorderLayout.SOUTH,buttons);

    JPanel relPanel = new JPanel();
    if(selEntity!=null)
    {
        relationClassList.setModel(new EntityComboBoxModel(selEntity.getKnowledgeModel()));
        entityClassList.setModel(new RelationComboBoxModel(selEntity.getKnowledgeModel()));
    }
    relPanel.add(relationClassList);
    relPanel.add(entityClassList);
    relPanel.add(relStrength);

    content.add(BorderLayout.CENTER,relPanel);

    pack();
    setLocationRelativeTo(getOwner());
    show();

  }

  protected void processWindowEvent(WindowEvent e) {
    if (e.getID() == WindowEvent.WINDOW_CLOSING) {
      cancel();
    }
    super.processWindowEvent(e);
  }

    void cancel() {
      hide();
      dispose();
    }

    public boolean isOK()
    {
        return isOk;
    }

    public void actionPerformed( ActionEvent e )
   {
        String command = e.getActionCommand();
        if ( command.equals(MainWindow.instanceOf().getResourceString("okLabel")))
        {
            RelationType rcd = getSelectedRelation();
              if ( rcd != null )
              {
                  Relation newRel;
                  Relation[] relations  = selEntity.getRelations(rcd);
                  Entity value = getSelectedValue();
                  for(int i=0;i<relations.length;i++)
                  {
                      if(relations[i].getValue().equals(value))
                          return;
                  }
                  if(value!=null)
                  {
                    newRel = rcd.newInstance(selEntity, value);
                    newRel.setExplanationStrength(((Double) relStrength.getValue()).doubleValue());
                  }
              }
              isOk = true;
              dispose();
        }
        else if(command.equals(MainWindow.instanceOf().getResourceString("cancelLabel")))
        {
              isOk = false;
              dispose();
        }
    }

    public RelationType getSelectedRelation()
    {
      RelationType rcd = null;
      if(relationClassList.getSelectedIndex()!=-1)
      {
            EntityComboBoxModel rcdModel = (EntityComboBoxModel) relationClassList.getModel();
            rcd = (RelationType) rcdModel.getConceptAt(relationClassList.getSelectedIndex());
      }
      return rcd;
    }//getSelectedRelation

    public Entity getSelectedValue()
    {
        Entity entity = null;
        if(entityClassList.getSelectedIndex()!=-1)
        {
            RelationComboBoxModel entityModel = (RelationComboBoxModel) entityClassList.getModel();
            entity = (Entity) entityModel.getConceptAt(entityClassList.getSelectedIndex());

        }
      return entity;
    }//getSelectedValue

}
