package jcreek.cke.visualizer;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import java.util.Collection;
import jcreek.representation.RelationType;
import jcreek.representation.Entity;
import jcreek.representation.KnowledgeModel;
import jcreek.cke.browser.Visualizer;
import jcreek.representation.Relation;
import jcreek.representation.cbr.*;
import jcreek.cke.CreekDocumentGroup;

import jcreek.cke.util.QuickSort;
import jcreek.cke.MainWindow;

public class PreferencesDialog extends JDialog implements ActionListener
{

    protected AppSettings props;
    protected OverviewPanel overPanel;
    protected ViewTabbedPanel viewPanel;
    protected Dimension vstrut10 = new Dimension( 0, 10 );
    protected Dimension hstrut25 = new Dimension( 25, 0 );


    RelationType sRel;
//    ClassDescriptor conceptClass;
    Entity concepts[];

    public static String [] relation;

    QuickSort quick = new QuickSort() {
      public boolean compare(Object ob1, Object ob2)
      {
        return ((Entity) ob1).getName().compareTo(((Entity) ob2).getName()) < 0;
      }
    };


    public PreferencesDialog( Frame owner ,KnowledgeModel kmm)
    {
        super( owner,"Preferences",true);
        props = AppSettings.instanceOf();

        MainWindow mw = MainWindow.instanceOf();

        //rh
        /*
        gets all the relation names
        */

        concepts = getConceptList(kmm);
        quick.sort(concepts);

        //sRel=kmm.getRelationType("Relation");
        //cDesc=sRel.getSubclasses();

        relation=new String[concepts.length];

        for(int i=0;i<concepts.length;i++)
          relation[i]=concepts[i].getName();

        Container pane = getContentPane();
        JTabbedPane tabbedPane = new JTabbedPane();
        pane.add( "North", tabbedPane );

        overPanel = new OverviewPanel( props );
        tabbedPane.addTab("View",overPanel);

        viewPanel = new ViewTabbedPanel( props,kmm );
        tabbedPane.addTab("OverView",viewPanel );

        // add space between tabbed pane and buttons
        pane.add( Box.createRigidArea( vstrut10 ), "Center" );

        // Create box for ok and cancel buttons
        Box buttonBox = Box.createHorizontalBox();
        pane.add( buttonBox, "South" );
        buttonBox.add( Box.createHorizontalGlue() );

        // OK button
        JButton button = new JButton(mw.getResourceString("okLabel"));
        button.addActionListener( this );
        buttonBox.add( button );

        // createRigidArea is more object effient than other Box methods
        buttonBox.add( Box.createRigidArea(hstrut25));

        // Cancel button
        button = new JButton(mw.getResourceString("cancelLabel"));
        button.addActionListener( this );
        buttonBox.add( button );

        getRootPane().setDefaultButton(button);

        // createRigidArea is more object efficient than other Box methods
        buttonBox.add( Box.createRigidArea( hstrut25 ) );

        setDefaultCloseOperation( WindowConstants.DISPOSE_ON_CLOSE );
        pack();
        //owner.centerComponent( this );
    }

    public void actionPerformed( ActionEvent e )
    {
        String command = e.getActionCommand();
        if ( command.equals(
          MainWindow.instanceOf().getResourceString("okLabel"))) {
	    // grab the settings and save them
            overPanel.setOptions( props );
            viewPanel.setOptions( props );
            props.commit();
        }
        dispose();
    }

    public PreferencesDialog()
    {
      try
      {
        jbInit();
      }
      catch(Exception e)
      {
        e.printStackTrace();
      }
    }

    private void jbInit() throws Exception
    {
        this.setForeground(Color.red);
    }

    protected Entity[] getConceptList(KnowledgeModel km)
    {
      Collection col = km.getEntity(CBRModel.RELATION).getAllSubclassesCollection();
      col.add(km.getEntity(CBRModel.RELATION));
      return (Entity[]) col.toArray(new Entity[col.size()]);
    }

}

class OverviewPanel extends JPanel
{
    protected JCheckBox relationName;
    //rh
    JPanel colorTab;
    Color selectedColor,relColor;
    JButton colorChooser;
    JComboBox colorList;
    JLabel name=new JLabel("Relation");
    JColorChooser choose=new JColorChooser();

    //rh
    int [] color=new int[PreferencesDialog.relation.length];
    public OverviewPanel( AppSettings props )
    {
        super();

        setLayout(new BorderLayout());
        relationName = new JCheckBox("Show Relation Name");
        add( relationName,BorderLayout.NORTH);
        relationName.setSelected( props.getBoolean( AppSettings.RELATION_NAME ));

        //rh 12/10/
        colorList=new JComboBox(PreferencesDialog.relation);

        colorList.addActionListener(new java.awt.event.ActionListener()  {
          public void actionPerformed(ActionEvent e) {
            colorListactionPerformed(e);
          }
        });

        colorChooser=new JButton();
        for(int i=0;i<PreferencesDialog.relation.length;i++)
          {
            Color relColor =new Color(AppSettings.instanceOf().getRelation(PreferencesDialog.relation[i]));
            color[i]=relColor.getRGB();
          }

        colorChooser.setBackground(new Color(color[colorList.getSelectedIndex()]));

        colorChooser.addActionListener(new java.awt.event.ActionListener() {
          public void actionPerformed(ActionEvent e) {
            colorChooseractionPerformed(e);
          }
        });

        colorTab = new JPanel();
        colorTab.setBorder( new TitledBorder
                         ( new EtchedBorder(),"Color Panel"));
        colorTab.add(name);
        colorTab.add(colorList);
        colorTab.add(colorChooser);
        add( colorTab,BorderLayout.CENTER);
     }

    public void colorChooseractionPerformed(ActionEvent e)
    {
        selectedColor = JColorChooser.showDialog(
                      colorTab,
                      "Choose Color",
                      colorChooser.getBackground());
        colorChooser.setBackground(selectedColor);
        color[colorList.getSelectedIndex()]=selectedColor.getRGB();
    }

    public void colorListactionPerformed(ActionEvent e)
    {
       colorChooser.setBackground(new Color(color[colorList.getSelectedIndex()]));
    }


    public void setOptions( AppSettings props )
    {
      // save new settings
        props.setBoolean( AppSettings.RELATION_NAME,relationName.isSelected() );
        for(int i=0;i<PreferencesDialog.relation.length;i++)
          {
            props.setRelation(PreferencesDialog.relation[i],color[i]);
          }
    }
}

class ViewTabbedPanel extends JPanel
{
    protected JRadioButton horizontalAllign;
    protected JRadioButton verticalAllign;
    protected JCheckBox isSubClassDisplay;
    protected JCheckBox isInstanceDisplay;

    RelationType rcd;
//    ClassDescriptor cD[];
    protected String str;


    public ViewTabbedPanel( AppSettings props,KnowledgeModel kmm )
    {
        super();
        setLayout(new BorderLayout());
        // Create the relationship label radio button group.
        ButtonGroup bg = new ButtonGroup();
        JPanel allignPanel = new JPanel();
        allignPanel.setBorder( new TitledBorder
                         ( new EtchedBorder(),"Allignment") );
//        gb.setConstraints( panel, gc );
        add( allignPanel,BorderLayout.NORTH );

        // horizontal allignment radio button
        horizontalAllign = new JRadioButton("Horizontal Allignment" );
        bg.add( horizontalAllign );
        allignPanel.add( horizontalAllign );
        horizontalAllign.setSelected( props.getBoolean( AppSettings.HORIZONTAL_ALLIGNMENT ) );

        // vertical allignment radio button
        verticalAllign = new JRadioButton("Vertical Allignment");
        bg.add( verticalAllign );
        allignPanel.add( verticalAllign );
        verticalAllign.setSelected( props.getBoolean( AppSettings.VERTICAL_ALLIGNMENT) );


        JPanel relPanel = new JPanel();
        relPanel.setBorder( new TitledBorder
                         ( new EtchedBorder(),"Relationships") );
        add( relPanel,BorderLayout.CENTER );

        //changed 28/11 rh
        rcd=kmm.getRelationType(CBRModel.SUBCLASS_OF);
        str=rcd.getName();

        // isSubclass checkbox
        isSubClassDisplay = new JCheckBox(str);
        relPanel.add( isSubClassDisplay );
        isSubClassDisplay.setSelected( props.getBoolean( AppSettings.ISSUBCLASS ) );
        //changed 28/11 rh
        rcd=kmm.getRelationType(CBRModel.INSTANCE_OF);
        str=rcd.getName();
        // isInstance checkbox
        isInstanceDisplay = new JCheckBox(str);
        relPanel.add( isInstanceDisplay );
        isInstanceDisplay.setSelected( props.getBoolean( AppSettings.ISINSTANCE) );
    }

    public void setOptions( AppSettings props )
    {
        props.setBoolean( AppSettings.ISSUBCLASS,isSubClassDisplay.isSelected() );
        props.setBoolean( AppSettings.ISINSTANCE,isInstanceDisplay.isSelected() );
        props.setBoolean( AppSettings.VERTICAL_ALLIGNMENT,verticalAllign.isSelected() );
        props.setBoolean( AppSettings.HORIZONTAL_ALLIGNMENT,horizontalAllign.isSelected() );
    }
}
