package jcreek.cke.visualizer;

import javax.swing.*;
import javax.swing.border.EmptyBorder;

import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;


import jcreek.cke.MainWindow;
import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.Context;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2003
 * Company:
 * @author
 * @version 1.0
 */

public class RenameViewDialog extends JDialog implements ActionListener {

  private JButton okButton      = new JButton();
  private JButton cancelButton  = new JButton();

  protected JTextField viewName = new JTextField(25);

  protected boolean isOk = false;

  protected String viewTitle;

  public RenameViewDialog(Frame owner,String title,String viewTitle) {
      super(owner,title,true);
      enableEvents(AWTEvent.WINDOW_EVENT_MASK);
      this.viewTitle = viewTitle;
      try {
        jbInit();
      }
      catch(Exception e) {
        e.printStackTrace();
      }
  }

    public boolean isOK()
    {
        return isOk;
    }

    protected void processWindowEvent(WindowEvent e) {
      if (e.getID() == WindowEvent.WINDOW_CLOSING) {
        cancel();
      }
      super.processWindowEvent(e);
    }

    protected void cancel() {
      hide();
      dispose();
    }


  private void jbInit() throws Exception {
    MainWindow mw = MainWindow.instanceOf();
    JPanel content = new JPanel(new BorderLayout());
    content.setBorder(new EmptyBorder(12,12,12,12));
    setContentPane(content);

    JPanel buttons = new JPanel();
    buttons.setLayout(new BoxLayout(buttons,BoxLayout.X_AXIS));
    buttons.setBorder(new EmptyBorder(12,0,0,0));
    buttons.add(Box.createGlue());
    okButton.setText(mw.getResourceString("okLabel"));
    okButton.addActionListener( this );
    buttons.add(okButton);
    buttons.add(Box.createHorizontalStrut(6));
    cancelButton.setText(mw.getResourceString("cancelLabel"));
    cancelButton.addActionListener( this );
    buttons.add(cancelButton);
    buttons.add(Box.createGlue());

    getRootPane().setDefaultButton(cancelButton);

    content.add(BorderLayout.SOUTH,buttons);

    viewName.setText(viewTitle);
    content.add(BorderLayout.CENTER,viewName);

    pack();
    setLocationRelativeTo(getOwner());
    show();

  }

    public void actionPerformed( ActionEvent e )
   {
        String command = e.getActionCommand();
        if ( command.equals(MainWindow.instanceOf().getResourceString("okLabel")))
        {
              if(checkTitle())
              {
                  isOk = true;
                  dispose();
              }
        }
        else if(command.equals(MainWindow.instanceOf().getResourceString("cancelLabel")))
        {
              isOk = false;
              dispose();
        }
    }

    protected boolean checkTitle()
    {
        String name = viewName.getText();
        if(viewTitle.equals(name))
          return true;
        if((name==null)||(name.length()==0))
        {
            showErrorMsg("invalidNameMsg");
            return false;
        }
        CreekDocumentGroup dg = CreekDocumentGroup.instanceOf();
        KnowledgeModel km = dg.getKM();
        Context[] contexts = km.getContexts();
        for(int i=0;i<contexts.length;i++)
        {
            if(((CreekDocument)contexts[i]).getTitle().equals( name ))
            {
                showErrorMsg("duplicateViewNameMsg");
                return false;
            }
        }
        return true;
    }

    protected void showErrorMsg(String key)
    {
        MainWindow mw = MainWindow.instanceOf();
            JOptionPane.showMessageDialog
                ( getOwner(),
		  mw.getResourceString( key ),
                  mw.getResourceString( "userErrorTitle" ),
                  JOptionPane.ERROR_MESSAGE );
    }

    public String getViewTitle()
    {
        return viewName.getText();
    }

}