package jcreek.cke.visualizer;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.plaf.ColorUIResource;
import java.util.Enumeration;
import javax.swing.event.*;
import javax.swing.border.*;
import java.util.Vector;

import jcreek.representation.KnowledgeModel;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.MainWindow;

/**
 * Title:        JavaCreek
 * Description:  A Case based Reasoning Approach in java
 * Copyright:    Copyright (c) 2002
 * Company:      Phil
 * @author Victor
 * @version 1.0
 */

public class SaveDialog extends JDialog {
  JPanel contentPanel;
  JPanel filePanel;
  JPanel buttonPanel;
  JButton cancelButton;
  JButton okButton;
  JButton noneButton;
  JButton allButton ;
  JTree fileTree;

  boolean ok =  false;

  public SaveDialog(Frame owner,String title,boolean modal) {
    super(owner,title,modal);
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }
  private void jbInit() throws Exception {

    MainWindow mw = MainWindow.instanceOf();

    this.getContentPane().setLayout(new BorderLayout());
    cancelButton = new JButton(mw.getResourceString("cancelLabel"));
    cancelButton.addActionListener(new ActionListener()
    {

        public void actionPerformed(ActionEvent event)
        {
            ok=false;
            setVisible(false);
        }

    });
    cancelButton.setAlignmentX((float) 0.9);

    getRootPane().setDefaultButton(cancelButton);

    okButton = new JButton(mw.getResourceString("okLabel"));
    okButton.addActionListener(new ActionListener()
    {

        public void actionPerformed(ActionEvent event)
        {
            ok=true;
            setVisible(false);
        }

    });
    okButton.setAlignmentX((float) 0.9);
    noneButton = new JButton(mw.getResourceString("noneLabel"));
    noneButton.addActionListener(new ActionListener()
    {

        public void actionPerformed(ActionEvent event)
        {
              setSelectedAll(false);
              fileTree.revalidate();
              fileTree.repaint();
        }

    });
    allButton = new JButton(mw.getResourceString("allLabel"));
    allButton.addActionListener(new ActionListener()
    {

        public void actionPerformed(ActionEvent event)
        {
              setSelectedAll(true);
              fileTree.revalidate();
              fileTree.repaint();
        }

    });
    buttonPanel = new JPanel();
    buttonPanel.add(allButton, null);
    buttonPanel.add(noneButton, null);
    buttonPanel.add(okButton, null);
    buttonPanel.add(cancelButton, null);
/*
    String[] strs = {"swing",     // 0
		     "platf",     // 1
		     "basic",     // 2
		     "metal",     // 3
		     "JTree"};    // 4

    CheckNode[] nodes = new CheckNode[strs.length];
    for (int i=0;i<strs.length;i++) {
      nodes[i] = new CheckNode(strs[i]);
    }
    nodes[0].add(nodes[1]);
    nodes[1].add(nodes[2]);
    nodes[1].add(nodes[3]);
    nodes[0].add(nodes[4]);
    nodes[3].setSelected(true);
*/
    fileTree = new JTree(new CheckNode("Model"));
    fileTree.setCellRenderer(new CheckRenderer());
    fileTree.getSelectionModel().setSelectionMode(
      TreeSelectionModel.SINGLE_TREE_SELECTION
    );

    fileTree.putClientProperty("JTree.lineStyle", "Angled");
    fileTree.addMouseListener(new NodeSelectionListener(fileTree));
    JScrollPane sp = new JScrollPane(fileTree);
    filePanel = new JPanel();
    filePanel.setLayout(new BorderLayout());
    filePanel.add(sp, BorderLayout.CENTER);
    contentPanel = new JPanel();
    contentPanel.setLayout(new BorderLayout());
    contentPanel.add(filePanel, BorderLayout.CENTER);
    contentPanel.add(buttonPanel,  BorderLayout.SOUTH);
    this.getContentPane().add(contentPanel, BorderLayout.CENTER);
 }


  public void initTree()
  {
      ok = false;
      DefaultTreeModel dtmd = (DefaultTreeModel)fileTree.getModel();
      CreekDocumentGroup cdg = CreekDocumentGroup.instanceOf();
      CheckNode topNode = new CheckNode(cdg.getKM().getName());
      topNode.setSelected(true);
      dtmd.setRoot(topNode);
      CreekDocument[] docs = cdg.getDocuments();
      for(int i=0;i<docs.length;i++)
      {
          if(docs[i].isDirty() || !(docs[i].isTitled()))
          {
              CheckNode docNode = new CheckNode(docs[i]);
              topNode.add(docNode);
          }
      }
      fileTree.expandPath(new TreePath(topNode.getPath()));
      pack();
  }


  public CreekDocument[] getSelectedDocuments()
  {
        Vector creekDocs = new Vector();
        CheckNode topNode = (CheckNode)((DefaultTreeModel)fileTree.getModel()).getRoot();
        Enumeration enum = topNode.breadthFirstEnumeration();
        while(enum.hasMoreElements())
        {
              CheckNode node = (CheckNode)enum.nextElement();
              if ((node.isSelected())&& (node != topNode))
                    creekDocs.add((CreekDocument)node.getUserObject());
        }
        CreekDocument[] selDocs = new CreekDocument[creekDocs.size()];
        creekDocs.copyInto(selDocs);
        return selDocs;
  }

  public CreekDocument[] getNonSelectedDocuments()
  {
        Vector creekDocs = new Vector();
        CheckNode topNode = (CheckNode)((DefaultTreeModel)fileTree.getModel()).getRoot();
        Enumeration enum = topNode.breadthFirstEnumeration();
        while(enum.hasMoreElements())
        {
              CheckNode node = (CheckNode)enum.nextElement();
              if ((!node.isSelected())&& (node != topNode))
                    creekDocs.add((CreekDocument)node.getUserObject());
        }
        CreekDocument[] selDocs = new CreekDocument[creekDocs.size()];
        creekDocs.copyInto(selDocs);
        return selDocs;
  }


  public boolean isOk()
  {
      return ok;
  }

  public boolean isModelSelected()
  {
        CheckNode topNode = (CheckNode)((DefaultTreeModel)fileTree.getModel()).getRoot();
        return topNode.isSelected();
  }

  protected void setSelectedAll(boolean state)
  {
        CheckNode topNode = (CheckNode)((DefaultTreeModel)fileTree.getModel()).getRoot();
        Enumeration enum = topNode.breadthFirstEnumeration();
        while(enum.hasMoreElements())
              ((CheckNode)enum.nextElement()).setSelected(state);
  }

  class NodeSelectionListener extends MouseAdapter {
    JTree tree;

    NodeSelectionListener(JTree tree) {
      this.tree = tree;
    }

    public void mouseClicked(MouseEvent e) {
      int x = e.getX();
      int y = e.getY();
      int row = tree.getRowForLocation(x, y);
      TreePath  path = tree.getPathForRow(row);
      //TreePath  path = tree.getSelectionPath();
      if (path != null) {
        CheckNode node = (CheckNode)path.getLastPathComponent();
        boolean isSelected = ! (node.isSelected());
        node.setSelected(isSelected);
        if (node.getSelectionMode() == CheckNode.SINGLE_SELECTION) {
          if ( isSelected ) {
            tree.expandPath(path);
          } else {
            tree.collapsePath(path);
          }
        }
        ((DefaultTreeModel)tree.getModel()).nodeChanged(node);
        // I need revalidate if node is root.  but why?
        if (row == 0) {
          tree.revalidate();
          tree.repaint();
        }
      }
    }
  }


}

class CheckNode extends DefaultMutableTreeNode {

  public final static int SINGLE_SELECTION = 0;
  public final static int DIG_IN_SELECTION = 4;
  protected int selectionMode;
  protected boolean isSelected;

  public CheckNode() {
    this(null);
  }

  public CheckNode(Object userObject) {
    this(userObject, true, false);
  }

  public CheckNode(Object userObject, boolean allowsChildren
                                    , boolean isSelected) {
    super(userObject, allowsChildren);
    this.isSelected = isSelected;
    setSelectionMode(SINGLE_SELECTION);
  }


  public void setSelectionMode(int mode) {
    selectionMode = mode;
  }

  public int getSelectionMode() {
    return selectionMode;
  }

  public void setSelected(boolean isSelected) {
    this.isSelected = isSelected;

    if ((selectionMode == DIG_IN_SELECTION)
        && (children != null)) {
      Enumeration enum = children.elements();
      while (enum.hasMoreElements()) {
        CheckNode node = (CheckNode)enum.nextElement();
        node.setSelected(isSelected);
      }
    }
  }

  public boolean isSelected() {
    return isSelected;
  }


  // If you want to change "isSelected" by CellEditor,
  /*
  public void setUserObject(Object obj) {
    if (obj instanceof Boolean) {
      setSelected(((Boolean)obj).booleanValue());
    } else {
      super.setUserObject(obj);
    }
  }
  */

}

class CheckRenderer extends JPanel implements TreeCellRenderer {
  protected JCheckBox check;
  protected TreeLabel label;

  public CheckRenderer() {
    setLayout(null);
    add(check = new JCheckBox());
    add(label = new TreeLabel());
    check.setBackground(UIManager.getColor("Tree.textBackground"));
    label.setForeground(UIManager.getColor("Tree.textForeground"));
  }

  public Component getTreeCellRendererComponent(JTree tree, Object value,
               boolean isSelected, boolean expanded,
               boolean leaf, int row, boolean hasFocus) {
    String  stringValue = tree.convertValueToText(value, isSelected,
			expanded, leaf, row, hasFocus);
    setEnabled(tree.isEnabled());
    if(stringValue!="JTree")//got to be a better way
      check.setSelected(((CheckNode)value).isSelected());
    label.setFont(tree.getFont());
    label.setText(stringValue);
    label.setSelected(isSelected);
    label.setFocus(hasFocus);
    if (leaf) {
      label.setIcon(UIManager.getIcon("Tree.leafIcon"));
    } else if (expanded) {
      label.setIcon(UIManager.getIcon("Tree.openIcon"));
    } else {
      label.setIcon(UIManager.getIcon("Tree.closedIcon"));
    }
    return this;
  }

  public Dimension getPreferredSize() {
    Dimension d_check = check.getPreferredSize();
    Dimension d_label = label.getPreferredSize();
    return new Dimension(d_check.width  + d_label.width,
      (d_check.height < d_label.height ?
       d_label.height : d_check.height));
  }

  public void doLayout() {
    Dimension d_check = check.getPreferredSize();
    Dimension d_label = label.getPreferredSize();
    int y_check = 0;
    int y_label = 0;
    if (d_check.height < d_label.height) {
      y_check = (d_label.height - d_check.height)/2;
    } else {
      y_label = (d_check.height - d_label.height)/2;
    }
    check.setLocation(0,y_check);
    check.setBounds(0,y_check,d_check.width,d_check.height);
    label.setLocation(d_check.width,y_label);
    label.setBounds(d_check.width,y_label,d_label.width,d_label.height);
  }


  public void setBackground(Color color) {
    if (color instanceof ColorUIResource)
      color = null;
    super.setBackground(color);
  }


  public class TreeLabel extends JLabel {
    boolean isSelected;
    boolean hasFocus;

    public TreeLabel() {
    }

    public void setBackground(Color color) {
	if(color instanceof ColorUIResource)
	    color = null;
	super.setBackground(color);
    }

    public void paint(Graphics g) {
      String str;
      if ((str = getText()) != null) {
        if (0 < str.length()) {
          if (isSelected) {
            g.setColor(UIManager.getColor("Tree.selectionBackground"));
          } else {
            g.setColor(UIManager.getColor("Tree.textBackground"));
          }
          Dimension d = getPreferredSize();
          int imageOffset = 0;
          Icon currentI = getIcon();
          if (currentI != null) {
            imageOffset = currentI.getIconWidth() + Math.max(0, getIconTextGap() - 1);
          }
          g.fillRect(imageOffset, 0, d.width -1 - imageOffset, d.height);
          if (hasFocus) {
            g.setColor(UIManager.getColor("Tree.selectionBorderColor"));
            g.drawRect(imageOffset, 0, d.width -1 - imageOffset, d.height -1);
         }
        }
      }
      super.paint(g);
    }

    public Dimension getPreferredSize() {
      Dimension retDimension = super.getPreferredSize();
      if (retDimension != null) {
        retDimension = new Dimension(retDimension.width + 3,
				 retDimension.height);
      }
      return retDimension;
    }

    public void setSelected(boolean isSelected) {
      this.isSelected = isSelected;
    }

    public void setFocus(boolean hasFocus) {
      this.hasFocus = hasFocus;
    }
  }
}




