package jcreek.cke.visualizer;

import javax.swing.JPanel;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import javax.swing.JScrollBar;
import javax.swing.JLabel;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import jcreek.cke.conceptview.*;


import jcreek.cke.command.ResizeListener;
import jcreek.cke.command.ResizeEvent;
import jcreek.cke.util.DPoint;
import jcreek.cke.util.DDimension;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:
 * @author
 * @version 1.0
 */

public class ScrollView extends LPanel implements AdjustmentListener , ResizeListener
{

  private DrawingSurface drawingSurface_;
  private JLabel		label_;

  // Paging a scrollbar will move the image by this fraction of the
  // window size - default is 1/4.
  private double		pageFraction_ = 0.25;

  private double		xoffset_ = 0.0, yoffset_ = 0.0;

  private DDimension	win_size;
  private DDimension	img_size;

  private JScrollBar	vscroll_, hscroll_;

  protected PropertyChangeSupport propSupport;


  public ScrollView(DrawingSurface drwSurface)
  {
         drawingSurface_ = drwSurface;
         drawingSurface_.addResizeListener(this);
	 spacing = 0;
	 constraints.insets.top = constraints.insets.bottom = 0;
//	 label_ = addLabel("", 0, -1, 1.0, 0.0, 1, 0);
         addComponent(drawingSurface_, 1, 0, 1.0, 1.0, 3, 0);
         vscroll_ = new JScrollBar(JScrollBar.VERTICAL);
         vscroll_.addAdjustmentListener(this);
         addComponent(vscroll_, 0, 0, 0.0, 1.0, 2, 0);
         hscroll_ = new JScrollBar(JScrollBar.HORIZONTAL);
         hscroll_.addAdjustmentListener(this);
         addComponent(hscroll_, 1, -1, 1.0, 0.0, 1, 0);
         propSupport = new PropertyChangeSupport( this );

        configure();

	 finish();

  }

   // Calculate new Scrollbar values, maximums, and sizes, and adjust the
   // offset of the OffsetCanvas if necessary.

      private void configure()
      {
         double	hmax, vmax, pageh, pagew, xoffset, yoffset;

         Dimension tmpD = drawingSurface_.getSize();

         win_size = new DDimension(tmpD.width, tmpD.height);
         img_size = drawingSurface_.contentsSize();



      	// offsetCanvas_ controls the new offset as well as the size, so it
      	// can remain stationary in an OffsetCanvas-dependent way.
         DPoint tmpdim = drawingSurface_.getOffset();

         xoffset = tmpdim.x;
         yoffset = tmpdim.y;



      	// Scrolls can move an amount of image size - window size (so the whole image
      	// can be seen by scrolling). If window is larger than image, no scrolling
      	// is allowed.

         hmax = Math.max(img_size.width - win_size.width, 0.0);
         vmax = Math.max(img_size.height - win_size.height, 0.0);

      	// Scroll positions remain the same unless they are now out of range.
         if(xoffset > (double)hmax)
            xoffset = (double)hmax;
         if(yoffset > (double)vmax)
            yoffset = (double)vmax;

      // Meaning of these values is different on SPARC and PC. Only way to make them close
      // to the same is to make visible equal to one.

      // On a SPARC, maximum is the maximum value of a Scrollbar.
      // On a PC, maximum - slider size is the maximum value.

      // With this setup, we can scroll one extra pixel on a SPARC (but feedback from
      // offsetCanvas_.setOffsets() can correct this), and the right amount on a PC.

      //	hscroll_.setValues(xoffset_, Math.min(win_size.width, img_size.width), 0, hmax);
      //	vscroll_.setValues(yoffset_, Math.min(win_size.height, img_size.height), 0, vmax);

         hscroll_.setValues((int)xoffset, 1, 0, (int)hmax + 1);
         vscroll_.setValues((int)yoffset, 1, 0, (int)vmax + 1);

         xoffset_ = xoffset;
         yoffset_ = yoffset;

      	// page increment is pageFraction_ of screen size but at least one
         pageh = Math.max((win_size.width * pageFraction_), 1.0);
         pagew = Math.max((win_size.height * pageFraction_), 1.0);
         hscroll_.setBlockIncrement((int)pageh);
         vscroll_.setBlockIncrement((int)pagew);

      	// Offsets will be equal to scroll amounts, unless the window is larger than the
      	// image, in which case the image is centered in the window.
         if(win_size.width > img_size.width || win_size.height > img_size.height)
         {
            if(win_size.width > img_size.width)
               xoffset_ = -(win_size.width - img_size.width + 1.0) / 2.0;
            if(win_size.height > img_size.height)
               yoffset_ = -(win_size.height - img_size.height + 1.0) / 2.0;
         }

      	// For some reason, a redraw here will cause an infinite loop. the size of
      	// offsetCanvas_ must be unstable during a resize.
         drawingSurface_.setOffsets(xoffset_, yoffset_, false);
      }


      public void resizePerformed(ResizeEvent re)
      {
            configure();
            fireResizeChange();
      }

      public DrawingSurface getDrawingSurface()
      {
          return drawingSurface_;
      }

      public void adjustmentValueChanged(AdjustmentEvent ae)
      {
            int eventType = ae.getAdjustmentType();
            if(
            (eventType == AdjustmentEvent.TRACK ||
             eventType == AdjustmentEvent.UNIT_DECREMENT ||
             eventType == AdjustmentEvent.UNIT_INCREMENT ||
             eventType == AdjustmentEvent.BLOCK_DECREMENT ||
             eventType == AdjustmentEvent.BLOCK_INCREMENT))
            {
            	// Scrollbar event
            	// Adjust the offset of the OffsetCanvas

               xoffset_ = (double)hscroll_.getValue();
               yoffset_ = (double)vscroll_.getValue();
               drawingSurface_.setOffsets(xoffset_, yoffset_, true);
               fireOffsetChange();
            }


      }

      public DDimension getPortSize()
      {
         return new DDimension(win_size.width, win_size.height);
      }


      public DDimension getContentSize()
      {
         return new DDimension(img_size.width, img_size.height);
      }


      public DDimension getOffset()
      {
         return new DDimension(xoffset_, yoffset_);
      }

   /**
   *	Scroll to any given location.
   *@param x horizontal offset to scroll to
   *@param y vertical offset to scroll to
   **/
      public void scrollTo(double x, double y)
      {
         if(x < 0.0)
            x = 0.0;
         if(y < 0.0)
            y = 0.0;

         if(x > (double)hscroll_.getMaximum())
            x = (double)hscroll_.getMaximum();
         if(y > (double)vscroll_.getMaximum())
            y = (double)vscroll_.getMaximum();

         xoffset_ = x;
         yoffset_ = y;

         hscroll_.setValue((int)x);
         vscroll_.setValue((int)y);

         drawingSurface_.setOffsets(x, y, true);
      }

    /**
     * Add a property change listener to this document group.
     *
     * @param  l  new listener to add notification list
     */
    public void addPropertyChangeListener( PropertyChangeListener l ) {
        propSupport.addPropertyChangeListener( l );
    } // addPropertyChangeListener

    /**
     * Add a property change listener to this document group.
     *
     * @param  name  name of property to listen to specifically
     * @param  l     new listener to add notification list
     */
    public void addPropertyChangeListener( String name,
                                           PropertyChangeListener l ) {
        propSupport.addPropertyChangeListener( name, l );
    } // addPropertyChangeListener

    /**
     * Remove a property change listener from this document group.
     *
     * @param  l  listener to remove from notification list
     */
    public void removePropertyChangeListener( PropertyChangeListener l ) {
        propSupport.removePropertyChangeListener( l );
    } // removePropertyChangeListener

    /**
     * Remove a property change listener from this document group.
     *
     * @param  name  property name to listen for specifically
     * @param  l     listener to remove from notification list
     */
    public void removePropertyChangeListener( String name,
                                              PropertyChangeListener l ) {
        propSupport.removePropertyChangeListener( name, l );
    } // removePropertyChangeListener


    protected void fireResizeChange( ) {
        propSupport.firePropertyChange( "resize", null,null);
    } // fireDocChange

    protected void fireOffsetChange( ) {
        propSupport.firePropertyChange( "offset", null,null);
    } // fireDocChange

}