package jcreek.cke.visualizer;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.Vector;

import jcreek.cke.browser.Visualizer;
import jcreek.representation.EntityTypeNotMatchedException;
import jcreek.representation.RelationType;
import jcreek.representation.Entity;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.cbr.*;
import jcreek.cke.util.QuickSort;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.graph.CreekModel;
import jcreek.cke.MainWindow;
import jcreek.cke.conceptview.*;

public class SubModelDialog extends JDialog implements ActionListener
{

  protected Container pane;
  protected CheckBoxPanel cbp;
  protected ViewOption viewPanel;
  protected Frame owner;
  protected Dimension vstrut10 = new Dimension( 0, 10 );
  protected Dimension hstrut25 = new Dimension( 25, 0 );
  protected Vector initSel = new Vector();
  protected Vector curSel  = new Vector();
  protected KnowledgeModel km;
  protected CreekDocument doc;
  protected CreekModel model;

  protected boolean canShow = false;

//  ClassDescriptor conceptClass;
  Entity relationConcepts[];
  VisualEntity selEntity;

  boolean ok = false;

  QuickSort quick = new QuickSort() {
      public boolean compare(Object ob1, Object ob2)
      {
        return ((Entity) ob1).getName().compareTo(((Entity) ob2).getName()) < 0;
      }
    };


  public SubModelDialog(Frame owner,VisualEntity selEntity)
  {

        super(owner,"Sub-Model",true);
        this.owner = owner;
        this.selEntity = selEntity;

        MainWindow mw = MainWindow.instanceOf();

        pane =  getContentPane();
        km = CreekDocumentGroup.instanceOf().getKM();
        doc = CreekDocumentGroup.instanceOf().getCurrentDocument();
        if(doc != null)
            model = doc.getModel();

        //Panel for holding relations CheckBox
        cbp = new CheckBoxPanel();
        createCheckBoxes();

        if(relationConcepts.length==0)
        {
          dispose();
          return;
        }

        checkList(selEntity);
        pane.add(cbp,"Center");

        //Panel for View Option
        viewPanel =  new ViewOption();
        pane.add(viewPanel,"North");

        //OK Cancel button
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        pane.add( buttonPanel, "South" );

        // OK button
        JButton button = new JButton(mw.getResourceString("okLabel"));
        button.addActionListener( this );
        buttonPanel.add( button );

        // Cancel button
        button = new JButton(mw.getResourceString("cancelLabel"));
        button.addActionListener( this );
        buttonPanel.add( button );

        getRootPane().setDefaultButton(button);

        setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE );
        pack();
        canShow = true;

  }

    public boolean getCanShow()
    {
        return canShow;
    }


    public void actionPerformed( ActionEvent e )
   {
        String command = e.getActionCommand();
        if ( command.equals(
                MainWindow.instanceOf().getResourceString("okLabel")))
            ok = true;
        else
          ok = false;
        dispose();
    }


   public RelationType[] getActivatedRelations()
   {
            curSel = cbp.getSelectedIndex();
            RelationType cd[] = new RelationType[curSel.size()];
            for(int i=curSel.size()-1;i>=0;i--)
            {
              try
              {
                cd[i] = new RelationType(relationConcepts[ ( (Integer) curSel.get(i)).
                                         intValue()]);
              }
              catch(EntityTypeNotMatchedException e)
              {
                e.printStackTrace();
              }
            }
            return cd;
   }


    public boolean isOk()
    {
        return ok;
    }

    protected void createCheckBoxes()
    {
        int rem = 1;

        relationConcepts = getConceptList();
        quick.sort(relationConcepts);
        int j=0;
        for(int i=0;i<relationConcepts.length;i++)
        {

              cbp.addCheckBox(relationConcepts[i].getName(),false, rem, -1, 1.0, 1.0, 0, 0);
          rem = (j>=3?0:1);
          j   = (j>=3?0:j+1);
        }

    } //createCheckBoxes
    protected Entity[] getConceptList()
    {
      return selEntity.getEntity().getKnowledgeModel().getRelationType(CBRModel.RELATION).getAllSubclasses();
    }

      protected void checkList(VisualEntity ve)
      {
          cbp.uncheckAll();
          initSel.removeAllElements();
          if(ve!=null)
          {
              for(int i=0;i<relationConcepts.length;i++)
              {
//                  if(model.relationIsActive((RelationType)relationConcepts[i],ve))
                  try
                  {
                    if (ve.relationIsActive(new RelationType(relationConcepts[i]))) {
                      JCheckBox relCheckBox = cbp.check(i);
                      initSel.add(new Integer(i));
                    }
                  }
                  catch(EntityTypeNotMatchedException e)
                  {
                    // This should not happen.
                    e.printStackTrace();
                  }
              }
          }
      }//checkList

      public boolean isNewView()
      {
          return viewPanel.isNewViewSelected();
      }

} //SubModelDialog

class CheckBoxPanel extends JPanel
{
    protected int spacing = 8;

    protected GridBagLayout layout;
    protected GridBagConstraints constraints;
    protected Vector checkBoxList = new Vector();
    protected Vector relationNames = new Vector();
    protected  TitledBorder relationTitle;

    public CheckBoxPanel( )
    {
        super();

        layout = new GridBagLayout();
        constraints = new GridBagConstraints();
        setLayout( layout );

        constraints.insets = new Insets(spacing, spacing, spacing,
                                         spacing);
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = constraints.ipady = 0;
        constraints.weightx = constraints.weighty = 1.0;
        relationTitle = new TitledBorder(BorderFactory.createEtchedBorder(
                                Color.white,new Color(134, 134, 134)),
                                "Relation Type");
        this.setBorder(relationTitle);
    }

    protected void addCheckBox(String string,
                                  boolean state, int width, int anchor,
                                  double weightx, double weighty, int fill, int shift)
    {
          JCheckBox relCheckBox = new JCheckBox(string, state);
          checkBoxList.add((JCheckBox)(addComponent(relCheckBox,
                                        width, anchor, weightx, weighty, fill, shift)));
          relationNames.add(string);
    }

      protected Component addComponent(Component component, int width,
                                    int anchor, double weightx, double weighty, int fill,
                                    int shift)
      {
         constraints.insets.left = spacing;
         constraints.insets.right = spacing;
         if(shift == 1 || shift == 3)
            constraints.insets.left = 0;
         if(shift == 2 || shift == 3)
            constraints.insets.right = 0;

         if(anchor < 0)
            constraints.anchor = GridBagConstraints.WEST;
         else if(anchor == 0)
            constraints.anchor = GridBagConstraints.CENTER;
         else
            constraints.anchor = GridBagConstraints.EAST;

         if(width == 0)
            constraints.gridwidth = GridBagConstraints.REMAINDER;
         else
            constraints.gridwidth = width;

         constraints.weightx = weightx;
         constraints.weighty = weighty;

         if(fill == 0)
            constraints.fill = GridBagConstraints.NONE;
         else if(fill == 1)
            constraints.fill = GridBagConstraints.HORIZONTAL;
         else if(fill == 2)
            constraints.fill = GridBagConstraints.VERTICAL;
         else
            constraints.fill = GridBagConstraints.BOTH;

         layout.setConstraints(component, constraints);
         add(component);

         if(width == 0)
            constraints.insets.top = constraints.insets.bottom = spacing;

         return component;
      }

      protected String[] getRelationNames()
      {
          Vector relNames = new Vector();
          JCheckBox relName;
          if(checkBoxList.size() != 0)
          {
              for(int i=checkBoxList.size()-1;i >= 0;i--)
              {
                  relName = (JCheckBox)checkBoxList.elementAt(i);
                  if(relName.isSelected())
                  {
                      relNames.addElement(relationNames.elementAt(i));

                  }
              }
                  String[] rNames = new String[relNames.size()];
                  relNames.copyInto(rNames);
                  return rNames;
          }
          return null;
      }


    public void uncheckAll()
    {
          JCheckBox relName;

          if(checkBoxList.size() != 0)
              for(int i=checkBoxList.size()-1;i >= 0;i--)
              {
                  relName = (JCheckBox)checkBoxList.elementAt(i);
                  relName.setSelected(false);
              }
    }

    public JCheckBox check(int selindex)
    {
        JCheckBox relName = (JCheckBox)checkBoxList.elementAt(selindex);
        relName.setSelected(true);
        return relName;
    }


    public Vector getSelectedIndex()
    {
          Vector curRel = new Vector();
          JCheckBox relName;
          if(checkBoxList.size() != 0)
          {
              for(int i=checkBoxList.size()-1;i >= 0;i--)
              {
                  relName = (JCheckBox)checkBoxList.elementAt(i);
                  if(relName.isSelected())
                  {
                      curRel.addElement(new Integer(i));

                  }
              }
//                  int[] rIndex = new int[curRel.size()];
//                  curRel.copyInto(rIndex);
                  return curRel;
          }
          return null;

    }


}//CheckBoxPanel

class ViewOption extends JPanel
{
  TitledBorder viewTitle;
  JRadioButton currentView = new JRadioButton();
  JRadioButton newView = new JRadioButton();
  ButtonGroup viewGroup = new ButtonGroup();

    public ViewOption()
    {
        super();
        init();
    }

    public void init()
    {
        viewTitle = new TitledBorder(BorderFactory.createEtchedBorder(Color.white,new Color(134, 134, 134)),"Open Relation In");
        currentView.setSelected(true);
        currentView.setText("Current View");
        currentView.addActionListener(new java.awt.event.ActionListener() {
          public void actionPerformed(ActionEvent e) {
            currentView_actionPerformed(e);
          }
        });
        this.setBorder(viewTitle);
        newView.setToolTipText("");
        newView.setText("New View");
        this.add(currentView, null);
        this.add(newView, null);
        viewGroup.add(currentView);
        viewGroup.add(newView);
    }

    void currentView_actionPerformed(ActionEvent e) {

    }

    public boolean isNewViewSelected()
    {
        return newView.isSelected();
    }
}

