package jcreek.cke.visualizer;

import javax.swing.JMenuItem;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JPopupMenu;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.Action;
import javax.swing.tree.*;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.ActionEvent;
import java.awt.GridLayout;
import java.awt.Component;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.event.ActionListener;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import java.net.URL;
import javax.swing.JButton;
import javax.swing.SwingUtilities;

import jcreek.cke.MainWindow;
import jcreek.cke.browser.Visualizer;
import jcreek.cke.CreekDocumentGroup;
import jcreek.cke.document.CreekDocument;
import jcreek.cke.graph.AbstractTreeModel;
import jcreek.representation.KnowledgeModel;
import jcreek.representation.Context;

/**
 * Title:        JavaCreek
 * Description:  A Case Based Reasoning approach developed in Java
 * Copyright:    Copyright (c) 1999
 * Company:      Phil Systems
 * @author Victor Dias
 * @version
 */

public class TreeView extends JPanel
          implements PropertyChangeListener,TreeSelectionListener
{

    protected JTree kmTree;
    protected TreeViewModel treeModel;
    final protected JPopupMenu popup = new JPopupMenu();
    protected CreekDocument selDoc;
    protected JMenuItem newWin;
    protected JMenuItem openWin;
    protected JMenuItem deleteWin;
    protected JMenuItem renameWin;

    public TreeView()
    {
        super();
        initTree();
    } //TreeView

    public void initTree()
    {

        treeModel = new TreeViewModel();
        kmTree = new JTree(treeModel);
        kmTree.setEditable(false);
        kmTree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);
        kmTree.getSelectionModel().addTreeSelectionListener(this);
        kmTree.setShowsRootHandles(true);

        JScrollPane scrollPane = new JScrollPane(kmTree);
        newWin  = createMenuItem("newConceptView");
        popup.add(newWin);
        openWin = createMenuItem("openConceptView");
        popup.add(openWin);
        popup.addSeparator();
        renameWin = createMenuItem("renameView");
        popup.add(renameWin);
        popup.addSeparator();
        deleteWin = createMenuItem("deleteView");
        popup.add(deleteWin);

        kmTree.addMouseListener(
        new MouseAdapter() {
          public void mousePressed(MouseEvent me) {
          if (SwingUtilities.isRightMouseButton(me)) {

               TreePath path = kmTree.getPathForLocation(me.getX(),
                                                       me.getY());

               if (path==null) {
               return;
               }

               kmTree.setSelectionPath(path);
           }
            if (me.isPopupTrigger())
            {
              DefaultMutableTreeNode dmt = getCurrentSelection(me);
              if(dmt != null)
              {
                  if(dmt.isRoot())
                      selDoc = null;
                  else
                      selDoc = (CreekDocument)dmt.getUserObject();
                  showPopup(me);
              }
            }
          }
          public void mouseReleased(MouseEvent me) {
            if (me.isPopupTrigger())
            {
              DefaultMutableTreeNode dmt = getCurrentSelection(me);
              if(dmt != null)
              {
                  if(dmt.isRoot())
                      selDoc = null;
                  else
                      selDoc = (CreekDocument)dmt.getUserObject();
                  showPopup(me);
              }
            }
            else if(me.getClickCount()==2)
            {

              DefaultMutableTreeNode dmt = getCurrentSelection(me);
              if(dmt != null)
              {
                  if(dmt.isRoot())
                      selDoc = null;
                  else
                      selDoc = (CreekDocument)dmt.getUserObject();
              }
              if(selDoc !=null)
              {
                Action a = MainWindow.instanceOf().getAction("openConceptView");
                if(a !=null)
                {
                  a.setEnabled(true);
                  a.putValue("selDoc",selDoc);
                  a.actionPerformed( new ActionEvent( kmTree,
                                            ActionEvent.ACTION_PERFORMED,
                                            "openConceptView" ) );
                }
              }
            }
          }
          void showPopup(MouseEvent me) {
            MainWindow mw  = MainWindow.instanceOf();
            Action a;
            if(selDoc == null)
            {

                a = mw.getAction("newConceptView");
                if(a !=null)
                {
                  a.setEnabled(true);
                  newWin.setEnabled( a.isEnabled() );
                }
                a = mw.getAction("openConceptView");
                if(a !=null)
                {
                  a.setEnabled(false);
                  openWin.setEnabled( a.isEnabled() );
                }
                a = mw.getAction("deleteView");
                if(a !=null)
                {
                  a.setEnabled(false);
                  deleteWin.setEnabled( a.isEnabled() );
                }
                a = mw.getAction("renameView");
                if(a !=null)
                {
                  a.setEnabled(false);
                  renameWin.setEnabled( a.isEnabled() );
                }

            }
            else
            {

                a = mw.getAction("newConceptView");
                if(a !=null)
                {
                  a.setEnabled(true);
                  newWin.setEnabled( a.isEnabled() );
                }

                a = mw.getAction("openConceptView");
                if(a !=null)
                {
                  a.setEnabled(true);
                  a.putValue("selDoc",selDoc);
                  openWin.setEnabled( a.isEnabled() );
                }
                a = mw.getAction("deleteView");
                if(a !=null)
                  {
                    a.setEnabled(true);
                    a.putValue("selDoc",selDoc);
                    deleteWin.setEnabled( a.isEnabled() );
                  }
                a = mw.getAction("renameView");
                if(a !=null)
                {
                  a.setEnabled(true);
                  a.putValue("selDoc",selDoc);
                  renameWin.setEnabled( a.isEnabled() );
                }
            }
            popup.show(kmTree, me.getX(), me.getY());
          }
        }
      );

        setLayout(new GridLayout(1,0));
        add(scrollPane);
        CreekDocumentGroup.instanceOf().addPropertyChangeListener(this);
    }//initTree


    /** Return the currently selected node. */
    public DefaultMutableTreeNode getCurrentSelection(MouseEvent me) {
        TreePath currentSelection = kmTree.getPathForLocation(me.getX(),me.getY());
        DefaultMutableTreeNode currentNode = null;
        if (currentSelection != null)
        {
            try
            {
              currentNode = (DefaultMutableTreeNode)(currentSelection.getLastPathComponent());
            }
            catch (Exception ex)
            {
                ex.printStackTrace();
            }
        }
        return currentNode;

    }

    public void propertyChange( PropertyChangeEvent e )
    {
        updateModelView();

    }//propertyChange


    public void updateModelView(/*CreekDocument newVal*/)
    {
      treeModel.updateModel();
    }//updateModelView


    public void valueChanged(TreeSelectionEvent te)
    {

        // ok got to check why this is null not right to do this
        if(te.getNewLeadSelectionPath() == null)
            return;
        DefaultMutableTreeNode dmt = (DefaultMutableTreeNode)(te.getNewLeadSelectionPath ().getLastPathComponent());
        if(!(dmt.isRoot()))
        {
            CreekDocument doc = (CreekDocument)dmt.getUserObject();
            selDoc = doc;
            Visualizer win = Visualizer.getFrame(this);
            win.activateWin(doc);
        }
    }//valueChanged

    public CreekDocument getSelDoc()
    {
        return selDoc;
    }

    /**
     * This is the hook through which all menu items are
     * created. Using the <code>cmd</code> string it finds
     * the menu item label and image (if any) in the resource
     * bundle.
     *
     * @param  cmd  action command string for this menu item;
     *              used to get the label and image
     * @return  new menu item
     * @see #createMenu
     */
    protected JMenuItem createMenuItem( String cmd ) {
        MainWindow mw = MainWindow.instanceOf();
        // create menu item and set image and text
        JMenuItem mi = new JMenuItem
            ( mw.getResourceString( cmd + Visualizer.labelSuffix ) );
        URL url = mw.getResource( cmd + Visualizer.imageSuffix );
        if ( url != null ) {
            mi.setHorizontalTextPosition( JButton.RIGHT );
            mi.setIcon( new ImageIcon( url ) );
        }

        // set menu action command
        mi.setActionCommand( cmd );
        Action a = mw.getAction( cmd );
        if ( a != null ) {
            // set up the action to listen for events
            mi.addActionListener( a );
            mi.setEnabled( a.isEnabled() );
        } else {
            // no action! disable menu item
            mi.setEnabled( false );
        }
        return mi;
    } // createMenuItem

}//TreeView

class TreeViewModel extends DefaultTreeModel
{
    CreekDocumentGroup cdg;

    public TreeViewModel()
    {
        super(new DefaultMutableTreeNode(
          MainWindow.instanceOf().getResourceString("rootView"+Visualizer.titleSuffix)));
        cdg = CreekDocumentGroup.instanceOf();
        updateModel();
    }

    public void insertNode(CreekDocument doc)
    {
        int childCount = root.getChildCount();
        insertNodeInto(new DefaultMutableTreeNode(doc),(DefaultMutableTreeNode)root,childCount);

    }//insertNode

    public boolean isLeaf(Object node)
    {
      if(((DefaultMutableTreeNode)node).isRoot())
          return false;
      return true;
    }//isLeaf

    public void updateModel()
    {
        ((DefaultMutableTreeNode)root).removeAllChildren();
        if(cdg != null)
        {
//          CreekDocument[] docs = cdg.getDocuments();
          Context[] cont = cdg.getContexts();
          for(int i= 0;i<cont.length;i++)
          {
              insertNode((CreekDocument)cont[i]);
          }
        }
        reload(root);
    }// updateModel



    public boolean isAdded(CreekDocument doc)
    {
        boolean isAdded = false;
        for(Enumeration e = root.children();e.hasMoreElements();)
        {
            DefaultMutableTreeNode dmt = (DefaultMutableTreeNode)e.nextElement();
            if(doc == (CreekDocument)(dmt.getUserObject()))
            {
               isAdded = true;
               break;
            }
        }
        return isAdded;
    }

    public DefaultMutableTreeNode getNode(CreekDocument doc)
    {
        DefaultMutableTreeNode dmt = null;
        for(Enumeration e = root.children();e.hasMoreElements();)
        {
            DefaultMutableTreeNode tmp = (DefaultMutableTreeNode)e.nextElement();
            if(doc == (CreekDocument)(tmp.getUserObject()))
            {
               dmt = tmp;
               break;
            }
        }
        return dmt;
    }

    public DefaultMutableTreeNode removeNode(CreekDocument doc)
    {
        DefaultMutableTreeNode dmt = null;
        for(Enumeration e = root.children();e.hasMoreElements();)
        {
            DefaultMutableTreeNode tmp = (DefaultMutableTreeNode)e.nextElement();
            if(doc == (CreekDocument)(tmp.getUserObject()))
            {
               dmt = tmp;
               removeNodeFromParent(tmp);
               break;
            }
        }
        return dmt;
    }


}//TreeViewModel





