package jcreek.examples;

import jcreek.representation.*;
import jcreek.util.CreekException;
import java.util.*;
import java.io.*;

/**
 * Creates the the java source code to produce the knowledge model contained
 * in a .km file.
 * @author Frode Srmo
 * @version 1.0
 */

public class BuildJavaModel
{
  public BuildJavaModel()
  {
  }

  public static void main(String argv[])
  {
/*    if(argv.length < 2)
    {
      System.out.println("Syntax: <Creek model file name> <class name>");
      System.exit(1);
    }*/

    KnowledgeModel model;
    String className = "TestModel";

    KnowledgeModel basic = new LocalKnowledgeModel(new BasicModel());
    KnowledgeModel shadow = new LocalKnowledgeModel(new BasicModel());

    model = new LocalKnowledgeModel();

    try
    {
      PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(className+".java")));

      out.println("import jcreek.representation.*;");
      out.println("import jcreek.util.CreekException;");
      out.println("public class "+className+" extends BasicModel");
      out.println("{");
      out.println("\tpublic static void main(String argv[])");
      out.println("\t{");
      out.println("\t\tLocalKnowledgeModel km = new LocalKnowledgeModel(new "+className+"());");
      out.println("\t\tLocalKnowledgeModel basic = new LocalKnowledgeModel(new BasicModel());");
      out.println("\t\tSystem.out.println(\"Is different? \"+jcreek.examples.BuildJavaModel.isModelDifferent(basic, km));");
      out.println("\t}");
      out.println("\tpublic void generateEpistemologicalModel(KnowledgeModel km)\n");
      out.println("\t{");
      out.println("\t\tsuper.generateEpistemologicalModel(km);");
      out.println("\t\ttry");
      out.println("\t\t{");

      RelationType[] relationTypes = (RelationType[]) model.matchEntityType(new RelationType[0]);

      ArrayList relationTypeQueue = new ArrayList();
      relationTypeQueue.add(model.getRelationType(BasicModel.RELATION));

      int counter = 0;
      while(!relationTypeQueue.isEmpty())
      {
        RelationType current = (RelationType) relationTypeQueue.get(0);
        relationTypeQueue.remove(0);
        relationTypeQueue.addAll(current.getImmediateSubclassesCollection());

        if(!shadow.hasEntity(current.getName()))
        {
          new RelationType(shadow, shadow.getRelationType(current.getImmediateSuperclasses()[0].getName()),
                           current.getName(),current.getDescription(),current.getExplanationStrength(),
                           current.getInverseType().getName(), current.getInverseType().getDescription(), current.getInverseType().getExplanationStrength());
          out.println("\t\t\tnew RelationType(km,\n"+
                      "\t\t\t\tkm.getRelationType(\""+current.getImmediateSuperclasses()[0].getName()+"\"),\n"+
                      "\t\t\t\t\""+current.getName()+"\",\n"+
                      "\t\t\t\t\""+current.getDescription()+"\",\n"+
                      "\t\t\t\t"+current.getExplanationStrength()+",\n"+
                      "\t\t\t\t\""+current.getInverseType().getName()+"\",\n"+
                      "\t\t\t\t\""+current.getInverseType().getDescription()+"\",\n"+
                      "\t\t\t\t"+current.getInverseType().getExplanationStrength()+");");

        }
        out.flush();
      }

      Entity[] entities = model.getEntities();

      for(int i=0;i<entities.length;i++)
      {
        Entity current = entities[i];
        if(!shadow.hasEntity(current.getName()) && (current.getEntityObject() == null || !(current.getEntityObject() instanceof Number)))
        {
          Entity ent = new Entity(shadow, current.getName(),
                                  current.getDescription());

          out.println("\t\t\tEntity e" + i + " = new Entity(km,\"" +
                      current.getName() + "\",\"" + current.getDescription() +
                      "\");");
          if (current.getEntityObject() != null)
          {
            Object ob = current.getEntityObject();
            if (ob instanceof Double)
            {
              ent.setEntityObject(new Double( ( (Double) ob).toString()));
              out.println("\t\t\te" + i + ".setEntityObject(new " +
                          ob.getClass().getName() + "(" + ( (Double) ob).toString() +
                          "));");
            }
            else if (ob instanceof Class)
            {
              ent.setEntityObject((Class) ob);
              out.println("\t\t\te" + i + ".setEntityObject(" +
                          ( (Class) ob).getName() + ".class);");
            }
          }
        }
        out.flush();
      }

      out.println();

      Relation[] relations = model.getRelations();
      for(int i=0;i<relations.length;i++)
      {
        if(!hasRelation(shadow, relations[i]) &&
           (relations[i].getFrom().getEntityObject() == null || !(relations[i].getFrom().getEntityObject() instanceof Number)) &&
           (relations[i].getValue().getEntityObject() == null || !(relations[i].getValue().getEntityObject() instanceof Number)))
        {
          out.println("\t\t\tnew Relation(km.getEntity(\"" +
                      relations[i].getFrom().getName() + "\"), " +
                      "km.getRelationType(\"" +
                      relations[i].getRelationType().getName() + "\"), " +
                      "km.getEntity(\"" + relations[i].getValue().getName() +
                      "\"), " +
                      relations[i].getExplanationStrength() + ");");
//          System.err.println("  rel: "+relations[i]);
          new Relation(shadow.getEntity(relations[i].getFrom().getName()),
                       shadow.getRelationType(relations[i].getRelationType().getName()),
                       shadow.getEntity(relations[i].getValue().getName()),
                       relations[i].getExplanationStrength());
        }
        out.flush();
      }

      out.println();

      relations = shadow.getRelations();
      for(int i=0;i<relations.length;i++)
      {
        if (!hasRelation(model, relations[i]) && hasRelation(shadow, relations[i]) &&
            (relations[i].getFrom().getEntityObject() == null || !(relations[i].getFrom().getEntityObject() instanceof Number)) &&
            (relations[i].getValue().getEntityObject() == null || !(relations[i].getValue().getEntityObject() instanceof Number)))

        {
          System.out.println("REMOVING "+relations[i]+" hasRel: "+hasRelation(model, relations[i]));
          shadow.getEntity(relations[i].getFrom().getName()).getLocalRelation(
            shadow.getRelationType(relations[i].getRelationType().getName()),
            shadow.getEntity(relations[i].getValue().getName()),
            relations[i].getExplanationStrength()).remove();

          out.println("\t\t\tkm.getEntity(\""+relations[i].getFrom().getName()+"\").getLocalRelation("+
            "km.getRelationType(\""+relations[i].getRelationType().getName()+"\"), "+
            "km.getEntity(\""+relations[i].getValue().getName()+"\"), "+
            relations[i].getExplanationStrength()+").remove();");
        }
        out.flush();
      }
      out.println("\t\t}");
      out.println("\t\tcatch(CreekException e)");
      out.println("\t\t{");
      out.println("\t\t\te.printStackTrace();");
      out.println("\t\t}");
      out.println("\t}");
      out.println("}");

      out.close();
      // Testing difference of shadow and target
      System.out.println("Different: "+isModelDifferent(shadow, model));

    }
    catch(IOException e)
    {
      e.printStackTrace();
    }
    catch(CreekException e)
    {
      e.printStackTrace();
    }
  }

  public static boolean hasRelation(KnowledgeModel km, Relation test)
  {
//    System.out.println(" has "+test);
    Entity from = km.getEntity(test.getFrom().getName());

    if(from == null)
      return true;
    Relation[] rels = from.getLocalRelations();
    for(int i=0;i<rels.length;i++)
    {
      if(rels[i].getRelationType().getName().equals(test.getRelationType().getName()) &&
         rels[i].getValue().getName().equals(test.getValue().getName()) &&
         rels[i].getExplanationStrength() == test.getExplanationStrength())
      {
        return true;
      }
//      else
//        System.out.println("  -> "+rels[i]);
    }
    return false;
  }

  public static boolean isModelDifferent(KnowledgeModel km1, KnowledgeModel km2)
  {
    Entity[] ents1 = km1.getEntities();
    HashSet entsIn1Names = new HashSet();
    for(int i=0;i<ents1.length;i++)
    {
      if(ents1[i].getEntityObject() == null)
        entsIn1Names.add(ents1[i].getName());
      else
        entsIn1Names.add(ents1[i].getEntityObject());
    }

    Entity[] ents2 = km2.getEntities();
    HashSet entsIn2Names = new HashSet();
    for(int i=0;i<ents2.length;i++)
    {

      if(ents2[i].getEntityObject() == null)
        entsIn2Names.add(ents2[i].getName());
      else
        entsIn2Names.add(ents2[i].getEntityObject());
    }

    HashSet entsOnlyIn1 = new HashSet(entsIn1Names);
    entsOnlyIn1.removeAll(entsIn2Names);
    System.out.println("entsOnlyIn1: "+entsOnlyIn1.toString());

    HashSet entsOnlyIn2 = new HashSet(entsIn2Names);
    entsOnlyIn2.removeAll(entsIn1Names);
    System.out.println("entsOnlyIn2: "+entsOnlyIn2.toString());

    // Relations

    Relation[] rels1 = km1.getRelations();
    ArrayList relsMap1 = new ArrayList();
    for(int i=0;i<rels1.length;i++)
    {
//      System.out.println("  "+new RelationDescriptor(rels1[i]));
      relsMap1.add(new RelationDescriptor(rels1[i]));
    }

    Relation[] rels2 = km2.getRelations();
    ArrayList relsMap2 = new ArrayList();
    for(int i=0;i<rels2.length;i++)
    {
      relsMap2.add(new RelationDescriptor(rels2[i]));
    }

    HashSet relsOnlyIn1 = new HashSet(relsMap1);
    relsOnlyIn1.removeAll(relsMap2);
    System.out.println("relsOnlyIn1: "+relsOnlyIn1.toString());

    HashSet relsOnlyIn2 = new HashSet(relsMap2);
    relsOnlyIn2.removeAll(relsMap1);
    System.out.println("relsOnlyIn2: "+relsOnlyIn2.toString());

    return !(entsOnlyIn1.isEmpty() && entsOnlyIn2.isEmpty());
  }

  public static Object describeEntity(Entity ent)
  {
    if(ent.getEntityObject() == null)
      return ent.getName();
    else
      return ent.getEntityObject();

  }
}
  class RelationDescriptor
  {
    Relation original;
    RelationDescriptor(Relation original)
    {
      this.original = original;
    }

    public boolean equals(Object ob)
    {
      if(ob instanceof RelationDescriptor)
      {
        Relation rel1 = original;
        Relation rel2 = ((RelationDescriptor) ob).original;

//        System.out.println("!!"+BuildJavaModel.describeEntity(rel1.getFrom()).equals(BuildJavaModel.describeEntity(rel2.getFrom()))+" 1: "+
//          BuildJavaModel.describeEntity(rel1.getValue()).equals(BuildJavaModel.describeEntity(rel2.getValue()))+" 2: "+
//          BuildJavaModel.describeEntity(rel1.getRelationType()).equals(BuildJavaModel.describeEntity(rel2.getRelationType())));


        return BuildJavaModel.describeEntity(rel1.getFrom()).equals(BuildJavaModel.describeEntity(rel2.getFrom())) &&
          BuildJavaModel.describeEntity(rel1.getValue()).equals(BuildJavaModel.describeEntity(rel2.getValue())) &&
          BuildJavaModel.describeEntity(rel1.getRelationType()).equals(BuildJavaModel.describeEntity(rel2.getRelationType()));
      }

      return false;
    }

    public int hashCode()
    {
        return BuildJavaModel.describeEntity(original.getFrom()).hashCode()+
        BuildJavaModel.describeEntity(original.getValue()).hashCode()*7+
        BuildJavaModel.describeEntity(original.getRelationType()).hashCode()*11;
    }

    public String toString()
    {
      return BuildJavaModel.describeEntity(original.getFrom())+" - "+
      BuildJavaModel.describeEntity(original.getRelationType())+" - "+
      BuildJavaModel.describeEntity(original.getValue());

    }
  }

