package jcreek.examples;

/**
 * Title:        JavaCreek
 * Description:
 * Copyright:    Copyright (c) 2000
 * Company:      NTNU
 * @author
 * @version 1.0
 */
import jcreek.representation.*; import jcreek.representation.cbr.*;

public class FruitModel
{
  public FruitModel(String filename)
  {
    long time = System.currentTimeMillis();
    LocalKnowledgeModel km = generateModel();
    System.out.println("Time to generate model: "+(System.currentTimeMillis()-time)+" ms");
/*
    try
    {
      km.saveAs(filename);
    }
    catch(java.io.IOException e)
    {
      e.printStackTrace();
    }*/
  }

  public static void main(String[] argv)
  {
    new FruitModel(argv[0]);
  }

  public static LocalKnowledgeModel generateModel()
  {
      LocalKnowledgeModel km = new LocalKnowledgeModel();
      km.setName("FruitDomain");
      km.setTitle("Example domain by Martha Drum Jre.");
      km.setDescription("This domain is used to demonstrate the use of cases and parameters.");

      try
      {


   //The parameters in the fruit-domain
     Entity topNode = new Entity(km,"top node","Top node of the parameter tree.");
     Entity observable = new Entity(km,"observable","Observable Entity.");
     observable.addRelation("instance of", km.getEntity("Symbol"));
     Entity measurable = new Entity(km,"measurable","Measurable Entity.");
     Relation measurableInstanceOf = measurable.addRelation("instance of", km.getEntity("Symbol"));
     Entity miscellaneous = new Entity(km,"miscellaneous","Miscellaneous Entity.");
     miscellaneous.addRelation("instance of", km.getEntity("Symbol"));
     Entity solution = new Entity(km,"fruit category","the fruit category contains the possible fruit types that a fruit object can belog to (Banana, Apple etc.). This is also the solution of a case regarding a fruit classification task.");
     solution.addRelation("instance of", km.getEntity("Symbol"));


     //The observable Entitys
     Entity surface = new Entity(km,"surface","Surface of the fruit.");
     Entity shellColour = new Entity(km,"colour","Colour of the fruitshell.");
     Entity shape = new Entity(km,"shape","Shape of the fruit.");
     Entity size = new Entity(km,"size","Size of the fruit.");
     Entity meatColour = new Entity(km,"meat colour","Colour of the fruitmeat.");
     Entity seed = new Entity(km,"seed","Seed Size of the fruit.");

     //The measurable Entitys
     Entity dense = new Entity(km,"dense","Dense of the fruitmeat.");
     Entity taste = new Entity(km,"taste","Taste of the fruit.");
     Entity smell = new Entity(km,"smell","Smell of the fruit.");
     Entity feeling = new Entity(km,"feeling","Feeling of eating the fruit.");
     Entity weight = new Entity(km,"weight", "The weight of the fruit.");
     weight.addRelation("instance of", km.getEntity("Number")).addOverriding(measurableInstanceOf);;

     RelationType findingRCD = km.getRelationType("has finding");
     Relation defaultValue = findingRCD.addRelation("has value type", topNode);

       new RelationType(km,
                               findingRCD,
                               "has size",
                               "the size of the fruit",
                               "size of",
                               "",
                               0.5).addRelation("has value type", size).addOverriding(defaultValue);
       new RelationType(km,
                               findingRCD,
                               "has weight",
                               "the size of the fruit",
                               "weight of",
                               "",
                               0.5).addRelation("has value type", weight).addOverriding(defaultValue);

       new RelationType(km,
                               findingRCD,
                               "has taste",
                               "the tast of the fruit",
                               "taste of",
                               "",
                               0.5).addRelation("has value type", taste).addOverriding(defaultValue);


     //The miscellaneous Entitys
     Entity primaryUse = new Entity(km,"primary use","What the fruit is most used for.");
     Entity imported = new Entity(km,"imported","If the fruit is imported.");
     Entity growsOn = new Entity(km,"grows on","What the fruit grows on/in.");

     //The Solutions in the fruit-domain
     Entity apple = new Entity(km, "apple", "Apple-Fruit");
     Entity orange = new Entity(km, "orange", "Orange-Fruit");
     Entity pear = new Entity(km, "pear", "Pear-Fruit");
     Entity banana = new Entity(km, "banana", "Banana-Fruit");
     Entity melon= new Entity(km, "melon", "Melon-Fruit");
     Entity grape = new Entity(km, "grape", "Grape-Fruit");
     Entity rasin = new Entity(km, "rasin", "Dry Grape");
     Entity noFruit = new Entity(km, "no known fruit", "This is no Fruit");



     //The possible surface Entitys
     Entity smooth = new Entity(km,"smooth","Sooth surface of the fruit. ");
     Entity rough = new Entity(km,"rough","Rough surface of the fruit. ");
     Entity thick = new Entity(km,"thick","Thick surface of the fruit. ");
     Entity shiny = new Entity(km,"shiny","Shiny surface of the fruit. ");

     //The possible shellcolour Entitys
     Entity shellColourRed = new Entity(km,"red shell","The shellcolour is red. ");
     Entity shellColourGreen = new Entity(km,"green shell","shellcolour is green. ");
     Entity shellColourYellow = new Entity(km,"yellow shell","shellcolour is yellow. ");
     Entity shellColourBrown = new Entity(km,"brown shell","shellcolour is brown. ");
     Entity shellColourOrange = new Entity(km,"orange shell","shellcolour is orange. ");
     Entity shellColourWhite = new Entity(km,"white shell","shellcolour is white. ");
     Entity shellColourBlue = new Entity(km,"blue shell","shellcolour is blue. ");

     //The posssible meatcolour Entitys
     Entity meatColourRed = new Entity(km,"red meat","The meatcolour is red. ");
     Entity meatColourGreen = new Entity(km,"green meat","meatcolour is green. ");
     Entity meatColourYellow = new Entity(km,"yellow meat","meatcolour is yellow. ");
     Entity meatColourBrown = new Entity(km,"brown meat","meatcolour is brown. ");
     Entity meatColourOrange = new Entity(km,"orange meat","meatcolour is orange. ");
     Entity meatColourWhite = new Entity(km,"white meat","meatcolour is white. ");
     Entity meatColourBlue = new Entity(km,"blue meat","meatcolour is blue. ");


     //The possible shape Entitys
     Entity round = new Entity(km,"round","Round Fruit. ");
     Entity oblong = new Entity(km,"oblong","Oblong Fruit. ");
     Entity bended = new Entity(km,"bended","Bended fruit. ");
     Entity pearShape = new Entity(km,"pearShape","Pear shaped fruit. ");
     Entity uneven = new Entity(km,"uneven","Uneven shaped fruit. ");

     //The possible size Entitys
     Entity big = new Entity(km,"big","Big fruit. ");
     Entity normal = new Entity(km,"normal","Normal size for a fruit. ");
     Entity small = new Entity(km,"small","Small fruit. ");

     //The possible seed Entitys
     Entity manySeeds = new Entity(km,"many seeds","Many seeds. ");
     Entity oneSeed = new Entity(km,"one seed","One seed. ");
     Entity noSeed = new Entity(km,"no seeds","No observable seeds. ");

     //The possible dense Entitys
     Entity softDense = new Entity(km,"mellow or soft","Fruit is mellow. ");
     Entity hardDense = new Entity(km,"hard density","Fruit is hard. ");

     //The possible taste Entitys
     Entity sweetTaste = new Entity(km,"sweet taste","Fruit has sweet taste. ");
     Entity fullTaste = new Entity(km,"full taste","Fruit has full taste. ");
     Entity bitterTaste = new Entity(km,"bitter taste","Fruit has bitter taste. ");
     Entity waterLikeTaste = new Entity(km,"taste like water","Fruit has water-like taste. ");
     Entity sourTaste = new Entity(km,"sour taste","Fruit has sour taste. ");
     Entity fruitfulTaste = new Entity(km,"fruitful taste","Fruit has fruitful taste. ");
     Entity rottenTaste = new Entity(km,"rotten taste","Fruit has rotten taste. ");

     //The possible smell Entitys
     Entity sweetSmell = new Entity(km,"sweet smell","Fruit has sweet smell. ");
     Entity fullSmell = new Entity(km,"full smell","Fruit has full smell. ");
     Entity bitterSmell = new Entity(km,"bitter smell","Fruit has bitter smell. ");
     Entity fruitfulSmell = new Entity(km,"fruitful smell","Fruit has fruitful smell. ");
     Entity rottenSmell = new Entity(km,"rotten smell","Fruit has rotten smell. ");

     //The possible feeling Entitys
     Entity fullFeeling = new Entity(km,"full","Full after eating this fruit. ");
     Entity hungryFeeling = new Entity(km,"hungry feeling","Hungry after eating this fruit. ");
     Entity drunkFeeling = new Entity(km,"drunk feeling","Full after eating this fruit. ");
     Entity happyFeeling = new Entity(km,"happy feeling","Happy after eating this fruit. ");
     Entity monkeyFeeling = new Entity(km,"monkey feeling","Feels like a monkey after eating this fruit. ");



     //The possible primary use findings
     Entity eatRaw = new Entity(km,"eaten raw","Fruit is eaten raw. ");
     Entity cooked = new Entity(km,"cooked","Fruit is cooked. ");
     Entity drink = new Entity(km,"drink","Fruit is used to make a drink. ");
     Entity jam = new Entity(km,"jam","Fruit is made for jam. ");

     //The possible Imported Entitys
     Entity africa = new Entity(km,"from africa","Fruit is imported from Africa. ");
     Entity asia = new Entity(km,"from asia","Fruit is imported from Asia. ");
     Entity southAmerica = new Entity(km,"from south america","Fruit is imported from South America ");
     Entity europe = new Entity(km,"from europe","Fruit is not imported. ");

     //The possible GrowsOn Entitys
     Entity tree = new Entity(km,"tree","Grows on tree. ");
     Entity bush = new Entity(km,"bush","Grows on bush. ");
     Entity palm = new Entity(km,"palm","Grows on palm.");
     Entity ground = new Entity(km,"ground","Grows on ground. ");

     // Making new fruit case templates

     Entity fruitCase = new Entity(km, "Fruit Case","Fruit Case");
     fruitCase.addRelation("has weight", km.getEntity(CBRModel.NO_VALUE));
     fruitCase.addRelation("has finding", km.getEntity(CBRModel.NO_VALUE));
     fruitCase.addRelation("has size", km.getEntity(CBRModel.NO_VALUE));
     fruitCase.addRelation("subclass of", km.getEntity("Case"));

     Entity tastyFruitCase = new Entity(km, "Tasty Fruit Case","Tasty Fruit Case");
     tastyFruitCase.addRelation("has weight", km.getEntity(CBRModel.NO_VALUE));
     tastyFruitCase.addRelation("has taste", km.getEntity(CBRModel.NO_VALUE));
     tastyFruitCase.addRelation("has size", km.getEntity(CBRModel.NO_VALUE));
     tastyFruitCase.addRelation("subclass of", km.getEntity("Case"));

     //The Subclasses of topNode
     new Relation(observable, km.getRelationType(CBRModel.SUBCLASS_OF), topNode);
     new Relation(measurable, km.getRelationType(CBRModel.SUBCLASS_OF),topNode);
     new Relation(miscellaneous, km.getRelationType(CBRModel.SUBCLASS_OF),topNode);
     new Relation(solution, km.getRelationType(CBRModel.SUBCLASS_OF),topNode);

     //The Subclasses of the observable Entitys
     new Relation(surface, km.getRelationType(CBRModel.SUBCLASS_OF),observable);
     new Relation(shellColour, km.getRelationType(CBRModel.SUBCLASS_OF),observable);
     new Relation(shape, km.getRelationType(CBRModel.SUBCLASS_OF),observable);
     new Relation(size, km.getRelationType(CBRModel.SUBCLASS_OF),observable);
     new Relation(meatColour, km.getRelationType(CBRModel.SUBCLASS_OF),observable);
     new Relation(seed, km.getRelationType(CBRModel.SUBCLASS_OF),observable);

     //The Subclasses of the measurable Entitys
     new Relation(dense, km.getRelationType(CBRModel.SUBCLASS_OF),measurable);
     new Relation(taste, km.getRelationType(CBRModel.SUBCLASS_OF),measurable);
     new Relation(smell, km.getRelationType(CBRModel.SUBCLASS_OF),measurable);
     new Relation(feeling, km.getRelationType(CBRModel.SUBCLASS_OF), measurable);

     //The subclasses og the miscellaneous Entitys
     new Relation(primaryUse, km.getRelationType(CBRModel.SUBCLASS_OF),miscellaneous);
     new Relation(imported, km.getRelationType(CBRModel.SUBCLASS_OF),miscellaneous);
     new Relation(growsOn, km.getRelationType(CBRModel.SUBCLASS_OF),miscellaneous);

     //The possible Soulutions (subclasses and solution)
     new Relation(solution, km.getRelationType(CBRModel.HAS_INSTANCE), apple);
     new Relation(solution, km.getRelationType(CBRModel.HAS_INSTANCE), orange);
     new Relation(solution, km.getRelationType(CBRModel.HAS_INSTANCE), pear);
     new Relation(solution, km.getRelationType(CBRModel.HAS_INSTANCE), banana);
     new Relation(solution, km.getRelationType(CBRModel.HAS_INSTANCE), melon);
     new Relation(solution, km.getRelationType(CBRModel.HAS_INSTANCE), grape);
     new Relation(solution, km.getRelationType(CBRModel.HAS_INSTANCE), rasin);
     new Relation(solution, km.getRelationType(CBRModel.HAS_INSTANCE), noFruit);

  //Possible findings for the observable parameters

     //The possible findings for shellColour
     new Relation(shellColour, km.getRelationType(CBRModel.HAS_INSTANCE),shellColourRed);
     new Relation(shellColour, km.getRelationType(CBRModel.HAS_INSTANCE),shellColourGreen);
     new Relation(shellColour, km.getRelationType(CBRModel.HAS_INSTANCE),shellColourOrange);
     new Relation(shellColour, km.getRelationType(CBRModel.HAS_INSTANCE),shellColourYellow);
     new Relation(shellColour, km.getRelationType(CBRModel.HAS_INSTANCE),shellColourBlue);
     new Relation(shellColour, km.getRelationType(CBRModel.HAS_INSTANCE),shellColourWhite);
     new Relation(shellColour, km.getRelationType(CBRModel.HAS_INSTANCE),shellColourBrown);

     //The possible findings for shellMeat
     new Relation(meatColour, km.getRelationType(CBRModel.HAS_INSTANCE),meatColourRed);
     new Relation(meatColour, km.getRelationType(CBRModel.HAS_INSTANCE),meatColourGreen);
     new Relation(meatColour, km.getRelationType(CBRModel.HAS_INSTANCE),meatColourOrange);
     new Relation(meatColour, km.getRelationType(CBRModel.HAS_INSTANCE),meatColourYellow);
     new Relation(meatColour, km.getRelationType(CBRModel.HAS_INSTANCE),meatColourBlue);
     new Relation(meatColour, km.getRelationType(CBRModel.HAS_INSTANCE),meatColourWhite);
     new Relation(meatColour, km.getRelationType(CBRModel.HAS_INSTANCE),meatColourBrown);

     //The possible findings for shape
     new Relation(shape, km.getRelationType(CBRModel.HAS_INSTANCE),oblong);
     new Relation(shape, km.getRelationType(CBRModel.HAS_INSTANCE),round);
     new Relation(shape, km.getRelationType(CBRModel.HAS_INSTANCE),bended);
     new Relation(shape, km.getRelationType(CBRModel.HAS_INSTANCE),pearShape);
     new Relation(shape, km.getRelationType(CBRModel.HAS_INSTANCE),uneven);

     //The possible findings for surface
     new Relation(surface, km.getRelationType(CBRModel.HAS_INSTANCE),smooth);
     new Relation(surface, km.getRelationType(CBRModel.HAS_INSTANCE),rough);
     new Relation(surface, km.getRelationType(CBRModel.HAS_INSTANCE),thick);
     new Relation(surface, km.getRelationType(CBRModel.HAS_INSTANCE),shiny);

     //The possible findings for size
     new Relation(size, km.getRelationType(CBRModel.HAS_INSTANCE),big);
     new Relation(size, km.getRelationType(CBRModel.HAS_INSTANCE),normal);
     new Relation(size, km.getRelationType(CBRModel.HAS_INSTANCE),small);

     //The possible findings for seed
     new Relation(seed, km.getRelationType(CBRModel.HAS_INSTANCE),manySeeds);
     new Relation(seed, km.getRelationType(CBRModel.HAS_INSTANCE),oneSeed);
     new Relation(seed, km.getRelationType(CBRModel.HAS_INSTANCE),noSeed);

   //Possible findings for the measurable parameters


     //The possible findings for dense
     new Relation(dense, km.getRelationType(CBRModel.HAS_INSTANCE),softDense);
     new Relation(dense, km.getRelationType(CBRModel.HAS_INSTANCE),hardDense);

     //The possible findings for taste
     new Relation(taste, km.getRelationType(CBRModel.HAS_INSTANCE),sweetTaste);
     new Relation(taste, km.getRelationType(CBRModel.HAS_INSTANCE),fullTaste);
     new Relation(taste, km.getRelationType(CBRModel.HAS_INSTANCE),bitterTaste);
     new Relation(taste, km.getRelationType(CBRModel.HAS_INSTANCE),waterLikeTaste);
     new Relation(taste, km.getRelationType(CBRModel.HAS_INSTANCE),sourTaste);
     new Relation(taste, km.getRelationType(CBRModel.HAS_INSTANCE),fruitfulTaste);
     new Relation(taste, km.getRelationType(CBRModel.HAS_INSTANCE),rottenTaste);

     //The possible findings for smell
     new Relation(smell, km.getRelationType(CBRModel.HAS_INSTANCE),sweetSmell);
     new Relation(smell, km.getRelationType(CBRModel.HAS_INSTANCE),fullSmell);
     new Relation(smell, km.getRelationType(CBRModel.HAS_INSTANCE),bitterSmell);
     new Relation(smell, km.getRelationType(CBRModel.HAS_INSTANCE),fruitfulSmell);
     new Relation(smell, km.getRelationType(CBRModel.HAS_INSTANCE),rottenSmell);

     //The possible findings for feeling
     new Relation(feeling, km.getRelationType(CBRModel.HAS_INSTANCE),fullFeeling);
     new Relation(feeling, km.getRelationType(CBRModel.HAS_INSTANCE),hungryFeeling);
     new Relation(feeling, km.getRelationType(CBRModel.HAS_INSTANCE),drunkFeeling);
     new Relation(feeling, km.getRelationType(CBRModel.HAS_INSTANCE),happyFeeling);
     new Relation(feeling, km.getRelationType(CBRModel.HAS_INSTANCE),monkeyFeeling);


   //Possible findings for the miscellaneous parameters

     //The possible findings for primary use
     new Relation(primaryUse, km.getRelationType(CBRModel.HAS_INSTANCE),eatRaw);
     new Relation(primaryUse, km.getRelationType(CBRModel.HAS_INSTANCE),cooked);
     new Relation(primaryUse, km.getRelationType(CBRModel.HAS_INSTANCE),drink);
     new Relation(primaryUse, km.getRelationType(CBRModel.HAS_INSTANCE),jam);

     //The possible findings for imported
     new Relation(imported, km.getRelationType(CBRModel.HAS_INSTANCE),asia);
     new Relation(imported, km.getRelationType(CBRModel.HAS_INSTANCE),africa);
     new Relation(imported, km.getRelationType(CBRModel.HAS_INSTANCE),southAmerica);
     new Relation(imported, km.getRelationType(CBRModel.HAS_INSTANCE),europe);

     //The possible findings for grows on
     new Relation(growsOn, km.getRelationType(CBRModel.HAS_INSTANCE),tree);
     new Relation(growsOn, km.getRelationType(CBRModel.HAS_INSTANCE),bush);
     new Relation(growsOn, km.getRelationType(CBRModel.HAS_INSTANCE),palm);
     new Relation(growsOn, km.getRelationType(CBRModel.HAS_INSTANCE),ground);

    //Some eksamples of implies
     new Relation(hardDense, km.getRelationType(CBRModel.IMPLIES),apple);
     new Relation(meatColourWhite, km.getRelationType(CBRModel.IMPLIES),apple);
     new Relation(bitterSmell, km.getRelationType(CBRModel.IMPLIES),banana);
     new Relation(noSeed, km.getRelationType(CBRModel.IMPLIES),banana);
     new Relation(shellColourBlue, km.getRelationType(CBRModel.IMPLIES),noFruit);
     new Relation(africa, km.getRelationType(CBRModel.IMPLIES),banana);
     new Relation(asia, km.getRelationType(CBRModel.IMPLIES),noFruit);
     new Relation(palm, km.getRelationType(CBRModel.IMPLIES),banana);
     new Relation(bitterTaste, km.getRelationType(CBRModel.IMPLIES),grape);
     //Some eksamples of causes
     new Relation(banana, km.getRelationType(CBRModel.CAUSES),monkeyFeeling);
     new Relation(melon, km.getRelationType(CBRModel.CAUSES),happyFeeling);
     new Relation(grape, km.getRelationType(CBRModel.CAUSES),drunkFeeling);
     new Relation(banana, km.getRelationType(CBRModel.CAUSES),fullFeeling);
     new Relation(grape, km.getRelationType(CBRModel.CAUSES),hungryFeeling);
     new Relation(pear, km.getRelationType(CBRModel.CAUSES),fullFeeling);
     new Relation(apple, km.getRelationType(CBRModel.CAUSES),fullFeeling);
     new Relation(melon, km.getRelationType(CBRModel.CAUSES),drunkFeeling);
     new Relation(palm, km.getRelationType(CBRModel.CAUSES),banana);
     new Relation(grape, km.getRelationType(CBRModel.CAUSES), rasin);


     //The cases
     Case case1 = new Case(km, "Case#1" , "a Banana from Brasil");
     new Relation(case1, km.getRelationType(CBRModel.FINDING_RELATION), shellColourRed);
     new Relation(case1, km.getRelationType(CBRModel.FINDING_RELATION), big);
     new Relation(case1, km.getRelationType(CBRModel.FINDING_RELATION), southAmerica);
     new Relation(case1, km.getRelationType(CBRModel.FINDING_RELATION), palm);
     new Relation(case1, km.getRelationType(CBRModel.FINDING_RELATION), bended);
     // af.setDescription("is very important for this case.");
     new Relation(case1, km.getRelationType(CBRModel.FINDING_RELATION), drink);
     // bf.setDescription("is not very important for this case");
     new Relation(case1, km.getRelationType(CBRModel.FINDING_RELATION), monkeyFeeling);
     case1.addRelation("has weight", new DoubleEntity(km, new Double(0.3), weight));

     new Relation(case1, km.getRelationType(CBRModel.HAS_SOLUTION), banana);
     case1.setStatus(Case.SOLVEDCASE);

     Case case2 = new Case(km, "Case#2" ,"a red orange grown in israel(Jaffa)");
     new Relation(case2, km.getRelationType(CBRModel.FINDING_RELATION), shellColourRed);
//     df.setDescription("Makes a rare art of this fruit");
     new Relation(case2, km.getRelationType(CBRModel.FINDING_RELATION), sweetTaste);
     new Relation(case2, km.getRelationType(CBRModel.FINDING_RELATION), europe);
     new Relation(case2, km.getRelationType(CBRModel.FINDING_RELATION), round);
//     cf.setDescription("Is essensial for this case");
     new Relation(case2, km.getRelationType(CBRModel.FINDING_RELATION), thick);
     new Relation(case2, km.getRelationType(CBRModel.FINDING_RELATION), meatColourYellow);
     new Relation(case2, km.getRelationType(CBRModel.FINDING_RELATION), bitterSmell);
     new Relation(case2, km.getRelationType(CBRModel.FINDING_RELATION), happyFeeling);
     case2.addRelation("has weight", new DoubleEntity(km, new Double(0.2), weight));

     new Relation(case2, km.getRelationType(CBRModel.HAS_SOLUTION), orange);
     new DoubleEntity(km, new Double(0.2), weight);
     case2.setStatus(Case.SOLVEDCASE);



     Case case3 = new Case(km, "Case#3" ,"A blue small melon");
     new Relation(case3, km.getRelationType(CBRModel.FINDING_RELATION), shellColourBlue );
     new Relation(case3, km.getRelationType(CBRModel.FINDING_RELATION), eatRaw);
     new Relation(case3, km.getRelationType(CBRModel.FINDING_RELATION), small);
//     ef.setDescription("Consentrated compared to the general art.");
     new Relation(case3, km.getRelationType(CBRModel.FINDING_RELATION), meatColourRed);
     new Relation(case3, km.getRelationType(CBRModel.FINDING_RELATION), fullTaste);
     new Relation(case3, km.getRelationType(CBRModel.FINDING_RELATION), drunkFeeling);
     case3.addRelation("has weight", new DoubleEntity(km, new Double(0.8), weight));

     new Relation(case3, km.getRelationType(CBRModel.FINDING_RELATION), thick);
//     gf.setDescription("Important for this fruit category.");
     new Relation(case3, km.getRelationType(CBRModel.FINDING_RELATION),melon);
     case3.setStatus(Case.SOLVEDCASE);



     Case case4 = new Case(km, "Case#4" ,"A green, sour apple from Norway");
     new Relation(case4, km.getRelationType(CBRModel.FINDING_RELATION), shellColourGreen);
//     ff.setDescription("Light green to yellow");
     new Relation(case4, km.getRelationType(CBRModel.FINDING_RELATION), sweetTaste);
     new Relation(case4, km.getRelationType(CBRModel.FINDING_RELATION), round);
     new Relation(case4, km.getRelationType(CBRModel.FINDING_RELATION), shiny);
     new Relation(case4, km.getRelationType(CBRModel.FINDING_RELATION), meatColourWhite);
     new Relation(case4, km.getRelationType(CBRModel.FINDING_RELATION), sweetSmell);
     case4.addRelation("has weight", new DoubleEntity(km, new Double(0.08), weight));

     new Relation(case4, km.getRelationType(CBRModel.FINDING_RELATION), apple);
     case4.setStatus(Case.SOLVEDCASE);


     Case case5 = new Case(km, "Case#5" ,"a red big apple from Norway");
     new Relation(case5, km.getRelationType(CBRModel.FINDING_RELATION), shellColourRed);
     new Relation(case5, km.getRelationType(CBRModel.FINDING_RELATION), fruitfulTaste);
     new Relation(case5, km.getRelationType(CBRModel.FINDING_RELATION), meatColourWhite);
//     jf.setDescription("Fruitful for this fruit.");
     new Relation(case5, km.getRelationType(CBRModel.FINDING_RELATION), normal);
     new Relation(case5, km.getRelationType(CBRModel.FINDING_RELATION), tree);
     new Relation(case5, km.getRelationType(CBRModel.FINDING_RELATION), sweetSmell);
     new Relation(case5, km.getRelationType(CBRModel.HAS_SOLUTION), apple);
     new Relation(case5, km.getRelationType(CBRModel.FINDING_RELATION), fullFeeling);
     case5.addRelation("has weight", new DoubleEntity(km, new Double(0.15), weight));

     case5.setStatus(Case.SOLVEDCASE);


     Case case6 = new Case(km, "Case#6" ,"a small brown banana");
     new Relation(case6, km.getRelationType(CBRModel.FINDING_RELATION), shellColourBrown);
     new Relation(case6, km.getRelationType(CBRModel.FINDING_RELATION), fruitfulTaste);
     new Relation(case6, km.getRelationType(CBRModel.FINDING_RELATION), palm);
//     kf.setDescription("This fruit has not been growing for a while.");
     new Relation(case6, km.getRelationType(CBRModel.FINDING_RELATION), small);
     new Relation(case6, km.getRelationType(CBRModel.FINDING_RELATION), softDense);
     new Relation(case6, km.getRelationType(CBRModel.FINDING_RELATION), fullFeeling);
     new Relation(case6, km.getRelationType(CBRModel.FINDING_RELATION), rottenSmell);
     case6.addRelation("has weight", new DoubleEntity(km, new Double(0.16), weight));

     new Relation(case6, km.getRelationType(CBRModel.HAS_SOLUTION), banana);
     case6.setStatus(Case.SOLVEDCASE);


     Case case7 = new Case(km, "Case#7" ,"a yellow melon from Kongo");
     new Relation(case7, km.getRelationType(CBRModel.FINDING_RELATION), manySeeds);
     new Relation(case7, km.getRelationType(CBRModel.FINDING_RELATION), fruitfulTaste);
//     lf.setDescription("Very hard and robust");
     new Relation(case7, km.getRelationType(CBRModel.FINDING_RELATION), meatColourWhite);
     new Relation(case7, km.getRelationType(CBRModel.FINDING_RELATION), ground);
     new Relation(case7, km.getRelationType(CBRModel.FINDING_RELATION), africa);
     new Relation(case7, km.getRelationType(CBRModel.HAS_SOLUTION), melon);
     new Relation(case7, km.getRelationType(CBRModel.FINDING_RELATION), happyFeeling);
     case7.addRelation("has weight", new DoubleEntity(km, new Double(0.5), weight));

     case7.setStatus(Case.SOLVEDCASE);


     Case case8 = new Case(km, "Case#8" ,"an apple");
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), uneven);
//     mf.setDescription("Very important.");
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), fruitfulTaste);
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), meatColourWhite);
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), normal);
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), tree);
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), sweetSmell);
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), hardDense);
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), europe);
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), fullFeeling);
     new Relation(case8, km.getRelationType(CBRModel.FINDING_RELATION), cooked);
     case8.addRelation("has weight", new DoubleEntity(km, new Double(0.1), weight));

     new Relation(case8, km.getRelationType(CBRModel.HAS_SOLUTION), apple);
     case8.setStatus(Case.SOLVEDCASE);


     Case case9 = new Case(km, "Case#9" ,"a very nice banana made for milkshake");
     new Relation(case9, km.getRelationType(CBRModel.FINDING_RELATION), drink);
     new Relation(case9, km.getRelationType(CBRModel.FINDING_RELATION), fruitfulTaste);
     new Relation(case9, km.getRelationType(CBRModel.FINDING_RELATION), shellColourYellow);
     new Relation(case9, km.getRelationType(CBRModel.FINDING_RELATION), normal);
     new Relation(case9, km.getRelationType(CBRModel.FINDING_RELATION), bended);
     new Relation(case9, km.getRelationType(CBRModel.FINDING_RELATION), bitterSmell);
     new Relation(case9, km.getRelationType(CBRModel.FINDING_RELATION), softDense);
     case9.addRelation("has weight", new DoubleEntity(km, new Double(0.2), weight));

     new Relation(case9, km.getRelationType(CBRModel.HAS_SOLUTION), banana);
     case9.setStatus(Case.SOLVEDCASE);


     Case case10 = new Case(km, "Case#10" ,"a white muscadet grape");
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), smooth);
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), bitterTaste);
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), small);
//     sf.setDescription("Very special drink for this fruit.");
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), shellColourGreen);
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), meatColourGreen);
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), softDense);
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), drunkFeeling);
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), monkeyFeeling);
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), asia);
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), bush);
     new Relation(case10, km.getRelationType(CBRModel.FINDING_RELATION), new DoubleEntity(km, new Double(0.01), weight));

     new Relation(case10, km.getRelationType(CBRModel.HAS_SOLUTION), grape);
     case10.setStatus(Case.SOLVEDCASE);


     Case case11 = new Case(km, "Case#11" ,"a unsolved copy of case#9");
     new Relation(case11, km.getRelationType(CBRModel.FINDING_RELATION), drink);
     new Relation(case11, km.getRelationType(CBRModel.FINDING_RELATION), fruitfulTaste);
     new Relation(case11, km.getRelationType(CBRModel.FINDING_RELATION), shellColourYellow);
     new Relation(case11, km.getRelationType(CBRModel.FINDING_RELATION), normal);
     new Relation(case11, km.getRelationType(CBRModel.FINDING_RELATION), bended);
     new Relation(case11, km.getRelationType(CBRModel.FINDING_RELATION), bitterSmell);
     new Relation(case11, km.getRelationType(CBRModel.FINDING_RELATION), softDense);
     new Relation(case11, km.getRelationType(CBRModel.FINDING_RELATION), meatColourBlue);
     case11.addRelation("has weight", new DoubleEntity(km, new Double(0.18), weight));

     case11.setStatus(Case.UNSOLVEDCASE);


     Case case12 = new Case(km, "Case#12" ,"a green banana");
     new Relation(case12, km.getRelationType(CBRModel.HAS_SOLUTION), eatRaw);
     case12.addRelation("has taste", fruitfulTaste);
     new Relation(case12, km.getRelationType(CBRModel.FINDING_RELATION), shellColourGreen);
     new Relation(case12, km.getRelationType(CBRModel.FINDING_RELATION), meatColourWhite);
     new Relation(case12, km.getRelationType(CBRModel.FINDING_RELATION), hungryFeeling);
     new Relation(case12, km.getRelationType(CBRModel.FINDING_RELATION), bended);
     new Relation(case12, km.getRelationType(CBRModel.FINDING_RELATION), bitterSmell);
     case12.addRelation("has weight", new DoubleEntity(km, new Double(0.15), weight));

     new Relation(case12, km.getRelationType(CBRModel.HAS_SOLUTION), banana);
     case12.setStatus(Case.SOLVEDCASE);

     Case case13 = new Case(km, "Case#13" ,"unsolved strangefruit");
     new Relation(case13, km.getRelationType(CBRModel.FINDING_RELATION), drink);
     new Relation(case13, km.getRelationType(CBRModel.FINDING_RELATION), fruitfulTaste);
     new Relation(case13, km.getRelationType(CBRModel.FINDING_RELATION), shellColourBlue);
     new Relation(case13, km.getRelationType(CBRModel.FINDING_RELATION), happyFeeling);
     new Relation(case13, km.getRelationType(CBRModel.FINDING_RELATION), meatColourRed);
     new Relation(case13, km.getRelationType(CBRModel.FINDING_RELATION), smooth);
     new Relation(case13, km.getRelationType(CBRModel.FINDING_RELATION), sweetSmell);
     case13.addRelation("has weight", new DoubleEntity(km, new Double(0.3), weight));
     new Relation(case13, km.getRelationType(CBRModel.FINDING_RELATION), tree);
     case13.setStatus(Case.UNSOLVEDCASE);


     }

     catch(NameAlreadyExistException ce)
      {
        System.out.println("Failed to make test model: "+ce.getMessage());
      }
    catch(NoSuchRelationTypeException e)
    {
      e.printStackTrace();
    }
    return km;
  }
}
