package jcreek.gui;

import java.awt.*;
import javax.swing.JPanel;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;
import jcreek.representation.*; import jcreek.representation.cbr.*;
import jcreek.representation.inference.*;
import jcreek.util.graph.*;
import jcreek.gui.conceptmap.*;
import jcreek.util.graph.layout.SpringEmbedderLayout;
import jcreek.gui.conceptmap.tool.DragTool;
import jcreek.util.*;

/**
 * Visualizes an ExplanationModel for an inherited relation. The complete submodel
 * for the relation is displayed in one pane and a list of explanation paths
 * in another. When an explanation path is chosen, this path is highlighted.
 *
 * @author Frode Srmo
 * @version 1.1
 */
public class ExplanationView extends JPanel
{

  private ConceptMapView        conceptView;
  private JTable                explanationList;
  private Relation              relation;
  private Explanation[]         explanations;
  private KnowledgeModel        km;
  private double                totalStrength;
  private Vector entities = new Vector();
  private Vector relations = new Vector();
  private JSplitPane mainPane;

  public ExplanationView(Relation target)
  {
    try
    {
      explanations = target.getExplanations();
    }
    catch(TaskTooComplexException e)
    {
      explanations = new Explanation[0];
    }

    km = target.getFrom().getKnowledgeModel();
    totalStrength = target.getExplanationStrength();
    init();
  }

  public ExplanationView(Explanation[] expl)
  {
    totalStrength = 1.0;
    if(expl.length > 0 && expl[0].size() > 0)
      km = expl[0].relationAt(0).getFrom().getKnowledgeModel();
    explanations = expl;
  }

  void init()
  {
    setLayout(new BorderLayout());

    // Setting up the ConceptView.

    SubModel explanationModel = Explanation.toSubModel(explanations);
    conceptView = new SubModelView(km, explanationModel, true, true);
    conceptView.setController(new ExplanationViewMapController(this));
    conceptView.setBackground(Color.WHITE);

    Entity[] ents = explanationModel.getEntities();
    for(int i=0;i<ents.length;i++)
      conceptView.getModel().addEntity(ents[i], new Point(0,0));

    Relation[] rels = explanationModel.getRelations();
    for(int i=0;i<rels.length;i++)
      conceptView.getModel().addRelation(rels[i]);

    explanationList = new JTable(new ExplanationTableModel(this))
    {
      ExplanationTableCellRenderer myRenderer = new ExplanationTableCellRenderer(conceptView, explanations);
      public TableCellRenderer getCellRenderer(int column, int row)
      {
        return myRenderer;
      }
    };

    explanationList.setRowHeight(50);
    mainPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, conceptView, explanationList);

    mainPane.addComponentListener(new ComponentListener()
                                  {
      public void componentHidden(java.awt.event.ComponentEvent e) { }
      public void componentShown(java.awt.event.ComponentEvent e) { }
      public void componentMoved(java.awt.event.ComponentEvent e) { }
      public void componentResized(java.awt.event.ComponentEvent e)
      {
        mainPane.setDividerLocation(0.6);
      }
                                  });

    add(mainPane, BorderLayout.CENTER);

    DragTool dg = new DragTool(conceptView);
/*
    SpringEmbedderLayout layout = new SpringEmbedderLayout(new Rectangle(0,0,300,300), conceptView.getGraph(), 5);
    layout.layout(100);
    conceptView.setGraph(conceptView.getGraph()); */

  }

  class ExplanationViewMapController extends DefaultConceptMapController
  {
    private ExplanationView master;

    ExplanationViewMapController(ExplanationView master)
    {
      super();
      this.master = master;
    }

    public Color getRelationColor(Relation relation)
    {
      if(master.explanations != null && master.explanations.length >= master.explanationList.getSelectedRow() && master.explanationList.getSelectedRow() > 0)
      {
        if(master.explanations[master.explanationList.getSelectedRow()-1].contains(relation))
          return super.getRelationSelectedColor(relation);
        else
          return super.getRelationColor(relation).brighter();
      }
      return super.getRelationColor(relation);
    }

    public Color getEntityColor(Entity entity)
    {
      if(master.explanations != null && master.explanations.length >= master.explanationList.getSelectedRow() && master.explanationList.getSelectedRow() > 0)
      {
        if(master.explanations[master.explanationList.getSelectedRow()-1].contains(entity))
          return super.getEntitySelectedColor(entity);
        else
          return super.getEntityColor(entity).brighter();
      }
      return super.getEntityColor(entity);
    }

    public Dimension getViewInsets()
    {
      return new Dimension(60, 60);
    }
  }

//----------------------------------------------------------------------------//
  class ExplanationTableCellRenderer extends DefaultTableCellRenderer

  {

    ConceptMapView cv;

    ExplanationTableCellRenderer(ConceptMapView cv, Explanation[] explanations)
    {
      this.cv = cv;

    }



    public Component getTableCellRendererComponent(JTable table,

                                                 Object value,

                                                 boolean isSelected,

                                                 boolean hasFocus,

                                                 int row,

                                                 int column)

    {

      Component ret = super.getTableCellRendererComponent(table,value,isSelected,hasFocus,row,column);

      if(JComponent.class.isAssignableFrom(ret.getClass()))

      {

//      System.out.println("In ExplanationTableCellRederer. Column: "+column);

        JComponent comp;

        if(column == 0 && row > 0)

        {
          JTextArea ta = new JTextArea(((Explanation) value).text(),3,1);

          ta.setLineWrap(true);

          ta.setWrapStyleWord(true);

          ta.setFont(new Font("Arial", Font.PLAIN, 10));

          ta.setBackground(ret.getBackground());

          ta.setForeground(ret.getForeground());

          comp = ta;
        }

        else

          comp = (JComponent) ret;

          if(isSelected && column == 0)
            cv.repaint();

        comp.setToolTipText(value.toString());

        return comp;

      }

      return ret;

    }

  }



  class ExplanationTableModel extends AbstractTableModel
  {
    ExplanationView parent;
    Vector         listeners = new Vector();

    ExplanationTableModel(ExplanationView parent)
    {
      this.parent = parent;
    }

    public int getRowCount()
    {
      try
      {
        return parent.explanations.length+1;
      }
      catch (NullPointerException ex)
      {
        return 0;
      }//catch
    }

    public int getColumnCount()
    {
      return 2;
    }

    public Object getValueAt(int row, int column)
    {
      if(row == 0)
      {
        if(column == 0)
          return "Total";
        else
          return new Double(parent.totalStrength).toString();
      }
      else
      {
        if(column == 0)
          return parent.explanations[row-1];
        else
          return new Double(parent.explanations[row-1].getStrength()).toString();
      }
    }



    public String getColumnName(int column)

    {

      if(column == 0)

        return "Explanation";

      else

        return "Strength";

    }

 }//ExplanationTableModel


}//Class ExplanationView



//----------------------------------------------------------------------------//

//----------------------------------------------------------------------------//









