package jcreek.gui.conceptmap;

/**
 * <p>Title: ConceptMapController</p>
 * <p>Description: The controller for how a ConceptMapView should display
 * a ConceptMapModel. This is a controller in the model-view-controller
 * design template sense.</p>
 * @author Frode Srmo
 * @version 1.0
 */

import jcreek.representation.Entity;
import jcreek.representation.Relation;
import java.awt.Color;
import java.awt.Font;
import java.awt.Dimension;

public interface ConceptMapController
{
  /**
   * The color to use when diplaying this entity.
   */
  public Color getEntityColor(Entity entity);

  /**
   * The color of the font used to draw the entity fonts.
   */
  public Color getEntityFontColor(Entity entity);

  /**
   * The color of the font used to draw the relation fonts.
   */
  public Color getRelationFontColor(Relation entity);

  /**
   * The color of a selected relation.
   */
  public Color getRelationSelectedColor(Relation relation);

  /**
   * The color to draw the "selection cross" on the entity if it is selected.
   */
  public Color getEntitySelectedColor(Entity entity);

  /**
   * The size of the circle representing an entity.
   */
  public Dimension getEntitySize();

  /**
   * The extra border space drawn outside the minimal square occupied by
   * all the entities in the model. This means that the entity with the lowest
   * x value will be drawn at exactly getViewInsets().width in the view.
   */
  public Dimension getViewInsets();

  /**
   * The font to use when displaying this entity.
   */
  public Font getEntityFont(Entity entity, double scale);

  /**
   * The color to use when diplaying this relation.
   */
  public Color getRelationColor(Relation relation);

  /**
   * The font to use when displaying this entity.
   */
  public Font getRelationFont(Relation relation, double scale);

  /**
   * Should entity names be displayed?
   *
   * @param scale The current scale (with 1.0 meaning normal scale).
   */
  public boolean showEntityNames(double scale);

  /**
   * Should relation names be displayed?
   *
   * @param scale The current scale (with 1.0 meaning normal scale).
   */
  public boolean showRelationNames(double scale);
}