package jcreek.gui.conceptmap;

/**
 * The default implementation of the ConceptMapController interface.
 *
 * @author Frode Srmo
 * @version 1.0
 */

import jcreek.representation.Entity;
import jcreek.representation.Relation;
import java.awt.Color;
import java.awt.Font;
import java.awt.Dimension;

public class DefaultConceptMapController implements ConceptMapController
{
  protected static final Dimension ENTITY_SIZE = new Dimension(20, 20);
  protected static final Color ENTITY_COLOR = new Color(120,120,240);
  protected static final Color ENTITY_SELECTED_COLOR = new Color(240,120,120);    // new Color(80,180,180);
  protected static final Color RELATION_COLOR = new Color(120,120,240);
  protected static final Color RELATION_SELECTED_COLOR = new Color(240,120,120);
  protected static final Color ENTITY_FONT_COLOR = new Color(0,0,0);
  protected static final Color RELATION_FONT_COLOR = new Color(0,0,0);
  protected static final Dimension INSETS = new Dimension(100, 100);

  protected Font entityFont;
  protected double currentScale;
  protected Font relationFont;

  /**
   * The color to use when diplaying this entity.
   */
  public Color getEntityColor(Entity entity)
  {
    return ENTITY_COLOR;
  }


  /**
   * The color of the font used to draw the entity fonts.
   */
  public Color getEntityFontColor(Entity entity)
  {
    return ENTITY_FONT_COLOR;
  }

  /**
   * The color to draw the "selection cross" on the entity if it is selected.
   */
  public Color getEntitySelectedColor(Entity entity)
  {
    return ENTITY_SELECTED_COLOR;
  }

  /**
   * The size of the circle representing an entity.
   */
  public Dimension getEntitySize()
  {
    return ENTITY_SIZE;
  }

  /**
   * The extra border space drawn outside the minimal square occupied by
   * all the entities in the model. This means that the entity with the lowest
   * x value will be drawn at exactly getViewInsets().width in the view.
   */
  public Dimension getViewInsets()
  {
    return INSETS;
  }

  protected void updateFonts(double scale)
  {
    entityFont = new Font("Arial",Font.PLAIN,(int) Math.round(14*scale));
    relationFont = new Font("Arial",Font.PLAIN,(int) Math.round(12*scale));
    currentScale = scale;
  }

  /**
   * The font to use when displaying this entity.
   */
  public synchronized Font getEntityFont(Entity entity, double scale)
  {
    if(scale != currentScale)
      updateFonts(scale);
    return entityFont;
  }

  /**
   * The color to use when diplaying this relation.
   */
  public Color getRelationColor(Relation relation)
  {
    return RELATION_COLOR;
  }

  /**
   * The color to use when diplaying this relation.
   */
  public Color getRelationSelectedColor(Relation relation)
  {
    return RELATION_SELECTED_COLOR;
  }

  /**
   * The color of the font used to draw the relation fonts.
   */
  public Color getRelationFontColor(Relation entity)
  {
    return RELATION_FONT_COLOR;
  }

  /**
   * The font to use when displaying this entity.
   */
  public Font getRelationFont(Relation relation, double scale)
  {
    if(scale != currentScale)
      updateFonts(scale);
    return relationFont;
  }

  /**
   * Should entity names be displayed?
   *
   * @param scale The current scale (with 1.0 meaning normal scale).
   */
  public boolean showEntityNames(double scale)
  {
    return scale > 0.2;
  }

  /**
   * Should relation names be displayed?
   *
   * @param scale The current scale (with 1.0 meaning normal scale).
   */
  public boolean showRelationNames(double scale)
  {
//    return false;
    return scale > 0.5;
  }
}
