package jcreek.gui.conceptmap;

import jcreek.representation.Relation;
import jcreek.representation.Entity;
import jcreek.representation.KnowledgeModel;

import jcreek.util.graph.Node;
import jcreek.util.graph.Edge;

/**
 * Represents a specialization of Edge (as in a graph edge) that encapsulates
 * a Creek Relation.
 *
 * @author Frode Srmo
 * @version 2.0
 */

public class VisualRelation extends Edge
{
  private VisualEntity       from;
  private VisualEntity       to;
  private transient Relation creekLRelation;
  private Object             relationID;

  private static final long serialVersionUID=5376994509922361648L;

  /**
   * Creates a new VisualRelation.
   *
   * @param model The ConceptMapModel this VisualRelation is a part of.
   * @param relation The Creek relation this VisualRelation encapsulates.
   */
  public VisualRelation(VisualEntity from, VisualEntity to, Relation relation)
  {
      super(from, to);
      creekLRelation = relation;
      relationID = relation.getID();
      if(from.getGraph() != null)
        from.getGraph().addEdge(this);
  }

  /**
   * Returns true if the supplied relation is the same as the relation
   * encapsulated by this VisualRelation.
   */
  public boolean represents(Relation r)
  {
    return creekLRelation.equals(r);
  }

  /**
   * Returns the relation encapsulated by this VisualRelation.
   */
  public Relation getRelation()
  {
    return creekLRelation;
  }

  /**
   * Overridden to fulfill the hashCode() contract since equals() has been
   * overridden.
   */
  public int hashCode()
  {
    return 7+creekLRelation.hashCode();
  }

  /**
   * Two VisualRelations are considered equal if they encapsulate the same
   * relation.
   */
  public boolean equals(Object ob)
  {
    if(ob instanceof VisualRelation)
      return ((VisualRelation) ob).creekLRelation.equals(creekLRelation);
    else
      return false;
  }

  public String toString()
  {
    return "VisualRelation for "+getRelation().toString();
  }

  public void connectAfterLoad(KnowledgeModel model)
  {
    creekLRelation = model.getRelation(relationID);
  }
}


