package jcreek.gui.reasoning;

import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.EmptyBorder;
import jcreek.representation.*; import jcreek.representation.cbr.*;
import jcreek.reasoning.*;
import jcreek.gui.conceptmap.ConceptMapModel;
// import jcreek.visualizer.VisualEntityOrganizer;

/**
 * This pane shows a comparison between two cases (a CaseComparison) by displaying
 * the list of findings direcly matched, partially matched and unmatched in the
 * two cases. When one of these are selected, details on this particular comparison
 * is displayed in the bottom of the pane.
 *
 * @author Frode Srmo 2002
 * @version 1.0
 */

public class CaseComparisonPane extends EntityComparisonPane
{
  private JPanel explanationPane;
  private FeaturePane featurePane;
  private JSplitPane mainPanel;
  private JPanel topPane;

  public CaseComparisonPane(CaseComparison comparison)
  {
    super(comparison);
    type = PANE_TYPE_LARGE;
  }

  public CaseComparison getCaseComparison()
  {
    return (CaseComparison) getEntityComparison();
  }

  protected void buildPane()
  {
    topPane = new JPanel();
    topPane.setLayout(new BorderLayout());
    topPane.setBorder(new javax.swing.border.EmptyBorder(10,10,10,10));

    JPanel namePanel = new JPanel();
    namePanel.setLayout(new GridLayout(1,3));
    JLabel case1 = new JLabel("  "+getEntityComparison().getOrigin().getName(), JLabel.LEFT);
    case1.setFont(new Font("Arial",Font.BOLD,18));
    case1.setForeground(new Color((float) 0.7, (float) 0.2, (float) 0.2));
    namePanel.add(case1);
    JLabel similarity = new JLabel(""+getEntityComparison().getPercentageStrength()+"%", JLabel.CENTER);
    similarity.setFont(new Font("Arial",Font.BOLD,16));
    similarity.setForeground(new Color((float) 0.8, (float) 0.2, (float) 0.2));
    namePanel.add(similarity);
    JLabel case2 = new JLabel(getEntityComparison().getTarget().getName()+"  ", JLabel.RIGHT);
    case2.setFont(new Font("Arial",Font.BOLD,18));
    case2.setForeground(new Color((float) 0.7, (float) 0.2, (float) 0.2));
    namePanel.add(case2);
    topPane.add(namePanel, BorderLayout.NORTH);

    explanationPane = new JPanel();
    featurePane = new FeaturePane();
    explanationPane.setMinimumSize(new Dimension(300, 300));
    explanationPane.setPreferredSize(new Dimension(300, 600));

    mainPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT, new JScrollPane(featurePane), explanationPane)
                {
      public void setBounds(int x, int y, int a, int b)
      {
        super.setBounds(x, y, a, b);
        int pos = getSize().height-explanationPane.getMinimumSize().height;
        int altPos = featurePane.getPreferredSize().height+10;
        if(altPos < pos);
          pos = altPos;
        this.setDividerLocation(pos);
      }
                };
    topPane.add(mainPanel, BorderLayout.CENTER);

    setLayout(new BorderLayout());
    add(topPane, BorderLayout.CENTER);
  }

/*  public static void main(String argv[])
  {
//    System.out.println("Generating model");
    try
    {
      LocalKnowledgeModel km = examples.FruitModel.generateModel();
//      km.saveAs("C:\\temp\\fruit.km");
//      LocalKnowledgeModel km = new LocalKnowledgeModel("C:\\temp\\fruit.km");
      System.out.println("Done. Resetting");
      RetrieveResult res = new RetrieveResult(new Case(km.getEntity("Case#9")), false);
      System.out.println("Done. Activating");
      res.activate();
      System.out.println("Done. Explaining");
      res.explain();
      System.out.println("Done. Focus");

      res.focus(100,0.0);

      CaseComparison comp = res.getComparison(1);
      CaseComparison comp = new CaseComparison(new Case(km.getEntity("Case#9")), new Case(km.getEntity("Case#12")));
      comp.activate();
      comp.explain();

      JFrame frame = new JFrame("Test");
//      res.update();
//      frame.setContentPane(new RetrieveResultPane(res));
      frame.setContentPane(new CaseComparisonPane(comp));
      frame.setSize(800,800);
      frame.show();
    }
    catch(jcreek.util.CreekException e)
    {
      e.printStackTrace();
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }*/
  public void showDetails(EntityComparison currentComparison)
  {
    if (currentComparison != null)
    {
      EntityComparisonPane ecp = EntityComparisonPane.createComparisonPane(
        currentComparison);
      if (ecp != null)
      {
        if (ecp.getPaneType() == EntityComparisonPane.PANE_TYPE_SMALL)
        {
          explanationPane.removeAll();
          explanationPane.setLayout(new BorderLayout());
          explanationPane.add(ecp, BorderLayout.CENTER);
          validate();
          repaint();
        }
        else
        {
          ecp.addBackButton(this, topPane);
          validate();
          repaint();
        }
      }
    }
  }
  class FeaturePane extends JPanel implements FocusListener
  {
    JTable directMatchTable;
    JTable unmatchedOriginTable;
    JTable unmatchedTargetTable;
    JTable explainedTable;
    JPanel explanationTextPanel;
    ComparisonSubModelView submodelPane;
    EntityComparison currentComparison = null;

    JTextArea textExplanationArea;
    JButton detailsButton;

    FeaturePane()
    {
      explanationTextPanel = new JPanel();
      explanationTextPanel.setLayout(new BorderLayout());

      textExplanationArea = new JTextArea(6, 25);
//    textExplanationArea.setRows(10);
      textExplanationArea.setEditable(false);
      textExplanationArea.setWrapStyleWord(true);
      textExplanationArea.setLineWrap(true);

      explanationTextPanel.add(new JScrollPane(textExplanationArea),
                               BorderLayout.CENTER);
      detailsButton = new JButton("Details");
      detailsButton.setEnabled(false);
      detailsButton.addActionListener(new ActionListener()
      {
        public void actionPerformed(ActionEvent e)
        {
          showDetails(currentComparison);
        }
      });
      explanationTextPanel.add(detailsButton, BorderLayout.SOUTH);

      explanationPane.setLayout(new BorderLayout());
      explanationPane.add(explanationTextPanel, BorderLayout.EAST);

      submodelPane = new ComparisonSubModelView(getCaseComparison().toSubModel(),
                                          getCaseComparison().getOrigin(),
                                          getCaseComparison().getTarget());

      explanationPane.add(submodelPane, BorderLayout.CENTER);

      explanationTextPanel.setPreferredSize(new Dimension(300, 10000));

      setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

      JLabel directMatchLabel = new JLabel("Directly matched features     ",
                                           JLabel.CENTER);
      directMatchLabel.setFont(new Font("Arial", Font.BOLD, 14));
      directMatchLabel.setBorder(new EmptyBorder(10, 0, 10, 0));
      directMatchLabel.setForeground(new Color( (float) 0.2, (float) 0.2,
                                               (float) 0.7));
      directMatchLabel.setMaximumSize(new Dimension(20000,
        directMatchLabel.getPreferredSize().height));
      add(directMatchLabel);

      ComparisonTableModel model = new ComparisonTableModel(getCaseComparison().
        getCompletelyMatchedComparisons(), getCaseComparison().getRelevanceFactorSum());
      directMatchTable = new JTable(model);
      directMatchTable.setDefaultRenderer(String.class,
                                          new RelevanceAwareCellRenderer(model));
      directMatchTable.setDefaultRenderer(Integer.class,
                                          new
                                          PercentageCellRenderer(Color.blue, model));
//    directMatchTable.setDefaultRenderer(Integer.class, new PercentageCellRenderer(Color.red, model));
      directMatchTable.addFocusListener(this);
//    directMatchTable.getColumnModel().getColumn(1).setCellRenderer(new AlignmentTableCellRenderer(JLabel.CENTER));
//    directMatchTable.getColumnModel().getColumn(2).setCellRenderer(new AlignmentTableCellRenderer(JLabel.RIGHT));
      directMatchTable.getSelectionModel().addListSelectionListener(new
        ListSelectionListener()
      {
        public void valueChanged(ListSelectionEvent e)
        {
//        if(directMatchTable.getSelectedRow() == -1 && explainedTable.getSelectedRow() == -1)
//          viewCaseExplanation();
          if (directMatchTable.getSelectedRow() != -1)
          {
            viewComparisonExplanation(getCaseComparison().
                                      getCompletelyMatchedComparisons()[
                                      directMatchTable.getSelectedRow()]);
          }
        }
      });
      add(directMatchTable.getTableHeader());
      add(directMatchTable);

      ComparisonTableModel explainedModel = new ComparisonTableModel(getCaseComparison().
        getPartlyMatchedComparisons(), getCaseComparison().getRelevanceFactorSum());
      explainedTable = new JTable(explainedModel);
      explainedTable.setDefaultRenderer(String.class,
                                        new
                                        RelevanceAwareCellRenderer(explainedModel));
      explainedTable.setDefaultRenderer(Integer.class,
                                        new
                                        PercentageCellRenderer(Color.red, explainedModel));

      JLabel partialMatchLabel = new JLabel("Partially matched features     ",
                                            JLabel.CENTER);
      partialMatchLabel.setBackground(Color.CYAN);
      partialMatchLabel.setBorder(new EmptyBorder(10, 0, 10, 0));
      partialMatchLabel.setFont(new Font("Arial", Font.BOLD, 14));
      partialMatchLabel.setForeground(new Color( (float) 0.2, (float) 0.2,
                                                (float) 0.7));
      partialMatchLabel.setMaximumSize(new Dimension(20000,
        directMatchLabel.getPreferredSize().height));
      add(partialMatchLabel);

      explainedTable.addFocusListener(this);
      explainedTable.getSelectionModel().addListSelectionListener(new
        ListSelectionListener()
      {
        public void valueChanged(ListSelectionEvent e)
        {
//        if(directMatchTable.getSelectedRow() == -1 && explainedTable.getSelectedRow() == -1 && )
//          viewCaseExplanation();
          if (explainedTable.getSelectedRow() != -1)
          {
//          System.out.println("Row: "+explainedTable.getSelectedRow());
            viewComparisonExplanation(getCaseComparison().
                                      getPartlyMatchedComparisons()[
                                      explainedTable.getSelectedRow()]);
          }
        }
      });
//    explainedTable.getColumnModel().getColumn(1).setCellRenderer(new AlignmentTableCellRenderer(JLabel.CENTER));
//    explainedTable.getColumnModel().getColumn(2).setCellRenderer(new AlignmentTableCellRenderer(JLabel.RIGHT));
//    explainedTable.setDefaultRenderer(String.class, new AlignmentTableCellRenderer());
      add(explainedTable.getTableHeader());
      add(explainedTable);

      JLabel unmatchedLabel = new JLabel("Unmatched features     ",
                                         JLabel.CENTER);
      unmatchedLabel.setBorder(new EmptyBorder(10, 0, 10, 0));
      unmatchedLabel.setFont(new Font("Arial", Font.BOLD, 14));
      unmatchedLabel.setForeground(new Color( (float) 0.2, (float) 0.2,
                                             (float) 0.7));
      unmatchedLabel.setMaximumSize(new Dimension(20000,
                                                  directMatchLabel.getPreferredSize().
                                                  height));

//    unmatchedLabel.setPreferredSize(new Dimension(10000, unmatchedLabel.getMinimumSize().height));

      add(unmatchedLabel);

      JPanel unmatchedPanel = new JPanel();
      unmatchedPanel.setLayout(new GridLayout(1, 2));

      unmatchedOriginTable = new JTable(new UnmatchedTableModel(getCaseComparison(),
        "Input Case Findings", true));
      unmatchedOriginTable.addFocusListener(this);
      unmatchedOriginTable.getSelectionModel().addListSelectionListener(new
        ListSelectionListener()
      {
        public void valueChanged(ListSelectionEvent e)
        {
          if (unmatchedOriginTable.getSelectedRow() != -1)
            viewUnmatched(getCaseComparison().getUnmatchedOriginRelations()[
                          unmatchedOriginTable.getSelectedRow()]);
        }
      });

      JPanel unmatchedOrigin = new JPanel(new BorderLayout());
      unmatchedOrigin.add(unmatchedOriginTable.getTableHeader(),
                          BorderLayout.NORTH);
      unmatchedOrigin.add(unmatchedOriginTable, BorderLayout.CENTER);
      unmatchedOrigin.setBorder(new EmptyBorder(0, 0, 0, 30));
      unmatchedPanel.add(unmatchedOrigin);

      unmatchedTargetTable = new JTable(new UnmatchedTableModel(getCaseComparison(),
        "Retrieved Case Finding", false));
      unmatchedTargetTable.addFocusListener(this);
      unmatchedTargetTable.getColumnModel().getColumn(0).setCellRenderer(new
        AlignmentTableCellRenderer(JLabel.RIGHT));
      unmatchedTargetTable.getSelectionModel().addListSelectionListener(new
        ListSelectionListener()
      {
        public void valueChanged(ListSelectionEvent e)
        {
          if (unmatchedTargetTable.getSelectedRow() != -1)
            viewUnmatched(getCaseComparison().getUnmatchedTargetRelations()[
                          unmatchedTargetTable.getSelectedRow()]);
        }
      });

      JPanel unmatchedTarget = new JPanel(new BorderLayout());
      unmatchedTarget.add(unmatchedTargetTable.getTableHeader(),
                          BorderLayout.NORTH);
      unmatchedTarget.add(unmatchedTargetTable, BorderLayout.CENTER);
      unmatchedTarget.setBorder(new EmptyBorder(0, 30, 0, 0));
      unmatchedPanel.add(unmatchedTarget);
      add(unmatchedPanel);
      unmatchedPanel.setMaximumSize(new Dimension(10000,
                                                  unmatchedPanel.getMinimumSize().
                                                  height));
//    JPanel invis = new JPanel();
//    invis.setMaximumSize(new Dimension(10000,10000));
//    add(invis);
      viewCaseExplanation();
    }

    public void doLayout()
    {
      super.doLayout();
    }

    public void focusLost(FocusEvent e)
    {
      if (JTable.class.isAssignableFrom(e.getSource().getClass()))
      {
        JTable table = (JTable) e.getSource();
//      table.getSelectionModel().clearSelection();
      }
    }

    public void focusGained(FocusEvent e)
    {
      if (e.getSource() != explainedTable)
        explainedTable.getSelectionModel().clearSelection();
      if (e.getSource() != unmatchedOriginTable)
        unmatchedOriginTable.getSelectionModel().clearSelection();
      if (e.getSource() != unmatchedTargetTable)
        unmatchedTargetTable.getSelectionModel().clearSelection();
      if (e.getSource() != directMatchTable)
        directMatchTable.getSelectionModel().clearSelection();

    }


    /*
      protected void checkSubmodelPane()
      {
        if(!java.util.Arrays.asList(explanationPanel.getComponents()).contains(submodelPane))
        {
     System.out.println("Removing component "+explanationPanel.getComponent(1));
          explanationPanel.remove(1);
          explanationPanel.add(submodelPane, BorderLayout.CENTER);
          explanationPanel.validate();
          explanationPanel.repaint();
        }
      }
     */
    public void viewCaseExplanation()
    {
      checkExplanationPane();
      textExplanationArea.setText(getCaseComparison().shortExplanation());
      detailsButton.setEnabled(false);
      submodelPane.getModel().getSelectionSet().clear();
      submodelPane.getModel().getSelectionSet().add(submodelPane.getModel().getVisualEntity(getCaseComparison().getOrigin()));
      submodelPane.getModel().getSelectionSet().add(submodelPane.getModel().getVisualEntity(getCaseComparison().getTarget()));
      explanationPane.validate();
      explanationPane.repaint();
    }

    public void viewUnmatched(Relation unmatched)
    {
      checkExplanationPane();
      SubModel model = new SubModel();
      model.add(unmatched.getValue());
      model.add(unmatched.getFrom());
      model.add(unmatched);

      submodelPane.getModel().getSelectionSet().clear();
      submodelPane.getModel().getSelectionSet().add(submodelPane.getModel().getVisualRelation(unmatched));
      submodelPane.getModel().getSelectionSet().add(submodelPane.getModel().getVisualEntity(unmatched.getValue()));
      textExplanationArea.setText("No finding match " + unmatched.getName() +
                                  " " + unmatched.getValue().getValueString());
      detailsButton.setEnabled(false);
      currentComparison = null;
      explanationPane.validate();
      explanationPane.repaint();
    }

    public void viewComparisonExplanation(EntityComparison comp)
    {
      checkExplanationPane();
      submodelPane.getModel().getSelectionSet().clear();
      highlight(comp.toSubModel(), submodelPane.getModel());
      textExplanationArea.setText(comp.shortExplanation());
      comp = ( (CaseFindingComparison) comp).getComparison();
      detailsButton.setEnabled(true);
      currentComparison = comp;
      explanationPane.validate();
      explanationPane.repaint();
    }

    private void highlight(SubModel model, ConceptMapModel mapModel)
    {
      Entity[] entities = model.getEntities();
      for(int i=0;i<entities.length;i++)
        mapModel.getSelectionSet().add(mapModel.getVisualEntity(entities[i]));

      Relation[] rels = model.getRelations();
      for(int i=0;i<rels.length;i++)
        mapModel.getSelectionSet().add(mapModel.getVisualRelation(rels[i]));
    }

    private void checkExplanationPane()
    {
      explanationPane.removeAll();
      explanationPane.add(submodelPane, BorderLayout.CENTER);
      explanationPane.add(explanationTextPanel, BorderLayout.EAST);
    }
  }

  class ComparisonTableModel extends AbstractTableModel
  {
    CaseFindingComparison[] comparisons;
    double relevanceSum = 0.0;
    ComparisonTableModel(CaseFindingComparison[] comparisons,
                         double relevanceSum)
    {
      this.comparisons = comparisons;
      this.relevanceSum = relevanceSum;
      /*    java.util.Arrays.sort(this.comparisons, new java.util.Comparator()
          {
            public int compare(Object o1, Object o2)
            {
              try
              {
                return Double.compare(((CaseFindingComparison) o2).getRelevance(), ((CaseFindingComparison) o1).getRelevance());
              }
              catch(Exception e)
              {

              }
              return 0;
            }
          }); */
    }

    public int getRowCount()
    {
      return comparisons.length;
    }

    public int getColumnCount()
    {
      return 3;
    }

    public String getColumnName(int column)
    {
      switch (column)
      {
        case 0:
          return "Input Case Finding";
//      case 1: return "Relevance Weight";
        case 1:
          return "Matching Strength";
        case 2:
          return "Retrieved Case Finding";
      }
      return "";
    }

    public Class getColumnClass(int column)
    {
      switch (column)
      {
        case 0:
          return String.class;
        case 1:
          return Integer.class;
        case 2:
          return String.class;
      }

      return String.class;
    }

    public Object getValueAt(int row, int column)
    {
      switch (column)
      {
//      case 0: return comparisons[row].getOriginRelation().getName()+" "+comparisons[row].getOriginRelation().getValue().getName();
        case 0:
          return comparisons[row].getOriginRelation().getValue().getValueString();
//      case 1: return new Double(comparisons[row].getRelevance());
        case 1:
          return new Integer(comparisons[row].getEntityComparison().
                             getPercentageStrength());
//      case 2: return comparisons[row].getTargetRelation().getName()+" "+comparisons[row].getTargetRelation().getValue().getName();
        case 2:
          return "(" +
            percentage(comparisons[row].getRelevance() / relevanceSum) +
            "% relevance) - " +
            comparisons[row].getTargetRelation().getValue().getValueString();
      }
      return "";
    }

    private int percentage(double d)
    {
      return (int) Math.round(d * 100.0);
    }
  }

  class UnmatchedTableModel extends AbstractTableModel
  {
    Relation[] relations;
    CaseComparison comp;
    String label;
    boolean origin = false;
    UnmatchedTableModel(CaseComparison comp, String label, boolean origin)
    {
      this.comp = comp;
      if (origin)
        this.relations = comp.getUnmatchedOriginRelations();
      else
        this.relations = comp.getUnmatchedTargetRelations();
      this.origin = origin;
      this.label = label;
    }

    public int getRowCount()
    {
      return relations.length;
    }

    public int getColumnCount()
    {
      return 1;
    }

    public String getColumnName(int column)
    {
      return label;
    }

    public Object getValueAt(int row, int column)
    {
      if (origin)
        return relations[row].getName() + " " +
          relations[row].getValue().getValueString() + " (penalty: " +
          percentage(comp.getUnmatchedPenalty()) + "%-points)";
      else
        return "(" +
          percentage(comp.getRelevanceFactor(relations[row]) / comp.getRelevanceFactorSum()) +
          "% relevance) - " + relations[row].getName() + " " +
          relations[row].getValue().getValueString();
    }

    private int percentage(double d)
    {
      return (int) Math.round(d * 100.0);
    }
  }

  class AlignmentTableCellRenderer extends DefaultTableCellRenderer
  {
    int align;
    AlignmentTableCellRenderer(int align)
    {
      super();
      this.align = align;
    }

    public Component getTableCellRendererComponent(JTable table,
      Object value,
      boolean isSelected,
      boolean hasFocus,
      int row,
      int column)
    {
      Component comp = super.getTableCellRendererComponent(table, value,
        isSelected, hasFocus, row, column);
      if (JLabel.class.isAssignableFrom(comp.getClass()))
      {
        JLabel labelComp = (JLabel) comp;
        labelComp.setHorizontalAlignment(align);
        return labelComp;
      }
      return comp;
    }
  }

  class RelevanceAwareCellRenderer extends DefaultTableCellRenderer
  {
    ComparisonTableModel model;
    RelevanceAwareCellRenderer(ComparisonTableModel model)
    {
      super();
      this.model = model;
    }

    public Component getTableCellRendererComponent(JTable table,
      Object value,
      boolean isSelected,
      boolean hasFocus,
      int row,
      int column)
    {
      Component comp = super.getTableCellRendererComponent(table, value,
        isSelected, hasFocus, row, column);
      if (!isSelected && model.comparisons[row].getRelevance() < 0.001)
      {
        comp.setForeground(Color.gray);
      }
      else
      {
        comp.setForeground(table.getForeground());
      }
      if (comp instanceof JLabel)
      {
        JLabel label = (JLabel) comp;
        if (column == 1)
          label.setHorizontalAlignment(JLabel.CENTER);
        else if (column == 2)
          label.setHorizontalAlignment(JLabel.RIGHT);
        else
          label.setHorizontalAlignment(JLabel.LEFT);
      }
      return comp;
    }
  }

  class PercentageCellRenderer extends JProgressBar implements
    TableCellRenderer
  {
    ComparisonTableModel model;
    Color defColor;
    PercentageCellRenderer(Color c, ComparisonTableModel model)
    {
      super();
      this.model = model;
      this.defColor = c;
      this.setForeground(c);
      this.setIndeterminate(false);
      this.setMaximum(100);
      this.setStringPainted(true);
      this.setValue(0);
      this.setString("0%");
      this.setBorder(BorderFactory.createEmptyBorder(1, 1, 1, 1));
    }

    public Component getTableCellRendererComponent(JTable table,
      Object value,
      boolean isSelected,
      boolean hasFocus,
      int row,
      int column)
    {
      setValue( ( (Integer) value).intValue());
      setString("" + value + "%");

      if (isSelected)
      {
        super.setBackground(table.getSelectionBackground());
      }
      else
      {
        super.setBackground(table.getBackground());
      }

      if (model.comparisons[row].getRelevance() < 0.001)
      {
        setForeground(Color.gray);
      }
      else
      {
        setForeground(defColor);
      }

      return this;
    }
  }
}
