package jcreek.gui.reasoning;

import jcreek.gui.SubModelView;
import jcreek.gui.conceptmap.DefaultConceptMapController;
import jcreek.representation.*; import jcreek.representation.cbr.*;
import java.awt.Point;
import java.awt.Dimension;


/**
 * Specialized type of SubModelView that will display a submodel that is somehow
 * locate "between" two compared entities. These entities will be fixed at
 * @author not attributable
 * @version 1.0
 */

public class ComparisonSubModelView extends SubModelView
{
  private Entity entity1;
  private Entity entity2;

  /**
   * Creates a SubModelView where the SubModel given is displayed and the two
   * entities supplied are fixed in the upper left and lower right corners.
   *
   * @param model The SubModel to lay out.
   * @param entity1 The first fixed entity (upper left corner)
   * @param entity2 The second fixed entity (upper right corner)
   */
  public ComparisonSubModelView(SubModel model, Entity entity1, Entity entity2)
  {
    super(entity1.getKnowledgeModel(), model, true, true);
    this.entity1 = entity1;
    this.entity2 = entity2;
    this.setScale(0.8);
    this.setController(new DefaultConceptMapController()
                       {

      public Dimension getViewInsets()
      {
        return new Dimension(50, 50);
      }
                       });
  }

  protected void firstResize()
  {
    fixEntity(entity1, new Point(0, 0));
    fixEntity(entity2, new Point((int) Math.round((getSize().width-100)/getScale()), (int) Math.round((getSize().height-100)/getScale())));
    super.firstResize();
  }
}
