package jcreek.gui.reasoning;

/**
 * Displays a PartitionComparison
 * @author Frode Srmo
 * @version 1.0
 */

import jcreek.representation.*; import jcreek.representation.cbr.*;
import jcreek.reasoning.*;
import jcreek.gui.conceptmap.*;
import jcreek.gui.*;
import javax.swing.*;
import java.awt.*;


public class PartitionComparisonPane extends EntityComparisonPane
{
  public PartitionComparisonPane(PartitionComparison comp)
  {
    super(comp);
  }

  protected JComponent buildDescriptionPane()
  {
    JPanel info = new JPanel();
    info.setLayout(new GridLayout(6,2));
    info.add(new JLabel("Origin"));
    info.add(new JLabel(getPartitionComparison().getOrigin().getValueString()));
    info.add(new JLabel("Target"));
    info.add(new JLabel(getPartitionComparison().getTarget().getValueString()));
    info.add(new JLabel("Status"));
    info.add(new JLabel(getPartitionComparison().getStatusText()));
    info.add(new JLabel("Activation strength"));
    info.add(new JLabel(""+(int) Math.round(getPartitionComparison().getActivationStrength()*100)+"%"));
    info.add(new JLabel("Explanation strength"));
    info.add(new JLabel(""+(int) Math.round(getPartitionComparison().getExplanationStrength()*100)+"%"));
    info.add(new JLabel("Total strength"));
    info.add(new JLabel(""+(int) Math.round(getPartitionComparison().getStrength()*100)+"%"));
    JPanel explanationPanel = new JPanel(new BorderLayout());
    explanationPanel.add(info, BorderLayout.NORTH);
    JTextArea explanation = new JTextArea(10, 30);
    explanation.setEditable(false);
    explanation.setWrapStyleWord(true);
    explanation.setLineWrap(true);
    explanation.setText(getPartitionComparison().toString());
    explanationPanel.add(new JScrollPane(explanation), BorderLayout.CENTER);
    return explanationPanel;
  }

  protected JComponent buildSubModelView()
  {
    SubModelView modelView = new SubModelView(getPartitionComparison().getKnowledgeModel(),
                                              getPartitionComparison().getUnionModel(),
                                              true, true);

    final SubModel overlapModel = getPartitionComparison().getOverlapModel();
    final SubModel originModel = getPartitionComparison().getKnowledgeModel().getPartition(getPartitionComparison().getOrigin().getEntityObject());

    modelView.setController(new DefaultConceptMapController()
    {
      Color originColor = new Color(120,120,240);
      Color targetColor = new Color(240,120,120);
      Color bothColor = new Color(180,120,180);

      public Color getEntityColor(Entity ent)
      {
        if(overlapModel.contains(ent))
          return bothColor;
        else if(originModel.contains(ent))
          return originColor;
        else
          return targetColor;
      }

      public Color getRelationColor(Relation rel)
      {
        if(overlapModel.contains(rel))
          return bothColor;
        else if(originModel.contains(rel))
          return originColor;
        else
          return targetColor;
      }

    });
    return modelView;
  }

  public PartitionComparison getPartitionComparison()
  {
    return (PartitionComparison) getEntityComparison();
  }

}

