package jcreek.gui.reasoning;

import jcreek.gui.reasoning.CaseComparisonPane;
import jcreek.gui.chart.*;
import jcreek.representation.*; import jcreek.representation.cbr.*;
import jcreek.representation.cbr.*;
import jcreek.reasoning.*;

import javax.swing.*;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.*;

/**
 * <p>Title: RetrieveResultPane</p>
 * <p>Description: Displays a RetrieveResult</p>
 * @author Frode Srmo
 * @version 1.0
 */

public class RetrieveResultPane extends JPanel implements ChartModel, ModelChangeListener
{
  private volatile RetrieveResult retrieveResult = null;
  private int selected = 0;
  private JChart chart;
  private CaseComparisonPane comparisonPane;
  private boolean active;
  private boolean reupdate = false;
  private Thread updateThread = null;
  private Case myCase;

  /**
   * Creates a new RetrieveResultPane encapsulating the given RetrieveResult.
   */
  public RetrieveResultPane(RetrieveResult retrieveResult)
  {
    super();
    this.retrieveResult = retrieveResult;
    init();
  }

  public void init()
  {

    chart = new JChart(this);
    chart.addMouseListener(new MouseAdapter() {
       public void mouseClicked(MouseEvent e)
       {
         setCaseSelected(chart.getValueForPos(e.getX(), e.getY()));
       }
    });
    chart.setMinimumSize(new Dimension(200, 70));
    chart.setPreferredSize(new Dimension(300, 100));
//    if(!retrieveResult.isFocused())
      updateContents();
//    else
//      updatePane();
    setPreferredSize(new Dimension(800, 800));
    setMinimumSize(new Dimension(400, 400));
  }

  /**
   * Creates a new RetrieveResult matching the supplied case against the
   * casebase, and uses this RetrieveResult as a basis for displaying the pane.
   */
  public RetrieveResultPane(Case myCase)
  {
    this.myCase = myCase;
    init();
  }

  public RetrieveResult getRetrieveResult()
  {
    return retrieveResult;
  }

  public void updatePane()
  {
//    System.out.println("updatePane()");
    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        buildPane();
        validate();
        repaint();
      }
    });
  }

  public void buildPane()
  {
    removeAll();
    setLayout(new BorderLayout());
    if(retrieveResult.size() < 1)
    {
      add(new JLabel("No cases matched this case.", JLabel.CENTER), BorderLayout.CENTER);
    }
    else
    {
      if(selected >= 0)
        comparisonPane = new CaseComparisonPane(retrieveResult.getComparison(selected));
      add(comparisonPane, BorderLayout.CENTER);
      add(chart, BorderLayout.SOUTH);
    }
  }

//----------------------------------------------------------------------------//
//   ChartModel methods.
//----------------------------------------------------------------------------//

    public double getValue(int col, int row)
    {
      CaseComparison comps[] = retrieveResult.getAllComparisons();
      if(col == 0)
        return comps[row].getActivationStrength();
      else
        return comps[row].getExplanationStrength();
    }

    public String getRowLabel(int row)
    {
      CaseComparison comps[] = retrieveResult.getAllComparisons();
      return comps[row].getTarget().getName();
    }

    public String getValueLabel(int col, int row)
    {
      if(col == 0)
        return "Activate";
      else
        return "Explain";
    }

    public int getRowSize()
    {
      CaseComparison comps[] = retrieveResult.getAllComparisons();
      return Math.min(comps.length, 10);
    }

    public int getColSize(int row)
    {
      return 2;
    }

    public double getMaxValue()
    {
      return 1.0;
    }

    public Color getValueColor(int col, int row)
    {
      if(col == 0)
        return Color.blue;
      else
        return Color.red;
    }

    public Color getBackgroundColor(int row)
    {
      if(row == selected)
        return new Color(150,150,240);
      else
        return Color.white;
    }

    public void setCaseSelected(int i)
    {
      selected = i;
//      RetrieveFocusResult result = new RetrieveFocusResult(getSelectedCase());
      //notifies listeners
//      fireReasoningStateChanged(result);
      buildPane();
      updatePane();
    }

    public void attach()
    {
      if(!active && retrieveResult != null)
        retrieveResult.getKnowledgeModel().addModelChangeListener(this);
      active = true;
    }

    public void detach()
    {
      if(active && retrieveResult != null)
        retrieveResult.getKnowledgeModel().removeModelChangeListener(this);
      active = false;
    }

    public void modelChanged(ModelChangeEvent e)
    {
      updateContents();
    }

    private String barText;
    void checkForWaitPane()
    {
      Thread waitAndSee = new Thread(new Runnable()
      {
        public void run()
        {
          try
          {
            Thread.sleep(500);
          }
          catch(InterruptedException e)
          {
          }
          if(updateThread != null)
          {
            buildWaitPane();
            while(updateThread != null)
            {
              try
              {
                Thread.sleep(100);
              }
              catch(InterruptedException e)
              {
              }

//              System.out.println("->"+bar.getModel().getValue());
              barText = null;
              if(retrieveResult == null)
                barText = "Matching case findings";
              else if(retrieveResult.isActivated())
                barText = "Explain";
              else if(retrieveResult.isExplained())
                barText = "Focus";
              else
                barText = "Activate";
              try
              {
                SwingUtilities.invokeAndWait(new Runnable() {
                  public void run() {
                    bar.setString(barText);
                    bar.setValue(bar.getModel().getValue());
                    repaint();
                  }
                });
              }
              catch(InterruptedException e) { }
              catch(java.lang.reflect.InvocationTargetException e) { }
            }
          }
        }
      });
      waitAndSee.start();
    }

    JProgressBar bar;

    synchronized void buildWaitPane()
    {
      try
      {
        SwingUtilities.invokeAndWait(new Runnable() {
          public void run()
          {
            removeAll();
            setLayout(new BorderLayout());
            bar = new JProgressBar(new DefaultBoundedRangeModel()
            {
              public int getMaximum()
              {
                return 220;
              }

              public int getMinimum()
              {
                return 0;
              }

              public int getValue()
              {
                if(retrieveResult == null)
                  return 0;
                int max = retrieveResult.getIterationMax();
                int current = retrieveResult.getIteration();
                if(max > 0)
                {
                  if(!retrieveResult.isActivated())
                  {
//                    System.out.println("activate getValue(): " + current + " of " +
//                                       max);
                    return 100 * current / max;
                  }
                  else if(!retrieveResult.isExplained())
                  {
//                    System.out.println("explain getValue(): " + current + " of " +
//                                       max);
                    return 100 + 100 * current / max;
                  }
                  else if(!retrieveResult.isFocused())
                    return 200+20*current/max;
                }
                if(retrieveResult.isFocused())
                  return 220;
                else
                  return 0;
              }
              public int getExtent()
              {
                return 1;
              }
            });
            bar.setIndeterminate(false);
            bar.setString("Activate");
            bar.setStringPainted(true);
            add(bar, BorderLayout.SOUTH);
            validate();
            repaint();
          }
        });
      }
      catch(InterruptedException e)
      {
      }
      catch(java.lang.reflect.InvocationTargetException e)
      {
      }
    }

    public void updateContents()
    {
//      System.out.println("updateContents()");
      reupdate = true;
      if(updateThread == null)
      {
        updateThread = new Thread(new Runnable()
        {
          public void run()
          {
//            System.out.println("run()");
            while(reupdate)
            {
              reupdate = false;
              if(myCase != null)
              {
                boolean act = active;
                if(active)
                  detach();
                retrieveResult = null;
                retrieveResult = new RetrieveResult(myCase, false);
                if(act)
                  attach();
              }

              if(!retrieveResult.isActivated())
              {
//                System.out.println("isActivated");
                retrieveResult.activate();
                retrieveResult.explain(10, 0.0);
                retrieveResult.focus(10, 0.0);
              }
              else
              {
//                System.out.println("updates");
                retrieveResult.update();
              }
              updateThread = null;
              updatePane();
            }
          }
        });
        updateThread.setPriority(Thread.MIN_PRIORITY);
        updateThread.start();
        checkForWaitPane();
      }
    }
}
