package jcreek.gui.representation;

/**
 * Convinience abstract class for implementing the EntityTypeField interface.
 * This class reduced the number of methods you need to implement to:
 * <li>public Component getRenderComponent(Entity entity, boolean isSelected, boolean hasFocus)
 * <li>public Component getEditorComponent(Entity entity, boolean isSelected)
 * <li>public Object getCellEditorValue()
 *
 *
 *
 * @author Frode Srmo
 * @version 1.0
 */

import javax.swing.JTree;
import javax.swing.JTable;
import javax.swing.JList;
import javax.swing.AbstractCellEditor;
import java.awt.Component;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import javax.swing.JComponent;
import jcreek.representation.Entity;


public abstract class AbstractEntityTypeField extends AbstractCellEditor implements EntityTypeField
{
  public AbstractEntityTypeField()
  {
    super();
  }

  public Component getTreeCellEditorComponent(JTree tree,
                                              Object value,
                                              boolean isSelected,
                                              boolean expanded,
                                              boolean leaf,
                                              int row)
  {
    Component c = getEditorComponent(value);
    addFocusListener(c);
    return c;
  }

  public Component getTableCellEditorComponent(JTable table,
                                               Object value,
                                               boolean isSelected,
                                               int row,
                                               int column)
  {
    Component c = getEditorComponent( value);
    addFocusListener(c);
    return c;
  }

  protected void addFocusListener(final Component comp)
  {
    comp.addFocusListener(new FocusAdapter()
    {
      public void focusLost(FocusEvent e)
      {
        stopCellEditing();
      }
    });
  }

  public Component getListCellRendererComponent(JList list,
                                                Object value,
                                                int index,
                                                boolean isSelected,
                                                boolean cellHasFocus)
  {
    return getRendererComponent( (Entity) value);
  }

  public Component getTableCellRendererComponent(JTable table,
                                                 Object value,
                                                 boolean isSelected,
                                                 boolean hasFocus,
                                                 int row,
                                                 int column)
  {
    return getRendererComponent( value);
  }

  public Object getCellEditorValue()
  {
    return null;
  }
}
