package jcreek.gui.representation;

/**
 * An element in a CasePane. An element encapsulated one "has finding" relation from the case.</p>
 * @author Frode Srmo
 * @version 1.0
 */

import javax.swing.*;
import javax.swing.event.*;
import java.util.Hashtable;
import java.util.Collection;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.*;
import jcreek.representation.*; import jcreek.representation.cbr.*;

public class CasePaneElement // implements ModelChangeListener
{
  protected Relation relation;
  protected boolean active = false;
  protected boolean editable = false;
  protected JComboBox valueBox;
  protected JComboBox typeBox;
  protected JSlider importanceSlider;
  protected JSlider predictiveSlider;
  protected ImageIcon removeIcon = new ImageIcon(getClass().getResource("/jcreek/cke/resources/delete.gif"));
  protected JButton removeButton = new JButton(removeIcon);

  /**
   * Creates a new CasePaneElement.
   *
   * @param rel The relation to encapsulate.
   */
  public CasePaneElement(Relation rel)
  {
    super();
    this.relation = rel;
//    removeButton.setOpaque(true);
//    removeButton.setPreferredSize(new Dimension(removeIcon.getIconWidth()+8, removeIcon.getIconHeight()+8));
    removeButton.setBorder(BorderFactory.createEmptyBorder(0,15,0,5));
    removeButton.addActionListener(new ActionListener()
                                   {
      public void actionPerformed(ActionEvent e)
      {
//        System.out.println("Removing "+relation);
        relation.remove();
      }
                                   });
//    buildCasePaneElement();
  }

  public void setEditable(boolean e)
  {
    boolean change = editable != e;
    editable = e;
//    if(change)
//      buildCasePaneElement();
  }

  public boolean isEditable()
  {
    return editable;
  }

  public void buildCasePaneElement(JPanel panel)
  {
    /*
    System.out.println("Is editable? "+isEditable());
    removeAll();

    if(isEditable())
      this.setBackground(Color.white);


    setLayout(new GridLayout(1, 3));
    */
    if(false)
    {
      RelationType rcd = relation.getKnowledgeModel().getRelationType(CBRModel.FINDING_RELATION);
      Collection finding = rcd.getAllSubclassesCollection();
      finding.add(rcd);

      Entity[] hasFindings = (Entity[]) finding.toArray(new RelationType[finding.size()]);

      typeBox = new JComboBox(hasFindings);
      typeBox.setBackground(panel.getBackground());
      typeBox.setSelectedItem(relation.getRelationType());
      typeBox.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e)
        {
          RelationType selected = (RelationType) typeBox.getSelectedItem();
          relation.setRelationType(selected);
        }
      });
      panel.add(typeBox);
    }
    else
    {
      JLabel nameLabel = new JLabel(relation.getName());
      nameLabel.setBackground(panel.getBackground());
      panel.add(nameLabel);
    }

    valueBox = null;

//    JComponent valueComp = ValueComponentFactory.getValueComponent(relation.getValue(), isEditable());
//    valueComp.setBackground(panel.getBackground());

    Entity valueEntity = relation.getValue();
    if(isEditable())
    {
      try
      {
        Relation typeEntity = valueEntity.getRelation(new String[] {"value of", "instance of", "subclass of"});
        Relation comparatorRel = typeEntity.getValue().getRelation("has comparator");
        if(comparatorRel.getValue().equals(valueEntity.getKnowledgeModel().getEntity("SymbolComparison")))
        {
          Relation[] valueRels = typeEntity.getValue().getRelations(new String[] {"value of", "instance of", "subclass of"});
          Entity valueEnts[] = new Entity[valueRels.length];
          for(int i=0;i<valueRels.length;i++)
            valueEnts[i] = valueRels[i].getValue();
          JComboBox selector = new JComboBox(valueEnts);
          selector.setBackground(panel.getBackground());
          valueBox = selector;
          selector.setBackground(panel.getBackground());
          selector.setSelectedItem(valueEntity);
          selector.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e)
            {
              Entity selected = (Entity) valueBox.getSelectedItem();
              relation.setValue(selected);
            }
          });
        }
      }
      catch(NoSuchRelationException e)
      {
        e.printStackTrace();
      }
      catch(NoSuchRelationTypeException e)
      {
        e.printStackTrace();
      }
    }

    if(valueBox == null)
    {
      JLabel l = new JLabel(valueEntity.toString());
      l.setBackground(panel.getBackground());
      panel.add(l);
    }
    else
      panel.add(valueBox);
//    panel.add(valueComp);

    JPanel secondaryPanel = new JPanel();
    JPanel sliderPanel = new JPanel();
    secondaryPanel.setLayout(new BorderLayout());
    sliderPanel.setLayout(new GridLayout(1, 2));

    int val = (int) Math.round(relation.getExplanationStrength()*100);
    if(val < 0)
       val = 0;
    if( val > 100)
      val = 100;

    importanceSlider = new JSlider(0, 100, val);
    importanceSlider.setToolTipText("Importance");
    importanceSlider.setBackground(panel.getBackground());
    Hashtable labelTable = new Hashtable();
//    labelTable.put(new Integer(20), new JLabel("Spurious"));
//    labelTable.put(new Integer(80), new JLabel("Necessary"));
//    importanceSlider.setLabelTable(labelTable);
//    importanceSlider.setPaintLabels(true);
    importanceSlider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e)
      {
        relation.setExplanationStrength((int) Math.round(importanceSlider.getValue()/100));
      }
    });
    sliderPanel.add(importanceSlider);

    val = (int) Math.round(relation.getInverse().getExplanationStrength()*100);
    if(val < 0)
       val = 0;
    if( val > 100)
      val = 100;

    predictiveSlider = new JSlider(0, 100, val);
    predictiveSlider.setToolTipText("Predictive Strength");
    predictiveSlider.setBackground(panel.getBackground());
    Hashtable labelTable2 = new Hashtable();
//    labelTable2.put(new Integer(10), new JLabel("Low"));
//    labelTable2.put(new Integer(90), new JLabel("High"));
//    predictiveSlider.setLabelTable(labelTable2);
//    predictiveSlider.setPaintLabels(true);
    predictiveSlider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e)
      {
        relation.getInverse().setExplanationStrength((int) Math.round(predictiveSlider.getValue()/100));
      }
    });
    sliderPanel.add(predictiveSlider);

    secondaryPanel.add(sliderPanel, BorderLayout.CENTER);
    panel.add(secondaryPanel);

//    this.setMaximumSize(new Dimension(400, 30));
//    .setPreferredSize(new Dimension(400, 40));
//    this.setMinimumSize(new Dimension(400, 30));


    if(isEditable())
    {
      removeButton.setBackground(panel.getBackground());
      secondaryPanel.add(removeButton, BorderLayout.EAST);
    }
//    doLayout();
  }
}


