package jcreek.gui.representation;

import jcreek.representation.*; import jcreek.representation.cbr.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.util.Vector;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import jcreek.util.EntityNameComparator;
import jcreek.representation.inference.ActivationConstants;

/**
 * A table model for showing case findings.
 *
 * @author Frode Srmo
 * @version 1.0
 */

public class CaseTableModel extends FrameTableModel
{
  private boolean showPredImp = false;

  public CaseTableModel(boolean showPredImp, Relation[] rels)
  {
    super(rels);
    this.showPredImp = showPredImp;
  }

  public void setShowStrengths(boolean show)
  {
    showPredImp = show;
  }

  public boolean showStrengths()
  {
    return showPredImp;
  }

  public void formatTable(JTable table, KnowledgeModel knowledgeModel)
  {
//    table.setRowHeight(table.getRowHeight()+4);
    table.getColumnModel().getColumn(2).setMinWidth(followIcon.getImage().getWidth(followIcon.getImageObserver())+5);
    table.getColumnModel().getColumn(2).setMaxWidth(followIcon.getImage().getWidth(followIcon.getImageObserver())+5);
    table.getColumnModel().getColumn(2).setResizable(false);
    if(table.getColumnModel().getColumnCount() > (showPredImp?5:3))
    {
      table.getColumnModel().getColumn(showPredImp?5:3).setMinWidth(removeIcon.getImage().getWidth(removeIcon.getImageObserver())+5);
      table.getColumnModel().getColumn(showPredImp?5:3).setMaxWidth(removeIcon.getImage().getWidth(removeIcon.getImageObserver())+5);
      table.getColumnModel().getColumn(showPredImp?5:3).setResizable(false);
    }
    table.removeMouseListener(actionListen);
    table.addMouseListener(actionListen);

    Entity[] concepts = knowledgeModel.getEntities();
    Arrays.sort(concepts, new EntityNameComparator());
    JComboBox entityBox = new JComboBox(concepts);


    table.setDefaultEditor(Entity.class, new DefaultCellEditor(new JComboBox()));

/*

    table.setDefaultEditor(Entity.class, new DefaultCellEditor(new JTextField())
    {
      public Component getTableCellEditorComponent(JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   int row,
                                             int column)
      {
        return EntityComponentFactory.getEntityComponent(relationAt(row), editable);
      }
    });
*/
    table.setDefaultEditor(Icon.class, new DefaultCellEditor(new JTextField())
    {
      public Component getTableCellEditorComponent(JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   int row,
                                             int column)
      {
        SwingUtilities.invokeLater(new IconClickedEvent(row, column, this));
        return table.getCellRenderer(row, column).getTableCellRendererComponent(table, value, isSelected, true, row, column);
      }
    });

    table.setDefaultEditor(PredictiveStrength.class, new DefaultCellEditor(new JComboBox(PredictiveStrength.getPossibleValues())));
    table.setDefaultEditor(ImportanceStrength.class, new DefaultCellEditor(new JComboBox(ImportanceStrength.getPossibleValues())));
  }



  public String getColumnName(int columnIndex)
  {
    switch(columnIndex)
    {
      case 0: return "Relation-type";
      case 1: return "Value";
      case 2: return "";
      case 3:
        if(showPredImp)
          return "Importance";
        else
          return " ";
      case 4: return "Predictive Strength";
      case 5: return " ";
    }
    return "WHAT?";
  }

  public boolean isCellEditable(int row, int cell)
  {
    if(cell == 2)
      return true;
    if(cell == 1 && editable)
      return true;
    if(editable && ((cell == 3 && !showPredImp) || (cell == 5 && showPredImp)))
      return true;
    if(showPredImp && editable && (cell == 3 || cell == 4))
       return true;
    return false;
  }

  public Class getColumnClass(int column)
  {
    switch(column)
    {
      case 0: return RelationType.class;
      case 1: return Entity.class;
      case 2: return Icon.class;
      case 3:
        if(showPredImp)
          return ImportanceStrength.class;
        else
          return Icon.class;
      case 4: return PredictiveStrength.class;
      case 5: return Icon.class;
    }

    return String.class;
  }

  public void setRelations(Relation[] rels)
  {
    relations = new Vector();
    for(int i=0;i<rels.length;i++)
      if(!rels[i].isNegated())
        relations.add(rels[i]);
    Collections.sort(relations, new Comparator()
    {
      public int compare(Object ob1, Object ob2)
      {
        return ((Relation) ob1).getName().compareTo(((Relation) ob2).getName());
      }
    });
    fireTableDataChanged();
  }

  /**
   * Returns the number of rows in the table
   */
  public int getColumnCount()
  {
    if(isEditable())
      return showPredImp?6:4;
    else
      return showPredImp?5:3;
  }

  public Object getValueAt(int rowIndex, int columnIndex)
  {
    if(rowIndex < relations.size())
    {
      switch(columnIndex)
      {
        case 0: return relationAt(rowIndex).getRelationType();
        case 1: return relationAt(rowIndex).getValue();
        case 2: return new ImageIcon(getClass().getResource("/jcreek/cke/resources/relation.gif"));
        case 3:
          if(showPredImp)
            return new ImportanceStrength(relationAt(rowIndex).getExplanationStrength());
          else
            return new ImageIcon(getClass().getResource("/jcreek/cke/resources/delete.gif"));
        case 4: return new PredictiveStrength(relationAt(rowIndex).getInverse().getExplanationStrength());
        case 5: return new ImageIcon(getClass().getResource("/jcreek/cke/resources/delete.gif"));
      }
    }
    else
    {
      return "Add";
    }
    return "WHAT?";
  }

  public void setValueAt(Object value, int row, int column)
  {
    if(showPredImp)
    {
      if(column == 4)
        relationAt(row).getInverse().setExplanationStrength(((PredictiveStrength) value).getPredictiveStrength());
      else if(column == 3)
        relationAt(row).setExplanationStrength(((ImportanceStrength) value).getImportanceStrength());
    }
  }

  void iconClicked(int row, int column)
  {
    switch(column)
    {
      case 2:
        fireLinkEvent(new LinkEvent(relationAt(row).getValue()));
        break;
      case 3:
      case 5:
        Relation rel = relationAt(row);
        if(rel.isInherited())
        {
          rel.negate();
        }
        else
        {
          removeRelation(rel);
          rel.remove();
        }
        break;
    }
  }
}
class PredictiveStrength
{
  double predStrength;
  PredictiveStrength(double strength)
  {
    predStrength = strength;
  }

  public double getPredictiveStrength()
  {
    return predStrength;
  }

  public String toString()
  {
    if(predStrength == ActivationConstants.SUFFICIENT)
      return "Sufficent";
    else if (predStrength == ActivationConstants.STRONGLY_INDICATIVE)
      return "Strongly Indicative";
    else if (predStrength == ActivationConstants.INDICATIVE)
      return "Indicative";
    else if (predStrength == ActivationConstants.SPURIOUS)
      return "Spurious";
    return "No value";
  }

  public static PredictiveStrength[] getPossibleValues()
  {
    return new PredictiveStrength[] {
      new PredictiveStrength(ActivationConstants.SUFFICIENT),
      new PredictiveStrength(ActivationConstants.STRONGLY_INDICATIVE),
      new PredictiveStrength(ActivationConstants.INDICATIVE),
      new PredictiveStrength(ActivationConstants.SPURIOUS) };
  }
}

class ImportanceStrength
{
  double impStrength;
  ImportanceStrength(double strength)
  {
    impStrength = strength;
  }

  public double getImportanceStrength()
  {
    return impStrength;
  }

  public String toString()
  {
    if(impStrength == ActivationConstants.NECCESSARY)
      return "Neccessary";
    else if (impStrength == ActivationConstants.CHARACTERISTIC)
      return "Characteristic";
    else if (impStrength == ActivationConstants.INFORMATIVE)
      return "Informative";
    else if (impStrength == ActivationConstants.IRRELEVANT)
      return "Irrelevant";
    return "No value";
  }

  public static ImportanceStrength[] getPossibleValues()
  {
    return new ImportanceStrength[] {
      new ImportanceStrength(ActivationConstants.NECCESSARY),
      new ImportanceStrength(ActivationConstants.CHARACTERISTIC),
      new ImportanceStrength(ActivationConstants.INFORMATIVE),
      new ImportanceStrength(ActivationConstants.IRRELEVANT) };
  }
}
