package jcreek.gui.representation;

/**
 * Represents a compontent for editing some value.
*
* Scenario 1: Existing relation with value. Value changes in table.
*                - relation points to different entity
*                - relation points to same entity containing different value
*
* Scenario 2: Relation not yet made.
*                - creates entity with given value
*                - points to already existing entity
*               ? What happens if builder aborts making relation? Is the value entity created?
*
* Scenario 3: No relation, change value directly in the entity.
*                - only entity-value changed.
*
* -> Separate entity-value editors from relation-value editors.
*         - entity-value editors may be used BY relation-value editors.
*
*
 *
 * @author Frode Srmo
 * @version 1.0
 */

import javax.swing.CellEditor;
import javax.swing.table.TableCellEditor;
import javax.swing.ListCellRenderer;
import javax.swing.table.TableCellRenderer;
import java.awt.Component;

public interface EntityTypeField extends CellEditor, TableCellEditor, ListCellRenderer, TableCellRenderer
{
  /**
   * Implement this method to make a generic editor component that can be
   * used in lists, tables and as parts of components.
   *
   * @param value The value of the field currently
   * @param isSelected If this field is selected (in tables etc.)
   * @return The editor component.
   */
  public abstract Component getEditorComponent(Object value);


  /**
  * Implement this method to make a generic render component that can be
  * used in lists, tables and as parts of components.
  *
  * @param value The value of the field currently
  * @param isSelected If this field is selected (in tables etc.)
  * @param hasFocus If this field has focus
  * @return The render component.
  */
  public abstract Component getRendererComponent(Object value);


}
