package jcreek.gui.representation;

/**
 * Factory class for generating GUI components that display or edit the object
 * encapsulated by an entity.
 *
 * @author Frode Srmo
 * @version 1.0
 */

import java.util.Properties;
import java.io.*;
import java.lang.reflect.*;
import java.util.Iterator;
import java.util.ArrayList;

public class EntityTypeFieldFactory
{
  private static Properties componentForType = null;

  /**
   * Should not be instanciated.
   */
  private EntityTypeFieldFactory()
  {
  }

  private static void checkLoadProperties()
  {
    if(componentForType == null)
    {
      try
      {

        componentForType = new Properties();
        InputStream in = EntityTypeFieldFactory.class.getResourceAsStream(
          "/jcreek/gui/representation/entitytypefields.properties");
        if (in != null)
          componentForType.load(in);
        else
          System.err.println("EntityTypeFieldFactory error: jcreek.gui.representation.entitytypefields.properties failed to load!");
      }
      catch (java.io.IOException ex)
      {
        System.err.println("EntityTypeFieldFactory error: jcreek.gui.representation.entitytypefields.properties failed to load!");
        ex.printStackTrace();
      }
      if (componentForType == null)
        componentForType = new Properties();
    }
  }

  public static EntityTypeField getEntityTypeField(Class javaClass)
  {
    checkLoadProperties();
    try
    {
      String entityTypeClass = componentForType.getProperty(javaClass.getName());
      if(entityTypeClass != null)
      {
        Class fieldClass = Class.forName(entityTypeClass);
        Constructor fieldConstructor = fieldClass.getConstructor(new Class[0]);
        return (EntityTypeField) fieldConstructor.newInstance(new Object[0]);
      }
      else
        return null;
    }
    catch (ClassNotFoundException e)
    {
      e.printStackTrace();
    }
    catch (NoSuchMethodException e)
    {
      e.printStackTrace();
    }
    catch (InstantiationException e)
    {
      e.printStackTrace();
    }
    catch (IllegalAccessException e)
    {
      e.printStackTrace();
    }
    catch (InvocationTargetException e)
    {
      e.printStackTrace();
    }
    catch (ClassCastException e)
    {
      e.printStackTrace();
    }

    return null;
  }

  /**
   * Returns an
   * @return Class[]
   */
  public static Class[] getClassesSupported()
  {
    checkLoadProperties();

    ArrayList supported = new ArrayList();

    int i=0;
    for(Iterator it = componentForType.keySet().iterator();it.hasNext();)
    {
      try
      {
        Class add = Class.forName((String) it.next());
        supported.add(add);
      }
      catch(ClassNotFoundException e) { }
    }
    return (Class[]) supported.toArray(new Class[supported.size()]);
  }
}
