package jcreek.gui.representation;

import jcreek.representation.*; import jcreek.representation.cbr.*;
import jcreek.util.EntityNameComparator;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;

/**
 * <p>Title: FrameTableModelionsTable</p>
 * <p>Description: A combineded TableModel, TableCellRenderer and TableCellEditor
 * for tables containg a set of relations. </p>
 * @author Frode Srmo
 * @version 1.0
 */

public class FrameTableModel extends AbstractTableModel
{
  protected Color inheritedColor = Color.darkGray;
  protected Color negatedColor = Color.red;
  protected static Border noFocusBorder = new EmptyBorder(1, 1, 1, 1);
  protected boolean includeAction = true;
  protected Vector relations = new Vector();
  protected Vector actionListeners = new Vector();
  protected JTable myTable;
  protected boolean editable = false;
  protected ImageIcon removeIcon = new ImageIcon(getClass().getResource("/jcreek/gui/resources/delete.gif"));
  protected ImageIcon followIcon = new ImageIcon(getClass().getResource("/jcreek/gui/resources/relation.gif"));
  private Vector listeners = new Vector();

  protected MouseAdapter actionListen = new MouseAdapter()
      {
        public void mouseClicked(MouseEvent e)
        {
          if(e.getClickCount() == 2 && myTable != null && myTable.columnAtPoint(e.getPoint()) != -1 && myTable.columnAtPoint(e.getPoint()) != 2 &&
             myTable.rowAtPoint(e.getPoint()) != -1)
          {
            fireLinkAction(new LinkEvent(relationAt(myTable.rowAtPoint(e.getPoint()))));
          }
        }
      };

  public FrameTableModel(Relation[] rels)
  {
    setRelations(rels);
  }

  public void formatTable(JTable table, KnowledgeModel knowledgeModel)
  {
//    table.setRowHeight(table.getRowHeight()+2);
    table.getColumnModel().getColumn(2).setMinWidth(followIcon.getImage().getWidth(followIcon.getImageObserver())+5);
    table.getColumnModel().getColumn(2).setMaxWidth(followIcon.getImage().getWidth(followIcon.getImageObserver())+5);
    table.getColumnModel().getColumn(2).setResizable(false);

    if(table.getColumnModel().getColumnCount() > 4)
    {
      table.getColumnModel().getColumn(4).setMinWidth(removeIcon.getImage().
          getWidth(removeIcon.getImageObserver()) + 5);
      table.getColumnModel().getColumn(4).setMaxWidth(removeIcon.getImage().
          getWidth(removeIcon.getImageObserver()) + 5);
      table.getColumnModel().getColumn(4).setResizable(false);
    }

    if(!java.util.Arrays.asList(table.getMouseListeners()).contains(actionListen))
      table.addMouseListener(actionListen);

    table.setDefaultRenderer(Object.class, new DefaultTableCellRenderer()
    {
      public Component getTableCellRendererComponent(JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   boolean hasFocus,
                                                   int row,
                                             int column)
      {
        Component c = null;
/*        if(value instanceof Entity && ((Entity) value).getEntityObject() != null)
        {
          Entity ent = (Entity) value;
          EntityTypeField field = EntityTypeFieldFactory.getEntityTypeField(ent.getEntityObject().getClass());
          if(field != null)
             c = field.getTableCellRendererComponent(table, ((Entity) value).getEntityObject(), isSelected, hasFocus, row, column);
        } */
        if(c == null)
          c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
        if(relationAt(row).isInherited())
        {
          c.setForeground(Color.GRAY);
        }
        else
        if(relationAt(row).isNegated())
        {
          c.setForeground(Color.RED.darker());
        }
        else
          c.setForeground(table.getForeground());
        return c;
      }
    });


    table.setDefaultRenderer(Double.class, new DefaultTableCellRenderer()
    {
      public Component getTableCellRendererComponent(JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   boolean hasFocus,
                                                   int row,
                                             int column)
      {
        Component c = super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);

        if(relationAt(row).isInherited())
        {
          c.setForeground(Color.GRAY);
        }
        else
        if(relationAt(row).isNegated())
        {
          c.setForeground(Color.RED.darker());
        }
        else
          c.setForeground(table.getForeground());

        if(c instanceof JLabel)
        {
          ((JLabel) c).setHorizontalAlignment(JLabel.RIGHT);
        }
        return c;
      }
    });

    Entity[] rcds = knowledgeModel.getRelationType(CBRModel.RELATION).getAllSubclasses();
    Arrays.sort(rcds, new EntityNameComparator());
    final JComboBox relationTypeBox = new JComboBox(rcds);

    table.setDefaultEditor(RelationType.class, new DefaultCellEditor(relationTypeBox));

  /*    {
        JComboBox box;
        public Component getTableCellEditorComponent(JTable table,
                                                     Object value,
                                                     boolean isSelected,
                                                     int row,
                                               int column)
        {
          Entity[] rcds = relationAt(row).getKnowledgeModel().getRelationType("Relation").getAllSubclasses();
          Arrays.sort(rcds, new EntityNameComparator());
          box = new JComboBox(rcds);
          box.setSelectedItem(value);
          return box;
        }

        public Object getCellEditorValue()
        {
            return box.getSelectedItem();
        }
      });
  */

      Entity[] concepts = knowledgeModel.getEntities();
      Arrays.sort(concepts, new EntityNameComparator());
      JComboBox entityBox = new JComboBox(concepts);

      table.setDefaultEditor(Entity.class, new DefaultCellEditor(entityBox));

/*      table.setDefaultEditor(Entity.class, new DefaultCellEditor(entityBox)
      {
        EntityTypeField field = null;
        Entity origValue;

        public Component getTableCellEditorComponent(JTable table,
                                                     Object value,
                                                     boolean isSelected,
                                                     int row,
                                               int column)
        {

          Component c = null;

          if (value instanceof Entity && ( (Entity) value).getEntityObject() != null) {
            origValue = (Entity) value;
            field = EntityTypeFieldFactory.getEntityTypeField(origValue.getEntityObject().
                getClass());
            if(field != null)
            {
              field.addCellEditorListener(new CellEditorListener()
              {
                public void editingCanceled(ChangeEvent e)
                {
                  cancelCellEditing();
                }

                public void editingStopped(ChangeEvent e)
                {
                  stopCellEditing();
                }
              });
              c = field.getTableCellEditorComponent(table, ((Entity) value).getEntityObject() , isSelected, row,
                                                    column);
            }
          }
          if(c == null)
          {
            field = null;
            c = super.getTableCellEditorComponent(table, value, isSelected, row,
                                   column);
          }
          return c;
        }

        public Object getCellEditorValue()
        {
          if(field != null)
          {
            System.out.println("Setting object in "+origValue+" to "+field.getCellEditorValue());
            origValue.setEntityObject((java.io.Serializable) field.getCellEditorValue());
            return origValue;
          }

          return super.getCellEditorValue();
        }
      });

      {
        public Component getTableCellEditorComponent(JTable table,
                                                     Object value,
                                                     boolean isSelected,
                                                     int row,
                                               int column)
        {
          box.setSelectedItem(value);
          box.addActionListener(this.delegate);
          return box;
        }

        public Object getCellEditorValue()
        {
            return box.getSelectedItem();
        }
      }); */

    table.setDefaultEditor(Icon.class, new DefaultCellEditor(new JTextField())
    {
      public Component getTableCellEditorComponent(JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   int row,
                                             int column)
      {
        SwingUtilities.invokeLater(new IconClickedEvent(row, column, this));
        return table.getCellRenderer(row, column).getTableCellRendererComponent(table, value, isSelected, true, row, column);
      }
    });
  }

  public void setTable(JTable table)
  {
    this.myTable = table;
  }

  /**
   * Returns the value at the given position.
   */

  public Object getValueAt(int rowIndex, int columnIndex)
  {
    if(rowIndex < relations.size())
    {
      switch(columnIndex)
      {
        case 0: return relationAt(rowIndex).getRelationType();
        case 1: return relationAt(rowIndex).getValue();
        case 2: return new ImageIcon(getClass().getResource("/jcreek/cke/resources/relation.gif"));
        case 3: return new Double(relationAt(rowIndex).getExplanationStrength());
        case 4: return new ImageIcon(getClass().getResource("/jcreek/cke/resources/delete.gif"));
      }
    }
    else
    {
      return "Add";
    }
    return "WHAT?";
  }

  /**
   *
   */
  public void setValueAt(Object value, int row, int column)
  {
    if(getValueAt(row, column).equals(value))
      return;

    try
    {
      switch (column) {
        case 0:
          relationAt(row).setRelationType( (RelationType)new RelationType( (
              Entity) value));
          break;
        case 1:
          relationAt(row).setValue( (Entity) value);
          break;
        case 3:
          relationAt(row).setExplanationStrength( ( (Double) value).doubleValue());
          break;
      }
    }
    catch(EntityTypeNotMatchedException e)
    {
      e.printStackTrace();
    }
    updateRelation(relationAt(row));
  }

  /**
   * Returns the number of rows in the table
   */
  public int getRowCount()
  {
    return relations.size();
  }

  /**
   * Returns the number of rows in the table
   */
  public int getColumnCount()
  {
    if(isEditable())
      return 5;
    else
      return 4;
  }

  public boolean isCellEditable(int row, int cell)
  {
    if(cell == 2)
      return true;
    return editable;
  }

  public void setEditable(boolean editable)
  {
    if(!editable && this.editable && !java.util.Arrays.asList(myTable.getMouseListeners()).contains(actionListen))
      myTable.addMouseListener(actionListen);
    else if (editable && !this.editable)
      myTable.removeMouseListener(actionListen);

    this.editable = editable;
    this.fireTableStructureChanged();
  }

  public boolean isEditable()
  {
    return editable;
  }

  /**
   * Returns the column names.
   */

  public String getColumnName(int columnIndex)
  {
    switch(columnIndex)
    {
      case 0: return "Relation-type";
      case 2: return "";
      case 1: return "Value";
      case 3: return "Strength";
      case 4: return "Del";
    }
    return "WHAT?";
  }

  public Class getColumnClass(int column)
  {
    switch(column)
    {
      case 0: return RelationType.class;
      case 2: return Icon.class;
      case 1: return Entity.class;
      case 3: return Double.class;
      case 4: return Icon.class;
    }

    return String.class;
  }


  public void removeRelation(Relation rel)
  {
    if(relations.contains(rel))
    {
      int pos = relations.indexOf(rel);
      relations.remove(rel);
      fireTableRowsDeleted(pos, pos);
    }
  }

  public void addRelation(Relation rel)
  {
    relations.add(rel);
    Collections.sort(relations, new Comparator()
    {
      public int compare(Object ob1, Object ob2)
      {
        return ((Relation) ob1).getName().compareTo(((Relation) ob2).getName());
      }
    });
    fireTableRowsInserted(relations.size()-1, relations.size()-1);
  }

  public Relation relationAt(int row)
  {
    return (Relation) relations.get(row);
  }

  public void setRelations(Relation[] rels)
  {
    relations = new Vector();
    relations.addAll(Arrays.asList(rels));
    Collections.sort(relations, new Comparator()
    {
      public int compare(Object ob1, Object ob2)
      {
        return ((Relation) ob1).getName().compareTo(((Relation) ob2).getName());
      }
    });
    fireTableDataChanged();
  }

  public Relation[] getRelations()
  {
    return (Relation[]) relations.toArray(new Relation[0]);
  }

  public void updateRelation(Relation rel)
  {
    int pos = relations.indexOf(rel);
    fireTableRowsUpdated(pos, pos);
    System.out.println(rel.toString());
  }

  public void addLinkListener(LinkListener listener)
  {
    listeners.add(listener);
  }

  public void removeLinkListener(LinkListener listener)
  {
    listeners.remove(listener);
  }

  protected void fireLinkEvent(LinkEvent e)
  {
    for(int i=0;i<listeners.size();i++)
      ((LinkListener) listeners.elementAt(i)).linkClicked(e);
  }

  protected void fireLinkAction(LinkEvent e)
  {

    for(int i=0;i<listeners.size();i++)
      ((LinkListener) listeners.elementAt(i)).linkAction(e);
  }

  void iconClicked(int row, int column)
  {
    switch(column)
    {
      case 2:
        fireLinkEvent(new LinkEvent(relationAt(row).getValue()));
        break;
      case 4:
        Relation rel = relationAt(row);
        if(rel.isInherited())
        {
          rel.negate();
        }
        else
        {
          removeRelation(rel);
          rel.remove();
        }
        break;
    }
  }

  class IconClickedEvent implements Runnable
  {
    private int row;
    private int column;
    private TableCellEditor cde;

    IconClickedEvent(int row, int column, TableCellEditor cde)
    {
      this.row = row;
      this.column = column;
      this.cde = cde;
    }

    public void run()
    {
      cde.stopCellEditing();
      iconClicked(row, column);
    }

  }
}

