package jcreek.gui.representation;

/**
 * Implements a cell editor and renderer for entities that are of the String
 * type.
 *
 * @author Elise Bakke
 * @version 1.0
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import jcreek.representation.*; import jcreek.representation.cbr.*;

public class StringTypeField extends AbstractEntityTypeField
{
  protected JLabel renderComponent = new JLabel();
  protected JTextField editorComponent = new JTextField();
  protected Class editingClass;

  public StringTypeField()
  {
    super();
  }

  /**
   * Makes a generic rendering component that can be used in lists, tables and as
   * parts of components.
   *
   * @param value The value of the field currently
   * @return The editor component.
   */



  public Component getRendererComponent(Object value)
  {
    if(value == null || !(value instanceof String))
     value = "Default String";

   renderComponent.setText( ( (String) value));
   editingClass = value.getClass();

   return renderComponent;
  }

  /**
   * Makes a generic editor component that can be used in lists, tables and as
   * parts of components.
   *
   * @param value The value of the field currently
   * @return The editor component.
   */

  public Component getEditorComponent(Object value)
  {
    if(value == null || !(value instanceof String))
     value = "";

    editorComponent.setText((String) value);
    editorComponent.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        stopCellEditing();
      }
    });

    return editorComponent;
  }

  public Object getCellEditorValue()
  {
    try
    {
      String value = String.valueOf(editorComponent.getText());
      return value;
    }
    catch(NumberFormatException e)
    {
    }
    return null;
  }

}
