package jcreek.gui.representation;

/**
 * Implements a cell editor and renderer for entities that are of the URL
 * type.
 *
 * @author Elise Bakke
 * @version 1.0
 */

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.net.URL;
import java.io.IOException;


public class URLTypeField extends AbstractEntityTypeField
{
  protected JPanel renderComponent;
  protected JTextField editorComponent;
//  protected JButton button;
  protected JLabel label;
//  protected Class editingClass;

  protected JEditorPane editorPane;

//  Frode: None of these fields are necessary.
//  protected URL url;
//  protected String urlString = null;
//  protected boolean b = true; //true if the url to use is a string, false if use URL

  // Added constant that will be used when when input is an invalid string or
  // we start with a new URL.
  protected static String EMPTY_URL = "http://";

  protected Object active;

  public URLTypeField()
  {
    super();

    //placcing GUI elements
    editorComponent = new JTextField();

    renderComponent = new JPanel();
    label = new JLabel("");
    renderComponent.add(label, BorderLayout.CENTER);
  }

  public Component getRendererComponent(Object u)
  {
    URL url = null;

    if(u == null || !(u instanceof URL))
    {
      if(u instanceof String)
      {
        // Frode: Changed things around a bit. It now only accepts strings that are
        // valid URLs. If not, an empty url object is used.
        try
        {
          url = new URL((String)u);
          label.setText(url.toString());
//          urlString = (String) u;
        }
        catch(MalformedURLException e)
        {
//          url = null;
          label.setText("<Malformed URL>");
        }
      }
      else
      {
        label.setText("An error occured. Please try again.");
        System.out.println("value is : "+u+" class "+u.getClass());
      }
    }
    else
    {
        url = (URL) u;
        label.setText(((URL) u).toString());
    }

    JPanel renderComponent = new JPanel(new BorderLayout());
    final JButton button = new JButton(new OpenURLFrameAction(url));
    renderComponent.add(label, BorderLayout.CENTER);
    renderComponent.add(button, BorderLayout.EAST);

    return renderComponent;
  }

  public Component getEditorComponent(Object u)
  {
    active = u;
    if(u == null || !(u instanceof URL)){
      if(u instanceof String){
        editorComponent.setText( (String) u);
      }
      else{
        editorComponent.setText(EMPTY_URL);
//                u = "Eit eller anna rart i getEditorComponent";
      }

    }
    else{
      editorComponent.setText(((URL) u).toString());
    }
    editorComponent.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
/*
        if(active == null || !(active instanceof URL)){
          if(active instanceof String){
            try{
             URL activeurl = new URL ((String) active);
             editorComponent.setText(((URL) activeurl).toString());
            }
            catch(MalformedURLException maue){
              editorComponent.setText("Malformed URL");
            }

          }
          else
          {
              active = "";
          }

            }
            else{
              editorComponent.setText(((URL) active).toString());
            } */

            stopCellEditing();
          }
        });

    return editorComponent;
  }

  public Object getCellEditorValue()
  {
    try
    {
      return new URL(editorComponent.getText());
    }
    catch(MalformedURLException e)
    {
      System.err.println(e);
    }
    return null;
  }


  class OpenURLFrameAction extends AbstractAction
  {
    URL urlToOpen = null;
    OpenURLFrameAction(URL url)
    {
      super("Display");
      urlToOpen = url;
      if(url == null)
        this.setEnabled(false);
    }
    public void actionPerformed(ActionEvent e)
    {
      URL url = null;
      try
      {
        url = new URL(label.getText());
      }
      catch(MalformedURLException ex)
      {
      }

      if(url != null)
      {
        //when pressing the button

        try {
          //test if b is set to true (the URL is a string representation


          /* Frode: Removing this. Assuming it is URL.
                         if (b){
              editorPane = new JEditorPane(urlString);
                         }
                         else { */
            editorPane = new JEditorPane(url);
//            }
          }
          catch (IOException ioe)
          {
            //something is wrong with either urlstring or url
            editorPane = new JEditorPane();
            editorPane.setText("An error occured (IOExcetion)");
          }
          editorPane.setEditable(false);

          JFrame frame = new JFrame();
          frame.getContentPane().add(editorPane, BorderLayout.CENTER);
          frame.setSize(800, 400);
          frame.setVisible(true);

        }
      }
  }
}
