package jcreek.reasoning;

import jcreek.representation.*; import jcreek.representation.cbr.*;
import jcreek.representation.inference.*;
import java.util.*;
import jcreek.util.LRUCache;


/**
 * <p>A comparison between two cases.
 * <p>The comparison is comparable if the entities have one parent in common.
 * Two entities are compared by comparing the relations of the entities,
 * i.e. by creating relation comparisons of every possible combination,
 * (followed by activation and explanation) and computing the strength.
 *
 * @author Ellen Lippe, modified by Frode Srmo of NTNU 2001-2002
 * @version 1.1
*/
public class CaseComparisonNew extends EntityComparison
{
  private double[] findingStrengths = new double[0];
  private double totalRelevance = 0.0;
  private Relation[] targetFindingRelations = new Relation[0];
  private Relation[] problemFindingRelations = new Relation[0];
  private Vector unmatchedTarget = new Vector();
  private Vector unmatchedOrigin = new Vector();
  private Hashtable problemFindingsToRelations = new Hashtable();
  private Set problemFindings = new HashSet();


  public CaseComparisonNew(Entity problem, Entity target)
  {
    super(problem, target);
  }//CaseComparison

  public void reset()
  {
    targetFindingRelations = getTarget().getRelations("has finding");
    problemFindingRelations = getOrigin().getRelations("has finding");
    findingStrengths = new double[targetFindingRelations.length];
    problemFindings = new HashSet(targetFindingRelations.length);
    for(int i=0;i<findingStrengths.length;i++)
      findingStrengths[i] = 0.0;
    problemFindingsToRelations = new Hashtable(targetFindingRelations.length);
  }

  // O(number of findings in problem case + number of findings in target case)
  public double updateActivationStrength()
  {
    // Cycle through the target findings, attempting to find a direct match.
    for(int i=0;i<targetFindingRelations.length;i++)
    {
      Vector matchedEntities = getEntitiesComparableTo(targetFindingRelations[i].getValue());
      // Check if the matchedEntities or the problemFindingRelations are the smallest set to iterate through.

    }
    return 0.0;
  }

  public double updateExplanationStrength()
  {
    return 0.0;
  }

  private Vector getEntitiesComparableTo(Entity ent)
  {
    LRUCache ic = getKnowledgeModel().getInferenceCache();
    Object key[] = new Object[] { "CaseComparison.getEntitiesComparableTo", ent };
    if(ic.isCached(key))
      return (Vector) ic.get(key);

    Vector comparable = new Vector();
    Entity[] allEntities = getKnowledgeModel().getEntities();
    for(int i=0;i<allEntities.length;i++)
      if(checkIsComparable(ent, allEntities[i]))
        comparable.add(allEntities[i]);

    return comparable;
  }

}//Class CaseComparison

