package jcreek.representation;

import java.io.*;
import java.util.*;

/**
 * Checks if the Creek model is valid. This should not normally be required,
 * as the constraints checked here should be forced by the representation,
 * but sometimes bugs happens.
 * @author Frode Srmo
 * @version 1.0
 */

public class CreekModelValidator
{
  public CreekModelValidator()
  {
  }

  public static void main(String argv[])
  {
    String filename = argv[0];
    boolean removeErrors = (argv.length > 0 && argv[1].equals("remove"));

    try
    {
      LocalKnowledgeModel km = new LocalKnowledgeModel(filename);

      // Check that all relations in main index is present in entity indexes.
      ArrayList errorRels = new ArrayList();
      Relation[] relses = km.getRelations();
      for(int k=0;k<relses.length;k++)
//      for(Iterator it = km.getRelationIterator();it.hasNext();)
      {
        boolean error = false;
//        Relation rel = (Relation) it.next();
        Relation rel = relses[k];
        Entity from = rel.getFrom();

        if(rel.getFrom() == null || rel.getValue() == null || rel.getRelationType() == null || rel.getInverse() == null)
        {
          System.out.println("Relation ["+rel.getFrom()+ " - " +rel.getRelationType()+ " - "+rel.getValue()+" ( "+rel.getInverse()+" ) has one or more crucial values as null.");
          error = true;
        }
        else
        {
        if(!Arrays.asList(from.getLocalRelations()).contains(rel))
        {
          error = true;
          System.out.println("Relation ["+rel+"] Does not exist in index of "+from);
        }
        Relation inverse = rel.getInverse();
        if(!inverse.equals(rel) && !km.getRelation(inverse.getID()).equals(inverse))
        {
          error = true;
          System.out.println("Relation ["+rel+"] Inverse does not exist in main index");
        }
        if(!Arrays.asList(rel.getRelationType().getInstances()).contains(rel))
        {
          error = true;
          System.out.println("Relation ["+rel+"] Relation type ('"+rel.getRelationType()+"') index does not include relation!");
          System.out.println(Arrays.asList(rel.getRelationType().getInstances()));
        }
        }
       if(error)
         errorRels.add(rel);
      }

      for(Iterator it = km.getEntityIterator();it.hasNext();)
      {
        boolean error = false;
        Entity ent = (Entity) it.next();
        Relation[] rels = ent.getLocalRelations();
        for(int i=0;i<rels.length;i++)
        {
          if(!km.getRelation(rels[i].getID()).equals(rels[i]))
          {
            error = true;
            System.out.println("Entity ["+ent+"] Relation ('"+rels[i]+"') does not exist in master index!");
          }
          if(!Arrays.asList(rels[i].getValue().getLocalRelations()).contains(rels[i].getInverse()))
          {
            error = true;
            System.out.println("Entity [" + ent + "] Relation ('" + rels[i] +
                               "') does not have an inverse registered!");
          }
        }
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }
}
