package jcreek.representation;

/**
 * A particular type of entity that encapsulates an instance of the
 * Java Double class.
 *
 * @author Frode Srmo
 * @version 1.0
 */

public class DoubleEntity extends NumberEntity
{
  /**
   * Creates a new DoubleEntity.
   *
   * @param model The KnowledgeModel to create the new entity in.
   * @param number The number to encapsulate in this entity.
   * @param description The description of the entity.
   */
  public DoubleEntity(KnowledgeModel model, double number, String description)
  {
    super(model, new Double(number), description);
  }

  /**
   * Creates a new NumberEntity that is a value of a supplied entity.
   */
  public DoubleEntity(KnowledgeModel model, Double number, Entity type)
  {
    super(model, number, type);
  }

  /**
   * Creates a new NumberEntity that is a value of a supplied entity.
   */
  public DoubleEntity(KnowledgeModel model, double number, Entity type)
  {
    super(model, new Double(number), type);
  }

  /**
   * Creates a NumberEntity from an existing Entity.
   */
  public DoubleEntity(Entity ent) throws EntityTypeNotMatchedException
  {
    super(ent);
  }

  /**
   * Returns the encapsulated number.
   */
  public Double asDouble()
  {
    try
    {
      return (Double) getEntityObject();
    }
    catch(ClassCastException e)
    {
      return new Double(Double.NaN);
    }
  }

  /**
   * An Entity is of type DoubleEntity if and only if it encapsulates an object
   * that is an instance of java.lang.Double.
   */
  public static boolean matches(Entity ent)
  {
    return ent.getEntityObject() != null && ent.getEntityObject() instanceof Double;
  }
}