package jcreek.representation;

import jcreek.util.CreekException;

/**
 * <p>Title: EntityTypeNotMatchedException</p>
 * <p>Description: Thrown when an attempt is made to instanciate an EntityType
 * with an entity that do not match the pattern expressed by this type.</p>
 * @author Frode Srmo
 * @version 1.0
 */

public class EntityTypeNotMatchedException extends CreekException
{
  private Entity entity;
  private Class entityTypeClass;

  /**
   * Creates a new EntityTypeNotMatchedException.
   *
   * @param entity The entity that is attempted instanciated.
   * @param entityType The subclass of EntityType this entity do not currently match.
   */
  public EntityTypeNotMatchedException(Entity entity, Class entityType)
  {
    super("The entity "+entity+" can not be matched to EntityType subclass "+entityType.getName()+".");
    this.entity = entity;
    this.entityTypeClass = entityType;
  }

  /**
   * Returns the entity that the was attempted to be instanciated.
   */
  public Entity getEntity()
  {
    return entity;
  }

  /**
   * Returns the class of the EntityType subclass.
   */
  public Class getEntityTypeClass()
  {
    return entityTypeClass;
  }
}