package jcreek.representation;

import java.util.EventObject;

public class ModelChangeEvent extends EventObject
{
  public static final int MODEL_NAME_CHANGED = 1;
  public static final int MODEL_TITLE_CHANGED = 2;
  public static final int MODEL_DESC_CHANGED = 3;
  public static final int MODEL_AUTHOR_CHANGED = 4;

  public static final int ENTITY_REMOVED = 101;
  public static final int ENTITY_ADDED = 102;
  public static final int ENTITY_NAME_CHANGED = 103;
  public static final int ENTITY_DESC_CHANGED = 104;
  public static final int ENTITY_RELATION_ADDED = 105;
  public static final int ENTITY_RELATION_REMOVED = 106;
  public static final int ENTITY_DATA_OBJECT_CHANGED = 107;

  public static final int RELATION_REMOVED = 201;
  public static final int RELATION_ADDED = 202;
  public static final int RELATION_NAME_CHANGED = 203;
  public static final int RELATION_DESC_CHANGED = 204;
  public static final int RELATION_VALUE_CHANGED = 205;
  public static final int RELATION_FROM_CHANGED = 206;
  public static final int RELATION_TYPE_CHANGED = 207;
  public static final int RELATION_EXPLANTION_STRENGTH_CHANGED = 208;
  public static final int RELATION_TITLE_CHANGED = 209;
  public static final int RELATION_OVERRIDE_ADDED = 210;
  public static final int RELATION_OVERRIDE_REMOVED = 211;
  public static final int RELATION_INVERSE_CHANGED = 212;

  public static final int PARTITION_ADDED = 301;
  public static final int PARTITION_REMOVED = 302;
  public static final int PARTITION_ENTITY_ADDED = 303;
  public static final int PARTITION_ENTITY_REMOVED = 304;
  public static final int PARTITION_RELATION_ADDED = 305;
  public static final int PARTITION_RELATION_REMOVED = 306;
  public static final int PARTITION_NAME_CHANGE = 307;

  protected int type;
  protected Object oldValue;
  protected Object newValue;

  /**
   * Creates a a ConceptChangeEvent.
   *
   * @param source The Entity, Relation or KnowledgeModel object affected by the change.
   * @param changeType The constant representing what kind of change this is.
   * @param oldValue The old value of the variable changed.
   * @param newValue The new value of the variable changed.
   */
  public ModelChangeEvent(Object source, int changeType, Object oldValue, Object newValue)
  {
    super(source);
    type = changeType;
    this.oldValue = oldValue;
    this.newValue = newValue;
  }

  /**
   * Creates a a ConceptChangeEvent. This should only be used for add/remove events where
   * there is no variable having an old and new value.
   *
   * @param source The Entity, Relation or KnowledgeModel object affected by the change.
   * @param changeType The constant representing what kind of change this is.
   */
  public ModelChangeEvent(Object source, int changeType)
  {
    super(source);
    type = changeType;
  }

  public int getType()
  {
    return type;
  }

  public Object getOldValue()
  {
    return oldValue;
  }

  public Object getNewValue()
  {
    return newValue;
  }

  public String toString()
  {
    return "ConceptChangeEvent: "+getSource().toString()+" type: "+type+ (oldValue==null?"":" old value is "+oldValue)+ (newValue==null?"":" new value is "+oldValue);
  }
}
