package jcreek.representation;

import jcreek.util.CreekException;

/**
 * Title:        NoSuchRelationException
 * Description:
 * Copyright:    Copyright (c) 2002
 * Company:      NTNU
 * @author       Frode Srmo
 * @version 1.0
 */

public class NoSuchRelationException extends CreekException
{
  RelationType[] rcd;
  Entity ent;

  /**
   * Creates a new NoSuchRelationException.
   *
   * @param rcd The type of the relation not found.
   * @param ent The enity this relation type was not found in.
   */
  public NoSuchRelationException(RelationType rcd, Entity ent)
  {
    super("No relation of type '"+rcd.getName()+"' found in entity '"+ent.getName()+"'");
    this.rcd = new RelationType[] {rcd};
    this.ent = ent;
  }

  /**
   * Creates a new NoSuchRelationException.
   *
   * @param rcd The arrays of types of the relation not found.
   * @param ent The enity this relation type was not found in.
   */
  public NoSuchRelationException(RelationType rcd[], Entity ent)
  {
    super("No relation of any of the "+rcd.length+" types was found in entity '"+ent.getName()+"'");
    this.rcd = rcd;
    this.ent = ent;
  }

  /**
   * Returns the relation type that could not be found.
   */
  public RelationType getRelationType()
  {
    if(rcd.length > 0)
      return rcd[0];
    else
      return null;
  }

  /**
   * Returns the relation types that could not be found.
   */
  public RelationType[] getRelationTypes()
  {
    return rcd;
  }

  /**
   * Returns the entity in which the relation could not be found.
   */
  public Entity getEntity()
  {
    return ent;
  }
}

