package jcreek.representation;

import jcreek.util.CreekException;

/**
 * Title:        NoSuchRelationTypeException
 * Description:  Thrown when attempting to reference a relation type by a string
 * and this string does not identify a relation type in the current knowledge model.
 * @author Frode Srmo
 * @version 1.0
 */

public class NoSuchRelationTypeException extends CreekException
{
  private String relationName;

  /**
   * Creates a new NoSuchRelationTypeException with the given string.
   */
  public NoSuchRelationTypeException(String relationName)
  {
    super("No relation type with the name '"+relationName+"' found");
    this.relationName = relationName;
  }

  /**
   * The string supplied as a relation type name for which there is no matching
   * relation type in the current knowledge model.
   */
  public String getRelationName()
  {
    return relationName;
  }
}

