package jcreek.representation;

/**
 * A particular type of entity that encapsulates an instance of any of the
 * Java Number classes.
 *
 * @author Frode Srmo
 * @version 1.0
 */

public class NumberEntity extends EntityType
{
  /**
   * Creates a new NumberEntity.
   *
   * @param model The KnowledgeModel to create the new entity in.
   * @param number The number to encapsulate in this entity.
   * @param description The description of the entity.
   */
  public NumberEntity(KnowledgeModel model, Number number, String description)
  {
    super(makeEntity(model, description), true);
    setEntityObject(number);
  }

  /**
   * Creates a new NumberEntity that is a value of a supplied entity.
   */
  public NumberEntity(KnowledgeModel model, Number number, Entity type)
  {
    this(model, number, "");
    try
    {
      addRelation(BasicModel.INSTANCE_OF, type);
    }
    catch(NoSuchRelationTypeException e)
    {
      // This should never happen.
      e.printStackTrace();
    }
  }

  /**
   * Creates a NumberEntity from an existing Entity.
   */
  public NumberEntity(Entity ent) throws EntityTypeNotMatchedException
  {
    super(ent);
  }

  private static Entity makeEntity(KnowledgeModel model, String description)
  {
    Entity entity = null;
    int i = model.entitySize()+1;
    while(entity == null)
    {
      try
      {
        entity = new Entity(model, "NumberEntity#"+i, description);
      }
      catch(NameAlreadyExistException e)
      {
        i++;
      }
    }
    return entity;
  }

  /**
   * Changes the encapsulated number of this entity.
   *
   * @param number The new Number.
   */
  public void setValue(Number number)
  {
    setEntityObject(number);
  }

  /**
   * Returns the encapsulated number.
   */
  public Number asNumber()
  {
    try
    {
      return (Number) getEntityObject();
    }
    catch(ClassCastException e)
    {
      return null;
    }
  }

  /**
   * An Entity is of type NumberEntity if and only if it encapsulates an object
   * that is an instance of java.lang.Number.
   */
  public static boolean matches(Entity ent)
  {
    return ent.getEntityObject() != null && ent.getEntityObject() instanceof Number;
  }
}
