package jcreek.representation.cbr;

import jcreek.representation.*; import jcreek.representation.cbr.*;
/**
 * Represents an entity type that encapsulates a Java Class object.
 *
 * @author Frode Srmo
 * @version 1.0
 */

public class ClassEntity extends EntityType
{
  /**
   * Creates a new ClassEntity.
   *
   * @param model The KnowledgeModel to create the new entity in.
   * @param myClass The class to encapsulate in this entity.
   * @param description The description of the entity.
   */
  public ClassEntity(KnowledgeModel model, Class myClass, String description) throws NameAlreadyExistException
  {
    super(model, formName(myClass), description);
    setEntityObject(myClass);
  }

  private static String formName(Class myClass)
  {
    int i = myClass.getName().lastIndexOf(".");
    if(i > 0 && i+1 < myClass.getName().length())
      return myClass.getName().substring(i+1);
    return myClass.getName();
  }

  /**
   * Creates a ClassEntity from an existing Entity.
   */
  public ClassEntity(Entity ent) throws EntityTypeNotMatchedException
  {
    super(ent);
  }

  /**
   * Sets the class encapsulated by this entity.
   *
   * param myClass The class to set as the new value.
   */
  public void setClass(Class myClass)
  {
    setEntityObject(myClass);
  }

  /**
   * Returns the class encapsulated by this entity
   */
  public Class asClass()
  {
    try
    {
      return (Class) getEntityObject();
    }
    catch(ClassCastException e)
    {
      return null;
    }
  }

  /**
   * Returns true if the given entitiy can be seen as a ClassEntity
   *
   * @param ent The entity to check.
   */
  public static boolean matches(Entity ent)
  {
    return ent.getEntityObject() != null && ent.getEntityObject() instanceof Class;
  }
}
