package jcreek.representation.cbr;

import jcreek.representation.*;
import jcreek.util.CreekException;

/**
 * Builds the basic model that is common to all jcreek models.
 *
 * This model is now deprecated. Use BasicModel if you only plan to use the
 *
 *
 * @author Frode Srmo, changed by Elise Bakke
 * @version 1.2
 */

//Denne er i prosjekt 2

public class IsoPodModel implements IsoPod, EpistemologicalModel
{
  public void generateEpistemologicalModel(KnowledgeModel model)
  {
    generateCBRModel(model);
  }

  public static void generateCBRModel(KnowledgeModel model)
  {
    try
    {
      model.getInferenceCache().suspendFlush(true);
      // Starts by creating the very basic hierarchy.
      Entity concept = new Entity(model, "Concept", "");
      Entity entity = new Entity(model, "Entity", "");

      // The "Relation", "Relation (inverse)", "has inverse" and "subclass of" relations are created
      // without using the RelationType normal constructor since they are used within the RelationType
      // class.
      RelationType relation = new RelationType(new Entity(model, "Relation", ""));
      model.registerEntity(RELATION, relation);
      RelationType inverseRelation = new RelationType(new Entity(model, "Relation (inverse)", ""));
      model.registerEntity(INVERSE_RELATION, inverseRelation);

      RelationType structuralRelation = new RelationType(new Entity(model, "Structural Relation", ""));
      model.registerEntity(STRUCTURAL_RELATION, relation);
      RelationType inverseStructuralRelation = new RelationType(new Entity(model, "Structural Relation (inverse)", ""));
      model.registerEntity(INVERSE_STRUCTURAL_RELATION, inverseRelation);

      RelationType hasInverse = new RelationType(new Entity(model, "has inverse", ""));
      model.registerEntity(HAS_INVERSE, hasInverse);

      RelationType subclassOf = new RelationType(new Entity(model, "subclass of", ""));
      model.registerEntity(SUBCLASS_OF, subclassOf);
      RelationType hasSubclass = new RelationType(new Entity(model, "has subclass", ""));
      model.registerEntity(HAS_SUBCLASS, hasSubclass);

      // We are now at the point where it is safe to add relations that do not have an inverse relation.
      // The "has inverse" relation itself is the only example of this so far, but is also all we need
      // to establish inverse relation types for the other relations.
      relation.addRelation(HAS_INVERSE, inverseRelation, 1.0);
      subclassOf.addRelation(HAS_INVERSE, hasSubclass, 1.0);
      structuralRelation.addRelation(HAS_INVERSE, inverseStructuralRelation, 1.0);

      // At this point, we can use "subclass of".
      concept.addRelation(HAS_SUBCLASS, relation, 1.0);
      concept.addRelation(HAS_SUBCLASS, entity, 1.0);
      relation.addRelation(HAS_SUBCLASS, structuralRelation, 1.0);
      relation.addRelation(HAS_SUBCLASS, inverseRelation, 1.0);
      inverseRelation.addRelation(HAS_SUBCLASS, inverseStructuralRelation, 1.0);
      structuralRelation.addRelation(HAS_SUBCLASS, hasSubclass, 1.0);
      inverseStructuralRelation.addRelation(HAS_SUBCLASS, subclassOf, 1.0);

      // Adds the rest of the relation types used by the normal RelationType constructor.
      RelationType hasDefaultExplanationStrength = new RelationType(new Entity(model, "has default explanation strength", ""));
      RelationType defaultExplanationStrengthOf = new RelationType(new Entity(model, "default explanation strength of", ""));

      hasDefaultExplanationStrength.addRelation(HAS_INVERSE, defaultExplanationStrengthOf, 1.0);

      model.registerEntity(HAS_DEFAULT_EXPLANATION_STRENGTH, hasDefaultExplanationStrength);
      model.registerEntity(DEFAULT_EXPLANATION_STRENGTH_OF, defaultExplanationStrengthOf);

      inverseRelation.addRelation(HAS_DEFAULT_EXPLANATION_STRENGTH, new DoubleEntity(model, 1.0, ""), 1.0);
      subclassOf.addRelation(HAS_DEFAULT_EXPLANATION_STRENGTH, new DoubleEntity(model, 0.9, ""), 1.0);
      hasSubclass.addRelation(HAS_DEFAULT_EXPLANATION_STRENGTH, new DoubleEntity(model, 0.9, ""), 1.0);
      hasDefaultExplanationStrength.addRelation(HAS_DEFAULT_EXPLANATION_STRENGTH, new DoubleEntity(model, 0.5, ""), 1.0);
      defaultExplanationStrengthOf.addRelation(HAS_DEFAULT_EXPLANATION_STRENGTH, new DoubleEntity(model, 0.5, ""), 1.0);

      // At this point we have established enough of the model to use the RelationType constructor and
      // the addRelation() method in Entity normally.


    // The rest of the relation hierarchy

      RelationType hasInstance = new RelationType(model, structuralRelation, "has instance", "The target is an instance of the origin",
          "instance of", "The target is the class of the origin.", 0.9);
      RelationType instanceOf = hasInstance.getInverseType();
      model.registerEntity(HAS_INSTANCE, hasInstance);
      model.registerEntity(INSTANCE_OF, instanceOf);

      RelationType hasPart = new RelationType(model, structuralRelation, "has part", "The target is a part of the origin",
          "part of", "The target is a part of the origin.", 0.7);
      RelationType partOf = hasPart.getInverseType();
      model.registerEntity(HAS_PART, hasPart);
      model.registerEntity(PART_OF, partOf);

      RelationType hasValue = new RelationType(model, structuralRelation, "has value", "The target is a value of the origin",
          "value of", "The target is a value of the origin.", 0.7);

      RelationType systemRelation = new RelationType(model, relation, "System Relation", "Relation used by the core Creek system.",
          "System Relation (inverse)", "Relation used by the core Creek system.", 0.9);
      model.registerEntity(SYSTEM_RELATION, systemRelation);
      model.registerEntity(INVERSE_SYSTEM_RELATION, systemRelation.getInverseType());

      hasDefaultExplanationStrength.addRelation(SUBCLASS_OF, relation);
      defaultExplanationStrengthOf.addRelation(SUBCLASS_OF, inverseRelation);
      structuralRelation.addRelation(HAS_SUBCLASS, hasInverse);
      inverseStructuralRelation.addRelation(HAS_SUBCLASS, hasInverse);

      RelationType transfers = new RelationType(model, systemRelation, "transfers", "The target relation-type is transferred over the origin relation-type.",
          "inherits over", "The target relation-type is inherited over the origin relation-type", 1.0);
      model.registerEntity(TRANSFERS, transfers);
      model.registerEntity(INHERITS_OVER, transfers.getInverseType());

      RelationType hasComparator = new RelationType(model, systemRelation, "has comparator", "The target entity encapsulates a java class that implements a comparator for the origin class.",
          "comparator of", "The target class is a type of entity compared by the encapsulated class of the origin entity.", 1.0);
      model.registerEntity(HAS_COMPARATOR, hasComparator);
      model.registerEntity(COMPARATOR_OF, hasComparator.getInverseType());

      RelationType hasCaseStatus = new RelationType(model, systemRelation, "has case status", "The target entity represents the status of the case (the origin).",
          "case status of", "The target is a case the origin is the status for.", 1.0);
      model.registerEntity(HAS_CASE_STATUS, hasCaseStatus);
      model.registerEntity(CASE_STATUS_OF, hasCaseStatus.getInverseType());

      RelationType hasSolution = new RelationType(model, systemRelation, "has solution", "The target is the solution of the origin case.",
          "solution of", "The target is a case the origin is the solution for.", 1.0);
      model.registerEntity(HAS_SOLUTION, hasSolution);
      model.registerEntity(SOLUTION_OF, hasSolution.getInverseType());

      RelationType hasValueType = new RelationType(model, systemRelation, "has value type", "The origin relation-type can only have instances of the target entity as its value type, i.e all relations of this type should have an instance of this entity as its value.",
          "value type of ", "The target is a relation-type this entity is the value type for", 1.0);
      model.registerEntity(HAS_VALUE_TYPE, hasValueType);
      model.registerEntity(VALUE_TYPE_OF, hasValueType.getInverseType());

      RelationType hasFromType = new RelationType(model, systemRelation, "has from type", "The origin relation-type can only have instances of the target entity as its from type, i.e all relations of this type should have an instance of this entity as its from value.",
          "from type of ", "The target is a relation-type this entity is the from type for", 1.0);
      model.registerEntity(HAS_FROM_TYPE, hasFromType);
      model.registerEntity(FROM_TYPE_OF, hasFromType.getInverseType());


      RelationType findingRelation = new RelationType(model, relation, "has finding", "Relation from a case to a finding.",
          "finding of", "Relation from a finding to a case", 0.5);
      model.registerEntity(FINDING_RELATION, findingRelation);
      model.registerEntity(INVERSE_FINDING_RELATION, findingRelation.getInverseType());

      RelationType causalRelation = new RelationType(model, relation, "Causal Relation", "A superclass for causal relations.",
          "Causal Relation (inverse)", "Superclass for invese causal relations.", 0.5);
      model.registerEntity(CAUSAL_RELATION, causalRelation);
      model.registerEntity(INVERSE_CAUSAL_RELATION, causalRelation.getInverseType());

      RelationType causes = new RelationType(model, causalRelation, "causes", "The target is caused by the origin",
          "caused by", "The target causes the origin", 0.7);
      model.registerEntity(CAUSES, causes);
      model.registerEntity(CAUSED_BY, causes.getInverseType());

      RelationType implies = new RelationType(model, causalRelation, "implies", "The target is implied (statistically) by the origin",
          "implied by", "The target implies (statistically) the origin", 0.4);
      model.registerEntity(IMPLIES, implies);
      model.registerEntity(IMPLIED_BY, implies.getInverseType());

      // Creating the entity hierarchy.

      Entity creekCase = new Entity(model, "Case", "");
      creekCase.addRelation(SUBCLASS_OF, entity);
      model.registerEntity(CASE, creekCase);

      Entity number = new Entity(model, "Number", "");
      number.addRelation(SUBCLASS_OF, entity);
      model.registerEntity(NUMBER, number);

      Entity string = new Entity(model, "String", "");
      string.addRelation(SUBCLASS_OF, entity);
      model.registerEntity(STRING, string);

      Entity url = new Entity(model, "URL", "");
      url.addRelation(SUBCLASS_OF, entity);
      model.registerEntity(URL, url);

      Entity symbol = new Entity(model, "Symbol", "");
      symbol.addRelation(SUBCLASS_OF, entity);
      model.registerEntity(SYMBOL, symbol);

      Entity javaClass = new Entity(model, "Java Class", "");
      javaClass.addRelation(SUBCLASS_OF, entity);
      model.registerEntity(JAVA_CLASS, javaClass);

      Entity noValue = new Entity(model, "No Value", "");
      noValue.addRelation(SUBCLASS_OF, entity);
      model.registerEntity(NO_VALUE, noValue);

      Entity caseStatus = new Entity(model, "Case Status", "");
      caseStatus.addRelation(SUBCLASS_OF, symbol);
      model.registerEntity(CASE_STATUS, caseStatus);

      Entity processedCase = new Entity(model, "Processed Case", "");
      processedCase.addRelation(INSTANCE_OF, caseStatus);
      model.registerEntity(PROCESSED_CASE, processedCase);

      Entity solvedCase = new Entity(model, "Solved Case", "");
      solvedCase.addRelation(INSTANCE_OF, caseStatus);
      model.registerEntity(SOLVED_CASE, solvedCase);

      Entity unsolvedCase = new Entity(model, "Unsolved Case", "");
      unsolvedCase.addRelation(INSTANCE_OF, caseStatus);
      model.registerEntity(UNSOLVED_CASE, unsolvedCase);

      Entity comparator = new Entity(model, "Comparator", "");
      comparator.addRelation(SUBCLASS_OF, javaClass);
//      model.registerEntity(UNSOLVED_CASE, unsolvedCase);

      Entity caseComparison = new ClassEntity(model, jcreek.reasoning.CaseComparison.class, "");
      caseComparison.addRelation(INSTANCE_OF, comparator);
      caseComparison.addRelation(COMPARATOR_OF, creekCase);

      Entity numberComparison = new ClassEntity(model, jcreek.reasoning.NumberComparison.class, "");
      numberComparison.addRelation(INSTANCE_OF, comparator);
      numberComparison.addRelation(COMPARATOR_OF, number);

      Entity stringComparison = new ClassEntity(model, jcreek.reasoning.StringComparison.class, "");
      stringComparison.addRelation(INSTANCE_OF, comparator);
      stringComparison.addRelation(COMPARATOR_OF, string);

      Entity urlComparison = new ClassEntity(model, jcreek.reasoning.URLComparison.class, "");
      urlComparison.addRelation(INSTANCE_OF, comparator);
      urlComparison.addRelation(COMPARATOR_OF, url);

      Entity symbolComparison = new ClassEntity(model, jcreek.reasoning.SymbolComparison.class, "");
      symbolComparison.addRelation(INSTANCE_OF, comparator);
      symbolComparison.addRelation(COMPARATOR_OF, symbol);

      // Transfer Rules

      Relation transferBySubclass = relation.addRelation(CBRModel.INHERITS_OVER, subclassOf);

      Relation transferByInstanceOf = relation.addRelation(CBRModel.INHERITS_OVER, instanceOf);

      // ...except yourself

      Relation r;

      r = subclassOf.addRelation(CBRModel.INHERITS_OVER, subclassOf);
      r.negate();
      r.getInverse().negate();
      r.addOverriding(transferBySubclass);

      r = hasSubclass.addRelation(CBRModel.INHERITS_OVER, subclassOf);
      r.negate();
      r.getInverse().negate();
      r.addOverriding(transferBySubclass);

      r = subclassOf.addRelation(CBRModel.INHERITS_OVER, instanceOf);
      r.negate();
      r.getInverse().negate();
      r.addOverriding(transferByInstanceOf);

      r = hasSubclass.addRelation(CBRModel.INHERITS_OVER, instanceOf);
      r.negate();
      r.getInverse().negate();
      r.addOverriding(transferByInstanceOf);


      r = instanceOf.addRelation(CBRModel.INHERITS_OVER, subclassOf);
      r.negate();
      r.getInverse().negate();

      r = hasInstance.addRelation(CBRModel.INHERITS_OVER, subclassOf);
      r.negate();
      r.getInverse().negate();
      r.addOverriding(transferBySubclass);

      r = instanceOf.addRelation(CBRModel.INHERITS_OVER, instanceOf);
      r.negate();
      r.getInverse().negate();
      r.addOverriding(transferByInstanceOf);

      r = hasInstance.addRelation(CBRModel.INHERITS_OVER, instanceOf);
      r.negate();
      r.getInverse().negate();
      r.addOverriding(transferByInstanceOf);

      // ...also the "has inverse" and "inverse of" relations should not be inherited.

      r = hasInverse.addRelation(CBRModel.INHERITS_OVER, subclassOf);
      r.negate();
      r.getInverse().negate();
      r.addOverriding(transferBySubclass);

      r = hasInverse.addRelation(CBRModel.INHERITS_OVER, instanceOf);
      r.negate();
      r.getInverse().negate();
      r.addOverriding(transferByInstanceOf);

      // Causal relations are transitive.

      causalRelation.addRelation(CBRModel.INHERITS_OVER, causalRelation);
      causalRelation.getInverseType().addRelation(CBRModel.INHERITS_OVER, causalRelation.getInverseType());

      // Has Part relations transfer causal relations.

      causalRelation.addRelation(CBRModel.INHERITS_OVER, partOf);
      causalRelation.getInverseType().addRelation(CBRModel.INHERITS_OVER, partOf);


    }
    catch(CreekException e)
    {
      e.printStackTrace();
    }
    model.getInferenceCache().suspendFlush(false);
  }

  public static void main(String argv[])
  {
    LocalKnowledgeModel km = new LocalKnowledgeModel();

    try
    {
      RelationType blah = new RelationType(km, km.getRelationType(RELATION), "has blah", "blah", "blah of", "halb", 0.7);
      blah.addRelation("transfers", blah);

      Entity frode = new Entity(km, "Frode", "Me");
      frode.addRelation("causes", new Entity(km, "Dust", "Bah"));
      frode.addRelation("subclass of", km.getEntity("Symbol"));
      km.getEntity("Concept").addRelation("has blah", new Entity(km, "test", "test"));
      km.getEntity("test").addRelation("has blah", new Entity(km, "test2", "test2"));
      km.getEntity("Entity").addRelation("has blah", km.getEntity("test"));
      km.getEntity("Symbol").addRelation("has blah", km.getEntity("test"));


/**     for(java.util.Iterator entityIt=km.getEntityIterator(); entityIt.hasNext();)
    {
      System.out.println(((Entity) entityIt.next()).getFrameString());
    } **/



//    Relation rels[] = km.getEntity("FRODE").getRelations();
    /*
    for(int i=0;i<rels.length;i++)
      if(rels[i].isInherited())
      {
        javax.swing.JFrame frame = new javax.swing.JFrame(rels[i].toString());
        frame.getContentPane().setLayout(new java.awt.BorderLayout());
        frame.getContentPane().add(new jcreek.gui.ExplanationView(rels[i]), java.awt.BorderLayout.CENTER);
        frame.setSize(800, 600);
        frame.show();
      }
*/
      try
      {

        km.saveAs("C:\\Temp\\Fruit");

        LocalKnowledgeModel km2 = new LocalKnowledgeModel("C:\\Temp\\Fruit.km");

        System.out.println("\n\n\n  GETTING FRAME STRING \n\n\n");

        System.out.println(km2.getEntity("FRODE").getFrameString());

        km2.getEntity("Frode").getRelation("causes").remove();
        System.out.println("In "+km2.toString()+" #"+km2.hashCode());
         System.out.println("Size of entities: "+km2.entitySize());
//        km2.getInferenceCache().flush();

        System.out.println("\n\n\n  GETTING FRAME STRING \n\n\n");

        System.out.println(km2.getEntity("FRODE").getFrameString());

        Entity[] ents = km2.getEntities();
        for(java.util.Iterator it=km2.getEntityIterator();it.hasNext();)
        {
          Entity next = (Entity) it.next();
          for(int j=0;j<ents.length;j++)
          {
            if(next.getKnowledgeModel() != ents[j].getKnowledgeModel())
              System.err.println("===> km in "+next+" and "+ents[j]);
            if(next.getKnowledgeModel().getInferenceCache() != ents[j].getKnowledgeModel().getInferenceCache())
              System.err.println("===> inf in "+next+" and "+ents[j]);
          }
        }

      }
      catch(java.io.IOException e)
      {
        e.printStackTrace();
      }
      catch(ClassNotFoundException e)
      {
        e.printStackTrace();
      }

    }
    catch(CreekException e)
    {
      e.printStackTrace();
    }
  }
}
