package jcreek.representation.cbr;

import jcreek.representation.*;
/**
 * A particular type of entity that encapsulates an instance of String.
 *
 * @author Elise Bakke
 * @version 1.0
 */

public class StringEntity extends EntityType
{
  /**
   * Creates a new StringEntity.
   *
   * @param model The KnowledgeModel to create the new entity in.
   * @param string The string to encapsulate in this entity.
   * @param description The description of the entity.
   */
  public StringEntity(KnowledgeModel model, String string, String description)
  {
    super(makeEntity(model, description), true);
    setEntityObject(string);
  }

  /**
   * Creates a new StringEntity that is a value of a supplied entity.
   */
  public StringEntity(KnowledgeModel model, String string, Entity type)
  {
    this(model, string, "");
    try
    {
      addRelation(CBRModel.INSTANCE_OF, type);
    }
    catch(NoSuchRelationTypeException e)
    {
      // This should never happen.
      e.printStackTrace();
    }
  }

  /**
   * Creates a StringEntity from an existing Entity.
   */
  public StringEntity(Entity ent) throws EntityTypeNotMatchedException
  {
    super(ent);
  }

  private static Entity makeEntity(KnowledgeModel model, String description)
  {
    Entity entity = null;
    int i = model.entitySize()+1;
    while(entity == null)
    {
      try
      {
        entity = new Entity(model, "StringEntity#"+i, description);
      }
      catch(NameAlreadyExistException e)
      {
        i++;
      }
    }
    return entity;
  }

  /**
   * Changes the encapsulated string of this entity.
   *
   * @param string The new String.
   */
  public void setString(String string)
  {
    setEntityObject(string);
  }

  /**
   * Returns the encapsulated string.
   */
  public String asString(){
    try
    {
      return (String) getEntityObject();
    }
    catch(ClassCastException e)
    {
      return null;
    }
  }

  /**
   * An Entity is of type StringEntity if and only if it encapsulates an object
   * that is an instance of java.lang.String.
   */
  public static boolean matches(Entity ent)
  {
    return ent.getEntityObject() != null && ent.getEntityObject() instanceof String;
  }
}
