package jcreek.representation.cbr;

import jcreek.representation.*;

import java.net.*;

/**
 * A particular type of entity that encapsulates an instance of URL.
 *
 * @author Elise Bakke
 * @version 1.0
 */

public class URLEntity extends EntityType
{
  /**
   * Creates a new URLEntity.
   *
   * @param model The KnowledgeModel to create the new entity in.
   * @param url The url to encapsulate in this entity.
   * @param description The description of the entity.
   */
  public URLEntity(KnowledgeModel model, URL url, String description)
  {
    super(makeEntity(model, description), true);
    setEntityObject(url);
  }

  /**
   * Creates a new URLEntity that is a value of a supplied entity.
   *
   * @param model The KnowledgeModel to create a new entity in
   * @param url The url to encapsulate in this entity.
   * @param enity The Entity to encapsulate
   */
  public URLEntity(KnowledgeModel model, URL url, Entity type)
  {
    this(model, url, "");
    try
    {
      addRelation(CBRModel.INSTANCE_OF, type);
    }
    catch(NoSuchRelationTypeException e)
    {
      // This should never happen.
      e.printStackTrace();
    }
  }

  /**
   * Creates a URLEntity from an existing Entity.
   *
   * @param ent The Entity to make the URLEntity from.
   */
  public URLEntity(Entity ent) throws EntityTypeNotMatchedException
  {
    super(ent);
  }

  private static Entity makeEntity(KnowledgeModel model, String description)
  {
    Entity entity = null;
    int i = model.entitySize()+1;
    while(entity == null)
    {
      try
      {
        entity = new Entity(model, "URLEntity#"+i, description);
      }
      catch(NameAlreadyExistException e)
      {
        i++;
      }
    }
    return entity;
  }

  /**
   * Changes the encapsulated url of this entity.
   *
   * @param url The new URL.
   */
  public void setURL(URL url)
  {
    setEntityObject(url);
  }

  /**
   * Returns the encapsulated url.
   */
  public URL asURL(){
    try
    {
      return (URL) getEntityObject();
    }
    catch(ClassCastException e)
    {
      return null;
    }
  }

  /**
   * An Entity is of type URLEntity if and only if it encapsulates an object
   * that is an instance of java.net.URL.
   */
  public static boolean matches(Entity ent)
  {
    return ent.getEntityObject() != null && ent.getEntityObject() instanceof URL;
  }
}
