package jcreek.representation.inference;

import java.util.Vector;
import java.util.Hashtable;
import java.util.HashSet;

import jcreek.representation.Entity;
import jcreek.representation.Relation;
import jcreek.representation.RelationType;

/**
 * Abstract class for implementing inheritance methods over the spreading
 * activation class.
 */

public abstract class InheritanceMethod extends SpreadingActivationRule
{
  /**
   * This method is ment as a wrapper for the execute() method of
   * SpreadingActivationRule, where the relations found by spreading
   * activation is inherited to the origin entity.
   */

  protected transient boolean DEBUG = false;

  public Relation[] inherit(Entity origin)
  {
    if(DEBUG)
     System.out.println("Starting INHERIT in "+origin.toString());

    Vector inherited = new Vector();
    Vector alreadyInherited = new Vector();
    Vector inheritSource = new Vector();

/*    if(origin.getKnowledgeModel().getInferenceCache().isCached("InheritanceMethod.inherit", origin))
      inherited = (Vector) origin.getKnowledgeModel().getInferenceCache().get("InheritanceMethod.inherit", origin);
    else
    {
*/
    ExplanationModel model = execute(origin);
    Relation[] rels = model.getRelations();

    for(int i=rels.length-1;i>=0;i--)
    {
      if(DEBUG)
        System.out.println("  -> "+rels[i].toString()+ "    origin is : "+origin);
      if(!rels[i].getFrom().equals(origin))
      {
        if(acceptInheritance(rels[i]))
        {
          if(DEBUG)
            System.out.println("     accepted - is not from origin but inheritable.");
//          RelationClassAndEntity rcae = new RelationClassAndEntity((Entity) rels[i].getValue(), rels[i].getRelationType());
          if(!alreadyInherited.contains(rels[i]))
          {
            Relation inheritedRelation = rels[i].inherit(origin, model);
//            if(inheritedRelation.getExplanationStrength() > 0.0)
//            {
              inherited.addElement(inheritedRelation);
              inheritSource.addElement(rels[i]);
              alreadyInherited.addElement(rels[i]);
//            }
          }
          else if(DEBUG)
            System.out.println(" -alreadyInherited.");
        }
      }
    }

    for(int i=rels.length-1;i>=0;i--)
    {
      if(DEBUG)
        System.out.println("  .-> "+rels[i].toString());
      if(rels[i].getFrom().equals(origin))
      {
        if(DEBUG)
          System.out.println("      accepted - is from origin");
        RelationClassAndEntity rcae = new RelationClassAndEntity((Entity) rels[i].getValue(), rels[i].getRelationType());
        if(!alreadyInherited.contains(rels[i]))
        {
          inherited.addElement(rels[i]);
          inheritSource.addElement(rels[i]);
          alreadyInherited.addElement(rels[i]);
        }
        else    if(DEBUG)
          System.out.println(" -alreadyInherited.");
      }
    }
    if(DEBUG)
         System.out.println("Overrider check");
    Vector checkedInherited = (Vector) inherited.clone();
    for(int i=inherited.size()-1;i>=0;i--)
    {
      Relation rel = (Relation) inherited.elementAt(i);
      if(DEBUG)
      {
        System.out.println("  -> " + rel.toString() + " I am overriding " +
                           java.util.Arrays.asList(rel.getOverridingRelations()));
        System.out.println("        overriders: "+java.util.Arrays.asList(rel.getOverridenBy()));
      }
      if(rel.isInherited())
      {
        boolean remove = false;
        Relation[] overriders = rel.getOverridenBy();
//        if(rel.isNegated())
//          remove = true;
//        else
          for(int j=overriders.length-1;j>=0;j--)
          {
            if(DEBUG)
              System.out.println("    --> "+overriders[j]);
            if(inheritSource.contains(overriders[j]))
              remove = true;
          }
        if(remove)
        {
          if(DEBUG)
            System.out.println("Relation "+rel+" is overridden. Removing.");
          checkedInherited.removeElement(rel);
        }
      }
    }
    inherited = checkedInherited;

/*    checkedInherited = new Vector();

    // Test to ensure that only one relation of each type to each value is inherited.
    // If more than one is found, it is added to the "original relation" list of
    // the inherited instead.
    Hashtable typeValueTable = new Hashtable();
    for(int i=inherited.size()-1;i>=0;i--)
    {
      Relation rel = (Relation) inherited.elementAt(i);
//      Integer hash = new Integer(rel.getRelationType().hashCode()+7*rel.getValue().hashCode();
      HashSet hash = new HashSet();
      hash.add(rel.getRelationType());
      hash.add(rel.getValue());

      if(rel.isInherited())
      {
        if(typeValueTable.containsKey(hash))
        {
          Relation first = (Relation) typeValueTable.get(hash);
          first.addOriginal(rel.getOriginal()[0]);
          first.getExplanationModel().joinWith(rel.getExplanationModel());
        }
        else
        {
          typeValueTable.put(hash, rel);
          checkedInherited.add(rel);
        }
      }
      else
        checkedInherited.add(rel);
    }
/*    origin.getKnowledgeModel().getInferenceCache().put("InheritanceMethod.inherit", origin, inherited);

    }*/

    inherited = checkedInherited;

    if(DEBUG)
      System.out.println("End INHERIT in "+origin.toString()+" "+inherited);
    Relation[] relArray = new Relation[inherited.size()];
    inherited.copyInto(relArray);

    return relArray;
  }

  public abstract boolean acceptInheritance(Relation r);
}

class RelationClassAndEntity
{
  RelationType r;
  Entity e;

  RelationClassAndEntity(Entity e, RelationType r)
  {
    this.e = e;
    this.r = r;
  }

  public boolean equals(Object ob)
  {
    if(ob.getClass() == getClass())
    {
      if(((RelationClassAndEntity) ob).e.equals(e) &&
        ((RelationClassAndEntity) ob).r.equals(r))
        return true;
    }
    return super.equals(ob);
  }
}

