package jcreek.representation.inference;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Hashtable;
import jcreek.representation.*; import jcreek.representation.cbr.*;

/**
 * <p>Title: SortedPlausibleInheritanceMethod</p>
 * <p>Description: Same as it's superclass, PlausibleInheritanceMethod, but the relations
 * inherited are sorted according to the relation-type and value-type before they are returned.
 * This is used when comparing features of cases.</p>
 * @author Frode Srom
 * @version 1.0
 */

public class SortedPlausibleInheritanceMethod extends PlausibleInheritanceMethod implements Comparator
{
  private Hashtable h;
  public SortedPlausibleInheritanceMethod(RelationType rcd)
  {
    super(rcd);
  }

  public SortedPlausibleInheritanceMethod(KnowledgeModel knowledgeModel)
  {
    super(knowledgeModel);
  }

  public Relation[] inherit(Entity e)
  {
    Relation[] rels = super.inherit(e);

//    h = new Hashtable(300);
    Arrays.sort(rels, this);
    return rels;
  }

  public int compare(Object a, Object b)
  {
    Relation r1 = (Relation) a;
    Relation r2 = (Relation) b;
/*
    return r1.getName().compareTo(r2.getName());
  }
*/
    if(r1.getRelationType().equals(r2.getRelationType()))
    {
      String r1val = null;
//      if(h.containsKey(r1))
//        r1val = (String) h.get(r1);
//      else
//      {
        try
        {
          Relation val = r1.getValue().getLocalRelation(new String[] {"value of", "instance of", "subclass of"});
          r1val = val.getValue().getName();
//          h.put(r1, r1val);
        }
        catch(NoSuchRelationException e)
        {
        }
        catch(NoSuchRelationTypeException e)
        {
        }
//      }

      String r2val = null;
//      if(h.containsKey(r2))
//        r2val = (String) h.get(r2);
//      else
//      {
        try
        {
          Relation val = r2.getValue().getLocalRelation(new String[] {"value of", "instance of", "subclass of"});
          r2val = val.getValue().getName();
//          h.put(r2, r2val);
        }
        catch(NoSuchRelationException e)
        {
        }
        catch(NoSuchRelationTypeException e)
        {
        }
//      }

      if(r1val == r2val)
        return 0;
      if(r1val == null)
        return -1;
      if(r2val == null)
        return 1;
      return r1val.compareTo(r2val);
    }
    else
      return r1.getRelationType().getName().compareTo(r2.getRelationType().getName());
  }
}
