package jcreek.util.graph;

import java.lang.Cloneable;
import java.io.Serializable;

public abstract class Edge implements Serializable, Cloneable
{
  protected Node from;
  protected Node to;
  protected Graph graph = null;

  /**
   * Creates a new edge. It is not automatically added to any graph.
   */
  public Edge(Node from, Node to)
  {
    this.from = from;
//    from.addEdge(this);
    this.to = to;
  }

  protected Edge()
  {
  }

  /**
   * Returns the node this edge is from.
   */
  public Node getFrom()
  {
    return from;
  }

  /**
   * Returns the node this edge is from.
   */
  public Node getTo()
  {
    return to;
  }

  /**
   * Returns a clone of this edge.
   */
  public Object clone()
  {
    try
    {
      return super.clone();
    }
    catch(CloneNotSupportedException e)
    {
      // Should never happen since this class implements clonable.
      return null;
    }
  }

  /**
   * Sets the graph to which this edge is a member.
   */
  void setGraph(Graph g)
  {
    graph = g;
  }

  /**
   * Ask to remove this edge from the current graph.
   */
  public void remove()
  {
    if(graph != null)
      graph.removeEdge(this);
  }

  /**
   * Notify the edge that it has been removed from a graph.
   */
  void removeP()
  {
    getFrom().removeEdgeP(this);
    getTo().removeEdgeP(this);
    graph = null;
  }
}