package weka.classifiers.lazy;

import java.util.Vector;

import weka.core.*;
import weka.classifiers.Classifier;
import weka.classifiers.Evaluation;
import weka.classifiers.functions.LinearRegression;
import weka.filters.unsupervised.attribute.MakeIndicator;
import weka.filters.unsupervised.attribute.Normalize;
import weka.filters.unsupervised.attribute.ReplaceMissingValues;
import weka.filters.unsupervised.attribute.NominalToBinary;
import weka.filters.Filter;

import jcreek.representation.*; import jcreek.representation.cbr.*;
import jcreek.reasoning.ReuseResult;
import jcreek.reasoning.RetrieveResult;

/**
 * <p>Title: </p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author unascribed
 * @version 1.0
 */

public class CreekCBR extends Classifier
{
  protected KnowledgeModel km;
  protected int classifiedCases = 0;
  protected Classifier[] regressor;

  public void buildClassifier(Instances instances) throws Exception
  {
    if (!instances.classAttribute().isNominal()) {
	throw new UnsupportedClassTypeException("Class attribute has to be nominal for Creek!");
    }

    km = new LocalKnowledgeModel();

    for(int i=0;i<instances.numAttributes();i++)
    {
      Attribute attribute = instances.attribute(i);
      Entity attributeEntity = new Entity(km, attribute.name(), attribute.name());
      attributeEntity.setEntityObject(new Integer(i));
      if(attribute.isNominal())
      {
//        System.out.println("Creating "+attributeEntity.getName());
        attributeEntity.addRelation("instance of", km.getEntity("Symbol"));
        for(int j=0;j<attribute.numValues();j++)
        {
          Entity valueEntity = new Entity(km, attribute.name()+"="+attribute.value(j), "");
          valueEntity.setEntityObject(new Integer(j));
          valueEntity.addRelation("instance of", attributeEntity);
//          System.out.println("  -> "+valueEntity.getName());
        }

      }
      else if(attribute.isNumeric())
        attributeEntity.addRelation("instance of", km.getEntity("Number"));



    }

    classifiedCases = instances.numInstances();

    km.getInferenceCache().suspendFlush(true);

    MakeIndicator indicatorFilter;
    Normalize normalizeFilter;
    for (int i = 0; i < instances.numClasses(); i++)
    {
      System.out.println("Creating regressor for "+instances.classAttribute().value(i));
      LinearRegression regressor = new LinearRegression();
      indicatorFilter = new MakeIndicator();
      indicatorFilter.setAttributeIndex(instances.classIndex());
      indicatorFilter.setValueIndex(i);
      indicatorFilter.setNumeric(true);
      indicatorFilter.setInputFormat(instances);
      Instances newInsts = Filter.useFilter(instances, indicatorFilter);
      normalizeFilter = new Normalize();
      normalizeFilter.setInputFormat(newInsts);
      newInsts = Filter.useFilter(newInsts, normalizeFilter);
      NominalToBinary nominalFilter = new NominalToBinary();
      nominalFilter.setInputFormat(newInsts);
      newInsts = Filter.useFilter(newInsts, nominalFilter);
      ReplaceMissingValues missingFilter = new ReplaceMissingValues();
      missingFilter.setInputFormat(newInsts);
      newInsts = Filter.useFilter(newInsts, missingFilter);
      newInsts.deleteWithMissingClass();

      regressor.turnChecksOff();

//      regressor.setEliminateColinearAttributes(false);
//      regressor.setAttributeSelectionMethod(new SelectedTag(LinearRegression.SELECTION_NONE, LinearRegression.TAGS_SELECTION));
      regressor.buildClassifier(newInsts);

      /*

      for(int j=0;j<newInsts.ncoeffs;j++)
      {
        System.out.println("  "+j+": "+regressor.coeffs[j]);
      }
   */
      double[] coeffs = regressor.coefficients();
      for(int j=0;j<newInsts.numAttributes();j++)
      {
//        System.out.println("Suggesting implication from "+newInsts.attribute(j).name()+" to "+instances.attribute(instances.classIndex()).value(i)+" of "+coeffs[j]);
        if(newInsts.classIndex() != j)
        {
          Entity solutionEntity = km.getEntity(instances.classAttribute().name()+"="+instances.classAttribute().value(i));

          Entity coeffEntity = null;
          if(newInsts.attribute(j).name().indexOf("=") == -1)
          {
            if(coeffs[j] >= 0.0)
              coeffEntity = km.getEntity(newInsts.attribute(j).name()+"=t");
            else
            {
              coeffEntity = km.getEntity(newInsts.attribute(j).name()+"=f");
              coeffs[j] = -coeffs[j];
            }
          }
          else
            coeffEntity = km.getEntity(newInsts.attribute(j).name());

          if(solutionEntity != null && coeffEntity != null && coeffs[j] > 0.01)
          {
//            System.out.println("Suggesting implication from "+coeffEntity.getName()+" to "+solutionEntity.getName()+" of "+coeffs[j]);
            coeffEntity.addRelation("implies", solutionEntity, coeffs[j]);
          }
        }
      }
    }
    for(int i=0;i<instances.numInstances();i++)
    {
        Instance wekaInstance = instances.instance(i);
        Case creekCase = new Case(km, "Case#"+i, "Case#", null);
        creekCase.setEntityObject(new Integer(i));

        buildCase(wekaInstance, creekCase);

        creekCase.setStatus(Case.SOLVEDCASE);
//        creekCase.calculateDefaultRelevance();
    }
//      System.out.println(regressor.toString());
//    km.save("audiology");
  }

  public double classifyInstance(Instance instance) throws Exception
  {
    classifiedCases++;
    Case creekCase = new Case(km, "Case#"+classifiedCases, "Case#"+classifiedCases, null);

    buildCase(instance, creekCase);

    creekCase.setStatus(Case.UNSOLVEDCASE);

    RetrieveResult retrieve = new RetrieveResult(creekCase);
    System.out.println(retrieve);
    ReuseResult reuse = new ReuseResult(retrieve);
    Entity solution = reuse.getSolution();
//    System.out.println("Solution for "+creekCase.getName()+" is "+solution.getName()+" (matched to "+reuse.getCase(0));
    System.out.println(reuse);
    return ((Integer) solution.getEntityObject()).doubleValue();
  }

  private void buildCase(Instance wekaInstance, Case creekCase) throws NoSuchRelationTypeException
  {
    creekCase.addRelation("instance of", km.getEntity("Case"));
    for(int j=0;j<wekaInstance.numAttributes();j++)
    {
      Attribute attr = wekaInstance.attribute(j);
      String relationType;
      if(wekaInstance.classAttribute().equals(attr))
        relationType = "has solution";
      else
        relationType = "has finding";

      if(!wekaInstance.isMissing(j))
      {
        if(attr.isNominal())
          creekCase.addRelation(relationType, km.getEntity(wekaInstance.attribute(j).name()+"="+wekaInstance.attribute(j).value((int) wekaInstance.value(j))));
        else if(attr.isNumeric())
        {
          Entity numeric = new jcreek.representation.NumberEntity(km, new Double(wekaInstance.value(j)), km.getEntity(wekaInstance.attribute(j).name()));
          creekCase.addRelation(relationType, numeric);
        }
      }
    }
  }

  public static void main(String argv[])
  {
    try {
      System.out.println(Evaluation.evaluateModel(new CreekCBR(), argv));
    } catch (Exception e) {
      System.err.println(e.getMessage());
    }
  }
}
